<?php
/** Adminer - Compact database management
* @link https://www.adminer.org/
* @author Jakub Vrana, https://www.vrana.cz/
* @copyright 2007 Jakub Vrana
* @license https://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
* @license https://www.gnu.org/licenses/gpl-2.0.html GNU General Public License, version 2 (one or other)
* @version 5.1.0
*/
// this is matched by compile.php

namespace Adminer;

?>
<?php
?>
<?php
$VERSION = "5.1.0";

?>
<?php
error_reporting(24575); // all but E_DEPRECATED (overriding mysqli methods without types is deprecated)
set_error_handler(function ($errno, $errstr) {
	// "offset on null" mutes $_GET["fields"][0] if there's no ?fields[]= (62017e3 is a wrong fix for this)
	// "Undefined array key" mutes $_GET["q"] if there's no ?q=
	// "Undefined offset" and "Undefined index" are older messages for the same thing
	return !!preg_match('~^(Trying to access array offset on( value of type)? null|Undefined (array key|offset|index))~', $errstr);
}, E_WARNING | E_NOTICE); // warning since PHP 8.0

// this is matched by compile.php


// disable filter.default
$filter = !preg_match('~^(unsafe_raw)?$~', ini_get("filter.default"));
if ($filter || ini_get("filter.default_flags")) {
	foreach (array('_GET', '_POST', '_COOKIE', '_SERVER') as $val) {
		$unsafe = filter_input_array(constant("INPUT$val"), FILTER_UNSAFE_RAW);
		if ($unsafe) {
			$$val = $unsafe;
		}
	}
}

if (function_exists("mb_internal_encoding")) {
	mb_internal_encoding("8bit");
}

?>
<?php
// This file is used both in Adminer and Adminer Editor.

/** Get database connection
* @return Db
*/
function connection() {
	// can be used in customization, $connection is minified
	global $connection;
	return $connection;
}

/** Get Adminer object
* @return Adminer
*/
function adminer() {
	global $adminer;
	return $adminer;
}

/** Get Driver object
* @return Driver
*/
function driver() {
	global $driver;
	return $driver;
}

/** Get Adminer version
* @return string
*/
function version() {
	global $VERSION;
	return $VERSION;
}

/** Unescape database identifier
* @param string text inside ``
* @return string
*/
function idf_unescape($idf) {
	if (!preg_match('~^[`\'"[]~', $idf)) {
		return $idf;
	}
	$last = substr($idf, -1);
	return str_replace($last . $last, $last, substr($idf, 1, -1));
}

/** Shortcut for $connection->quote($string)
* @param string
* @return string
*/
function q($string) {
	global $connection;
	return $connection->quote($string);
}

/** Escape string to use inside ''
* @param string
* @return string
*/
function escape_string($val) {
	return substr(q($val), 1, -1);
}

/** Remove non-digits from a string
* @param string
* @return string
*/
function number($val) {
	return preg_replace('~[^0-9]+~', '', $val);
}

/** Get regular expression to match numeric types
* @return string
*/
function number_type() {
	return '((?<!o)int(?!er)|numeric|real|float|double|decimal|money)'; // not point, not interval
}

/** Disable magic_quotes_gpc
* @param array e.g. (&$_GET, &$_POST, &$_COOKIE)
* @param bool whether to leave values as is
* @return null modified in place
*/
function remove_slashes($process, $filter = false) {
	if (function_exists("get_magic_quotes_gpc") && get_magic_quotes_gpc()) {
		while (list($key, $val) = each($process)) {
			foreach ($val as $k => $v) {
				unset($process[$key][$k]);
				if (is_array($v)) {
					$process[$key][stripslashes($k)] = $v;
					$process[] = &$process[$key][stripslashes($k)];
				} else {
					$process[$key][stripslashes($k)] = ($filter ? $v : stripslashes($v));
				}
			}
		}
	}
}

/** Escape or unescape string to use inside form []
* @param string
* @param bool
* @return string
*/
function bracket_escape($idf, $back = false) {
	// escape brackets inside name="x[]"
	static $trans = array(':' => ':1', ']' => ':2', '[' => ':3', '"' => ':4');
	return strtr($idf, ($back ? array_flip($trans) : $trans));
}

/** Check if connection has at least the given version
* @param string required version
* @param string required MariaDB version
* @param Db defaults to $connection
* @return bool
*/
function min_version($version, $maria_db = "", $connection2 = null) {
	global $connection;
	if (!$connection2) {
		$connection2 = $connection;
	}
	$server_info = $connection2->server_info;
	if ($maria_db && preg_match('~([\d.]+)-MariaDB~', $server_info, $match)) {
		$server_info = $match[1];
		$version = $maria_db;
	}
	return $version && version_compare($server_info, $version) >= 0;
}

/** Get connection charset
* @param Db
* @return string
*/
function charset($connection) {
	return (min_version("5.5.3", 0, $connection) ? "utf8mb4" : "utf8"); // SHOW CHARSET would require an extra query
}

/** Get INI boolean value
* @param string
* @return bool
*/
function ini_bool($ini) {
	$val = ini_get($ini);
	return (preg_match('~^(on|true|yes)$~i', $val) || (int) $val); // boolean values set by php_value are strings
}

/** Check if SID is necessary
* @return bool
*/
function sid() {
	static $return;
	if ($return === null) { // restart_session() defines SID
		$return = (SID && !($_COOKIE && ini_bool("session.use_cookies"))); // $_COOKIE - don't pass SID with permanent login
	}
	return $return;
}

/** Set password to session
* @param string
* @param string
* @param string
* @param string
* @return null
*/
function set_password($vendor, $server, $username, $password) {
	$_SESSION["pwds"][$vendor][$server][$username] = ($_COOKIE["adminer_key"] && is_string($password)
		? array(encrypt_string($password, $_COOKIE["adminer_key"]))
		: $password
	);
}

/** Get password from session
* @return string or null for missing password or false for expired password
*/
function get_password() {
	$return = get_session("pwds");
	if (is_array($return)) {
		$return = ($_COOKIE["adminer_key"]
			? decrypt_string($return[0], $_COOKIE["adminer_key"])
			: false
		);
	}
	return $return;
}

/** Get single value from database
* @param string
* @param int
* @return string or false if error
*/
function get_val($query, $field = 0) {
	global $connection;
	return $connection->result($query, $field);
}

/** Get list of values from database
* @param string
* @param mixed
* @return array
*/
function get_vals($query, $column = 0) {
	global $connection;
	$return = array();
	$result = $connection->query($query);
	if (is_object($result)) {
		while ($row = $result->fetch_row()) {
			$return[] = $row[$column];
		}
	}
	return $return;
}

/** Get keys from first column and values from second
* @param string
* @param Db
* @param bool
* @return array
*/
function get_key_vals($query, $connection2 = null, $set_keys = true) {
	global $connection;
	if (!is_object($connection2)) {
		$connection2 = $connection;
	}
	$return = array();
	$result = $connection2->query($query);
	if (is_object($result)) {
		while ($row = $result->fetch_row()) {
			if ($set_keys) {
				$return[$row[0]] = $row[1];
			} else {
				$return[] = $row[0];
			}
		}
	}
	return $return;
}

/** Get all rows of result
* @param string
* @param Db
* @param string
* @return array of associative arrays
*/
function get_rows($query, $connection2 = null, $error = "<p class='error'>") {
	global $connection;
	$conn = (is_object($connection2) ? $connection2 : $connection);
	$return = array();
	$result = $conn->query($query);
	if (is_object($result)) { // can return true
		while ($row = $result->fetch_assoc()) {
			$return[] = $row;
		}
	} elseif (!$result && !is_object($connection2) && $error && (defined('Adminer\PAGE_HEADER') || $error == "-- ")) {
		echo $error . error() . "\n";
	}
	return $return;
}

/** Find unique identifier of a row
* @param array
* @param array result of indexes()
* @return array or null if there is no unique identifier
*/
function unique_array($row, $indexes) {
	foreach ($indexes as $index) {
		if (preg_match("~PRIMARY|UNIQUE~", $index["type"])) {
			$return = array();
			foreach ($index["columns"] as $key) {
				if (!isset($row[$key])) { // NULL is ambiguous
					continue 2;
				}
				$return[$key] = $row[$key];
			}
			return $return;
		}
	}
}

/** Escape column key used in where()
* @param string
* @return string
*/
function escape_key($key) {
	if (preg_match('(^([\w(]+)(' . str_replace("_", ".*", preg_quote(idf_escape("_"))) . ')([ \w)]+)$)', $key, $match)) { //! columns looking like functions
		return $match[1] . idf_escape(idf_unescape($match[2])) . $match[3]; //! SQL injection
	}
	return idf_escape($key);
}

/** Create SQL condition from parsed query string
* @param array parsed query string
* @param array
* @return string
*/
function where($where, $fields = array()) {
	global $connection;
	$return = array();
	foreach ((array) $where["where"] as $key => $val) {
		$key = bracket_escape($key, 1); // 1 - back
		$column = escape_key($key);
		$field_type = $fields[$key]["type"];
		$return[] = $column
			. (JUSH == "sql" && $field_type == "json" ? " = CAST(" . q($val) . " AS JSON)"
				: (JUSH == "sql" && is_numeric($val) && preg_match('~\.~', $val) ? " LIKE " . q($val) // LIKE because of floats but slow with ints
				: (JUSH == "mssql" && strpos($field_type, "datetime") === false ? " LIKE " . q(preg_replace('~[_%[]~', '[\0]', $val)) // LIKE because of text but it does not work with datetime
				: " = " . unconvert_field($fields[$key], q($val)))))
		; //! enum and set
		if (JUSH == "sql" && preg_match('~char|text~', $field_type) && preg_match("~[^ -@]~", $val)) { // not just [a-z] to catch non-ASCII characters
			$return[] = "$column = " . q($val) . " COLLATE " . charset($connection) . "_bin";
		}
	}
	foreach ((array) $where["null"] as $key) {
		$return[] = escape_key($key) . " IS NULL";
	}
	return implode(" AND ", $return);
}

/** Create SQL condition from query string
* @param string
* @param array
* @return string
*/
function where_check($val, $fields = array()) {
	parse_str($val, $check);
	remove_slashes(array(&$check));
	return where($check, $fields);
}

/** Create query string where condition from value
* @param int condition order
* @param string column identifier
* @param string
* @param string
* @return string
*/
function where_link($i, $column, $value, $operator = "=") {
	return "&where%5B$i%5D%5Bcol%5D=" . urlencode($column) . "&where%5B$i%5D%5Bop%5D=" . urlencode(($value !== null ? $operator : "IS NULL")) . "&where%5B$i%5D%5Bval%5D=" . urlencode($value);
}

/** Get select clause for convertible fields
* @param array
* @param array
* @param array
* @return string
*/
function convert_fields($columns, $fields, $select = array()) {
	$return = "";
	foreach ($columns as $key => $val) {
		if ($select && !in_array(idf_escape($key), $select)) {
			continue;
		}
		$as = convert_field($fields[$key]);
		if ($as) {
			$return .= ", $as AS " . idf_escape($key);
		}
	}
	return $return;
}

/** Set cookie valid on current path
* @param string
* @param string
* @param int number of seconds, 0 for session cookie, 2592000 - 30 days
* @return bool
*/
function cookie($name, $value, $lifetime = 2592000) {
	global $HTTPS;
	return header(
		"Set-Cookie: $name=" . urlencode($value)
			. ($lifetime ? "; expires=" . gmdate("D, d M Y H:i:s", time() + $lifetime) . " GMT" : "")
			. "; path=" . preg_replace('~\?.*~', '', $_SERVER["REQUEST_URI"])
			. ($HTTPS ? "; secure" : "")
			. "; HttpOnly; SameSite=lax",
		false
	);
}

/** Get settings stored in a cookie
* @param string
* @return array
*/
function get_settings($cookie) {
	parse_str($_COOKIE[$cookie], $settings);
	return $settings;
}

/** Get setting stored in a cookie
* @param string
* @param string
* @return mixed
*/
function get_setting($key, $cookie = "adminer_settings") {
	$settings = get_settings($cookie);
	return $settings[$key];
}

/** Store settings to a cookie
* @param array
* @param string
* @return bool
*/
function save_settings($settings, $cookie = "adminer_settings") {
	return cookie($cookie, http_build_query($settings + get_settings($cookie)));
}

/** Restart stopped session
* @return null
*/
function restart_session() {
	if (!ini_bool("session.use_cookies") && (!function_exists('session_status') || session_status() == 1)) { // 1 - PHP_SESSION_NONE, session_status() available since PHP 5.4
		session_start();
	}
}

/** Stop session if possible
* @param bool
* @return null
*/
function stop_session($force = false) {
	$use_cookies = ini_bool("session.use_cookies");
	if (!$use_cookies || $force) {
		session_write_close(); // improves concurrency if a user opens several pages at once, may be restarted later
		if ($use_cookies && @ini_set("session.use_cookies", false) === false) { // @ - may be disabled
			session_start();
		}
	}
}

/** Get session variable for current server
* @param string
* @return mixed
*/
function &get_session($key) {
	return $_SESSION[$key][DRIVER][SERVER][$_GET["username"]];
}

/** Set session variable for current server
* @param string
* @param mixed
* @return mixed
*/
function set_session($key, $val) {
	$_SESSION[$key][DRIVER][SERVER][$_GET["username"]] = $val; // used also in auth.inc.php
}

/** Get authenticated URL
* @param string
* @param string
* @param string
* @param string
* @return string
*/
function auth_url($vendor, $server, $username, $db = null) {
	global $drivers;
	$uri = remove_from_uri(implode("|", array_keys($drivers))
		. "|username|ext|"
		. ($db !== null ? "db|" : "")
		. ($vendor == 'mssql' || $vendor == 'pgsql' ? "" : "ns|") // we don't have access to support() here
		. session_name())
	;
	preg_match('~([^?]*)\??(.*)~', $uri, $match);
	return "$match[1]?"
		. (sid() ? SID . "&" : "")
		. ($vendor != "server" || $server != "" ? urlencode($vendor) . "=" . urlencode($server) . "&" : "")
		. ($_GET["ext"] ? "ext=" . urlencode($_GET["ext"]) . "&" : "")
		. "username=" . urlencode($username)
		. ($db != "" ? "&db=" . urlencode($db) : "")
		. ($match[2] ? "&$match[2]" : "")
	;
}

/** Find whether it is an AJAX request
* @return bool
*/
function is_ajax() {
	return ($_SERVER["HTTP_X_REQUESTED_WITH"] == "XMLHttpRequest");
}

/** Send Location header and exit
* @param string null to only set a message
* @param string
* @return null
*/
function redirect($location, $message = null) {
	if ($message !== null) {
		restart_session();
		$_SESSION["messages"][preg_replace('~^[^?]*~', '', ($location !== null ? $location : $_SERVER["REQUEST_URI"]))][] = $message;
	}
	if ($location !== null) {
		if ($location == "") {
			$location = ".";
		}
		header("Location: $location");
		exit;
	}
}

/** Execute query and redirect if successful
* @param string
* @param string
* @param string
* @param bool
* @param bool
* @param bool
* @param string
* @return bool
*/
function query_redirect($query, $location, $message, $redirect = true, $execute = true, $failed = false, $time = "") {
	global $connection, $error, $adminer;
	if ($execute) {
		$start = microtime(true);
		$failed = !$connection->query($query);
		$time = format_time($start);
	}
	$sql = "";
	if ($query) {
		$sql = $adminer->messageQuery($query, $time, $failed);
	}
	if ($failed) {
		$error = error() . $sql . script("messagesPrint();");
		return false;
	}
	if ($redirect) {
		redirect($location, $message . $sql);
	}
	return true;
}

/** Execute and remember query
* @param string or null to return remembered queries, end with ';' to use DELIMITER
* @return Result or [$queries, $time] if $query = null
*/
function queries($query) {
	global $connection;
	static $queries = array();
	static $start;
	if (!$start) {
		$start = microtime(true);
	}
	if ($query === null) {
		// return executed queries
		return array(implode("\n", $queries), format_time($start));
	}
	$queries[] = (preg_match('~;$~', $query) ? "DELIMITER ;;\n$query;\nDELIMITER " : $query) . ";";
	return $connection->query($query);
}

/** Apply command to all array items
* @param string
* @param array
* @param callback
* @return bool
*/
function apply_queries($query, $tables, $escape = 'Adminer\table') {
	foreach ($tables as $table) {
		if (!queries("$query " . $escape($table))) {
			return false;
		}
	}
	return true;
}

/** Redirect by remembered queries
* @param string
* @param string
* @param bool
* @return bool
*/
function queries_redirect($location, $message, $redirect) {
	list($queries, $time) = queries(null);
	return query_redirect($queries, $location, $message, $redirect, false, !$redirect, $time);
}

/** Format elapsed time
* @param float output of microtime(true)
* @return string HTML code
*/
function format_time($start) {
	return lang(0, max(0, microtime(true) - $start));
}

/** Get relative REQUEST_URI
* @return string
*/
function relative_uri() {
	return str_replace(":", "%3a", preg_replace('~^[^?]*/([^?]*)~', '\1', $_SERVER["REQUEST_URI"]));
}

/** Remove parameter from query string
* @param string
* @return string
*/
function remove_from_uri($param = "") {
	return substr(preg_replace("~(?<=[?&])($param" . (SID ? "" : "|" . session_name()) . ")=[^&]*&~", '', relative_uri() . "&"), 0, -1);
}

/** Get file contents from $_FILES
* @param string
* @param bool
* @param string
* @return mixed int for error, string otherwise
*/
function get_file($key, $decompress = false, $delimiter = "") {
	$file = $_FILES[$key];
	if (!$file) {
		return null;
	}
	foreach ($file as $key => $val) {
		$file[$key] = (array) $val;
	}
	$return = '';
	foreach ($file["error"] as $key => $error) {
		if ($error) {
			return $error;
		}
		$name = $file["name"][$key];
		$tmp_name = $file["tmp_name"][$key];
		$content = file_get_contents(
			$decompress && preg_match('~\.gz$~', $name)
			? "compress.zlib://$tmp_name"
			: $tmp_name
		); //! may not be reachable because of open_basedir
		if ($decompress) {
			$start = substr($content, 0, 3);
			if (function_exists("iconv") && preg_match("~^\xFE\xFF|^\xFF\xFE~", $start)) { // not ternary operator to save memory
				$content = iconv("utf-16", "utf-8", $content);
			} elseif ($start == "\xEF\xBB\xBF") { // UTF-8 BOM
				$content = substr($content, 3);
			}
		}
		$return .= $content;
		if ($delimiter) {
			$return .= (preg_match("($delimiter\\s*\$)", $content) ? "" : $delimiter) . "\n\n";
		}
	}
	return $return;
}

/** Determine upload error
* @param int
* @return string
*/
function upload_error($error) {
	$max_size = ($error == UPLOAD_ERR_INI_SIZE ? ini_get("upload_max_filesize") : 0); // post_max_size is checked in index.php
	return ($error ? lang(1) . ($max_size ? " " . lang(2, $max_size) : "") : lang(3));
}

/** Create repeat pattern for preg
* @param string
* @param int
* @return string
*/
function repeat_pattern($pattern, $length) {
	// fix for Compilation failed: number too big in {} quantifier
	return str_repeat("$pattern{0,65535}", $length / 65535) . "$pattern{0," . ($length % 65535) . "}"; // can create {0,0} which is OK
}

/** Check whether the string is in UTF-8
* @param string
* @return bool
*/
function is_utf8($val) {
	// don't print control chars except \t\r\n
	return (preg_match('~~u', $val) && !preg_match('~[\0-\x8\xB\xC\xE-\x1F]~', $val));
}

/** Shorten UTF-8 string
* @param string
* @param int
* @param string
* @return string escaped string with appended ...
*/
function shorten_utf8($string, $length = 80, $suffix = "") {
	if (!preg_match("(^(" . repeat_pattern("[\t\r\n -\x{10FFFF}]", $length) . ")($)?)u", $string, $match)) { // ~s causes trash in $match[2] under some PHP versions, (.|\n) is slow
		preg_match("(^(" . repeat_pattern("[\t\r\n -~]", $length) . ")($)?)", $string, $match);
	}
	return h($match[1]) . $suffix . (isset($match[2]) ? "" : "<i>…</i>");
}

/** Format decimal number
* @param int
* @return string
*/
function format_number($val) {
	return strtr(number_format($val, 0, ".", lang(4)), preg_split('~~u', lang(5), -1, PREG_SPLIT_NO_EMPTY));
}

/** Generate friendly URL
* @param string
* @return string
*/
function friendly_url($val) {
	// used for blobs and export
	return preg_replace('~\W~i', '-', $val);
}

/** Get status of a single table and fall back to name on error
* @param string
* @param bool
* @return array
*/
function table_status1($table, $fast = false) {
	$return = table_status($table, $fast);
	return ($return ?: array("Name" => $table));
}

/** Find out foreign keys for each column
* @param string
* @return array [$col => []]
*/
function column_foreign_keys($table) {
	global $adminer;
	$return = array();
	foreach ($adminer->foreignKeys($table) as $foreign_key) {
		foreach ($foreign_key["source"] as $val) {
			$return[$val][] = $foreign_key;
		}
	}
	return $return;
}

/** Compute fields() from $_POST edit data
* @return array
*/
function fields_from_edit() {
	global $driver;
	$return = array();
	foreach ((array) $_POST["field_keys"] as $key => $val) {
		if ($val != "") {
			$val = bracket_escape($val);
			$_POST["function"][$val] = $_POST["field_funs"][$key];
			$_POST["fields"][$val] = $_POST["field_vals"][$key];
		}
	}
	foreach ((array) $_POST["fields"] as $key => $val) {
		$name = bracket_escape($key, 1); // 1 - back
		$return[$name] = array(
			"field" => $name,
			"privileges" => array("insert" => 1, "update" => 1, "where" => 1, "order" => 1),
			"null" => 1,
			"auto_increment" => ($key == $driver->primary),
		);
	}
	return $return;
}

/** Send headers for export
* @param string
* @param bool
* @return string extension
*/
function dump_headers($identifier, $multi_table = false) {
	global $adminer;
	$return = $adminer->dumpHeaders($identifier, $multi_table);
	$output = $_POST["output"];
	if ($output != "text") {
		header("Content-Disposition: attachment; filename=" . $adminer->dumpFilename($identifier) . ".$return" . ($output != "file" && preg_match('~^[0-9a-z]+$~', $output) ? ".$output" : ""));
	}
	session_write_close();
	if (!ob_get_level()) {
		ob_start(null, 4096);
	}
	ob_flush();
	flush();
	return $return;
}

/** Print CSV row
* @param array
* @return null
*/
function dump_csv($row) {
	foreach ($row as $key => $val) {
		if (preg_match('~["\n,;\t]|^0|\.\d*0$~', $val) || $val === "") {
			$row[$key] = '"' . str_replace('"', '""', $val) . '"';
		}
	}
	echo implode(($_POST["format"] == "csv" ? "," : ($_POST["format"] == "tsv" ? "\t" : ";")), $row) . "\r\n";
}

/** Apply SQL function
* @param string
* @param string escaped column identifier
* @return string
*/
function apply_sql_function($function, $column) {
	return ($function ? ($function == "unixepoch" ? "DATETIME($column, '$function')" : ($function == "count distinct" ? "COUNT(DISTINCT " : strtoupper("$function(")) . "$column)") : $column);
}

/** Get path of the temporary directory
* @return string
*/
function get_temp_dir() {
	$return = ini_get("upload_tmp_dir"); // session_save_path() may contain other storage path
	if (!$return) {
		if (function_exists('sys_get_temp_dir')) {
			$return = sys_get_temp_dir();
		} else {
			$filename = @tempnam("", ""); // @ - temp directory can be disabled by open_basedir
			if (!$filename) {
				return false;
			}
			$return = dirname($filename);
			unlink($filename);
		}
	}
	return $return;
}

/** Open and exclusively lock a file
* @param string
* @return resource or null for error
*/
function file_open_lock($filename) {
	if (is_link($filename)) {
		return; // https://cwe.mitre.org/data/definitions/61.html
	}
	$fp = @fopen($filename, "c+"); // @ - may not be writable
	if (!$fp) {
		return;
	}
	chmod($filename, 0660);
	if (!flock($fp, LOCK_EX)) {
		fclose($fp);
		return;
	}
	return $fp;
}

/** Write and unlock a file
* @param resource
* @param string
*/
function file_write_unlock($fp, $data) {
	rewind($fp);
	fwrite($fp, $data);
	ftruncate($fp, strlen($data));
	file_unlock($fp);
}

/** Unlock and close a file
* @param resource
*/
function file_unlock($fp) {
	flock($fp, LOCK_UN);
	fclose($fp);
}

/** Get first element of an array
* @param array
* @return mixed or false if not found
*/
function first($array) {
	// reset(f()) triggers a notice
	return reset($array);
}

/** Read password from file adminer.key in temporary directory or create one
* @param bool
* @return string or false if the file can not be created
*/
function password_file($create) {
	$filename = get_temp_dir() . "/adminer.key";
	if (!$create && !file_exists($filename)) {
		return false;
	}
	$fp = file_open_lock($filename);
	if (!$fp) {
		return false;
	}
	$return = stream_get_contents($fp);
	if (!$return) {
		$return = rand_string();
		file_write_unlock($fp, $return);
	} else {
		file_unlock($fp);
	}
	return $return;
}

/** Get a random string
* @return string 32 hexadecimal characters
*/
function rand_string() {
	return md5(uniqid(mt_rand(), true));
}

/** Format value to use in select
* @param string
* @param string
* @param array
* @param int
* @return string HTML
*/
function select_value($val, $link, $field, $text_length) {
	global $adminer;
	if (is_array($val)) {
		$return = "";
		foreach ($val as $k => $v) {
			$return .= "<tr>"
				. ($val != array_values($val) ? "<th>" . h($k) : "")
				. "<td>" . select_value($v, $link, $field, $text_length)
			;
		}
		return "<table>$return</table>";
	}
	if (!$link) {
		$link = $adminer->selectLink($val, $field);
	}
	if ($link === null) {
		if (is_mail($val)) {
			$link = "mailto:$val";
		}
		if (is_url($val)) {
			$link = $val; // IE 11 and all modern browsers hide referrer
		}
	}
	$return = $adminer->editVal($val, $field);
	if ($return !== null) {
		if (!is_utf8($return)) {
			$return = "\0"; // htmlspecialchars of binary data returns an empty string
		} elseif ($text_length != "" && is_shortable($field)) {
			$return = shorten_utf8($return, max(0, +$text_length)); // usage of LEFT() would reduce traffic but complicate query - expected average speedup: .001 s VS .01 s on local network
		} else {
			$return = h($return);
		}
	}
	return $adminer->selectVal($return, $link, $field, $val);
}

/** Check whether the string is e-mail address
* @param string
* @return bool
*/
function is_mail($email) {
	$atom = '[-a-z0-9!#$%&\'*+/=?^_`{|}~]'; // characters of local-name
	$domain = '[a-z0-9]([-a-z0-9]{0,61}[a-z0-9])'; // one domain component
	$pattern = "$atom+(\\.$atom+)*@($domain?\\.)+$domain";
	return is_string($email) && preg_match("(^$pattern(,\\s*$pattern)*\$)i", $email);
}

/** Check whether the string is URL address
* @param string
* @return bool
*/
function is_url($string) {
	$domain = '[a-z0-9]([-a-z0-9]{0,61}[a-z0-9])'; // one domain component //! IDN
	return preg_match("~^(https?)://($domain?\\.)+$domain(:\\d+)?(/.*)?(\\?.*)?(#.*)?\$~i", $string); //! restrict path, query and fragment characters
}

/** Check if field should be shortened
* @param array
* @return bool
*/
function is_shortable($field) {
	return preg_match('~char|text|json|lob|geometry|point|linestring|polygon|string|bytea~', $field["type"]);
}

/** Get query to compute number of found rows
* @param string
* @param array
* @param bool
* @param array
* @return string
*/
function count_rows($table, $where, $is_group, $group) {
	$query = " FROM " . table($table) . ($where ? " WHERE " . implode(" AND ", $where) : "");
	return ($is_group && (JUSH == "sql" || count($group) == 1)
		? "SELECT COUNT(DISTINCT " . implode(", ", $group) . ")$query"
		: "SELECT COUNT(*)" . ($is_group ? " FROM (SELECT 1$query GROUP BY " . implode(", ", $group) . ") x" : $query)
	);
}

/** Run query which can be killed by AJAX call after timing out
* @param string
* @return array of strings
*/
function slow_query($query) {
	global $adminer, $token, $driver;
	$db = $adminer->database();
	$timeout = $adminer->queryTimeout();
	$slow_query = $driver->slowQuery($query, $timeout);
	$connection2 = null;
	if (!$slow_query && support("kill") && is_object($connection2 = connect($adminer->credentials())) && ($db == "" || $connection2->select_db($db))) {
		$kill = $connection2->result(connection_id()); // MySQL and MySQLi can use thread_id but it's not in PDO_MySQL
		echo script("const timeout = setTimeout(() => { ajax('" . js_escape(ME) . "script=kill', function () {}, 'kill=$kill&token=$token'); }, 1000 * $timeout);");
	}
	ob_flush();
	flush();
	$return = @get_key_vals(($slow_query ?: $query), $connection2, false); // @ - may be killed
	if ($connection2) {
		echo script("clearTimeout(timeout);");
		ob_flush();
		flush();
	}
	return $return;
}

/** Generate BREACH resistant CSRF token
* @return string
*/
function get_token() {
	$rand = rand(1, 1e6);
	return ($rand ^ $_SESSION["token"]) . ":$rand";
}

/** Verify if supplied CSRF token is valid
* @return bool
*/
function verify_token() {
	list($token, $rand) = explode(":", $_POST["token"]);
	return ($rand ^ $_SESSION["token"]) == $token;
}

// used in compiled version
function lzw_decompress($binary) {
	// convert binary string to codes
	$dictionary_count = 256;
	$bits = 8; // ceil(log($dictionary_count, 2))
	$codes = array();
	$rest = 0;
	$rest_length = 0;
	for ($i=0; $i < strlen($binary); $i++) {
		$rest = ($rest << 8) + ord($binary[$i]);
		$rest_length += 8;
		if ($rest_length >= $bits) {
			$rest_length -= $bits;
			$codes[] = $rest >> $rest_length;
			$rest &= (1 << $rest_length) - 1;
			$dictionary_count++;
			if ($dictionary_count >> $bits) {
				$bits++;
			}
		}
	}
	// decompression
	$dictionary = range("\0", "\xFF");
	$return = "";
	foreach ($codes as $i => $code) {
		$element = $dictionary[$code];
		if (!isset($element)) {
			$element = $word . $word[0];
		}
		$return .= $element;
		if ($i) {
			$dictionary[] = $word . $element[0];
		}
		$word = $element;
	}
	return $return;
}

?>
<?php
/** Return <script> element
* @param string
* @param string
* @return string
*/
function script($source, $trailing = "\n") {
	return "<script" . nonce() . ">$source</script>$trailing";
}

/** Return <script src> element
* @param string
* @return string
*/
function script_src($url) {
	return "<script src='" . h($url) . "'" . nonce() . "></script>\n";
}

/** Get a nonce="" attribute with CSP nonce
* @return string
*/
function nonce() {
	return ' nonce="' . get_nonce() . '"';
}

/** Get <input type="hidden">
* @param string
* @param string
* @return string HTML
*/
function input_hidden($name, $value = "") {
	return "<input type='hidden' name='" . h($name) . "' value='" . h($value) . "'>\n";
}

/** Get <input type="hidden" name="token">
* @param string token to use instead of global $token
* @return string HTML
*/
function input_token($special = "") {
	global $token;
	return input_hidden("token", ($special ?: $token));
}

/** Get a target="_blank" attribute
* @return string
*/
function target_blank() {
	return ' target="_blank" rel="noreferrer noopener"';
}

/** Escape for HTML
* @param string
* @return string
*/
function h($string) {
	return str_replace("\0", "&#0;", htmlspecialchars($string, ENT_QUOTES, 'utf-8'));
}

/** Convert \n to <br>
* @param string
* @return string
*/
function nl_br($string) {
	return str_replace("\n", "<br>", $string); // nl2br() uses XHTML before PHP 5.3
}

/** Generate HTML checkbox
* @param string
* @param string
* @param bool
* @param string
* @param string
* @param string
* @param string
* @return string
*/
function checkbox($name, $value, $checked, $label = "", $onclick = "", $class = "", $labelled_by = "") {
	$return = "<input type='checkbox' name='$name' value='" . h($value) . "'"
		. ($checked ? " checked" : "")
		. ($labelled_by ? " aria-labelledby='$labelled_by'" : "")
		. ">"
		. ($onclick ? script("qsl('input').onclick = function () { $onclick };", "") : "")
	;
	return ($label != "" || $class ? "<label" . ($class ? " class='$class'" : "") . ">$return" . h($label) . "</label>" : $return);
}

/** Generate list of HTML options
* @param array array of strings or arrays (creates optgroup)
* @param mixed
* @param bool always use array keys for value="", otherwise only string keys are used
* @return string
*/
function optionlist($options, $selected = null, $use_keys = false) {
	$return = "";
	foreach ($options as $k => $v) {
		$opts = array($k => $v);
		if (is_array($v)) {
			$return .= '<optgroup label="' . h($k) . '">';
			$opts = $v;
		}
		foreach ($opts as $key => $val) {
			$return .= '<option'
				. ($use_keys || is_string($key) ? ' value="' . h($key) . '"' : '')
				. ($selected !== null && ($use_keys || is_string($key) ? (string) $key : $val) === $selected ? ' selected' : '')
				. '>' . h($val)
			;
		}
		if (is_array($v)) {
			$return .= '</optgroup>';
		}
	}
	return $return;
}

/** Generate HTML <select>
* @param string
* @param array
* @param string
* @param string
* @param string
* @return string
*/
function html_select($name, $options, $value = "", $onchange = "", $labelled_by = "") {
	return "<select name='" . h($name) . "'"
		. ($labelled_by ? " aria-labelledby='$labelled_by'" : "")
		. ">" . optionlist($options, $value) . "</select>"
		. ($onchange ? script("qsl('select').onchange = function () { $onchange };", "") : "")
	;
}

/** Generate HTML radio list
* @param string
* @param array
* @param string
* @return string
*/
function html_radios($name, $options, $value = "") {
	$return = "";
	foreach ($options as $key => $val) {
		$return .= "<label><input type='radio' name='" . h($name) . "' value='" . h($key) . "'" . ($key == $value ? " checked" : "") . ">" . h($val) . "</label>";
	}
	return $return;
}

/** Get onclick confirmation
* @param string
* @param string
* @return string
*/
function confirm($message = "", $selector = "qsl('input')") {
	return script("$selector.onclick = () => confirm('" . ($message ? js_escape($message) : lang(6)) . "');", "");
}

/** Print header for hidden fieldset (close by </div></fieldset>)
* @param string
* @param string
* @param bool
* @return null
*/
function print_fieldset($id, $legend, $visible = false) {
	echo "<fieldset><legend>";
	echo "<a href='#fieldset-$id'>$legend</a>";
	echo script("qsl('a').onclick = partial(toggle, 'fieldset-$id');", "");
	echo "</legend>";
	echo "<div id='fieldset-$id'" . ($visible ? "" : " class='hidden'") . ">\n";
}

/** Return class='active' if $bold is true
* @param bool
* @param string
* @return string
*/
function bold($bold, $class = "") {
	return ($bold ? " class='active $class'" : ($class ? " class='$class'" : ""));
}

/** Escape string for JavaScript apostrophes
* @param string
* @return string
*/
function js_escape($string) {
	return addcslashes($string, "\r\n'\\/"); // slash for <script>
}

/** Generate page number for pagination
* @param int
* @param int
* @return string
*/
function pagination($page, $current) {
	return " " . ($page == $current
		? $page + 1
		: '<a href="' . h(remove_from_uri("page") . ($page ? "&page=$page" . ($_GET["next"] ? "&next=" . urlencode($_GET["next"]) : "") : "")) . '">' . ($page + 1) . "</a>"
	);
}

/** Print hidden fields
* @param array
* @param array
* @param string
* @return bool
*/
function hidden_fields($process, $ignore = array(), $prefix = '') {
	$return = false;
	foreach ($process as $key => $val) {
		if (!in_array($key, $ignore)) {
			if (is_array($val)) {
				hidden_fields($val, array(), $key);
			} else {
				$return = true;
				echo input_hidden(($prefix ? $prefix . "[$key]" : $key), $val);
			}
		}
	}
	return $return;
}

/** Print hidden fields for GET forms
* @return null
*/
function hidden_fields_get() {
	echo (sid() ? input_hidden(session_name(), session_id()) : '');
	echo (SERVER !== null ? input_hidden(DRIVER, SERVER) : "");
	echo input_hidden("username", $_GET["username"]);
}

/** Print enum or set input field
* @param string "radio"|"checkbox"
* @param string
* @param array
* @param mixed string|array
* @param string
* @return null
*/
function enum_input($type, $attrs, $field, $value, $empty = null) {
	global $adminer;
	preg_match_all("~'((?:[^']|'')*)'~", $field["length"], $matches);
	$return = ($empty !== null ? "<label><input type='$type'$attrs value='$empty'" . ((is_array($value) ? in_array($empty, $value) : $value === $empty) ? " checked" : "") . "><i>" . lang(7) . "</i></label>" : "");
	foreach ($matches[1] as $i => $val) {
		$val = stripcslashes(str_replace("''", "'", $val));
		$checked = (is_array($value) ? in_array($val, $value) : $value === $val);
		$return .= " <label><input type='$type'$attrs value='" . h($val) . "'" . ($checked ? ' checked' : '') . '>' . h($adminer->editVal($val, $field)) . '</label>';
	}
	return $return;
}

/** Print edit input field
* @param array one field from fields()
* @param mixed
* @param string
* @param bool
* @return null
*/
function input($field, $value, $function, $autofocus = false) {
	global $driver, $adminer;
	$name = h(bracket_escape($field["field"]));
	echo "<td class='function'>";
	if (is_array($value) && !$function) {
		$value = json_encode($value, 128 | 64 | 256); // 128 - JSON_PRETTY_PRINT, 64 - JSON_UNESCAPED_SLASHES, 256 - JSON_UNESCAPED_UNICODE available since PHP 5.4
		$function = "json";
	}
	$reset = (JUSH == "mssql" && $field["auto_increment"]);
	if ($reset && !$_POST["save"]) {
		$function = null;
	}
	$functions = (isset($_GET["select"]) || $reset ? array("orig" => lang(8)) : array()) + $adminer->editFunctions($field);
	$disabled = stripos($field["default"], "GENERATED ALWAYS AS ") === 0 ? " disabled=''" : "";
	$attrs = " name='fields[$name]'$disabled" . ($autofocus ? " autofocus" : "");
	$enums = $driver->enumLength($field);
	if ($enums) {
		$field["type"] = "enum";
		$field["length"] = $enums;
	}
	echo $driver->unconvertFunction($field) . " ";
	if ($field["type"] == "enum") {
		echo h($functions[""]) . "<td>" . $adminer->editInput($_GET["edit"], $field, $attrs, $value);
	} else {
		$has_function = (in_array($function, $functions) || isset($functions[$function]));
		echo (count($functions) > 1
			? "<select name='function[$name]'$disabled>" . optionlist($functions, $function === null || $has_function ? $function : "") . "</select>"
				. on_help("event.target.value.replace(/^SQL\$/, '')", 1)
				. script("qsl('select').onchange = functionChange;", "")
			: h(reset($functions))
		) . '<td>';
		$input = $adminer->editInput($_GET["edit"], $field, $attrs, $value); // usage in call is without a table
		if ($input != "") {
			echo $input;
		} elseif (preg_match('~bool~', $field["type"])) {
			echo "<input type='hidden'$attrs value='0'>"
				. "<input type='checkbox'" . (preg_match('~^(1|t|true|y|yes|on)$~i', $value) ? " checked='checked'" : "") . "$attrs value='1'>";
		} elseif ($field["type"] == "set") {
			preg_match_all("~'((?:[^']|'')*)'~", $field["length"], $matches);
			foreach ($matches[1] as $i => $val) {
				$val = stripcslashes(str_replace("''", "'", $val));
				$checked = in_array($val, explode(",", $value), true);
				echo " <label><input type='checkbox' name='fields[$name][$i]' value='" . h($val) . "'" . ($checked ? ' checked' : '') . ">" . h($adminer->editVal($val, $field)) . '</label>';
			}
		} elseif (preg_match('~blob|bytea|raw|file~', $field["type"]) && ini_bool("file_uploads")) {
			echo "<input type='file' name='fields-$name'>";
		} elseif ($function == "json" || preg_match('~^jsonb?$~', $field["type"])) {
			echo "<textarea$attrs cols='50' rows='12' class='jush-js'>" . h($value) . '</textarea>';
		} elseif (($text = preg_match('~text|lob|memo~i', $field["type"])) || preg_match("~\n~", $value)) {
			if ($text && JUSH != "sqlite") {
				$attrs .= " cols='50' rows='12'";
			} else {
				$rows = min(12, substr_count($value, "\n") + 1);
				$attrs .= " cols='30' rows='$rows'";
			}
			echo "<textarea$attrs>" . h($value) . '</textarea>';
		} else {
			// int(3) is only a display hint
			$types = $driver->types();
			$maxlength = (!preg_match('~int~', $field["type"]) && preg_match('~^(\d+)(,(\d+))?$~', $field["length"], $match)
				? ((preg_match("~binary~", $field["type"]) ? 2 : 1) * $match[1] + ($match[3] ? 1 : 0) + ($match[2] && !$field["unsigned"] ? 1 : 0))
				: ($types[$field["type"]] ? $types[$field["type"]] + ($field["unsigned"] ? 0 : 1) : 0)
			);
			if (JUSH == 'sql' && min_version(5.6) && preg_match('~time~', $field["type"])) {
				$maxlength += 7; // microtime
			}
			// type='date' and type='time' display localized value which may be confusing, type='datetime' uses 'T' as date and time separator
			echo "<input"
				. ((!$has_function || $function === "") && preg_match('~(?<!o)int(?!er)~', $field["type"]) && !preg_match('~\[\]~', $field["full_type"]) ? " type='number'" : "")
				. " value='" . h($value) . "'" . ($maxlength ? " data-maxlength='$maxlength'" : "")
				. (preg_match('~char|binary~', $field["type"]) && $maxlength > 20 ? " size='" . ($maxlength > 99 ? 60 : 40) . "'" : "")
				. "$attrs>"
			;
		}
		echo $adminer->editHint($_GET["edit"], $field, $value);
		// skip 'original'
		$first = 0;
		foreach ($functions as $key => $val) {
			if ($key === "" || !$val) {
				break;
			}
			$first++;
		}
		if ($first && count($functions) > 1) {
			echo script("qsl('td').oninput = partial(skipOriginal, $first);");
		}
	}
}

/** Process edit input field
* @param one field from fields()
* @return string or false to leave the original value
*/
function process_input($field) {
	global $adminer, $driver;
	if (stripos($field["default"], "GENERATED ALWAYS AS ") === 0) {
		return null;
	}
	$idf = bracket_escape($field["field"]);
	$function = $_POST["function"][$idf];
	$value = $_POST["fields"][$idf];
	if ($field["type"] == "enum" || $driver->enumLength($field)) {
		if ($value == -1) {
			return false;
		}
		if ($value == "") {
			return "NULL";
		}
	}
	if ($field["auto_increment"] && $value == "") {
		return null;
	}
	if ($function == "orig") {
		return (preg_match('~^CURRENT_TIMESTAMP~i', $field["on_update"]) ? idf_escape($field["field"]) : false);
	}
	if ($function == "NULL") {
		return "NULL";
	}
	if ($field["type"] == "set") {
		$value = implode(",", (array) $value);
	}
	if ($function == "json") {
		$function = "";
		$value = json_decode($value, true);
		if (!is_array($value)) {
			return false; //! report errors
		}
		return $value;
	}
	if (preg_match('~blob|bytea|raw|file~', $field["type"]) && ini_bool("file_uploads")) {
		$file = get_file("fields-$idf");
		if (!is_string($file)) {
			return false; //! report errors
		}
		return $driver->quoteBinary($file);
	}
	return $adminer->processInput($field, $value, $function);
}

/** Print results of search in all tables
* @uses $_GET["where"][0]
* @uses $_POST["tables"]
* @return null
*/
function search_tables() {
	global $adminer, $connection;
	$_GET["where"][0]["val"] = $_POST["query"];
	$sep = "<ul>\n";
	foreach (table_status('', true) as $table => $table_status) {
		$name = $adminer->tableName($table_status);
		if (isset($table_status["Engine"]) && $name != "" && (!$_POST["tables"] || in_array($table, $_POST["tables"]))) {
			$result = $connection->query("SELECT" . limit("1 FROM " . table($table), " WHERE " . implode(" AND ", $adminer->selectSearchProcess(fields($table), array())), 1));
			if (!$result || $result->fetch_row()) {
				$print = "<a href='" . h(ME . "select=" . urlencode($table) . "&where[0][op]=" . urlencode($_GET["where"][0]["op"]) . "&where[0][val]=" . urlencode($_GET["where"][0]["val"])) . "'>$name</a>";
				echo "$sep<li>" . ($result ? $print : "<p class='error'>$print: " . error()) . "\n";
				$sep = "";
			}
		}
	}
	echo ($sep ? "<p class='message'>" . lang(9) : "</ul>") . "\n";
}

/** Return events to display help on mouse over
* @param string JS expression
* @param bool JS expression
* @return string
*/
function on_help($command, $side = 0) {
	return script("mixin(qsl('select, input'), {onmouseover: function (event) { helpMouseover.call(this, event, $command, $side) }, onmouseout: helpMouseout});", "");
}

/** Print edit data form
* @param string
* @param array
* @param mixed
* @param bool
* @return null
*/
function edit_form($table, $fields, $row, $update) {
	global $adminer, $error;
	$table_name = $adminer->tableName(table_status1($table, true));
	page_header(
		($update ? lang(10) : lang(11)),
		$error,
		array("select" => array($table, $table_name)),
		$table_name
	);
	$adminer->editRowPrint($table, $fields, $row, $update);
	if ($row === false) {
		echo "<p class='error'>" . lang(12) . "\n";
		return;
	}
	echo "<form action='' method='post' enctype='multipart/form-data' id='form'>\n";
	if (!$fields) {
		echo "<p class='error'>" . lang(13) . "\n";
	} else {
		echo "<table class='layout'>" . script("qsl('table').onkeydown = editingKeydown;");
		$autofocus = !$_POST;
		foreach ($fields as $name => $field) {
			echo "<tr><th>" . $adminer->fieldName($field);
			$default = $_GET["set"][bracket_escape($name)];
			if ($default === null) {
				$default = $field["default"];
				if ($field["type"] == "bit" && preg_match("~^b'([01]*)'\$~", $default, $regs)) {
					$default = $regs[1];
				}
				if (JUSH == "sql" && preg_match('~binary~', $field["type"])) {
					$default = bin2hex($default); // same as UNHEX
				}
			}
			$value = ($row !== null
				? ($row[$name] != "" && JUSH == "sql" && preg_match("~enum|set~", $field["type"]) && is_array($row[$name])
					? implode(",", $row[$name])
					: (is_bool($row[$name]) ? +$row[$name] : $row[$name])
				)
				: (!$update && $field["auto_increment"]
					? ""
					: (isset($_GET["select"]) ? false : $default)
				)
			);
			if (!$_POST["save"] && is_string($value)) {
				$value = $adminer->editVal($value, $field);
			}
			$function = ($_POST["save"]
				? (string) $_POST["function"][$name]
				: ($update && preg_match('~^CURRENT_TIMESTAMP~i', $field["on_update"])
					? "now"
					: ($value === false ? null : ($value !== null ? '' : 'NULL'))
				)
			);
			if (!$_POST && !$update && $value == $field["default"] && preg_match('~^[\w.]+\(~', $value)) {
				$function = "SQL";
			}
			if (preg_match("~time~", $field["type"]) && preg_match('~^CURRENT_TIMESTAMP~i', $value)) {
				$value = "";
				$function = "now";
			}
			if ($field["type"] == "uuid" && $value == "uuid()") {
				$value = "";
				$function = "uuid";
			}
			if ($autofocus !== false) {
				$autofocus = ($field["auto_increment"] || $function == "now" || $function == "uuid" ? null : true); // null - don't autofocus this input but check the next one
			}
			input($field, $value, $function, $autofocus);
			if ($autofocus) {
				$autofocus = false;
			}
			echo "\n";
		}
		if (!support("table") && !fields($table)) {
			echo "<tr>"
				. "<th><input name='field_keys[]'>"
				. script("qsl('input').oninput = fieldChange;")
				. "<td class='function'>" . html_select("field_funs[]", $adminer->editFunctions(array("null" => isset($_GET["select"]))))
				. "<td><input name='field_vals[]'>"
				. "\n"
			;
		}
		echo "</table>\n";
	}
	echo "<p>\n";
	if ($fields) {
		echo "<input type='submit' value='" . lang(14) . "'>\n";
		if (!isset($_GET["select"])) {
			echo "<input type='submit' name='insert' value='" . ($update
				? lang(15)
				: lang(16)
			) . "' title='Ctrl+Shift+Enter'>\n";
			echo ($update ? script("qsl('input').onclick = function () { return !ajaxForm(this.form, '" . lang(17) . "…', this); };") : "");
		}
	}
	echo ($update ? "<input type='submit' name='delete' value='" . lang(18) . "'>" . confirm() . "\n" : "");
	if (isset($_GET["select"])) {
		hidden_fields(array("check" => (array) $_POST["check"], "clone" => $_POST["clone"], "all" => $_POST["all"]));
	}
	echo input_hidden("referer", (isset($_POST["referer"]) ? $_POST["referer"] : $_SERVER["HTTP_REFERER"]));
	echo input_hidden("save", 1);
	echo input_token();
	echo "</form>\n";
}


// used only in compiled file
if (isset($_GET["file"])) {
	?>
<?php
if (substr($VERSION, -4) != '-dev') {
	if ($_SERVER["HTTP_IF_MODIFIED_SINCE"]) {
		header("HTTP/1.1 304 Not Modified");
		exit;
	}
	header("Expires: " . gmdate("D, d M Y H:i:s", time() + 365*24*60*60) . " GMT");
	header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
	header("Cache-Control: immutable");
}

if ($_GET["file"] == "favicon.ico") {
	header("Content-Type: image/x-icon");
	echo lzw_decompress("\0\0\0` \0\0\n @\0C\"\0`EQ?tvM'Jdd\\b0\0\"fӈs5AXPaJ08#RTz`#.cXȀ?-\0Im?.M\0ȯ(̉/(%\0");
} elseif ($_GET["file"] == "default.css") {
	header("Content-Type: text/css; charset=utf-8");
	echo lzw_decompress("b7'o9c`a1#ydC1tFQx\\2\nSn0'#I,\$Mc)c1iXi3ͦn)Tid:FcI[c	Févt2+C,aGF:;Nu)Ǜ!tlF|,`pwS-oQk nEO+,=4mMƋGSZh6. uOMC@M'(b5ҩHa2)qиpe6?t#Z-ox<s;H4\$䥍ۚa4\"(!C,DN;Jj@@!K6jX\r@ 2@b(ZApl8h.=*H4q3AЂ.K!fqr!1Ȏc*+ (\n2j(dYADtϑm*RPQb#J+1N̙>AwKC`P=<@̊c]RrCHZմ\0=P\$@o(0CG-ZAʅt3:tc͵JlcR\nrlΨk9Gp!pj_X]hXfu0̳ҝ0=C6#˞75s,(f#zX>3[n?1m\\XPXeggJ4> \\T?H3y~hxt6%u8Lm\nXiM[,YP9|~O1*5+ӣ1d{o=gx,EPKsкE^4f;*J#j=kt8W1vhԃ3Btȧ.^̀os`?7}?Z]y[iPnW\"zx\"4ؽr?'\"@gf!637QCA\rYTP}@M}ͽ+{!ղݜ4׹ G	I\0^RN}EDY{/>#Ɓ*n:Fphn%w#p΍\\\0P+E\r)sLӱd\r@-\$V]Ã;d0\0X%ܰQ* w]dXya&3MM8rXElQ%ֽB2t!\01,ʈTI-4hee3ņDn.P'3ՆZRsxi8.3FI֒ԚbSii2)+\r+pCc9..ipaIxn԰RjaO)\raĘ@Y\rrNPjBC`d\07̻'@#֩9\"	u\rtqՒ\"ae0Y*`rBMDl597<13geD>Ң{\\Xyr3m̨*.3(7.Y[/!3(sC()fQ'N@*LaŜ\"dA8ݴ\":MI	(*1C)Lyf4S(ƻ\"QۄTWA{&.+6o1:\\Gȣ[Z3CuH	Ɯqؼ	0Ss-&&eCb@(\r(R5bH\rNlҘHe|aÈ1.!x蓂YCH)>JI =AڧɥCTpr +;IOe].5<[vS|̔4݅9UܩQsⴓ-ߘ\nn\$\n[1ʊ'z҂9LVz*XgxT~'E+Y,AKGxsa]5ƨA>TiRLjq=|Y58o<j\rLP0N&|f9b#,[TYu֚c`ֈdOv & VO-j<B7p#x&AChQ5%<y+\$~Tm9?ck!jo cV\ng]4SJIPT_	>~;śOfQ\nwM#uեXo8TK%e\\d-,mSFKEg\0\$4Lٖ㈑M\0P-ꧼÁA)kz5Qz[4de}/ݟh\rWWs<kI*[@icsB@5Gj7	_/-A6\0ЉzZ7ΏhOe4Q]G߾tJyQN	S\\ P\nJOnbOOΐތnNdNTcoF/cΞ]G\nP9ƴ)2!n\ngOM!OB#D0#l^J\0PoFQt\0pp\nFO_,L.,Y#R-Ԓh\0v00\0APOp/P'cڜ\0-\\{PSY.W`PN\njԢhn%p}I{͟b6QQZ)z1|pP邗ZɪǢ`Z C\n\r\0\0fqY1S4M`ZL@`*]@Z\r` >ѼKbeRG&gFn-ўQXKҤ/\r ċq.?\"`J#qi\$Q4Q%HzM&1&k&H'8cR@QJS8\r ڏ*{R)+`r'2*R,@+)-a,,#)6\0/Ư Xr(R0e1J+2؆,-3\0\rWr/r,{2>(xK,0T_Y5334Dfs)2R-^\r7Q7O260S)b\r.٣Q.2r-J3)/-:s;/ҊNY\$,hRK1s(ѩ\$c314sH?Tr^t	6t#;қAW4%T\r+4 8s)J%2W24=7@`E+A3tUE8!6%@At	CBYCS3CD [\$4IHtLr\0:SHTKF9@:R1F4SKF\$ \03A3#iJuEtB	B493H1!4+0	N.\0.JfP4Oԕ44FHԩ\"[9-P4;/?=9Z.A<aTRW,t;\0TG,u4\0VqVOTڞkyV WsXu;rVu,cPeuXՑ-ZuMQLӉ12wI3;C]s\\Nc455:3]2? 2%\\3\"z\rG[7\" u'*Q3Jz-4d &S@j{M6x0/SoX'+5\nZac0`!`iKKn \\SYuy27Xi8,[3֚voEhth_֍Q7i3iMfi6e`e:_#e@Y\$c14C40\rgg+j:\"\\MJ,");
} elseif ($_GET["file"] == "dark.css") {
	header("Content-Type: text/css; charset=utf-8");
	echo lzw_decompress("b7'o9cbFr7MHP`2\r\"'\r\rF#s1p;Ɠe2IY.GFI:4S3㔚Yu(Ìc(`h#%0[LhC!Eb5Úyfbw	z#1P6l2MQde#Qm>5ًn1Se0o#GyGxA׍]]ԻLƓ)s25]68-ļo NZ)[Y#)Pkp8C(\\5w\0C-&\n(:rʖclLtA HR @	xFtB8LBI0\r@3|s s\$)rsI2TF9r:t0;Ar*͔=!ʹ.*r\r.U534h@@1?RQnC4(9#87aՒQOP2LBHK%YTUUe\\E`!ȅn(@M#0Ьb04\noXtӃtXj1sHĢۈʭ+\\	H,@60,9ζʊ٣.2㢩c@565PQ\\!fޯ+@0\rXOdZq\n*y:B£È-gC>Wc[ocJ=e;ۢ)S#W7麭Gà8n3a@`aXqpn׭4ttCʡC/fpOp`AHYXao[\04ەskvɀGT\0٠:9m.~9mCy{㳠zv0ūF1D.H9O\r0	,]Ma6\0d7x͍%ts1GV	m֛Q!AEwhaw\0A<=<pAqWVfx{X\nA,4㘢tǰ'H[Miܾ2M");
} elseif ($_GET["file"] == "functions.js") {
	header("Content-Type: text/javascript; charset=utf-8");
	echo lzw_decompress("f:gCI\n83)781x:\nOg#)r7\n\"`|2gSiH)NS\r\"0@)`(\$s6O!V/=g@T4=iS6IOG#XVCsZ1.hp8,[H~Cz2lc3sIb4\nF8NPC\r\$nT=\\r9O\"	l<\r\\xI,s\nAel1EƯL][M'=Ke\0lӝә\nivc]EHפHoֽΈ麮쨋6#Ґ\":cz>ߣC2vCX<P*50X97N\"3!!#\nZ%ć#CH!rX&:c7JHp~# A:86&|ˠ3@2<sΆPN]!0M-˲40\0\\ԗ響33d!>H5.,4pX25Mڢ*,(\n,(!apΎn\0RփvmI.\r][H<J8Ѷ5NC<ɠ%mCr#`XDooSL)^B{`wJJZ/XC(>a@OӚ%Dpӣ8a^X2`\\~\$`W!Qcذ!\"#\"iiZnJ\r\$.\$:j]1C{`7ܻ<pA`nS#F3`bA6:u:n8VԃNa2I0*iri%8g/5ܭ,t3Ygr9u=C[ʦ:c@{^˨zXRȺ7!ut0⼸&EBo4k\"#P<9Hg6!bHk\\MAGRPK\"o7*K CAa2(&z2\$+f^\n0zE I\rH<Y!`tR!e]C[2.\\P((2ppMp!YV@\"l&`4(qБ\n,/2)#_\0C8E]/2 W )A	\rϺqLRTˣO=ncI1܍x6p\$!QQ0R`=Kr|Ԋ\n,`c\rq<I~et1y#	4HX NHz&Mi@6%Ƀ6̘DA6\$8h[M=:Ni:'Q\rѝ o\\u0=RX*8hr\$PAB@t\r%AEB[3O!\\X9~\nx\\@3\0<οBa2nwV*\"ETZQVbE\n)\r,9ZduN,Y2eRՂ[䪕нTN21QCŗt,BydRQ \rMdQ\n#WUUjtz,ܫgVLZÂ1Y8\rzl׫!i&Br+8lHe)[[0HrJVYn3baW@ :˨׊` =ɾN@x\nXԅ ;R~@Mu2Yoj|q&T6XZn{fdKwwZTo`d`KjV{L!̙0Yf܉O!InB#+;`CZ\\GFci!z:dIʜAѓHܐX5׼Isa3NK'мBy!(&Qtr	Tz:W(#>[^e:U+A,7~aՁ\"OcfV@|t5_2ֲBsI[}[aȩ9S3dcݷz͘Wqe\n]^dk\0=3za޼~20hw7ap-	܆ۿ\0Gቃa	/gwl^ָCi/t0t	hI}!'fnwic7O9om~L8\rz'UFbVScZ4]s꒿[-iWom5.t=KbcX͞=82iEGQ*)CuEAJt\"p_B\"p &T+H{(2^O,t+hLe͐C7~4dJ3kjA@7sSqpSF(,)ib4@ir\0A\0K;FĊ\r8X\\-I\$%%/<%E.]W bLMF-RM:0	z^cЖGZB:.?iӀ3(+@L&vY `p#nМ`-^qv^ \re^jlBb](X\rFX&%:Pf M@ΐD!&fzK(cU8ob1>@Ч^<1N-m垠lhrvdHrb.?Ce Z@^`^)h^Hd(NƷJ`YёQѠQN֣pMNݍdժﮚ*;Hx@j	PXf;n3G`% \$-t-FjGd f\$FDK!`z a\r##GEP22G\0'T&r\"ml\"(@\rCRn0b!*bX\r\"\"	D#R\rR6F-%NdNkbI#\rE'm\nqHk'R!qD®2β#RV`ķ@l y3(-B\rb53&14h\r\"Dk>`r4@)\$(K!IvNbMDze5B%[63F٭䭢\"fWJ[)S(*7FK7Dr8\r8B`DBV!ED9,\$(@#TuZC29	u#Հ29#@tXt`L\nbtxLqFDd4@n\n'NNSEtZch\0sCC]\$\$RV2[\$bRPDI-?FDӫDĦ#ut0 _FiKޭ24C̮\nK)&\r+62Jޮ4>RtBDEDVPTwGu@DO	YO*D隺qNQ`N䦧\\\n%<rqF޲UTY&2t;hNBlBrԁHRLh([u%f?tGWQtID++땓%Y`Lz#JlMBEU}3Q\"2U-b{eB\$!]UB,jҔ~ً/^BR+6'7Jb ãD;\rO)P\rJqS\0R@CB\$D)CJPhqf9\0&pH5<oSkseC_L\"%`\nd &ezPTVԭn^	\nbnldLk 5Mp}\rE6n'o6!vuIln5|aήœrLEr!u%aub- zp`t(&\rH<eX\0u.3;;,V3 e_,]M]}_h\\1qD |(u)׭W%11/0W8cJb~2ϹA`A1\"Q\"OxV'}@B&@ VwvBeb8&L\"~&^Çf*lWCEWQx*xkÇp?݊\$2#ȶ oS4`A+BXYD̒x_38()Hd?xXę= 8ބ(%Ȥz\n\0\n`*%@Izm7#ksN\$[tAKp%(jDe˔\\F9LjKtsqᙶ 9B8\"ǋ<ǖ'9*qT#R\\`e-i/4U\0ǆm@/1}E@]yyqx]ohX#ۍ dhl%4x}HIH:\nS\\`Q֌8#\"xb\n81˟u31w`z	@	\r8&*!.R<h@уLi2GDjONE4O0CыƾuBC+4݂,\$L\$IpKgkknN˖! \r)*MtKx6֏P(TbERq=N]BIS%M;vT67vtT\r J\n	*\r*7.&hy;V\0Qz(lHzu{5'FBDUE׬#_\0@@K%@rP5*Mj\\Qj8]v[U\0ښ_D|?1ZMUڡ(<ˬ)©D6SXq>	yf@PR=F#Tj\rBkp\0ϗoxœU|`^J(:;tX{\"i,\$[1&5F5-ZTXM\"~T#]`m;R0a\\c;h<̼PD{ib:\0l~/bN2շ`Q5\ncT[HtlD\">/d8MowZ\riBhR槫Z'd@oiR9GAxGWjGybKV}F\\3\"\0@zcϣX\"	Y v6yl<R(R]25_>bB,\$Ŕ8q{Ȍ˛ %NPnP-#s;XLR r\nnr-3^_+6im\0k2|@`\0uI^\\v,=4.*bx7kRԦo	y΂sQv&.#]D\rϭԒ8I\"r((\nt甼̩>UKd/}#0MT&۠\rr{	YÝJ	<aX2:,uQZG]}k_{= ؤ3i~, }488Hr	M2#Pbq'N@ZԞPI	rЩ?Nw)9mMsHŁgN ]p1]:\$t71yFAg!Y8`,oDh\\\0x:?\"+	S:Q#A*޸h@X@V I	>A\0	JPꏋBset\rerR.	rr\nQ}h\0t\rmt0 F#}h`.L`=*z| 5<2AFJ@ȳnhymtbF\02Bޓ5K\\aoZQ	.3TiHVe!^\\T K!FP`W	L3Vi\$Coav[\rn\0d0!C40t4MHj&AlXlC)eM>;mSQCXT\\n2#VR7\"eЕtn|)5`}z~rBto]@X28'N9ǆ~%yO\"bBzDT\0 qE*'0TT\\\"#,Hz}3h!WK\"HEhQ!ZD\$Q	Ҁy@5rظ&.0i(wyn\"E# q뀻c?< ЧHICBpfVATUnqK2dUܮ!w`-]u1I'Rȣ6\rJ5qA\\(.AmpZ\$f\n\0sA)(;\$!#u@51`\0#\0mp[HF8XAܒ~\0o{ yt忐`jM܅b l\"WTX5y/Mi޲69iJl\$;\"H~BɶIN0\$+FDdӦB:B5q3@ZdSjGW\rKE06rQz5\0bH/=o- rY@1/l&-C\032fɕu8['2INJ@ZCrC	՗!xɠi)9X\"҆@w()Ɯ\$)E2T`d,e(pJx!ҩl^R-2my&|\r\0R\0K +pU҈Ce0#X\"H;	(yk6r\$d!nk@u.QH'x.!L#ZABpȔR0ʌj`5b%C0@,i\$HJR;r+Da[H>:ly/&e\n	7'9@+(};s\0S;P>p7nIl(VRtm.(>_DNnIYM\\+K#/XBP(`עt\"M'!~ҹK#axO1%1m\nIr&4GO[W_u]_(	ͼhxs1%!r%llDD)7!j>\rOb| 3A\nHhי~60\0A!෩u䃰2O<&6L覎(!C^6dx'Efd^,cLBK\$Mf^3e@PnTJUH>~ 1īK˫)Hיʱ**(gfFdwK\0\0*%\0\"|H@\0J>*d(&mjeV܂1ʆ)Y\0\0\n5h-#G,UWqjJxP(utR`AJ\"-8\"MA*\"Q%\rhD.nD`-W_W*+m\"9S\na'DF!2Ik՟hXӀ}Yf5:Rai8_\0ouŁ/)U\nHe|/>4b9oǎHRSXM2#b]L8<wy	o9\0Mи=}i\\	i@^\$,|Y.(MZoum5YO}Dy!\$ё\0P\0\n!32\\*jh@R3ݏDF(#;|	%L0˄(|hPhEMY*h+8\"JrO9ܩ\r7pD3J|6aI%NbpޒS&8Bi>u(E0(kJY09UMeWPԒH<O:ZUE<.V']]*%D\n	.W΅(uP5(	2^BClzP`3\0(1Tjb]pN\\:!,Lg`PHHҮ(\n<Etb0G\nؗD[p00Sq Ⱦ6iI	j{\0PKq\"ɯSƺt;>MpScɦЖx6E4N'ΰu}\0_eH!7B\\k!^W\rlq,oXh6=_cS؈#V\$olQb!䋤)1\"<+VZI`KND_<eMoaa93e*Vb Ukc;jb]Vn*JVHHbjdAV#NB6@n0c8dՎgN@Ӯ\0Εq9Z¶gE6KUJնUz mde;I~eTr]m~kn[5Eijq+MYx.\0lÿD+bOŭmHP#_:1\n,(1rFp*:Wא0P|hh9G[K5kLtyBHX.*YKA_7Ja}t2DYtBՀP֏c{/S~F`-T#M\nKECZ+\nȍ&U'8wD0F-qxƔtx>CӓL^la9\$3E;+B%t뭋[(kLPJVWɐl5`iåU^ b2nePtC	!y%b^U>aPTVʶIUׄV!4kj㥤_F;JXW`i`++<X/]=a4LhV!{:;P\"1Ы]HGhd.RhPǈ>\0 iқs`,otk<=`q3DK*)HGG\"to1i-#]FupeB(Zu@,!jO\\*.EgiX轄-]]o87faRpFN!7DRipx\nhὕBCy@Ql\$a*KMXZ:OJmya4Q~bT@E	Ekx	L+%G\n\0'j2[C6\$EtK<}v/OP7>V2	\"W[*ٖDqmʉ*rK'E[A\0\0[ORdiPXXh[g4(:Bg8\\r yEˎs_mr#\08160ȕKDw*`E,a@>2_q~XøJWK<#>X@v;^YBE.]\\dNR͢q,g_b>5eeƈ%-,n]\nx>t%ˀ1F,d62f-՛gF,BWA+Wm\\\$mkC5@@#uA.ڇ6#8dI\\f#98ggmIow!1Ӛd9Vfp\"=q	+IC-o.7<60'@mkzi i.biL:u_4r-ܓQ`,\0\0CSɽNiٙ״pLvΠ@E+!g}Z!۞oX\nXlpJ4\rl\rDۼ%-ƫ0ryX\\g:/;Y {_7H%ƉB0ި~<z,GhywO=y|=YzA7>W̱CG=-2`sD&C݀&r3Ԣ3-&@k3Up^^ϓ\"S%YUO+sKWXc1e\"Ri儳5Y\$TeNo\0Ja+VuL3a\nmO\rMw:DDA,8~Ȁ \$\\\0my/VC-TEP\r%+]L@VJ[v٨\03RQG߉j7`%{98\$2[#T1Jr)S%T\$4\nV  8Kz)ZMsW\0Š5٭Vh)#D57\$'BY4\r<|Lnhy:]R/gSY\nc&kQ\$I	@E2޼㽼0B,ԝz{dN=bL\0fȣEN9\"еzCf6ʥJ S)ZuV	҉5՗b)t_?\"Z@Mh&ۿx*Il\$\$Um)`dKK'L5r'YN9xR.|2SMNBG-rjQ| 'ֹN;HCZ|+Fd/6Cdnp]wB\r)!JB\rsed#>Hm఼WdYP{a3\nc&qystC棵D>'snsy2bb.{B\n=O!;\\*+WXD\\XOhFX]4.#M<jMtS&GUP&tdB0!{>2:Qaf^B`eF*0Z\\|'&hdgx(;G]@\"_%n,jPfCr0oi--K5a{9яܒZ=\0M-a~DMFw!Sk1|G=_@[A˗VHW\\et:0y],Mp\\ﶲ),\$w\$j8%)-*Rq*SHn2VjUh[*8y!jw;Ĵ	f>]͇3*}\n\\/\n;6m@b;h-ȒU:INP}VlnU%.9B@Ip\0\nK7\$`ExD]\nF|!nԄߺ\\Ȼy?,Wͭ:u\\q&4g5H;QǠ\"˒s֝vhInfU(VV#If;:{~r󭜨;O1F\$\\CcV_}I9'VOltn\nBR1m|wK6O ֝鑘\0jO=/i>Fvn;_؁|A9?+cܰ#F\0P\n|!:>/N{Ū;0%>w@N`6;\"/Xg	D:gi##kKrRM:@\$w@\0we<ݕַ{wP2:<⪇\$&:~륽>nDw83<VqZLdS\0,k?A\rfΫ=I\nh<MI:E0@\r2S+\"chI\0d(\n[ϔ:6KA(8#g@ZCP ^UC mx{IBn6\n[DA33JLNIddRq2(7uAfp`zמ@|YGkoOy>%ԄHbNГ02jnr9jШ]c6 ԛsp\\k\rj'D*<>y`prW\$!lG||(qBR0{⏸T*\02GOEI)\r^Eop4\"^P/\0^7h4[.>.];Q:؂ߤ[8-+>綽F46~8]_\0002_JpKP7<XNJ%ţt\nc!Pא3_L_?2EO4a~8X7PT6z3 'M)%a>f|'QnF#_O}]؀q-_HzHpÅ>\rp.|B@QR\0Fh\nJ0\n`K!:)s'*;a^\"-b+fNnd#\n~\0zSgUphG@T`g,HMM\\x!8\0O'\$\$CA&s+?1'7^a >2N,%ʴH+1	2i h@\0M朽	FE7fm\rj\rȏ\"a\$w(-Yla`ZPK!{P BɔށM퀆z0y,TPW4zPL>lN\r4B@\rX@PTYd.A`d.Ajz{w?m<+:j\r\"=5Kbe[U(`P+p YG,e? \rM7\0");
} elseif ($_GET["file"] == "jush.js") {
	header("Content-Type: text/javascript; charset=utf-8");
	echo lzw_decompress("v0F==FS	_6MƳr:ECIo:CXc\r؄J(:=Ea28x?'iSANNxsNBVl0S	Ul(D|҄P>E㩶yHch-3Eb bpEp9.~\n?Kbiw|`d.x8EN!23\rYy6GFmY8o7\n\r0<d4E'\n#\r.C!^t(bqH.s2Nq٤9#{c3nӸ2r:<+9CȨ\n<\r`/b\\!H2SڙF#8ЈI78K*ں!鎑+:+&2|:9:NpA/# 0D\\'12a@+J.c,1@^.Bь`OK=`BP6>(eK%! ^!ϬBHSs8^93O1.Xj+M	#+F:7S\$0V(FQ\r!I*X/̊67=۪X3݆؇^gf#Wg8ߋh7Ek\rŹG)tWe4V؝&7\0RN!01WyCP!i|gn.\r09Aݸ۶^8vl\"b|yHY290߅.:y6:ؿn\0Q7bk<\0湸-B{;W&/nw2A׵A0yu)kLƹtk\0;d=%m.ŏc5f*@4 cƸ܆|\"맳h\\fPNqsf~PpHp\n~>T_QOQ\$VSpn1ʚ}=LJeucaA|;ȓN-Z@Rͳ 	.2`RE^iP1&ވ(\$CY5؃axh@=Ʋ+>`ע\r!br2p(=!esX4GHhc MS.|YjHzBSV0j\nf\rDo%\\1MI`(:!-3=0SgWe5z(hdrӫKi@Y.\$@sѱEI&DfSR}rڽ?x\"@ngPI\\U<5X\"E0t8Y=`=>Q4Bk+p`(8/NqSKriO*[JRJY&u7#>Xû?APCDD\$Y<X[dd:a\$ΠW/ɂ!+eYIw=9i;q\r\n1x0]Q<zI9~W9RDKI6LCz\"0NWWzH4xgתx&FaӃ\\x=^ԓKHxٓ0EÝ҂ɚXk,R~	̛NySz6\0D	؏hs|.=Ix}/uN'[R`N95\0CXْ6w1PuL\0VʲO9[O>PKtÈu\r|̮RpOUDrf9LcSvnQo@o(ްàpa*^O>Oɹ<e\"ٓP>H^	psTO\r0d{Z\$	2,7C!u}B^?DڃFݱHΙ`'@J3|OܹBMbf1n@1(ղ!oowf)IL\\[8[1)!)u~c-6-y*	>\"m61ӕ.~*x諍qǚG |rlO*%݅AbRAxgDfV\\R5lޤ`5`w|SgOB;Ϯ^LÖW?5 ac}sݏIArݺO0;wxP(bmL'~wh\0c¨pE߲:C{g&ܾ/Ƒ>[ۜ)	a}n͡wN˼x]V^ye&@A	P\" E?P>@|!8 ЊH	\\`@E	Â4\0Da!nr쯜\\8o`Hf&̒<r(jNeN)6EO4.n06\r \$\$ N<|αNjOY\0Rn`omkH*-Ϙw	OzNZ*ʛnO\n#n⏓p[P_bjPPГ\0}\n/ӐП	o}S'`b\nPdp ?Po0sq\n:bLUu\r.L`SP1mq~]%&ʚQ \rDpqpV|f8\$p&ׂF&mOwG	1/elրD\0`~`K\\b&QQ`ʾAVEWn: ؓBƌ\r*l\0NDr뭦[&GhrH4A'bP>VƱM~R%2rm\$\02cMhʇvc}cjgs%lDȺ2D+A9#\$\0\$RHl@Q!%\$RFVNy+F\n	 %fz*ֿMɾR%@ڝ6\"TN kփ~@F@LQBv6OD^hhm|6nL7`zr֍Z@ր@܇3h\$@ѫt7zI P\rkf D\"b`E@\$\0RZ1&\"~0`\nbG)	c>[>ήe\"6N4@dn9ɴD4&2\"/|7u:ӱ;T3 ԓi<TO`ZB؃90S>Qhr\0A28\0W!ttwHOA\0eIFJT4xsAAGJ2i%:=#^ g7cr7s%MsD vsZ5\rb\$@P\r\$=%4nX\\Xd,lpOx9bm\"&g4O\\(ൔ5&rs M8.IY5U5IP3db/M\03y^u^\"UbIgT?U4Nh`5t\r2}5-2W(f7@e/\rJKd7- Sli3qUz\0)\$coF?@]LJbDҿ0s?[gʜ%\rjUn^R5,֪tFE\"xzm\n`-W#S(l	p%CU辚F&T|jbZ8	/4L*nɦyB:(8^98U K{`Z\nF\0Cl\r'(`meR6MBC6vn%#nvDjGo,^:``sl\r_X5CoV-8RZ@y13q GSBtvѢt#bB]#pfZCĲOZN]slԂEL,+Q@Yw~9I\"8!մV5&r\\7W&ܼ[\r\ri\r~L|dܷ,|i@,\0\"g\$B~!)5v0V b|M\$Df\r8;}fficԄV0,Fx\rR`a&nȧQB.# Y>wgE[ƗX~ROY]8]rK}-?8vL@~A*fJMtג-v[#xL'L>l8Pg\n\rQѱ\rM\":xw\$b-=kRXoQ乇9;ˈ過sՃ͋)~geBBt,,Ky-,mӀ+07yC˃IzƍY^GGWuv0#kXRJ\$JP+6x18Yg{?\0X\r	XFWהV/̓dIg9߆іy1-GX@ORy!GuY5ZF\r㕵-\$Oeu-ZFZdi9+쵘`Mz\rҫIyAVp:OJ:V:#::c{klZsWP0#9g@Mczw[9U\\k69Ӆ y,f6n-Zufًc,[o[gd :w#!W\\@n`߱\rɡ\$۟\$%ߡ۷z#\$imYcɂkI_yLϹ\$`V[F2C8\$ؼG[¼=Uυ[qKY݋Q?8aXm*G\\?U\0ϢKĤ|CR͓-|ɜaeRYƺ饘ܒPJE=u\${8X{ŏٓٗՙ\rͰ٬&Yҹ(ټM2)V u7\0S Z_o]\\|٩Ec7S΄[<<;-i }l!,}%-۬=Ӭ=Y8PV|zE.\rbLfƸh*;	ַ;؇Q{9\n_b\$5lUzXnz\0xbkM	2 Z\rc|ג/}%`NA\0*=`F^Q3WX<tR>r`uģ>izNاÝi\$\0rs^C>U5^a)	J+>uB@?J-HOJ'-TʀToUhF{ԏJ[NVoJ&SB\"I^5I2T龽]\0\rkL%}t۷~I0H|PkL5_T<w=<x\"esaK\"JH+Ua'Y~7)W<6=_Nh?6ܘy,aw\rİ#-V@k?ib*%޺p?yЀΆp-p|nCaf8A8+#\rR@npm~ۈ{`H?v*%Ǽv%G``Z.,6zU8|yV/p^פm]zcӞ\$IB0|@pR\nj9 G7읤#p߭?'=6Hlψ.YOY_VGO]I=x\$=|Ϫ{\n<;{:f^L'SA1%8*^p75W\n\0S⟕\02\nX(u[rpB0ڭx:n	ZI3C{[&C(@}rw2闌nt{CɆY!\0He>P\"9t5o!\$@\\7SS\rC P㄄@InhG	IS`x70b+v5^gr%bpU%)<+S/Z@ 4!j8\0vN-6a[>X,e\ned/PX`}kORN+1O\$πF6B-:wڨNTD>xY)n1&7}&xZ\nޖW:U@a⺃@.RhbcT\"x\n E|߈\r-\0\"QAIh\0	 FP\0MHFSB؎@\0*9s\00'	@EtOCx@\"G81`ϾP(G=1ˏ\0\"f>Qꎸ@`'>;l82>zI IG\nRH	c\"\0;1ێn)8B`(V@Q8c\"2E4r\09\rԑ \0'GzH5E!#\rAJЉJ(FC&d I\"IV솣GSAXZ~`'UA@+A\npi%ѿGZ`\$>~?E\0} <Q'Ewئ#\rɂ7rQ }'iMIO0dm% Hʰ\"-h#XFMt\$!Rt,(H88!J5Ixr\nThړ~Pe@&eg\"[hؖ4|2zDlw#9	v{lb/~\0&I8%,IKA\0/GYK*>O/2teھفP93=\$Xd-&|#154LUG.i2`M.B\00036ISJ-~쩦jF\\3	o4u	(@a3A\0c`P( 0\$\\}/d\0-3%b0\ncz`))%*6\"ٖE4FqJd(Ӏ1iLm2A.)&q@\$`L2Lrse .vss\riKQ󤙬 0()|MbtU9!ED	(	`8*pa<80s\r N8O0Ξd0OVx@'<OlJ)	~}\0U=O'Ňd~\0OfXH	LҠ(]'@EPLWE'=\0'\nN\$iIZy	>iOH6f'߁x.\"}@-wa2vӅAL><0/PB͢T\n<sSQ~|ӂPfiOφlq9T\rѕgÄFӧ%O(1h⺶nmv;|gSaFRȤNr9z%&X\0007\"2t-\rh%fŦֽ3!\"(7I\$s/ -7*J\rΕCLxw֗铴(ҪB,+h\nf\rF7Rf*:\"Δ4tPiX*\0P.(#+HoJAGq.57+N	:-m`&HJOUvi\0\nGN:gRn2i)}#	F駩>d`qHƕe5J);HQ\nHϓGRWԞ/Jj)K*URib8za.RG!4ͣ@9c: E.F|T*s<Z]_Oi\r@2qTlVUkCQ\rOe\"\n.TEUZԠ@i^ܪLaMUBV'U+Q VWmGԺu0*PT+!u\\kVy@Ƥj+H䁐\"EP,`<HՔpğ%	l\nK \0\$T!8@@2h4Lŝ+&,|\"TQ霋b#w)umŵ[ޒ)E}[Exd)p	n-AK1}W\\IUnF^\n` \$m)oZ	PDPVD r%R)bұl^w)JB-KD.18\0; le,L(\"mN\nZKgHe\0\0t7]Kk\$yNX\06(Yf\\\rK1y,`0qo\0h\$\n_dRzECh<Yp!\0ro;'g'*!YXv%K4RV\rZ}Z\rompN]N5xUay\rjWkb~+medyٯʰZksO4;Tal@4[]M7n 7>6ϓ=h*0HΫj\$[`,y	>7pD\$u9H ;R~0[DH삕6ܐ>-LxjZkNȢndg;C\\\nPb[h)3McD40uR#bP5:aEqH: :.X?c9%nKa5J`7X\nq=ȿvrE<(~CȷPQxHbKܪ-]\"QCU.aQv& 7]Ĩ媻>.9\0=K=)T _OX5!bUhAP-\r%zPޔ߀<xc7|4qpC<NY5ь)澈}AN_RCTxF*3g.`*B`&T:**7ƷEWR\\cW[Kb\roHru 2~/խ	@aI ,%b \0¡+{[,`_67.@̆)?mmba\nv]`W8!W`:Fpo-`7	\reXXzKI:bD_5>ŗf+<Yvg,%H\\  d\$@q\nA \n68F'|IRT{sm38b)	@LcMF@#Y`NDXCxzYc0y3hDZ6\"t\\7SE;U#R^ީs\0CfbܚrrI\"Y	tå8ZB/.`EK|b\n|_}KC. p1:#Y\nTC	%,,\r#@+dqŁ\${D	\\J\0񒫇-`m!|gdzVIvv&A`MH\\I|EjB0ۊ@ѡnUKތ>]ݸhiX9upra\$7vQCA>1xifR7*;8%\"ɄwPTByH'\n攏bظvT5xcH\$\\ۏXlKa`#tEwgh1 zp4:\nC2HK<X	(!J;㏨,u3ysMC9pwz\0ՠ9ǈxǃ1Bي`r)=hLƂ`?z9E?J1QR<\rL\n8(#rp>LQ|\"4(*8fpiWaQ\nQ*\\0@H;VYΆOZx<F'IA\n<]dP_NT!\r˧@*~ІB=%z;:AB}&lch`TO))\0yIۦ8NyјG\r\0T\"hn5W@}ՆB}ZkVФy=s	zӔ;\r쌚,hTi|jza&ր\$iS°Hi>IB{Z*UӘInO}XMsQ8IЊ	v&! k@#<TZ.jZ:	^B}YvO3BTC6=keS~?]ijOѦm,\0}!mF![J.gUlZP٦O[;&]Oht	`aILAkbkiNvYm:vvkg7)>b&؞p\05I]dp=+:;) Dx@^oѸAL't w&Ug3B`/='d>/dbF\0w\0y9nZ[6TubZ~~\nzd'@Ra\n\n@G0;vS={~\0@_c0ov1~xe\0po>83|pp<Ilo˄O; %8Gx.>oO=^uLG\rN7ݶq8~&n5l]ڀI..4ओ_ۼ=xPI5][\0_\0̓  <:eo By/Eq瑻f'Jw#7NxF(yD7\\'Y2˕?߯)9enGrvQ	./.Yܪ<zkMޠcMB+\"ہ\rgl\0^\0B@-	T61\nP@ \"\"@F0tU\04!_|(B\0Oc<'t\"m)TWF P?f9CMmkDސ|	&3`dΞ\0O8y@\n\0I?@@@/O\n0<d\r\n\0HC>knm_:Gb\$\0ђ|(vI60\"KB JrK`|6FT'9Y9>r@y@%ʄd7<\$p>t\r\0|yr́k9+6#\"97 NڮͪEnp{s^_;\"I\0J <w6ejc%85րL&F{2/w;&CD+p%#BYo:d4#H!A,݃\nsα8#=gjl:UBYX\0eտtmd(v@k\\9vQ2-{&/¶A<%N`EKJPպ,s&8+-1T@W8lDx76@\$v\"tXvj@tH'Ey@5ك<ɏ{vOY{LWr:(,̗\n+:(5䏤02%DQB{x-(*~.CJ\nSў#K|䆮ɨ2C@aBbCqyL7K4OfQ='<!ٙfP+`gNDUҡ!\$\$-/3Az_@d~Q3'>\n\011>J5Tk8;dY^ƥ\0Ӈ(F왕`kQ+I}Zg0>0MW{z_BkП;`(-wJe&ؤ;FA%L\r?!̋\"V_5G3s?-eتQ,Ys?24~l\$߱eؤ޷G\rrHA~O,G@ldϲYlbЂ?#:Sߒknü,3Jy\rgfπv/4ݒkdA}OY|tKAޗ?|ށ-&W`_\0S\"os~Gr\$Dr{#'Eͽg/?<?:0'Zn79h@?b@(3o(.,o>{I\"䑂\"`9ډ^-F7%hҰ*֬@|	\0i@@~C\0XX\r,3\0ZT 6.<;C;2b\0K=1#! 5:T\nꙪMtᵀil@9Sb@(81iA @\r+8KB6~\r8-RL\n*`61wB[Oٻ:t A\n@J\"A8kl[Co<_#AFXnl(W,ꮈZ6ȭXn\0J3Pu>>d!=V{KGecF龪Ɍm/0LXOi*˻\0B/3z(}0+IBPp\nBשIui,)0%f	ShϜ{:P#_'Tk2h Ⱦi¸B\r 0kΐOn#>l	\nB\n2̐VOiаYbs\0dIſ	16B[,\\+2(&\0\0\rp^Z)@<ALzɐU\r\rtdH\rl0DV1 9d0Lt@[5P	P/+<Bzzn;f \"\nxgj`T24X @;7\"ț9hۮ>c<C-a\nD\np9bZk *2Bʡ\\1XC'ɁDD6; 9;+Ȯ`ʃJC\0002oPH>\rc`2A@F`ۂ%\$\"D8+A\\`սy&74x\0ºtѢp iZHeHRD#LZp).bɀ,pB\$%xB&TɈ`E(Rb\0;F1ioTⲀ4/k<U*\0K\rQZe]\0ɑLEK:),Xc(?N,WVGBʯRqhŀih<SoŗYEM_YYE]Q]ųWKŻ45qvzEB^r4.9\nal*+,`SUb/QEkQ5XcmTPT{`%=	P\n\0x{HqB!R5P`]	i>¤hF\nN<<| hOjᝐtڝC)F88(18NRi\0߯i蓀-@'2!K@%X\0Dk(Z\0\0룆#ii(/-\$ػ`t\$[;^ ׃;O/:Θӽ]\nJaL9FRS劣\$TdՃ~`62	jD2\\OGQ8 XE4nlCfA\0@bX	bXd4bk#V\rt~W5ћFEN`m#HFOX\08\$%\n;()0\n:D@@)p	r)0jM\n\08\0(\n#!`QQ\r(8J5R?M(X)(<~QG졀Rѹ6䀑 dmǴ]\"b\rȵʁ &>A\$h?c(\n\0>	}R~\rhH{,G<m(VN\"\0_h7:،2A_>R\$1\"\\27\"z#Gl~rDGml[I-#Srr@u ;d* I/\"1']<\nHwAI 8#	[v\0001^l#27\\}ɒ3#7E&|i9l&v\r9'zC./3'@j+h农*r@hY;'2~(96{A(9HCTD[҅](,0u(}3Q)<R2(RL\rd'\nF2{J|u((SAȱ(o%( °\0[.ʐ3򙆚J1(T2\"jʫ*7ү]*I:0.!H\n+C`(P?ҸLaF+2ʀ9 +σ*AFL60\0+c\$@cP?R# RXy:6pD ,G5(QQԤcP\r+į'JB8,m8-PpMx̥BV}|G,< 6\n\rҲJS 9ZĻ2.E1K8:ՌG*A &5-ĸ!jKAe-9'#/U's0'\nLUJN.mĶ\nK049Lcp\0<L0t2B\$<LBLsLJxhs1ln'|WdLm,\"w*tLo-Yhߤ\"Z1ȥx焨Ĥ /1U9̤ʒK2s.'(̂vI|̇.cS\r\$a3r3\rJ#i<\r 1+΀J4\$N#-4jjM\no/34tHʘlȒ8L/4SN0Q4ҳRM0]K3>%0')L?*Ts|3`̋6|Rͅ3aJ&rMxs92<s+̅6(l͑1>9͟5ۉT6<x\0\\slM/}GJ\0006M7j7;3gM7C+\"K7s#~<ˑ8di\"\$+, 08Y&67xb/}#3\08L	29Mu9K1*-/䲟\n54qKœwD栏o1She~#slr:ӜN|\"4L79?O}\0[KӉ7eE(\raN)3ܳJ.k2BFKL)I2o9%|2fsI'D̒u'pSBy>/|-\0sʖr|O8DH-N<uJm:=X%)0Y32o\nդt	M,lDͣ=K=+ق6OU>I>\0MR\nгOY'ASOM=DSϫ=r;ssO=2?N[.D3ɣ?O=\0\"LO[?u\07@T4v+p+\$9L.1,HJ̎GP7F5>U'A5P?A\\%?Y@MC4LAhd<P'TN?4%̢\roBE\nҁqALLaPDT	T.B\nЯ.422؈)\rP?UT1P@D54\0ԶL9II}'M*3\$`6ɫ'Hrv9\nPP?lP<QUC_QGB悌P4J2|q,}菦>0\$f`)PY(+\00 ޕbWQ0p\0\ne\$rPs\nQQFn0(@#J@&ў3\0*FZ9\"#>	(Qn	FmhEF\n`(N?r;\0\\R&>`'\0x	cꎮ(\n@F&\0n\nƏR/rD#đ(cQG\n>ďTFRGќ%	ѥGxtjѮkTJpArGJ,-Ү(ԁ#!e+HH*4RK04Ar>tGRJ}'QG	rQGE0\0H\0eF6ҍJ9Km)nPGJ8tK,R .tSHT\0L+n((1Gu|G\"H5t!@>S?M5\"4RN4H#`#ԐI5c#I=%4IIl?6RL%0ԂILQ3S@(\nTұN`0kM\0I&'qIT\rI0NR52rE7  G, RoI{Pe(5Ҋe5%#>2`\"UKe?heK\\\0	X*7kTH(#ѻKM2#	R\n%*-!TQ= UT?T1O\r.T\\% ,UR]K!Q%+MQp\ni[\0JJ!SQT^}47JTS5HMSO9KQ`\\WS+\0+%MPaQM`GG?.Q㨉@#p*='RtӬ>USPPrR\$\0%UC0?\\.UuL(u7(\0U7dNIfME\$5K?쎃?0jJ\rT@\"Hx5oUVUW)yS)M]TS\$p>FcOZU.?S5mU8%<(QFuFV\nMTK_U@=\\5qL?\rbusY\r4wgY!1#eXa@U>d4\0\0#p	>\0=  h?	?L.՜Ԩ	@'nX	5`\$J4eK@V-nֱKuV]WիDUZm6hVX[\rVM-DվYui;uU)BU[\$ģsTMG4kH!]uWR}oHOoI\$?EqH; \nTԙG:#\0tTMncT-DVJuق?T%vCʏeG2;y]hh\$W:)CWs^wuuV`M^E\\W^*ՙWRRWVzNן_Jtא>׿WgV5wG\0S}FZUV)ZuhWK	4qHUU7XhUD_y6F\\T`MV\n`}4XS݃e`H\nGpGU&#%}r	eW\"?=1IZe*֞饄ܣT,Xdt	\0&kTbMP-TN`%^BU\0!\0a<&GH?D%eM9=Le}Q6=֤k@R\ne(AWWu WB]oY']8U@єVԢ-L5yb kHWh\rVO\0Vj?UPOhӫQ	#\rmWcb}\$Le?4jVk!Q`'U%^hREN\0Tn휂u\rT_*\0-\$]76mٻY4TmfU&8;p?5RU\"F*?g-x4X쏅IuSRfi[RSb8	4ٽg56g*Yb͠VUE n6t}O5l#M+\"i5+t#yV ] QԆQMZoFե=Zl魥6'Zi͇YZgQucUQ/5sZ T0>&cU@Q!ZMU\0.\$YP8R?}kiցNMITDK#x'TRH7G卵Tގ-p\niUltU|VV0l\0D[+lݎc[ πcM5|\0l:ҤfG6і\r1=m] \\TmQg1ہX᣺>fuebkam ݣkmQ:\0>##sn}'g\0ñZU\"XukT>2URO %\\b\$\0`%78[:mm7mH\\H=vKL\$pKFm\$SHZ=W%c0>cto%X}L\0\"S%Zo7\0#Hw\n{*i	nh?]\rqHT`VmeUꀿKi#v	 \"\0Ű#PM7Ihԝ\n?gT7PEATRPrM5`S\n5x@69hE!6xTZ4\r;Qr(-K;` tUK/VN@S PVm@nvbTt>E5;jC?#rLcT[` yT\0p-W38-IS+T]\"::=N)XOo:9\0q6ݯr@! Waۑ]e#@/?2tT]wUv%mܒQ'o\\շ֑H<4\\YxSaYU\$0XqHŔSb W)! >Yyb-\0>UYKG\0kwדSEy-nck-	؟P@\0WY`\rgtUD1=M޳!u<ĦCר\$t`d9́\0z}cJD@b;\$.{iTP#\\ɑȍxTk|&e<<D,B'|8WBzk-^p!Pf%:\r\r.\\_1z\r\$=0G|BŢ{z|Շ#='ڭ*Rź}._nF7C}kP10ZJ/_eJ 7 <n?-!X],\n`+UQy]6Tr8UfӏNMDRO0&ӑm=5i6׍];@=KTj]5YY]\rwhԑRP0]u2Ӏ#_iG*?	\n_Qn̔}40m 0\0t*: ,7.; UX*\00049e. J	%\nMX>;!Bz@MtHa>1[?\0N\\<,+ЖAv8D	Dv\r(ujƔ2(܃nIjH\$/^!s@a\nv&d/A{lNƠ`'Tn,!<k:݄S@]c`،hTT`^ T?;{p5x4Dx=XkA\nA M\$S Nìo& ȕ:kN[	nҙB߮/Hz:,t0+2;a)vPLz)	{#ڂ63b/};) *Qb,pb&5pPΕY1\rX\r!%a<O\$h\0006/oi{)[*'4Gpa!Vh@-bH? JxJc->*fb&A_\"%-=W{JYb~%;%X/ \$QbG8f,\rxc(\ra:v1`>c&aa%b@qLHkWt\n	7ɤ+V|?NcQ` cgh 6F086xߝA]9\088JՃcη1@ 0ab7x\$?82NS\$J'D\\5A%1v3O3!7Nrh#;7{&%Aw\$:;pK8c5ܘLn,ȔȀ#	\0@:RNEB3˯.hS=.3\"ELscRv)ǭ\$iOFImљn!Jb\rTd|`On;(h5wd;kNʪ73T-78\nUY7Ds7@\n5.	Tsf~kn)	mA7BNdͦ>@E&P@ ツbҝ:ҜAE\0<\"Qk7X:\0atl;\rq\0)|\\S;(Ys_^c&(|Yj^~ZDƸK+\0܄;=ї +A(6\\iBz2mXB_}6߉.}_ӛe [B2e|(fzZcf}ن\0P@2AdbyfbYNmA2×d93f\rvde9dYfnace/fٓf9fe~4?_{f-l~7ں}bYvMLLveш\n9EuUY\\	#\$ngB< ~w\ruCW-d|ǬyTz	1,k9QVpRO,hCB~nY˸QpjY#NXWumZ(g3VL^oygq!gz!]p.:q)	gtJa|u܃a6	/燃4d\$6\n2#1.gsž\\&u+,gwyYK1 09:מۭf6˞xY9Qb\$~tX'6z.m`19s@4̓hDy2☾vqζVD.\06<\"\0綊k>P91vzϏ\rN՟FYV}\$:6`::';OOd\$yF~8\"턚.5y6O,Q!=t%e\0\0yf6}R\nA`Pr,C\0k@SzBQCX!I\0.vN\$@TcFHiZ2֑K\n)]i>77߀MbŸ?ŽC;CޓcI4#0hTMD=zMXCYi@`,yCݑic;zV%,M%~:ENY.NYN/N7h<A j\\\naW-x`ډdi~KP0M*i\$Fz|QAVI=j!,:tB0-zNV?@KAzxDbVK\08KD^;GgjeÝF|oC9un(\0*4A1j\nBf=nQzxb܂D47i,!vJP!XΎxP{ZvUӀjB^!dj\rK:4z4bplCCܢyAo\$)6zQ?A\r`\\zEיִ\r݃s:Ehe>Ќnfnڥ;B管jn~wThoM[(KKɮt!ˤTx4oyƝEKR6:KG#.\$t&7c-@]QQ:ʊ߾Ҩi-,lQnéqO+GH:f:ꓯID_BoMAj9\nW3F~/f9	0>GdD\\A]bK\"\rF~[c\r˸BOs1d!y/Ѕn\r07\r	%h\n2lJבց8\" hBhjJ7-b*K!FCV4SKًF-~2;FKÛ4nZ1vR9\"L:.νdQhkank#9N9ƲdU\0N6OV5+iǢd]{جc	gAM^=U{vl\$P5/(\r):`F_:Ɨ=	!yV9ϟEQ5>:5<cƓz	M1[ndn/F9F#`vX<BFjdN`Q5󞾴K5o	h;#BZ>o@ck*@֓D\\S)pۭsC6pU[G4?.e\na	>W@{.£훭̵\\9ژ>CAץ`0d]fM1I7[\n],qVJۑ?tz]um*p+틽.\0HW;+Bzox;^nEtKhqꟓE!+n=T瓗xkj6{#h#[o}qPDղÝo1xc8D\0񲆜J	v=WFzzmkhOޓ5j\$X}<A>n{~h]\"\rGDxQ)=:5G:PD8p	sH2pzt\\ڀk|)Yt	PE\\D0¾|p1Ɛs=&`hIO\n,M틂>Ae\\}\\>գG7Nl\\L4!5c,T!p}Ĭ<QH艞89!=F1jːA@o6ۏU9Ĺq\nM<_}3q\0\$no>\$z/	+q}1o\0F8?Pr;<NGEc\$*qU}sF8bC6\rkGm 4K<~4H!jm8Nkr	f.Uzh#SrU(	Zsnz!/%\0/&}ں6rxW`5cGObW\$bM]\$?z\rޭ\"qJΘnـA&}#[%ɸ-'gt\$ƕjLwNre\0\$8Z#:;s\0M\\s\nDMeAf4IBԾp`@%Z\00040}O.\"L4]\"'Hfי1nыRetFޮ.MY6ȏlc>h5ӂ}<Ɍ(7FLrm2(%b7C\0[͸Ms#V6Χ5M	&v797@!\0|N6\$ݔvn!TȠ<WD@M؀_(;'hLd+rQˤHiʱ3,)t]+p=<tq1o3	Fe}%\0001R,SO_Iͥҍ)lt8LIt:&\0Ҥ!?_^}0d\0i\r'gA)4?/LtθIE|4W?mi7g	Уu/C1IyI?C{SZMemKP \0~\0A5#.\$sY)|ҊM9yd]ϫA =9	h^rE@SO#>0LHKHE%t.mOfѸR{~F%8sKBYw]/#Qcc)HT_GX\\pr>ՕFlXcVnu@ud85lB -hETV\0h=`-TuvrTg^5Q=b4lZMUYxu'vC^McٓUESU1#d&ven@Rn%?d_vOeŗWiTwf[)?a=_/iVMX]VodeڏfEI'j,mpRcj͍8?^Vg5Zc+}sk\nWueVZ۽vTlU^UU[S=÷kٝ\\ݛ;W7guxҿU86vv(vUOsէ۽ow_U?iY׳\\utyQuVM^]cknW5eYG^%]P_[cWs|Vo=XwuY\$ݕXYq:w]fd=CUd=v=Va]H`\n]w?wiQlOjzguI{Yx4ViHFVl+{FÕ>\\sErVrܟwY}\\uuŮyd<cptq]9]!j=Uc;ybGSREהT?s'ׇQ̅TwF}=Umw-6SC.ag&x{-;߁i^1|\0u	Z^(I7c;VU%h͜Yg\rt\0Qhv9cPHy?8axDg-!3Yg\$Yݯj7P>eeXbshaYD/fnn=	^μﳞ:V[LNax+w9/x>+a\$L;(SFto;lyxs\"	Eߍ-@׿5>~=!\01BUSb\0O8L}ѫ4q8L:.63.YroɀYz[_+Qp?62/xb2ځ~-0+r~mCX!b\0A89&Rh	H?ɖ^WdE梾bϟz?\\<j. Jc;\$);N[yj	_HI:B*ļ3:S.lfPQöhF[6Ý@p\r{ӝe;|VsFNP+kog̝6[>֘{l+7{+f\ncl=ypy;B\nìmǒy%h@L4``{cnF{kz^[OU|\0.dwy(gnJdϼAOQF_:bPPha,	1:']Pg}66XЗŘ/P/-I>Mx1b޷ U#`d3zŔ?6Ctxǻ:L׻#,?0|SmwTi68/˰%*hwç,@`2M}E %oa)_QNM׿\"Yά)PwRMƇ?ա.B\r5TbX\$X/t!)	)I7Ľ[1}n`o`~AΪbtoʒwڟhn/{Iԟ}<v b(>8	\r3\"(\rp\r7ޟ{l:o^.}~ݯ/.m7\0s?T~?><|oMN:ƠyJq\0o\r,<}2	PJL~?;W-i_ݼ\\}:\"PA;5\r @+8~fDߤr\rٟ,t_\"ƿY?'ߣ}cٯ4\"l]efȐy[ILNa2!fPS#	4_J?߽Ġ[~EN箒4*ÂU\0%8ʇQ`SH??h\\@P2 J[xLG?\0ȁ>ü/R\"3HB{<.~܄l}}<|_^w/_J:ަ&whklN[T@(z~M0#h+ܓ6GEThckѝ tS2(q[ŠZ_>Y\nTTE\r\";(Xs-@D kSJ{(p a^\0bZf{#diDL<2lĈ_vP擯	\0%S0*D!ֽgЅ;v4dP'1qZXb.Yfմ[<cS['+Ђ|^p Vbn1(p\n\02*ge G} -/;1^\ntqzP[ 	p\"%Z\0d\"9+.FOL1o}jOPhCDE\\d_j9Lc&9xV75|te16P5B\0}*2Jn=fBQ'rR	}RɎB8>KưMC>Qɪ`P3inկwPa	#c3YHEh1_k0\npeGǟ1eh=\n29t*\0h(!sQV\0{j&+@D[ַ0ula#M;\rtXǁjhQε4CM3SM_w6;A0n{l֠Xxz	zfHBrl	K!dO# n~ps].1jh0!!r0ppd9iD%rf\0P4	3g7>J\rLM2k+8*ZhFߌґ1ZhdFٌ.Aй. mNY\0փKXAx6Q|h8fc/%}帠qcnWA``PBL惁ɂj`+\\f;gݘ,<C;>gS:8\n,۳XA	c}H?òS=*8@7R(č^ˁ7gj߀W8z8Y|CܰAFD}#PxE\n#8P5nMFX 6rݟOzB_`LԐbENMZȁ\nP>Am7PGGx91\09B^kt97P<7VqJN)_u-daG`<oĳ\$'JMM	yp܍B4i(@8Uhb~<(\"Yw4X7fzPA \"āAbTTmT!9.PBLh.UM_ĕ#VpB([e^	zG- 9gtEd?C 2VɈSO'<Zu(ҍ{e=C\0vpO&Ki Cಷ4n|,/'MPU~lxv(֛(NQP۰d\\TsΑڨȢˀ@\0HN\$xNo_)wYxq<8\\9sN͖'HC\"b !RIN \"KG8	\$sKDF!&i @b7;hC{HQ(=5q0TOK4+{pO%\n	m>JWlCRr\$5)VLp JE\rؐԤB8i\\6nb&\r2<8mۇ%\$ࣧ_f!_7\r+63pǴ:V#d'dMt9jJ#CYr䔾L:u~=:t!)A]if%Up)V.J9nyGnn{ȇW\nU;w^G*\n\$ޣLrgixdte:bݎ>\0Ku%S*xݫ7^ ^%)V\\LbrT6T\$M\nD<,cS죉LA?KaDT2 @!.U\$}#ۮUT.6vj巎Cvⵍp֕WK[	\\'p.ߖ;ZbiRKV-_inQ#}nU|ZfrG]v˶ՀU[Yoj8V*w\"y*E+YHZ9Re p#aZ8}Ek+xhMx1L'P	:v_eAփu=Qx@h+ܝ\\I\"\$nC&\0t4@b p[\"KDVMMKY^A?d)X!lIDk~?Kg7\nF (,,l9'Q8DoX j`մhryMn\0<ǵsF6;Bugs׶\0yl|2\r]sj2B+у=p DO~2++!^H{_li\\ˆ`\nK&/j 9ݢcdD'o@cD/?P\n.Y\r%\0(LEDGәҹ|xkA!Ic4Aeoq '9XXxCsWґ\"{Ӏ\rY!u)\"5fFNEPHHl	&Ӭ\"mQtZʑW+Ų\$ .Ǌ-`a	F8oX#ឺ&R>> }\\X9v~.o/#xS,4c>pC4hg\rE1@O|4(e\\6*	d!ҋxMp`\0007D4)cdPZV\nɸ)@\0001\0na\04\0ga\05P@\rF\0l\0XƱ#wxƥ,\0dƱ@FH\01dd(8Zx@F.:1Xhш6\0a2a@\rӂ`\0g2\\ac(F7wepc53LbQ7\0sV2\\b`1cF8\0d\02<eF\0aB4\$b`эM [\0l\03f8Z:hXȱOF4ɑF\0ir5eQ@\0001\0m0iq`+g@\000520kQPF;\0o4dk \0\rcbFna3|kHQciF0{1e#(Fj|\"qFepdj7dqGF7nhQ9B2\\k1#OFM>3Lj5\05gq=݌T\02g1ǣ(FP!5Hhѯ#^<\01\$p@\r@FbI8ccFHۑCG1HѺ\r\0i.2;QclƂI^9Td \rFFe2\$bq7[f8\\lߑqGe񇣧3,exőGAoX \rcFPaϠ#Ƅ5<q QF6lѡcH<,h`ck2/gadȱc;q3lF8j44{qc8O<c-Ƈ~8sьF1F8lfiǌ92lxqc]\0g8aʀ53lQG\$A?mqLNZz6uc=܍G68sıG@D~0QXfGs=|gq\$G}oz?dCFSF6oc<9*9hhvGG]4e\0001G\0c3YэH.!9qIH=U;hbQˣGWAq(\\ B,s(\$ƁlqҤY]2x/%дpaM&7m1GNBt&֏4<e1#O8QOFCR9{1dF~25c,E=LlQ+E\"2|ȱ#AGIaHČDdXбcƍc=4QcL3= 9Tj#*C\"Ffxѡ#3G#\"?͑VG#28}XcB;fy#1GZe2\$^{9c(GCoHDc&73R=b9ңHy=x#v@O R:|Ѳ#&\$\"3܅L#F#3Lñ,G/3eNc=ȭI v4,q(1%HБ*F<|1cIQ?lQ.I\$3<\ncvGu\"*GY<Ԍ(ױdGJ(YFSA\$1dS5#6ܒH(Ix\"Z8q#\$ǯ; Z6LtģGJ\0e\$34n1I\"GHݑ#^qY3|bY3#nH-<>i1#גFY\0QFFDMdc?HLJBbIT3I@T|(U5\0bLJBs	4>ǌm:b@r HAW1̇pcˑu'BTa.#3GzW4ĲG#>>u4&?\\dFK#cI2KJ}r`Ɉ#=bi?q#5m(^:k#R6dVI'3<yҒ_.G4&:x2G\$G{r:pZHmv?c9Cqc\"H!v3wq\$H(KLY	3#4?1)\$ǣ'7k*d\nHWr2X#Ex23e!k(b988<v44u뒓A*6OI%GH	<GI'RKlhcW)<d	?iRǌ%L1)KqZb?fGtz9RcГF=}RKnII!F?<Gqj~%\"3Č(;\$Jō>09*3I؍e\"&St(Ų#ܓM!6Bԡ01УYHVAtpZ]ʤw&\"G2jG#5kNҥdƹ X	,`RdGC3\";zO2#b\r'>mkI_'1<91xc\\t\"%jV,ΣbC@')\ngV݇\$ڻQJ͉hk\rU*`M-<EdBcMUU-<BiY(wؚ娋GeoJŕ^BQKZ\"[b^>(Y`LM?%?% -fTZ<[p Ľ]v-Jmrѫv-an` ,pqs:%PאWb\0hGc%˷%|z0Gސya)4p#\nTO0}2/p?e;W&0ĶE^nT3zc[v%<]Q4A}ԁVT}R<.\$4쿷Fܗ#0NY\ri\0kGZIk\$kNms\n5!KB%K``\0'\n}Df\0֢<,-@ǍiK_,fe/Zu`S0jX5@WDQgp\nubZx=-\"a:\0J\$x1m` \\@!-ZHJ)Ց	4M\neke5zb|@P09ZFf\0\n/=˞dRCK-atlJ-iT\0GDUƬ\n]Gjŕ\n;fGKW!2eX}j%L_2\$+c&U+Xd\nƕ\n_\$N]\$0%z-^2s\0VIKY\$D?Iv?Lt,εRUmJf\\(P#֖L\$cwjg<~bPi>Գ\$s <<fg%~pZf@kKʁ,%Q0d,MT\0(^jvhϐ*ȘVJWY\"hB&k)v.]YC-gU\\C\$4]dYu%W+w&>[M7v-sR)K\$04Zɇ˴\"S8!P\n@!\0tDWe#)Kve[E<C1j\n5MWO.k9#)YR.fk4+Df/3Fl+*lR6%ZE23	i lЇrf͙%-ayZMqQj^seՍΚ-Zekx	5s{c温v`1^ԹJWLx2^%A̳RZ]_UĪ^VMY_kY+UUMjm7)ZBuZDm6:jxf`7d`\nM,HY9J[ȯfmܥrM}XF{W	L&	f;Ηﰍq1L8q!TU&O2Zod \\5qb9'efJVN'wa)UgVM^r3٭fy.?tMsJkk&-5\$\0\nf8m,ɱ&;~wl9Q'3͞%6mOE:	Xz'3A]:![I3\$XֲՆnWͻT:A`aMM^V,s<܉ԫCf<K2]t)Ө&ꉌQedwa3UN:o	 ҪRo4s'.;^s:fӴfm:Rvtsc\0Y.s:8{PN8.vY'V0u&˯g<2xqz;Yh6iNfNMy1wºb3\\4RQ8r4ߪ*KG<5Sc;Y5br\\˖g(OU)9NxÀ*윳;-ewjS0NI4:Sg9<	TIK>O)SBlyѝNxnt3\09\nO<~v{6y2՝M=UTi<'Y|Va<YI=Xb'Ϊ7-eS\0N\0=uuTW!QtiveR=q,sg+\"<o<YygNʚ9ntsNϖ&ty%Cʈ)=MWd+;[igy;&pM,\nCGDӬU%?(:ũ(\0_VH?x	3'GќS<I}*GKT³6Y	gKFJ:zCʤ+O~O9r\0&&y\rZ/*fK1<0+W,˖`H- Yj_gH,*dr-B[өM_#p<Eɓԥ\nT@%g*@)·.]jhP\$\$S%͟Y8h i,ʂz%'I\0(rgFW]aYiE\0)q֣QwDD|3I)[,ftPS(7;/HuYݢ9l:3ԯT*P_Ab:!O֡BQjZgNc#BlZSVО>Gʦ'gPIBqtCJSW)ZҐq!Tղ8Yg5o^#uA*eNkUn-,ЬC.E04(R=y q4LO\rm(W?YRB}8\"kC*`hnP]C*lb9kR*'Cb?Q\0Xgt%vLXTDf,Hȵr*?h_!1Dvl9Ћ(D)Czr)tIX>'.SJ6a&k29uدS5b&,#YPa*i4RrPYE5eKJV<-QTEMHZh)|`\$!kH}J*d(0;X,,EQ}A>奡ԃLCS5?:/.h,^TniZ/y,,1Ewa!Q\0Ef9I41]˪SћDq碼:,\$Vmd&6pjf[Q@iKZeͼ^BB\n ֒wG߼@Tf<`-E*\\X*\0\$Gv* t]Hpz0h?{Vvuq-fHN]rȴL*#\\BJU&4	O]@J0\rTu=bʣu*N<\"EFV\\ϛ\n@E/WR\rhI\" qb?T(YZBՋ2ΩNB;{kLWJeJFJԾ)HJҺGhQr%ZS/+1bʛ\n64whݢ\n_%=''vI~rSi/.ĩabE@ -Y2?Q螰RZUJR^:3`KUѐTHjQ?f\0RXYjl'Y~,Y}Z\n(R8Y)Td\0Qs@H\n\"-DTJJJU4|?O\\]IySUƢe;ɩ\nh-[ʖ(!&/'6JVjVk4gخHvQ#I(Ι:}%u1Dy	nԙ~Ҝ7Jf*1>G\\\r!tRKQe/4YXRo\0Pp(*)\"#\$Si)ra\\/(O\$jF3f挀(t\0`dU	>hecH\rp`gPc[=Lf\0002\0/\05\0b!`&\0]*px)gCZud-<\$k%AzdҾY\0֛5k\0006Қc@@\rx7Zn4@zM5\noiSTtF5\"U8@Ɣd]M]5pg)bMڜ\\r`n/M7Js)SyGN*7t\$ρ\$6ʜjuiӍsN&dZvt#PFvҝ;jqG}N<jxT)SƦNޞ%<\$\r)߀_ME;\nxS=otS!OU>jxiGO2tzyt)ӒMf:==J~#اNR:==xtSPP@juF~eP0@~tiFfOZtSϨM?*S?NB@*~	)Sը-O=:jӔoN\nz}*Gq]N̕Jj!*yMf0Az*\$Q&B-wPRډ\$QMImB*uiT[Q5%j1jQBEZ}*#PE5*!TtMQJkQA0\0CCJT8QKG(˵\$=Sސ#RJE*ITl \"5I0j+ޑQnD`b#Gƒ0j4T[PBF\rJU(8ԸR>kڔ+\$-%'~5Gژ/*\"TS:J*.[F&~L:r`%TRMʕu6ӖSr=ꛕ-iܩ\03Am<c3KQRuOJ5=*4TSҥOJITSƣPz)UT\"uPDő{%[TV-HJ5BS§MuEH#RN5Q8F*UATRHj!;T0\r1cCF9S2ZQcƿWS5RrujsU4\rܭJ\0}USUZ3}UE*X?S<\rUh5X	`S:UV*5E[bT2MV*=?UVeUګ=Qhĵ]	Uګ}Q8]UZ:X=OՈ]S1XuaKoV\"-PXU6UTZ2Yʱ=k՜'VjTug*UZ0[\nAcհGTZ1%[\nkQeV­mTUUEīgTN4=\\JAcīwW\nUHq*UL	%Etr*TR	%\\Œ*ϫ֮St@I*ꁻC\rR:R_0vWE]TͪW֦}_UFU9WlbUj%WE`JU'Wb6maՅD1X&N_}daX^5YXUbXk\n)X֚]ՎUXcjõ\\kGOUֱsJ}GXmbڼ_\$l\0F|EuvB2k\n˕/%cu*8Ye^걵(VrWZ4]55VxAVnY5cV)ZZ5cVWZqmkGۭ'Yޯtcuj8Zz%4kQ֘Z2i4*ɟ	&~2c93CQFۭ[2]3@ɟWP.kڡ5Z[Wj*dVYKk*kV̬W1eھkLYv˵#.ΌgZ3]nׁ֖T b<jKl>+a+<]7\0><s+=LR5S+]\rH2Ȅ[:%cs*OviL<5V@בwC25S'v|i\0^n{jfVJHduBs;W8Yt'y6xBDʤ'WN|ً.P-q@	fsr͟8*í`1C=o5Y%\r5idE&ѻ8B\$5m:k+iƋI&W\r,Jڸq%<k\"!qUf ^\nrYUV^#6Uz9S+)W7*eD)f2v\\f:)On3 \nk\$;piՔˮ8+*\nث}9QSzh\"UاBo;ripPw]vjUkLҥߧ;;eLpzU&]ed5W0=UF@}Wә\$;^uZ:tj݇&(\\?{\rkNН_pSUX͝_>='ikPw;Ey:ja񬮉Y0AE\n浾rh53hWw\ri:i+R@FVx|W1^Njx\\IfGpa,TN|zcs0ELxm5*5ά@Dg(Q\0\"HX:Ɍ4\"1u(.`Ӂ屑y,0R`5A-	~v	+XqMs;[0Bf((&hq_Fڃ8~6Ob8'	Xdfu`42\nؿ|.KuPH3:^/G|<Y(<\nb>	Z;'zc\r3b2\rNL2xU:\r6X-b\0tT%PXSX!kćQua?vg.SS:ldtH\0=/`_3mF%lbуB0 ڦk5ň(PO??Ί<\nЋS=5j\n{*\03b!eTF3<bʃ*	5Fc	N	H=ga6e\r 6;\0&ĚaQe4ЂhYadL	\nդl*G_ׅ	yH1e.XjtY2Mw46J]MȐϽș\n,jxFG@*g\0_ XY	f\rm9yà߇>o(jG;8\"yA3׃f9	LmgQ[{ds(Y~~@@:	Y6Fa\$l)Ob=<VxYuvxč5YYjuh\r/c^x \rA,m*yw\0٫hu7U̫HA{#S{>Lh]Ђ&~fÞѸxm].B&emelH+6ZĿ(\0ǅٝ,:YPZakQ.~	[-a_:ɜbPcA/\rheh'uiӬ\0=m\0i\0JPh-6`rfi=mCR\"'^õTqlSUݐ^Q3T.A=&gvlM3@-T+P ƏQA.!\0jD[\"W,Z'QRݫU&vYX[i0\"՗{Y l{\"{P\"aWZd\0BPV.mm=0kv\r55Zൾh24lOZܵOɖ.,:FZ('`-NBڅխ6,§aal<6ܽ\0000@lM4ZcRՕaloڝ|&GIb3\n\r0(5[/fH\rŮZ`L^d\$LΐU(5-[;(8*v̓~|a64dl\n/Ly*>2?d!|'O(kP6!itx\"I\0A ,7bzJ2EC\nB5@!Fh+-:\0NMCsHہ=nA;so*:qB\0ۨNnnV܄4}k6Zʗ_tv3>w9\nL(Yy-B{G\$6ye̋td]2");
} else {
	header("Content-Type: image/gif");
	switch ($_GET["file"]) {
		case "plus.gif":
			echo "GIF89a\0\0\0001\0\0\0\0\0!\0\0\0,\0\0\0\0\0\0!M*)o) qe#L\0;";
			break;
		case "cross.gif":
			echo "GIF89a\0\0\0001\0\0\0\0\0!\0\0\0,\0\0\0\0\0\0##\naFo~y._wa1JGL6]\0\0;";
			break;
		case "up.gif":
			echo "GIF89a\0\0\0001\0\0\0\0\0!\0\0\0,\0\0\0\0\0\0 MQN\n}a8yaŶ\0\0;";
			break;
		case "down.gif":
			echo "GIF89a\0\0\0001\0\0\0\0\0!\0\0\0,\0\0\0\0\0\0 M*)[W\\L&ٜƶ\0\0;";
			break;
		case "arrow.gif":
			echo "GIF89a\0\n\0\0\0!\0\0\0,\0\0\0\0\0\n\0\0iӲ޻\0\0;";
			break;
	}
}
exit;

}

if ($_GET["script"] == "version") {
	$filename = get_temp_dir() . "/adminer.version";
	@unlink($filename); // it may not be writable by us, @ - it may not exist
	$fp = file_open_lock($filename);
	if ($fp) {
		file_write_unlock($fp, serialize(array("signature" => $_POST["signature"], "version" => $_POST["version"])));
	}
	exit;
}

global $adminer, $connection, $driver, $drivers, $error, $HTTPS, $LANG, $langs, $permanent, $has_token, $token, $translations, $VERSION; // allows including Adminer inside a function

if (!$_SERVER["REQUEST_URI"]) { // IIS 5 compatibility
	$_SERVER["REQUEST_URI"] = $_SERVER["ORIG_PATH_INFO"];
}
if (!strpos($_SERVER["REQUEST_URI"], '?') && $_SERVER["QUERY_STRING"] != "") { // IIS 7 compatibility
	$_SERVER["REQUEST_URI"] .= "?$_SERVER[QUERY_STRING]";
}
if ($_SERVER["HTTP_X_FORWARDED_PREFIX"]) {
	$_SERVER["REQUEST_URI"] = $_SERVER["HTTP_X_FORWARDED_PREFIX"] . $_SERVER["REQUEST_URI"];
}
$HTTPS = ($_SERVER["HTTPS"] && strcasecmp($_SERVER["HTTPS"], "off")) || ini_bool("session.cookie_secure"); // session.cookie_secure could be set on HTTP if we are behind a reverse proxy

@ini_set("session.use_trans_sid", false); // protect links in export, @ - may be disabled
if (!defined("SID")) {
	session_cache_limiter(""); // to allow restarting session
	session_name("adminer_sid"); // use specific session name to get own namespace
	session_set_cookie_params(0, preg_replace('~\?.*~', '', $_SERVER["REQUEST_URI"]), "", $HTTPS, true); // ini_set() may be disabled
	session_start();
}

// disable magic quotes to be able to use database escaping function
remove_slashes(array(&$_GET, &$_POST, &$_COOKIE), $filter);
if (function_exists("get_magic_quotes_runtime") && get_magic_quotes_runtime()) {
	set_magic_quotes_runtime(false);
}
@set_time_limit(0); // @ - can be disabled
@ini_set("precision", 15); // @ - can be disabled, 15 - internal PHP precision

?>
<?php
// not used in a single language version

$langs = array(
	'en' => 'English', // Jakub Vrána - https://www.vrana.cz
	'ar' => 'العربية', // Y.M Amine - Algeria - nbr7@live.fr
	'bg' => 'Български', // Deyan Delchev
	'bn' => 'বাংলা', // Dipak Kumar - dipak.ndc@gmail.com, Hossain Ahmed Saiman - hossain.ahmed@altscope.com
	'bs' => 'Bosanski', // Emir Kurtovic
	'ca' => 'Català', // Joan Llosas
	'cs' => 'Čeština', // Jakub Vrána - https://www.vrana.cz
	'da' => 'Dansk', // Jarne W. Beutnagel - jarne@beutnagel.dk
	'de' => 'Deutsch', // Klemens Häckel - http://clickdimension.wordpress.com
	'el' => 'Ελληνικά', // Dimitrios T. Tanis - jtanis@tanisfood.gr
	'es' => 'Español', // Klemens Häckel - http://clickdimension.wordpress.com
	'et' => 'Eesti', // Priit Kallas
	'fa' => 'فارسی', // mojtaba barghbani - Iran - mbarghbani@gmail.com, Nima Amini - http://nimlog.com
	'fi' => 'Suomi', // Finnish - Kari Eveli - http://www.lexitec.fi/
	'fr' => 'Français', // Francis Gagné, Aurélien Royer
	'gl' => 'Galego', // Eduardo Penabad Ramos
	'he' => 'עברית', // Binyamin Yawitz - https://stuff-group.com/
	'hu' => 'Magyar', // Borsos Szilárd (Borsosfi) - http://www.borsosfi.hu, info@borsosfi.hu
	'id' => 'Bahasa Indonesia', // Ivan Lanin - http://ivan.lanin.org
	'it' => 'Italiano', // Alessandro Fiorotto, Paolo Asperti
	'ja' => '日本語', // Hitoshi Ozawa - http://sourceforge.jp/projects/oss-ja-jpn/releases/
	'ka' => 'ქართული', // Saba Khmaladze skhmaladze@uglt.org
	'ko' => '한국어', // dalli - skcha67@gmail.com
	'lt' => 'Lietuvių', // Paulius Leščinskas - http://www.lescinskas.lt
	'lv' => 'Latviešu', // Kristaps Lediņš - https://krysits.com
	'ms' => 'Bahasa Melayu', // Pisyek
	'nl' => 'Nederlands', // Maarten Balliauw - http://blog.maartenballiauw.be
	'no' => 'Norsk', // Iver Odin Kvello, mupublishing.com
	'pl' => 'Polski', // Radosław Kowalewski - http://srsbiz.pl/
	'pt' => 'Português', // André Dias
	'pt-br' => 'Português (Brazil)', // Gian Live - gian@live.com, Davi Alexandre davi@davialexandre.com.br, RobertoPC - http://www.robertopc.com.br
	'ro' => 'Limba Română', // .nick .messing - dot.nick.dot.messing@gmail.com
	'ru' => 'Русский', // Maksim Izmaylov; Andre Polykanine - https://github.com/Oire/
	'sk' => 'Slovenčina', // Ivan Suchy - http://www.ivansuchy.com, Juraj Krivda - http://www.jstudio.cz
	'sl' => 'Slovenski', // Matej Ferlan - www.itdinamik.com, matej.ferlan@itdinamik.com
	'sr' => 'Српски', // Nikola Radovanović - cobisimo@gmail.com
	'sv' => 'Svenska', // rasmusolle - https://github.com/rasmusolle
	'ta' => 'த‌மிழ்', // G. Sampath Kumar, Chennai, India, sampathkumar11@gmail.com
	'th' => 'ภาษาไทย', // Panya Saraphi, elect.tu@gmail.com - http://www.opencart2u.com/
	'tr' => 'Türkçe', // Bilgehan Korkmaz - turktron.com
	'uk' => 'Українська', // Valerii Kryzhov
	'uz' => 'Oʻzbekcha', // Junaydullaev Inoyatullokhon - https://av.uz/
	'vi' => 'Tiếng Việt', // Giang Manh @ manhgd google mail
	'zh' => '简体中文', // Mr. Lodar, vea - urn2.net - vea.urn2@gmail.com
	'zh-tw' => '繁體中文', // http://tzangms.com
);

/** Get current language
* @return string
*/
function get_lang() {
	global $LANG;
	return $LANG;
}

/** Translate string
* @param string
* @param int
* @return string
*/
// this is matched by compile.php
function lang($idf, $number = null) {
	if (is_string($idf)) { // compiled version uses numbers, string comes from a plugin
		// English translation is closest to the original identifiers //! pluralized translations are not found
		$pos = array_search($idf, get_translations("en")); //! this should be cached
		if ($pos !== false) {
			$idf = $pos;
		}
	}
	global $LANG, $translations;
	$translation = ($translations[$idf] ?: $idf);
	if (is_array($translation)) {
		// this is matched by compile.php
		$pos = ($number == 1 ? 0
			: ($LANG == 'cs' || $LANG == 'sk' ? ($number && $number < 5 ? 1 : 2) // different forms for 1, 2-4, other
			: ($LANG == 'fr' ? (!$number ? 0 : 1) // different forms for 0-1, other
			: ($LANG == 'pl' ? ($number % 10 > 1 && $number % 10 < 5 && $number / 10 % 10 != 1 ? 1 : 2) // different forms for 1, 2-4 except 12-14, other
			: ($LANG == 'sl' ? ($number % 100 == 1 ? 0 : ($number % 100 == 2 ? 1 : ($number % 100 == 3 || $number % 100 == 4 ? 2 : 3))) // different forms for 1, 2, 3-4, other
			: ($LANG == 'lt' ? ($number % 10 == 1 && $number % 100 != 11 ? 0 : ($number % 10 > 1 && $number / 10 % 10 != 1 ? 1 : 2)) // different forms for 1, 12-19, other
			: ($LANG == 'lv' ? ($number % 10 == 1 && $number % 100 != 11 ? 0 : ($number ? 1 : 2)) // different forms for 1 except 11, other, 0
			: (in_array($LANG, array('bs', 'ru', 'sr', 'uk')) ? ($number % 10 == 1 && $number % 100 != 11 ? 0 : ($number % 10 > 1 && $number % 10 < 5 && $number / 10 % 10 != 1 ? 1 : 2)) // different forms for 1 except 11, 2-4 except 12-14, other
			: 1)))))))) // different forms for 1, other
		; // http://www.gnu.org/software/gettext/manual/html_node/Plural-forms.html
		$translation = $translation[$pos];
	}
	$translation = str_replace("'", '’', $translation); // translations can contain HTML or be used in optionlist (we couldn't escape them here) but they can also be used e.g. in title='' //! escape plaintext translations
	$args = func_get_args();
	array_shift($args);
	$format = str_replace("%d", "%s", $translation);
	if ($format != $translation) {
		$args[0] = format_number($number);
	}
	return vsprintf($format, $args);
}

function switch_lang() {
	global $LANG, $langs;
	echo "<form action='' method='post'>\n<div id='lang'>";
	echo lang(19) . ": " . html_select("lang", $langs, $LANG, "this.form.submit();");
	echo " <input type='submit' value='" . lang(20) . "' class='hidden'>\n";
	echo input_token(get_token()); // $token may be empty in auth.inc.php
	echo "</div>\n</form>\n";
}

if (isset($_POST["lang"]) && verify_token()) { // $error not yet available
	cookie("adminer_lang", $_POST["lang"]);
	$_SESSION["lang"] = $_POST["lang"]; // cookies may be disabled
	$_SESSION["translations"] = array(); // used in compiled version
	redirect(remove_from_uri());
}

$LANG = "en";
if (isset($langs[$_COOKIE["adminer_lang"]])) {
	cookie("adminer_lang", $_COOKIE["adminer_lang"]);
	$LANG = $_COOKIE["adminer_lang"];
} elseif (isset($langs[$_SESSION["lang"]])) {
	$LANG = $_SESSION["lang"];
} else {
	$accept_language = array();
	preg_match_all('~([-a-z]+)(;q=([0-9.]+))?~', str_replace("_", "-", strtolower($_SERVER["HTTP_ACCEPT_LANGUAGE"])), $matches, PREG_SET_ORDER);
	foreach ($matches as $match) {
		$accept_language[$match[1]] = (isset($match[3]) ? $match[3] : 1);
	}
	arsort($accept_language);
	foreach ($accept_language as $key => $q) {
		if (isset($langs[$key])) {
			$LANG = $key;
			break;
		}
		$key = preg_replace('~-.*~', '', $key);
		if (!isset($accept_language[$key]) && isset($langs[$key])) {
			$LANG = $key;
			break;
		}
	}
}

$translations = $_SESSION["translations"];
if ($_SESSION["translations_version"] != 4206916610) {
	$translations = array();
	$_SESSION["translations_version"] = 4206916610;
}

function get_translations($lang) {
	switch ($lang) {
		case "en": $compressed = "%(n0Q :\r	@a0p(a<MSl\\;bѨ\\zNb)̅#FCyfn7Y	h5\rQ<ΰC\\~\n2NC(r40`(:Bag8i:&㙔yFY\r2 8Zӣ<'Ha2܌Ҟ0\nb豌n:ZΰUQŭwDmfpQqacqw7PX3to	ZB9Nzs;̑҄/:|<4j'J:0rH1/+7(jDӊc栢0K(25B87\$B/h8'@,-BƎQEP#O7Ct\r`j[z0c|9h\$>\0\r\n҄=Û\0x\r\nC@:t\"~8_)9xDjΘ2(-xx!H㌣.-D;W+863@Ɍ^F+u\0ᠡ*,1,i8cxؒI¤fۣlZ*/cs.00~0YWB07U\":3xuc@#пC`2'3XIl*83ʮ߷ 553#\r9{5Uwxj0B͡.'(ǧh\$FJŋļC.!n(hڂ߉@N*㢺칈05=RtP5&ĻɓEĄ:ew%8beUz+q3dɓ*\r|7([Ӱzʲcw<]0|]G]MvXl̳?wG핷W:*1ngQW 2<1_OlT@PT%PwQG\"p/'!RD[	9Ufi*<&xޫx{ApEh*P*\rBukI)G2zSj/i\rYuLOjPJF\nzS	m!&hbd]IF\",C3DwIBܜlcgAlW\r!+Oz=G\0\n@QXj\n[16\":9\\5reH_qIy9s	Kn4lL|L@FLQH7J12h匳uDX!0O)8kxDI\rF'F8d&e3[#-k^Y+e܋r	9ɠ*ٷ`+QD4ʾwbKL	L*4\0*Vv10K1?R4s	50G]#4o\$DTHSfDe20fFhH)vbz7F\$p \n@\"@Up\"]J\\0PbJ1H&26J <Q樞LT2ЋSd_Nl/Ś !L2r_j#KB%I((+\"	r*3g6yӐ3:p#sP\\#Oѻ01Ri4\\0[`k4W\$DY2MUq#Keh.ivrꉁX+|<c4bZ5\neh9&k#>/``Z)elL1n,EP]d\$LƦUoң:u_ݟNIlSli8ªAa QB7~,5`N8;9y^.i0ak#AB{V,Zy[̋@Ny/><4BmL5vsRhגGVlјSG_ vu˹0 uԏ%)9%2aݞ,x	G(+PŲ,שFʆYˑ;%+#	f8>YQGگL%1dם0r%,&(F`fZdaAo=vM~mlY4!S⌷[aA8e,Զqa^j[dJ605l^d^,BDKH){6quB(q#GCcpN	md\\JԵ\"Wq~ŸoI7!MzݴFIݒR[}rUߎYWhǓM't7;^-*V4:<ޓ_߹	Ó@Pb&mZhz.FHK&y.o|e~\r\n(\$m6mĢ9?,S߶ENfi'`Ճ?Zz~)`n\0l&`\"`b:| n6:B#Y*<㐂\"'v4./20b60gNB;4#%D~Bwf\00B\"\rrHn_LW0GP޶	(/^\\p/bТ3i,e\0,VUc.fduvF0i|Q9os9;BL]\07\r})1P <F&H|7R%Ц \r)K\n#b>mU\nk2όﱖ\"q|[!-YmO&SM\\I>C@j>\r,ZD:vH\r vAZq`\nprc\\.\r&Dќlߑֶ\" ]fֿi..	:\n6.Q\"&quS͆K\"8Hc\n%=%x\"l`rrn.4r\\2+g#bKLRl&ëQpHI<\0@dH+#-B-&=5Њi7'\"?|	0,B@-ι&\n\"-LBĂ[\0#on/So7 Rx("; break;
		case "ar": $compressed = "%)l*C(Xl\"qd+aN.6d^\"(<elV&,lS\nA#RNd|X\nFC1l7`ӄ\$F`!2\rl'E<>!!%9J*\rrSUTe#}J*d*V́il(nTIdu'c(oFe3Nbp2NSӳ:LZ&\\b\\uZuJ+ωBHdNl#d2ޯR\n)&<:\\%7%aSpl|0~(7\rm87(9\r@\"7N9 4x6x;#\"~2ѰW,\nNlERv9j\nV:Οh\\pO*Xs')ir*&V3J;l1B+l>j\\z1,t*4܅NA/ډH%-=lLHBPG)\n\$R2Et,]4R25 k(3\r1C351A(4.00@9`@Y@2D49Ax^;܁pVT3(ã?x\rKJH\r#x|1mNR*))U8I\"TL\"8I[R3Qӌ>,j\$WB9\r\n0!VPH9CMyRSDBY({*QT:0E\$D)*0)0kZJJI16HRY.\nGKk\$#kch5 05+\$&-k[96]MTlZT=&gNĈX[1r/tّQ!oݏ#ۦRJBǕ{!LP!N}sYth9ki\\C`ā0|f\r!X =h(i_	@4(x7a_0\$x΃RPA:g343`ث)TJm\"ĘE`\n\n/A\0u]\0l\r5H!08&0RaI1cSrHΌ}-CD\rQ%_@pn@H1`4+]l\n\\@`\\jBPp:0`^9aF&#\nHx|2ȋ:S0*U>Hj7qx5Cp\r+]-5#[ˁq.@D]Av.t%\$kj>ΕM!}\rS.ʑFR.d#\r*,p=NrC4C5yD%XPgH s2u	ѩ\nF⼈bDBhOA'`jC@p\0ZI#N A9;5Vra7Ue?a|N&hБٳY\$\r\r1w74Fη˿0Oa@!07\",GE*(qhĭIj %\$ʚ8Rb:F:޼`L6>g,AK-MZ3yI2K(I&}XzAt7k\0fA?͕a`P cHB4:lmhFT\"řj`.\"5>sZA3\"\$({Rd917\\YpD4W. (gn@).uX s2@kV`F\nA39Ƚ7#HqG%VrJXj_yLQpp \n@\"ra&\\ΞKe\$aK6{)¾\\NI\0T̶yjX\$~Ih.T LQ>ԼK[ʹ,#lTDi` r,=ؾzdu-3]O{0lcBցF/+-|c~\0D]Li'g{nD'>v	uomzƥ\n^q,m裊8S\r!SUCHx*if@ԺrkC(we2bifGkf?igL1st(Х\$界A0ABTz7׬B(Icq(КK?˯5;WdŦyNY*hR]l+DIJOǆK6&yI]͗x Q\rûD\"X)H~0Z^),ȇ1ceʻ2~H֜hׯ8j=\$휼./_v1Hֶ̄toC\0RRT(n8OV-,C(-OO#g%IdO\"R0lMV\$\"`0k%dh)AV\"'c\$>wJ&Gt3\"\$!\\xM6DTB6fT!vNSRq``-	!c	p(dO	nm\"]wĕgSĲFb湥48-lKA\n'~\"dOltwzM\r:JE\rdbpMD\0lFFޮ޶0LpHj-pv\rPG!Q.%Bud	o\r`hت\rFDc;1yy1\nq\rG|17Dq*\rk)F1y\reeg\0hHyaQ}ER1<1vH#P~PDՂ/P+/e0p*gMe(I\$d12RbK?H&.l@:I#AL2Pz[\$2o&%Rr\0#\r^rc(Tm쪘b#ѼvpMR\r.!c[Sr	pF\"0R[1Pwr<sKCc,q,`8øcSO3#\rS\r0FZ13_1s+56s0,qb{#%/gc%Ŀ.-8P51@\"h&3c8K\r8;	i;S#9S);-=s\nHhC;2Msw-Ӣm?\"->n4\$ ]cJ&nAf4k4\rB@BAVD`k%`95QSa<E6E):tds2vNpDb14U;bEE_I?PN(BhSfSA&v>b-8mhlC*Oi-	3ϸMEoVg0J&hMΕLOFeon;h\rV@BjYkxȐY@|~KвH&\nZ60=O/\"/]tg!֌Pd; R2&~Ƣv17-SzGr\0E=\"2MZ,mV\r\$\rZ8/5V=\"-,/|n:LJGqP\r00P0IҼ(>SW`hf4CIC\r\\{5u.F)NZm'y9,\$bCGrce҇Hy-M]Ve\nt\rB\$q,1B\$gtI/^_14Q#rtж\$6Hu5cvNJ3*Jn&Dt?f\0	\0t	@\n`"; break;
		case "bg": $compressed = "%)h-Z(6Q\rA| P\rAtX4P)	EVLh.du\r4eܞ/-O!AH#8:ʥ4lcZ2͠.(\nY(\$\$1`(`1ƃQp9(g+8]*OqJ_\rGiTh~McN\\4P򂞴[1UkINq呺6}rZ״)\"Qr#Y]7O㬸2]f,D5(7'1|F'7QLs*ns0,{ ī(H4ʴ\0\np7\r7I00c(@2\r(D:Q;\">P!\$p9r낏02Pb&٩;BҫC2izꤨRF-\"؅-KAOłJ<\$i؃,ߚJ)(fl ĚhQ̴-r:Hz-;RƵ*4l\nԍK\$6h=?TavW)\n7(OƁ\"OLf\$hii̝;P;l# 4,gw\01qpTgEфd;#7H#\"Ɂ4C(C@:t,6@p_cJ|63ElX4px!\\&Nze7iCT.)>6N8:bztjJh4Sbʰ((B9\rׂ\nO4N%*M)뎕THhpңH1 )<SHd6t,m?\"h&I%.gх\"⊿	qYKo<\"Ups봷4IrA,4K-htz֋ s;j)Ϋ;z%v|kչ(İh925=mdƢZwBgTuZk@T\nsV`Ԅw\$b\n9BA'4T<A;'	<(~ BVP6z!8\n+`6P掘3*a!:Ԉxr!図gC>l3C,mɔ.ʊHsEEPtCŰ!]3*O#\r98RPcy%ɭBޡpaE;;@taHq(sRLD#ihʤSe}1}Ju*p\\U}]ʕ	g-K<TlɘJc{2ynI,J5ʘ'-܉Xr]\"|L h(y\0\"uo\r!q0`@SaA1F,t91BѤё }LYn\$9QXI432>.B)HIIRdZ=>2\"A\\( al5&Xq2E#ntՕ &iQJ>=;'L*=LIU\\TA̰KT8#-]Nw!-DX\"Q@hFKP@`lKChen3Iê]:۠<~_/e  l&fCd 2A'(\$+Vi[)hdJP	B3ХSlzg[\"9: L	#o Qo_#S=d\rTT@\rOYr7_ ?\$\$ɭh\r!4Eí2!0ˉ6)Ti%[WkhEY}T*dMTJ!tQYlHq9;\0j.ohG{8bkhI%cH+eyǥ\r(TL0NuΑJz'J,)S)iPNE9Մ'WFi+崲)CaqB֕-CVKER1t(eFÝ(o^3YW\$M\njڛgVyI)nJi*V\0/3>[h,X q	m2}E;Rno/`QlL ŕ+sk88BP_2.u?ˁ)B!<FO7|G	7t&B[=iAJ:ʩBשFYA#q ﭛŊcZ;JuVfי\r_F>\$Xe6tU]>]?^dĠų	Vb@A~㨿\"\n40&bҍ+`(&3lFnu;'ulG砥Ay3!\ne<Gh'zhו`gt.{ϴׯ\"ݭP/u/-0K L)~(2P(8).Hkd` a\$XFLTx)P<(s-ţ4\"%ngDJuǲ)֣F8<0π挄hWE+\"W\n 	\0@ DFa,BGK]\0\$S\"|kK<<(n^\nÊoFR['\\o2'r}i^oq-R!)\0 \$1%kU6q>HM:>)D.cY'pnQ\"3('QSQ,Б21N@\rQLi^|br;{}G%.Wq։0%ܔT.um+<\"ҾjqG ,x,\$|,/\$~\$0]	Ҋ^-*	lhr4\"\"&Ĩ Jg#6T0\"gƆuO?F\"Jʄ\nۊT+\$R(x(L'j((d+ϲ/m,	\0R\0\$⫒܉.2rc'\0.&kȘ<hM@V(FsBiЈmP2,[`DL\\uIR.(30Ns0cHDs10|>Aq~z)_\"\"7'n\0X%.vSJ	ҊVz\0*:g)>x39^S3N;;p;-3xK<F:RpnU\n2>M=S;)H)X;-f\n4iAC-HXCjBTE2TIIq:rH2N;?#@@GCP:\rBHMԍAAR.NlZT(#DAv<|4KN:Y.(5yBHIMzޑY3fLJѧΛ(PfI/==4),vu49p?nP44O\0J7JR*~jS,Iu^B4/Vs;:B55>7YܰmWDKz;'f41 7uQ\$I\\Kr77>J/^..&4GHE[I2BC`masUrZ`_\nԳ/uvƔ\\&vm?2`C\\tib4	dj?U~cc4o	\"	l<0upQc5|AVvVchW|1:v;=ԽBVv?YbnRh{kki0hXrY6jLm	ӁQutoalY̷UV9[y0@VDMGfa4&PQ@֣`&BFs0rUsDn:-72WJ:`)6\\Pcoqbv-BCIB,Wy\ri+?}wmVRk4L7z~7 ~ Ş>]}WҗYSxRj}2`47Z'w|\\woK\\K.S+TkeIi\ns)1YJNO\nJ)-BqebW.n2,7p'2vr맔'3X,4}	(XA+eET,bK>dl6Dޗd,&MhbN\0@\nZ,uga~\"Yx}DlB: \ry.ҜJ\n\"g؈g]_oVRO'sb6@2`)o	5dY\":=\$<>.E^=\$Rf%d}ŋnxբ,V	/M71.m\r2qRzs+h>99ɚuix6Yőۛ2)ms2Şi3QMEVV6g=1\0e\\=e`SOVTQ)*8P6+4V|C'*QZc6䁠qZoj\rFµ/Jů&rs+4C< 5;ڙC'6J,7mmKn]Xu%`Il;WQl"; break;
		case "bn": $compressed = "%)Ut<d sNb\nda\n 6#k:jKMŐD)RA%4}O&S+&e<Jаy#Fj4IjhjV\0B`ULcqؽ2`S4C- dOTSTLZ(JyBHW΢Jhj_\rmyioCZNr,N%Dn৮еU8O2nŭr`(:NS7]|􆇵82ɼ 4NQ 8'cIg2Oy2#:\rK:#:E3nm ;KB+M	Ь#G.S9h6ԫmTd\nQD\rIlj'@Ep{LDЦ\r#pΝ2*b+\nDNtɨÄ+H*[;\09Cx0o`7/h: AR9C7KOêx߫N%4譖}4k[Ư#mq8	CP{]G:\rQ-R(1T47AMc4	ҚDa*;6Bj(c֍etZ-QElLt}CEX'J]6\nY4]f'1OQa\nB*9z,ilKL6JvP;B6˳AEѯu 9T@0c9ˣ>LRK#4C(C@:t9sAC8^2},9^,A=A\r7^0SeCIxɵ|EY@ BYQM^\$Lp^2be쳕 EcxRCݥm4C.4ˌ'4B9\rJ\nMr>Ǵ-.ϊWk*/zN9]c_u/asIN*k/PDlA: lq/p-S*:r|E\0:*HNL'e喺d؄.\"mUK!C\\@>S2CCB|/&e,FAa\r()QG+Ck6!ABC2eb\\	%;g?]\$XDt.\\Q1\$Y0ˉcʔxPFy*\"A@q'xF'BC<FT.1	iU:5 לa44Ða.9 ɠCq4yf>,.εU	?:U<  `srK-l0rG30f\r2r\"AyoAP7 :V( \r3r!3;^8\r\n\\<+(مF谦3tf\"A,2c8*~8rM!/mN\\t4Ft׏Calm[{q\r97f\\- }^\\R^53Ad̿bf8v\$-[LIQu*Afjᡃrj@Uiִp\r-MֿZ#fm\r6nO๺v?'\0\$}kx|\\O q!©TGUbPYH`M;,prqB'\"Ɖ^` j!*#Y/d0kFi3|RP!\r9A(h<MTU7:.@\$,H<-^%_d=ؖ;VGr'nAۢ|O>0'agD\$zX񃽭 fԳF̩8P)Woet\nEҜ_l{\"Hi)\0C\naH#]IiL\\-H>Fbfp8\\6)˕#BɎPHJ1z\"VZLRZY퀶%Z(ΫHy<~.?M82\rzܳڟC+Q8+3)c:%%JŐ@xS\no~FN8\$qѩ@MV,G5>++řw/.?#smŔ[qiar7FJѫ`\r75-Ѻ\"=j}gAQ%aQ3K>5]yFĬF b*0\\dO#Ɛ4JYtvRg`˃Ruf)<JGq+Gn˾Lea,\r'´=c2yg\$v|q!LIds>°+ȭR\n*{G/4Ӝگ۾s`L黎ظ]bNJ6vÄOM}b[\\Ub\\\"+ZN(*^8\0`Ofc\rɃL !	<̔BLxdHK\n`>̈ɄʤLROܑ;4blO(p\\{#'GXuBZ%6C.|{H2VIwxNT \",L:84OoJ~l` l0\rpT얍ヿD*P<uOŘ~-4p1JP}+iJv>bTfhbPV#/[Ne`	xfj\n 	\0@ܰQ\"@E6'MDx)|B{ ^Ee4{\$5cqCvkNCÚtT\n^)@TŅoXaH(SqWn9_!R,1EvJ!W1\"ŘFʳ!QJ>D<*0\$hFCQ_<Ejo%RJrm'Y')e1c(\$I,Ғbs2n\"䮗f	9Ƿ+,3D#.WcB*J4\"\r]'%Ī2+rt\$(WRW/328h-pYP>/\$,bC.zxDԂ/iE4*0bGPGVp1Bd8Su/)\".9(#.^'TS(}u54#\$X-A3V16x|D_0K^Ӭ皐=0Ea\n\\,r5'P\$exNt\$A\ncUh>>S-GCO#A3A5B\rT\$sHb^'T30Ie;R%	5_\0D56R^V:ԍGQ:4T`0fnӒW:4-A@=J'Bt( ~4E&\"L4<)3N/q(ưHxTPD4Dt%tJ=\"C@5T5Qt;\$v0?RSR5INvRO]Oo(Diis#\"=@rQi6.+:}PRI\"tVSESe#4v\0QK(4@2L)\"\\./Y4<40IէGZϾY/OTc	 TB%\\}Bu7-5O5W^`BbYXR//ţUH~Lz+5	Z!cNNB6F/|N沯c̀\\,u6HDsKLpZ6Ŵd./iVAqIK6Set5`31fr88p֏0B](9XϽ\n!Lu/VkS%nK{YvQ%P_.6b)wV2FuI;V@+(ط-%4vqUarQs8r7Msue7?E2UWc9eueuA<VY\"rhA]k61+w[lX6KZBV\rzsp3|u\rAnS)u|5}wAYWs(dmӦo[m\$qxA`5m8-XhW!wx\\G!/';eB'\rDv\$sp1puQVc'}dYwաu퓆Ulmtda}Vq5UKqtsxP}؛ ؞^Qt(5L73}釕A(Ez}8n/C\0=ϠowF?K퐈lY\ry\r(gL;ԃ;ņO \$aw^{w	YW0?R}UH]\r:|H'~U݀c_ɕ8fUSׂъugv{CYNXۜe9v'Hq\"t|5GU^982{у%\"]qc]R3d6I*yI2ZJ{T]07YU>h.*EDrY:Ew3>Ӳ r@jb\rV ӏjDBjDt\nZ	zK#Ze]º㸕V'pS usږӢ;SD:h(V`E@:2N]yK|MJCg}oF\\s.J@:Wt^!2]Gߴx<Cdpl@[m\$0M<-\\ٰ'\"pE.VtyV84:XE㍮yi@鰶Exˉ8߻QR=D,3עUY{C{8zUpUOT\n?C=y\ngr[	@LTg[4TM|Yg+\rhDI{RJ]OktԏR310Փ{C4SS䜆 \nƈ\r3{HQf;:42+@}\0/Bt&;OORbo19\\ȅ۾r|юG\r@\\x`t\\:6y	\0t	@\n`"; break;
		case "bs": $compressed = "%(l0FQt7aNg).&0M磱7JdKia20%9IH)7C@iCf4* A\"PCIrGn7+,lbdѶ.e)zCy\n,΢AJ -e3Nw|d\r]ŧ3cXݣw1@ay2Go7X搳\$eiMpVtbM Uk{Cn59.jc(4:\nX:4N@;c\"@&H\ro4n\r#8@@H;*\0ߨ\rò腱P.\"k\$b#{:Gshl5ϪҠϠ6J9>0Fî,&%iJ3*5'-I)#U.AЀ1mPS<P(@;C5IB#'\n\0xC@:tt4:p˘x	BJ\0|6hbp3.cj4px!=/;=ʃC ɋ8?C;N2^w|6'2LQ9\nL ,MNC!>ãdvCH4r4# kkҿG(0d c	pP:\rpb^p58ɤ&!\"V:5;#'֜M5T5HChӱfӠJHJnz]_^:=3w\"9|kwjĆF)A1<zsખO*X#\n/<]GkCMg\0@6PWg2WKhm۩93,gEmZqm1%|g#(\rԽ9dfC2fQ\r3BH{SP7լ~J)`K\0o9CM|o(TA@s\$*zI%PfXC:~\rZQDiLo4V\nZ+epW%`%L1>ef貵\$Yu\nj)#	5RQcn.rXdgʩVDbժW*^\0bnX?i\$^InƝGRDA㧕r :!rC,ĝ.4JKEF\rCARql*\0J?\"`̂=*/;4w^K*O	'r*2&`H\nԒPPI0䨦Hn)!3Xh4)jj;\"BLL;05lcS\r\r*UNsue2fldjB/aL)g\0\0;SGC*kf`u\$\r)k9FkRX6f*S% 4r&lޝ48I a؊Tqfՙ MR\r~dz9ͳhMM[2D`TNC\$)'XRvb\$1NS\\\0n{%\n4?0oWiP@ {Ғ'ۂMDث/Cg7)Ph	 =aLbiA	8PT2hQ s\$1B\n@U\"~=Jl\r	z4ҒpvB%)Ҽمv!V!2u%s#]8 GeŎLdmtѹ3.slͼ#υ+;xU4v^ɩ'9\0ql\rW4.{aq/d#j>P`IfP\"~AWD3;\$0P\0RpWL1AB~^eQ\n7xvȿyAwΒF7MлtLiC(wƫɤ'tHn;/:~ˎn\0i_cBo1/k7kLXs\0q6={whpS#'\$pR6m!()2~Wѻj?HLMGy@BHq `Z\r.dI\$(QQAy1@^\0SL`N '1R5 \\05	US̴]N3m\\B<Bhz\$sợnll&u.!WdLs?:upG^uӁe1Dܯ~a{O	U=cjK'EO!\0)\$4Lk|Б́<(3\n/}9Px:N\nٸorQZfqK?'\\ˠ\r:G\$Cæ	H}cͦH=e|Lo^+.\nbTxlX2F0o\r؆#\0@f<\r\r~i|hG&+\0-PD0`.0b!fLC\nPT=mCʣ\n`FdMHPPpNx@uJ`\nD\0ɐp'4tMho\nE\rA`ʶ%n\"ƢY^*.ޜPg,q'\"1B& 颡gY.cC(2mC&%-cΑZP3oB.b:%c>!O6i8BdQ\r'1SJَ:Roq麞l	'\$\"^܍\na`^O؄Mur\r0r\n_qj)m i8!22vur0r\"b@M%RJkE\"XFkrcOE#Q\$dc<h	!.2E)2\$5%E\"*CE+	)U%rm+_2b *-F#p\$sw.C#%.,v2^0&C`Kҡr+d'\0=\$dM3#!Z#Jm3V0>.I3@4<MRfݓ\\-7qq/c/t1Vӌ&9sPx0\$d76-5zK)d;<9\0	/)LTuj=`)`),VĹ iZ'T*@t?l]4&1F7(`@BQ9F¤h`QTJh<U(\ngM\nZ\rȆ0\$ jT@p1Pԣj>x43kPxFxLgsE0&=p>z/c6 dH3j;~\0)`=,N`OeL)ed f\";C(R Lnk7RD?OIS̭TԎ\r>\r>M:: iK]l7@YR'\"pL#DC\r	Jvuhx@\$\0g&\"hRpr(n-z2L|C34\0,̺rU01b]bL.\$\$'9ؠ(rPH\""; break;
		case "ca": $compressed = "%(m8g3IeLa9t<NBQ0 6Lsk\r@x4d	s#q2T\0Bc@n7Ʀ3xCf4(TPfS9?Qi3M`(Q4D9pEΦ\r\$0ֳX~`6#+yedya;D*i+p4(8\$\"M<kXXēYNT^y=E\n)joM|*u4r9]֡횠: 9@9Ȓ\nl`6=:*z2\n&4욊9*Zz\rI<H4H*̈;I!/HȈ+2\"*\r#&!<&:Oh\"D׌06r0.Pʯ<\".(r\"\n\$H4bfQM&P2%434C|	7<0c7>44Y8ʌD49Ax^;r%LAr3_!uJ0|6Ȃ3.ɓxBR j:`Nʣ:-̍ǣ5ꓵ7\nLc0]I+\rP @1*x9ڽRĴx/8+\"0З=22ш}\"2+A((3%Ei\$6-cގ\r`ߝ	0ܝ9aV474#KK]Pl,H 0l#*\nbZ%81Oj\"WxWӀзHϲo=+?Y`x2sr06dh!.C7vK7G,J2Q	ʹ߃I\"/<ܷ:Ia7#k+[(ir/|'Q\$LCi\$	qPؕ@Q4`81&Ҹo*\$7*-zdU3+S(h2\n`sIO\"*CPBgx)u;!茫XKc,zY=h^KFwd\$嬅3NTiGetT\n#\0ISOU#+Q *Y]ȲVX%e\"G;P5]*}E`ƃ!o \"ȱ\$Hdq^DT%\$t #P U1hI6cA3HH\\#l\\n8&pʶ_&bC1`х0'JP	A%AF%\roQ2Xpldw	<D[oF]Nӵ Ĺ/3]h)SC	\\rZA*GsA(/\nngkZ`FT@#'FAב4R	IH@_Œ^LS!3f	; <ZƐmfꎙWHUM:ONu(lݭ	L*E}\$AIW@\$ZW^a?^\".UZ\rqE/~5^UH>ٱ%TÌq@RLh#5PԒ4䅢BARd\nXm	8PT\"@-MsµL\\#(9ZeI)PŰîvNriI<&dy/TPR+T1]-bQfSCS5DSHŜPHHE`rW%qآP*QDcHT:riHGކY\"LA2t3F*,Jy]s̐@	C~I1\"a}UAѝ[8LZdI\ng0A5SHriR#mj)RO\n\nHj1PjH<22\r\n!@yOg}I1~i&~dY\0/E<BXw\\]4TZ\"W \\JAnkKMȉY.؋xb9R;%I>-&e9r^I會~s9^RèloUb!ur:q	ƴӈ\"IIuF<|@Klr2갲g%I#\"2Mޑ.ثW~\$JYճ~\\Ju\\+#BIʁ-(:Bcb+hHyE9hq}/f#1!ߊ{.l,>9_9^?%|Vqʶ'~nz/ԋrX|ضKM}@?#(fr@#l(\r\0.\n@@6Z(K)`'C.lEP#фp/j̊vK<#R,nvl`0v0~0onЌ\nƏum%\ngt3nZՐ:&JJДfM΀݂44W	p?a\n7bP06boP̀4ȒRPp,H/p1BJ6h*dN\0N.sE2%\r0Ced1d`^/kҎtF^}Pcb0c*\rQdfqsp䆫l(k20v&\npfiްJީ\rEpe\nmq ލ&\$t00NNF#&Lʕ_1\roЯ\$N8Gs@LTẽ%'-&\"ģeD{\nP-`;a%(`)y)\$R[)r)ҠRX'c)*2}*P<'q^Br&M[.W,.&Xؠ4ℚq@T.s0JS1c	0`#M\02m-	\$?Q+s@.o4-rO5Fd&Rt,侬sKN\05L8\"\$?ɜэ\$\"RNki	C_dj(X04\"/;;?%L!bPT4b\"/;8p<ʾN_<LcpdI\0Jk PETƂ\$`@grzk`\nZUp|SE?QDEyn\$N!BpǅګxB6jP('~=c\r1M)@# °/H(d[V@\"CeV(e%<DjRGnІuHt0 fagN/BINO3ƂOyP1?DD(C53>=Rӕ 1lU\nNF'+ZO\rTJnXnB8npVnCACl -ƈndS:2t,/d9IC\"2\"3 ִ\"lRGoZNOgP껠.Q+HH\$.[h*	\0@	t\n`"; break;
		case "cs": $compressed = "%(e8̆*dl7qraNCyo9D	m\r5hv7e6MflTLJs!Ht	PeONY0cAn8d:VH+TتX\nbc7eHa1M̈dNA^/J{HLlPDܮZe2bclu:Do\rbʻP.7Dn[6j1F7761T7r{āE3iǓ^0b穦p@c4{2\"&\0cr!*\r(\$B%k:CP艨z=	1c((R99*^F!Ac~()LH=c(!\r)<iaRB874BB`5k<<񌣒n99ZBDFo\0B4B9*MCI l4HhL\rx[f!\02ÐLb~0z\r8a^\\04Bx9r9\0J8|;%A\"1^00n=EC{P#57CkH77L^w޶l:[\\d+0}P(S0bc*:.o :(\n\0%	F P\"\"L>9Ŋz^d\rzW@:\\HWQCX&#H4!#A*CFMBbNȈ@WZt.z' LB&%̹[@V23:-{4;#Ӝ(^?6<\rЕ/-C;O0@6X<w/wJvbWr=\n8@6WZC x@o\\	9pH:пDBƆпڧ#`6~xvPQĵ '\"1?\"D\"OE=)ZNNh˘ )?(L'~CVRA`Id &SaL+!ja4\"F'́!B 	'.%C\0{цdF)XtM1\"}\"!s(,0ߐ(nZi:\\²V\\+xXabp^yP\0|Hs=6g5%	6LF«%B}!tGd +PmA:4Rz%%UKtY%UH5jʻW_&*\\7Kܴɝ\"+z=!E\0It!E2e-x<Г{&RHvTխbn	]JHv1LX\"XRQ>KqTH{\0\n\04u\raRSH\n\0\0h((3hH5C=UgԨ74\$i/Ci\\ç\r\n1PʃcBA'DCBJNdM(~L5ĿfS\nA0N2~d9=Z&aE!8c8Jz&V:&wmO,?b1Гs\r\"_tF1\0\"%*O%92)aK|4'ͦÇ\0et3xS\n&t_LHH[1C|KMK8^\rz?\0N	<Sl`؊r3Dho #Jo;8n0Y\$a\"a=Vp]TF:W4=CˑkP > :xN˄#m-ĘzufeHa#x0H:ɰT\$Ő|QxUT]=F943Ӫ4L1(n9(/rx_pt\nj/JQiGFFpQo	}1T'u1nJ&=\$Gaa8'DŅ;#Ia1a?S\\=_42\"a\$D5\r+\"_D2/662fl<g:Eቈ-DPC/AnE%xFW(Wq1LKߺt;xl`tӜ8]4\nw\nq16P/O(= @\rfiBSr4t|QHvɒ3hǬC	:ڤk'@_c\01[2r? kd1Þi}9'5YQvZq><<GayXX3xGu\0HW\",c@PSr\"7i,\$Lc9=/}G(>y_?=~_Rm[\$7\"ѡbb`ޯ%\$R\rDB&PL^BP~#u~MfP\$N~\$r%ʣ=]%sJ&},N)R6\$j8j2#,7a#~~lɫ@sc~|c^\"*Abr&yЌ&p&k@haP@%o\\40ètC4f0dĤxg~Dhl Kz(\nEH'*&MdN&hb044e%>B`fq2\rжv@M<\0gPg`PynixDKy1毑u0P,b=\rX+\rQO*|jQ0<	Q\r`2h,f ѿ2Ȍq%R,l#/1!̤4=.H#Qxg,\rc|>Bdl%P|B`g`DKU*ornZdTdǤ% T%bHZjC̮BjG*Sw\$/%A2<0 #*&Jd8j5/GvYR\\\r1\\e\ns0\"\"	1 7Z*#j94y13-ed~d<Ƅ<K\r0s/8r\rΝ91.n i㓮`X#{;&9x:\02z[(eH\rbnC|+Im>fZя5>s?9ЬEDT<YA3CR5?2:e0@> 5U6RӍDNc:RI?NNgtSsFA:N\\&.fJ`Q032LyG/ԅGe)\$tc%FwI<]IxEG.5D2''G	KnK2MIT1PH\r5_ Ӣ^g2'K2=z7- 3AHOb#PGOBD	b Gol@g#O#OhT:o\"@P)Tbr4O_\"2\"5uZv5Q\"]c'Fŉ\n&20'eX^Ҫ{'A\0D)* g6i@ۆ\nZyrTcU Pwےoi]ui]tەs Og\"R! i'&	F\n/FI\"	uKhK&B,9aZ/6!Z㰨dm(PM`b!\$5bq(fo\0W`gf-2\$PpV*xK}vXa̛6h79Qi{k>c'0^hՊjV&\\9##AP#PBwdso.#+d &*'z\r1pPrv`lgCGr\"YlcvVBi-<:\"rW*3rs`A\n1^p "; break;
		case "da": $compressed = "%(u7I:\r	f4is4N2l\"ц9Ü,r	Nd(2e7Lo7C\0(`1ƃQp9gC9GCyo9Lq\n\$	)36Me#)76遹NXZQ6DL7+dtD 0\\AΗk6G2ٶCy@f0as܁[1Z7bm8rGS8(n5z߯47cNo2-\"p܈ә2#nӸ\0ص%0h&i'#z(!BrFOKB7L2B.C+02b5,h.ۀ:#<0-܃\r5c	2\n	\$\r&6@>O#1)4\\H4\rD0O9`@PBd3t4\"B-8^xDjЅ\$x|(X+(\$V:cd3JH;Vmx<M\n&(J227=uXމ P אL7\0 Q\"1W\"0:uzSWPY#Ml7ځu;-ͦraCcPBbC?^Z64>h\$2c\$b\"k,5ڤ*'v24+V懲@PF&0ʏbk;`+	#k9%Ql>x!E;]I@A4eXp\\#q]#|u03c/6B<*ǨԭW\\ݏ2	PܐVlD8lB,-L760\n|I/!Ns͆9Q_0\n]2R-/ʔ:#bɣbSLxCI'\n0(TtR]L:U\0rTJ5jP>r:r6;L<x.@܁)Kd0đ:Q\\K.Qj5G(q3\\U wBV6%tѤB+#@h9/k!+3'!:uI88[\"1!a͞\nVgɛ4w?DYЃo{*PS\$(9a\r%\"\"JȳBȖ0g\0P	@dJ@('@R3E2-1К	*\$:R	X0_HU\"I\"9>J9\rA(UN@w\r1Z5R):a4A) K!(0OSPD5pDI3ᘙXoI-6fa8A\$荄^)~]	Iqb(\$3rI\0ģ.|sԶ`ɫ8|+Мzj\rk֓/I24X˄l7̛\"DR\\BZBN9J@PD12CIƱf%XT2MO	*\0BEl\"Pn\nP^𠸥3}I<&Wv\$[Ș3Jl[20j&>NՀ4^P):d,5}{5l͠WoN\0]:&e.cNP:u+EBL)1LB438u<YPmDR~a{:3LAB:7VCY\$iHзְ:N}[]co5дV8l,Gk/\"FYMb\"df%lmD3aLh*JQlB:\"p\0R]Ɂ1p_\r4,X+@A K<z^p/C	\0[]LN\0001`VG8G\"-j5wLRj4[|<%cykD9B^֖dmͽHآrnjJQ_mC]AmBX. mpɻ6	 ɚsDuù'|2*Ů6BfUC	=\$բCI6nE%8?JSz>A~ih!ؗj_@+!-q!#aDRpÃ^(o[8K;Z#R~kO/_i{cqdD()bK-\n+{@qqh>`<AϿ5	q̷9KcbumzĿGN39W,]\nG`2{I{n=xv_]gս2ϿiC\n7ŽN\0'PoQ0iޯnT9d/Q&=\0o	lf8͆bli\0d\n ::\n4%QH>FL4Kft`؈0@\r8 7(d>C=(yP\$B#pJfl,-ΐ>VžX.oo[fZEԏNkϣ omKJ\rK\rm1	-E,b0I%:+=aBpq8kcL{c4ìBtT14QbNYF\rd\nwqE(qONYwQ^ϸHOjot#qHBJh=\$.ױ#.naqTנ pЍ8\nY/2#\"CU#dC\"K\$LE\\Ir/դYo%ch\03c\\?F'D-\n1\r.m(d`̐Nd)* )bȦ@2[p\0,q,E``@>h&CNn60bVx\n\n@J%#m)Bp.S12kM\"+堿T킁\\/PB7ΠG5C./B/hCXOD\nlB9&bhK&4D&+,()4FA;3;n.<n4N\$\$@\$3>F2de\"ph	F| #f\\b\\\0>úl-0\0Dp\"ڼL/&DC,0;4 (,l^[=\0ҼtA8\$i`@-jpP!@"; break;
		case "de": $compressed = "%(o1\r! ;C	i9	MQ4x4L&:Xg904@i9S\nI5eLn4NA\0(`1ƃQp9& >9M(e)V\n%⡄e6[`rbQfa\$Wn9ԇCіIg/* )jFQ`M94x 0·Y]rgxLSڸ@wŎBx(6nBh:KC%-|iz9#A:W7/X7=p@##kx䣩*P@ȳL9Cx䩰Rfʡk1Cˆ:)J\0ߨHЉ\$6(R[74ã!,l	+8CX#x-.+	ƣ3,q=#(,6)p츰th@;Co&\r:PQF;O[ j9.^C-sH-3 cꁴ\$\rBhx0.\0CD8a^H\\SKz	5LcC\$4DGh|\r\":Ct:p:;K`%&K(2*=B?\r̘JC,a: BV`r1Losƃx[p\rn[C\nփG.(H҂lA}H\$kt4ԸΜ:+/n8ⱘ:1&ժK~,>76\n2ف#,/5N/33N-&^Y~ô\"4\\	QSXNϭ}\n\\-*T5-X\$\n߽(D}:- VF#kWvۀݻþu\$h3/(cj;\r#|T~@ק(tQ.pfrIb:=zZNpoUM@l!1R[hr\\A3lAQgNX.dH7	q4y2K4'ZXPl&\0Ot#BayņIQ)CP\r\rqbD!17D-\0dpp)Ի50P1\"OFڽ`5HwYjeMHh/r!%Ag<4p!||6\"@4k\\G`}D ZVBGF1KUU-ᔉ[-`, 2YK2J&*m8ݠp->xkKHԎK_QD#l	SX\$OC#+ɑ:a@\$\0Z3\0rr6%GCøiMQh\$v;` Bf6HjYKe1U2:y\$/*l?\0t,eBe*#S	C(uF:0	@PQI?(-9&`\"\r &2ͻ\r/Š&+e@B`L201~SyY\\ru@q2%	H1dѭUMl}OT-IKAz4bHR*Lwl-IO2-5.ӌÊb!%F\0II+ʨGАcA̳\"Qt.4bKA:-f+ͅѽWńш|!\r t_(\0 -hP@p251=F}i99'dX?t%z^&Pq˄+	kV6/3N\\O.Q2HٗxɃ2#/ef iBD!P\"bhB`E͇ewDƧ[pL<8.V<L'RUa!kf0\"rtpv\r2Qq&Btx/-U]AR|΢(xPV0᤿k .>thC4ըkyD>vܐر_J\r2dPةj[r7fB36zl\rw\r3\n|ulLWSH(ANs>Mܴ-rSq+WfYNtNxZ`M(aGF2^29̉e1RxN Bˏ>&t;a0c\nK^t1pgv))?V6:|'aP 0K-.J>(-UD RBɸ^UYbl4Oq#PMoQH#%	~Mwozύ~<辪C!86#Y+4qN38CP-^ʃKOJOWđ~]cӏ3~s:ꉶVvt3CȉrZQ#9IlrțIS߰좒zDC	@g(lGDD-ZR:o&9H\0'D|/7FMP@s]F0N!Db\r,I}+T\"=\r`xh\$Lg\nLB9IҨVLrv)|gj'}ML-Gm~DR\r=j\rj+.x#D(mZ\nT	,px7dH@c@NBЕ4UgtyC3~\$H\nqSUc\nNGeg*o|A[`\$`λ0>ONW1~϶'ц\\ˢ@jiB[cnS,qaqQ00@a\0>΢\r4\"IJ=CH2E/CZD@G0pr)HW̲ðH\$\r!Q\rD0\rDNd4.^e_2y1n(rDr~Ҝ(*n+)=@DѨ~oNkN0qIq/0'TD/+iMf\n\rhP,J)13:s,e TG\nC\"榤/x#&m{F35BR3a6SY*6W6\$q6;S?Ņ\r0m3a	.x1p汸ċSBsu6hsw~蒠߲N:s=S<f=\$\r2SD1 M3I@5bLAs0\$H	>-`.@J=\0\r\$T^nFC3\rj<\r\nD#E4V/LlBk\n;B'z4Yo@F^`6Đ0S@\\dG}@\np4GDQ%,GY	\0tI. 2A+\$LRk#\r(\0!NdU:H(yJ<\$#dV]=\0ۄ\"KJLb@:NЊu1T!j\rASN!(nF|>!NUV1=jwYMEX`ТSY5J|C=0٦2Sn'\r'mdRJT\r31#ЄvSM1#D*Z%dޣRZVr\"REC,ue^|keW\\ qՐ˿c7?@d\$#!FI,`"; break;
		case "el": $compressed = "%)g-Vrg/x\"ZАzgcLK=[QeDXŢJr͜F1z#@Cf+Y.SD,ZO.DS\nlΜ/*	D+9YXfad3\rFqck[)>Hj!uq*?#BWe<\$]b^2n>z< TM5'Q+^rJU)qs+4,er5-3J7g?g+1]_CFx|-UtLꢻ)9n?O+;)IjtP#0\nQ!s'\n|W+IHsH<?5RP9~%3ٞG(-4COT\np7\r7I00c(@2\r(K:9@;\"P#K[Dr())JNO1~+LR0=8*ªqt.:Mcδizbm\n:ĺQnIr\"MUqљĤ E>FH	>!dhӷkAFv%PQwKjOzިOT:gE[4L]DӃhTAr,֍	z]jh2N)uw.Jb6#t5ͳ|9γ@0c9#>g5̓45H4\r0=ϡ`@i@2D49Ax^;pÓ2]0x7V9xDl5e#46̃H7x6+]|\"ᇬ1A^œEz/{&DTFͼ|/t^'<	C[j3W(1;r|PH+#ݜ(C2>7Mإ\$^d=b\$}Vv5ʍæ)v6ޥ۔F̊\"\$ap\n'|he]y(AM):E\"32vfE'/hL0GM*2.\n78!\\,\\=J\$Ԓu`i[!°1(fP{L(\r~Kd@QuGϠgBȳ	YՕXUGĆIA{RL#af5ZH%҅q(\"FqHr9#~Dx BzQwW9d:4S34Ca.i`Co4yhe>TTCf\rU&MZUPs8ob8lɄsg:&+\\Ϝňsu5:6PL%}*ȕ8c@)qr5*zӜїcI]jOQ1L.Q	r.\" bhڋzȧ\$u}ƞzDE!VRJE-#bY!RJtWSo\"Cɂ&P4rLPriqڀ jMQ5`퉲WKklF>7yYĺ!؄,O)i\"I\0I\"W0zb!Ijq	IiEVZ]k텱׆SlӂqY\$(jQǡ|1Q*=Lv5ET=\n5?fɓ^xν<a65Z2bB4&ie h!6<q\rs՛`C`o50@T`1:a\rg, D	nkQR|*BWsY=e\0\0(,RQM^%P(.-ɴ\r̨1k;{IA;cINh'\0\"YŎ5\$b )H*]zN޺3Éu&6o k0њJzODcc;#)4P̹S<03\nd&,\$\$%km8.@TI	vSp,1-gAp;l(Zr=ԚҬBAp8<kS0\0\\ƥV|))3z*\"GUC*sZ\0¦څJD!me!G3xb|FF).d>!gcVwБ䘨,0RxDG[=?U0&)LI\0F\n#=D'6P:^{ȍcTwߕ{+\0U|črZ9'\nNbF_p\$}Pδ1Lth(}<!\$<o_'rN2cp/|)ʑ^se5-t9}ݛ~Nt+	\$)<~bAXEؑ<>h*ܧ̐ML\$.Wg}qJKE	22B|P85	RˎCp9\"Dn\",\"6OLbx2utϠAzD*zL,q\$W	\r \ri<Gl@\rrve'@ @㒿+hEzHÆ͚r|mEʎ0JrooQ\$P>òZè趌DQAnt`G&\0W]P&\\HuG\$O>pYbD::ªl:` jdMMb8\\iQz5./f8f>,b&YNw\"#gg\ra%9<0@&E,_О\$ \n`\0\r\$ffFbGOǤH-0X47g@Gb)!Xy&u\"%d%,Sé%5-4\nx%.?'%D(lQ%B]'((ۇ?().&0s*.c&̾b+3+2+HT{+&\0_g2,,R*q(3	(0&2)+(쪵\n]E\0H(+0^pb#>|-!krrQF=~*\"d5F\$\$OCb+ԥo@Cy3bY/o\$.X3X:@\\'o+\0\"Q*\"\"B\$Ǩ 2RNZC2ɤK\0i\0B6l*M\"INW,wdӬE2lwACD#WbE©DN~4RHҥeF9#&cn\rI>o#E0%1b1G:8<(F#IeB8S 8DFBe擔jT(rx~t*+-:J0Wo8ʩNT\$1D\"/:J*uQ\0 \n^OPEp9DW-TҾe\"RU/t \\jKEtVUIVIk7u)Wc]0S+T'VEQ\rV!3RD1XqZԸ(>'<(0#()c(g*rܫc\\ȹ]]R2\n5uқ^r^)/XO3`-TBeBwY[68[b1b'X@ՇRԏc-c F6B+ZI1ZeQszuȞ\n>8DH,#[Qc9&(BR>\\\"f=32Wh/,!gd&3eh6/ccFM7e9dA_4JҥUbq,8Qrv\nCH<.U#'ID'Ns4c\\5=vejUssU էK0BLt'Q\r!b#U7*-w4/XWYYxS4W\rWVlqJdwH^]fwa!WF\$\"2L&|}@bhw5^\$tAkrU(UTEwSVux\\xrVJ_uxwBQZXiv\$8sHkd77pl,!	77{MlX(2B2NGjm>q4*BCw5xzlyx8׽X%6RXMRPeUYxvo~sksx=dzj>I-7ueQ\rkMLmdNlQ\$%*M0L`qy(ADȠ2H)eq ]WyDxapGչIz6TYa{fsɃ\r%O1ۑwul1Úfw(8=v3R_]#S[-d׏w˜fͣu .5r8J4\nbqp@Y,*Tx7T.?\nb'*pM`6PS!::5Y4F8@JuN/A%o,u)H5Mv1F,,^:})-ig}ahr:1``\rR~{ds\$y(s:Q!-kKO6`iz\0\np&I*VGtG6tr\$\rF#?.s/0EtJwНo4r^yB89%8MVQ2i\$/\\BBXƼqz˧6,M8<@B-\$Z'n@*]Bb,&Nh3R@ϵjԂiB\rL/R&\rg/I0֡u[ER)6Q[,uYq8\\!{,rKxحHzS)rJՅAb%l\"\\A%BR&OSQ;уU1.6zv1*Gu4(25A^zu\n |]O:Z?^3EToN*85|Sm@ĺyh%qcϨjnzsB<fAAd^%Լ3B@&r"; break;
		case "es": $compressed = "%(oNbi1gBMi;,la6XkA<M\$N;abS\nFE9͎Q 2NgC,@\nFC1l7AL%\0/LS~\n7M:8(r4FdJx#&̆1*rL+Z	oX˕.ifS {4gCcpt:\r'*O{0dd}ɎE!(o7-[NNn2\\AjH}C2f5Hl\\S9㈧+/js1\r3OFF&5~:5L7Z8/Ø3ȷ\0s[ B'@+Z,F'e2P2k4-!)DOP\nL2(9el*\r(jK<9x'D,<QLnDr0  `޹\rΛzH@202\rTx!\\ی#ƅ##DN%\rD R),:5C3:xaM27Gr3_p(I|+SiN6\"Cx!F+F81:L,ڲë^H°Lެ#+J*s(K.\$/PJΌCʘuF,Hx:\\ےMBH14B\\PO\"6ht3 S)\$Γ\r&gF.&{|1ϽGJ`&=;-;Zgi3r >.)\"`ܮB1׵|sW:qW8xr|r*SB\$C΂JӬRz\n\$<uG.2;\"\$:\0絥60o<\nGa¬lgE]]#^#>\nPCʅ\nIC1_.y<l>Fkв	#hIܔhM^I.XU\nA`BKXć0ڑzL&\"&( @YaGuujq V\\z\nL)䲈\rwU9E@ȢI^I\$0ʆ	!LЫ\"3d\"&ز\0T!tXKcD蠳s2H)WeׄB\nN!*x%<od۔ӶB)(2PZ~%RPԐdP80TD᱌>'7GBQ+!<xHԔXgxi.\$q\$*dFD藄,(Aao\$x\ne`0s0%\0Q1ip0Sj! uɦrdm'4@y 1HQ-G\$NHCD\$4x07vTECg0o\rr!0nļH CcꡆHP\$#p9<4J\\	\$\njގQ3̘\nE]mU.q#B,l>\0T@@rqJAmD1CC+Eg7jeG\n<)D|ݜGDMAp J!:*9)%d!CPϑ.!{yآ&aTZڴ0o2`=yAʞ{Fw-\n\$D%BT\n\$MA<)+]hb12AN@@v.Mߑ1f4Gmh8܆t)DF!\$ia5N)X/&r5t#lnwVe\rݎrQ<kSS8O(''dKX~IfaAɼEcn|EHvÇ&kbCόsUFZW\"E0hS2/(a֟\n`,8֋|%,.CJđXtxhu)c7FM,D϶5hS̕jp[Kot*9)\"V%vtNBQ\rP*_bTȄX\"71D#1^S˽24hgc9ou<lj\\p	i܈M>*,ܭ^Ѽ6G%<ZDkO%1=҂؅p='̬s1c=<uz7㾲I:QjePFI}(dx^De8\$l9[Kyh'oqeOwuS|_tX'bn%h^rM{{	m9\n}:`\0q	\"߰v!QyQa}xZ[fdFG*13MzcDZ\"PK?fČHuJ?l\rխl!g\\fZKh\"mj0eFIPm8IDqB0q~5C(\"P>\"1̡OPLs6ldsZ0lPqDRvCHP@@.\\cPrF2}\n:HLC\$p5=vTʎ@	BD\n-r/C>^/CP6S &c@!:&^2|\ng\$LLLq8	Vx&PFx8rR\r\rdONCl׭ck8?-PPrP<.k֨0W_bN\$ei-щ񦤐V'\"`A\r3eOGH1\"6	2J2oGS1֭-֭qC	Qƌ1_#Hݒ;vb3\"V248+ RmG}\0G@ʱ(>Mp:Z`;]&\nj\"b2(2rb̀҂B\$aR0`0(qO`-*+V2ِ+\"P2#ҶJC\0IC .M,o{'É.2.qv\0A/ͨۄsID)01[){2*b`b.m\r13\$*RP3HAi4̑4C:[gd#G=0DteU73\0JjJ0:}RPڃ6b9CD:C@\0\np\$p*T.a8D{5'K(!\"jH@vr̂O.3#+<@ݮ5iHs,uC\rl\$2I(l^-XFx&D!N(fp/|b	Ϧ4x	0)?(tDG-T[Jة^9E2+IdL8Ѡb CoHf`mp40)LA\"(\nk1G`@àJPZ,\$EBJ-1`VSi~OCO1\0a.\r\$Li 2MNg8J	\0@	t\n`"; break;
		case "et": $compressed = "%(a4\r\"e9&!i7D|<@vabQ\\\n&Mg92 3B!G3u92	apIdCf4(aLA0d2ࣤ4iF<b&l&+\r\nBQ(ԉDa'89\rfupNI9du'hѸ&S<@@tϝNhgP9NI9;|)@jjC,@m\"ٳq|F=ZqF̶`*y㹸@e9Rr!eX\rl#8+/H:Zh,\$4k§C|7[־Hēè1-i5N;:*-\"#HKp9BB9\ra\0P<B87走\n0)xQ >\"xHH.1>H2:\n&\rjP¹*+2;@?[8@/1hX\rX\04p|4C(C@:t㽄\"-C8_\n#Wv\rR4P\r#x|țR'8j+|ܦZ-j2h\"죷͒䚭`P5P!΂>8A@ˊ\n*41\rhB\"V뜣^*%L\0^IdՌzHBxL}jDk#,r\nH90ycH5J;nX64<9lc(h@Pvf-h(7-b\r3z\$^xm\08Zm=<5chabq#lB*Wݭ8@+\"LGH&'n;X6jOsP#p\\I=eivV\0}2b\r5O3}2\rhӟ'aeC#A3bF`!dRÀ\\SQ1fhe	?`a >Dv8eC\$R\$0Z2nI'\nN *@`,,jʻW`5V:Ya2VљnmT[9΃\n:@BBi^u0!BRNJMʒYY[UZ+epX;HFc	֌k-X\0V=G\rpA`P[delGIXt(	rolCU+3بM!M6BUL\r<riT5jXi\$iޔ2^BމI#DPQ\\\n (O(IID,ω28FiL{`'&BxwNԭZi[sD3@u^q@jFrA-jN4]\rm`@R8L4j@9Jb%j\nC\r!4s;V6?9Hy2MRU,al養8*\$mR}IS)TpTXS#\nMWjG[k`H<uL\"Vj,7Ʉ3\0	#`CWCwM\\\"A'H#jr(3Rem\0\0U\n @oD0\"\"!^\nNɺUIJ;bx!@5;{8ʶa.(1HLfLXVQ\$?TPI]O:Lnqh>M^8[[<#̊3')-tUNsի,g'fBQ\rPUU_:]r	*]1>g\rSd7a\rg֬4h(wHbLj<LNlx{nEc:+\\NNRBh[d+SSPkmWF9%oY%@B8GI(8qxK ~HT5cB6RMf~/LCS++PPsk&pWO1*DXft+p␵D\"nc-@\nk\\CF\"\0sVZBA2ȾLGCĝnIrhv*2	quWz[Eȉ\"S5\\8q6:^ÑrN!!삐4g,LD/ka?iMS9zb'?VLӕ.s\"wO߲%أ=y>Ǣ}=d3Yq%>:(MS:JVJǹ6j䝉eW0|,iᒵyNj@B\0l\"\rM`,D5φ&rpCNeaxe nf	l+.0,?Ee\$/\\0`\$\$o\$tEt\nox#jygKo+f	ξ/\nm\nƈBnLߨNnTnzbGΆ'BHj!o	mjMkn-pKb5ZFXWC8>I.D\$J[djfDc|eV.2:϶o.iF؟\rl2TDG΁82q<%@-eW@ZV0\r#P;_D_\0101_\nMWqȂ.n#Q`# \0Qp.cX1+0+-h \0vLG\"brH@\$)7@f\$9o8C]4 m\n	lrjl^'&~r#\"`iis;'x,D=R[*(HV1S\"})1`--9)2r\"˱2eD\n/\0\n<2!\"2ʒ1-\$)B&	/#32L-3!3k.[rRJ1ͦmd-S'/3g5)r\"ͫ5j]2-*y5\r\r݄GR!cKc/X\$QO6/҈\$Nb	\r,\0+,Og2`r11pns1S?\0	p\r]N#sP?\$b|m5\"9O2J`\$F'˥ #.z Z*;ND(dNd/f!Hv\"P`߀nHD',Q#9m\r(\"V'Bl\"	kn!ûvbNhl_q\rP:bl	Tu\"\rD!URS'ooL3C92#RJ\"LU9:4*s㘕IXB\0gXBdEW*d>\"j`MVujTP-\0[Į#ObIưOmPd4 G<ǬAR]R\r5/04\r:8yȟ辇`.\0	\0t	@\n`"; break;
		case "fa": $compressed = "%)l)\n@T6PD&چ,\"0@@c\$}\rl,\nB\\\n	Nd(z	m*[\nl=NCMK(~B%	2ID6MB\0Sm`ێ,k6Ѷmkvᶹ![vM@2ka>\nl+2H#0\n]SPU!uxd)cZ\"%zB1C2o\r*u\\o1g{-PsW㤵>--#JK<֋TsFT/\nS0&>l`Q\r{US!\\8(7\rcp;\09Cx䗈0C2 2a: 8AP	c2)d\"rԢŒ>_%,r6N\"|%mT\$͊S%楨J>BM[&%ES<HPW;'ﲲZ%nS,+>'.r%!R@ȩbҥҡ',2Ϣ8N\$#F0ҒЪ@XO,P2\r\\\n7@0c09c=o\nЄ%\nH4\r80`@Y@2D49Ax^;܁pVU\\3(ր2d\rT(P\r#x})ǒCHIAhHS,sH3\$̻8~Ƒ	#\0Q%<^\n7W(J2<nTSB?9+2KʨLZ)3TD%D2嶘HQ,O/,kJ,/EM\r/,.j\n+bMyjV.2E˭<8ڟy2oA*(p(*Ll3#u-'NXdK--'⭦0Н߇S1sa%3wY̐P!x7ǸZ &A\0:p{.dy`p3\0F# ЀlB ``2軕H\$B@PrDC \$4T`0q&4WQ&8@r(J5&yHEel&TTB{cɵ:g5* 2X yjDl8AVhW{ab1׺4;)!ѐc(\n\n02C ~*Z:p4E^[in- w\\ˢP.p/B|:0`37Ib~KeУ%QQ&^2\"!3Vp|Z!IDHTXGRU-V\\ksZ]S/׻Y%5#;{LbtVB0x%HOaFELVόQdK	.A\n+z\r\$(<\r	s]`O`oQbp@Ф`1yLa\r'fdcjG\"*N+tQ\n (L>J@Fd\nd9QIYX S/!P9z{*oY44\"D(f޲MJ6଍eDK_hef4F8,2!40bܧR0-\naH#YֻQ<.βW;Y%!PPR\"u(IEԲ]F)&6828U5\r3!I adYKi\rcUCC;-}X@:s#\$6\rb=G%0ldR`=wfnE:t.Ck_ɢFhr\"M&HуA>)\$JfB\$li|NRަCKwѥBO^hE6!)[*I,qQ\0PO	*\0BE:L\"Ptֆ|Q,VgWLUMɅ2\\E\"Ձ➥ɌdRSL7iţ{|5`DNi뫅%\rPSqG\$-#?GWWG/n;d1oTT l㘁QL< \0=::s) HcM}kD^vGC)|]6V}E κ`e1a!7O3Z!}ObEK/8;=ۙ>`G%'db4a5,9hKݨm!B{w5%U	}yus~ZY`* @+u VǚsMrq*\"?pU45G#o.q,SDMzu_1?mϚjN3ߓ,oT/yͅ6F<̮k#\0,>%O*O?Dc͹lJ!<hϰ\0000Ol .f:p@S8xN'/{.Iemm90leP82IL\0~2Rԏp<Pv1fHr磒87w0݋	s.#\$EI|pǐkbBkA H~Le\n2CMD Feb+p xbte.+\"qq­0Xvcpu4ypc7%8.e\nQ2Sî̎Q`&.^x1v6}xlp!{,bP2oxk\n@\"fQbϦk\"ѻ	e5ފMqF_	:qb*2jsG'FP#jVLДlF_Q#BJN:3f\"o7q0\n0q2pRZ42eл&̉\nq	(=w2v3қ*#dF1b>RdN+j?A\nN)[c8r(n.1'2c)xnҎ\n8E D/*q\\IyO`:n312ަS3Rv^17%E1R5&1A2%3a4]6ST3[1.`֕\$\$kܨPSzsC*|\nĩi93~ ة\n@_8eC4_=.44Ss sĿ+~Ȁ37#e0F4'2m@),̈́LلfEAt:gJ{&zޭN15!TL(q\"zR1+z(³DmzԃcQTwz.LOu+\rVŋ7ࣰGth3\"1\nZ	,#1MMPvlvH@\$\$_6|{*.7]O9({ԝ\0(qJ26/.RHMZ4(Ժ#u.4HND6+h*0||R#Uj\$Nm{\0y\$nN&3poFu5\$zUwZPKKMXmVFhvZgʍc{STJ\"dr5^k3΂D\"UR.MUg sKS?SqWC0a\"XTʍ=-Xi c*]bN#N\r@OU&83אMgQ3`"; break;
		case "fi": $compressed = "%(i2\r3 2Dcy6bHyl;MlegSnGgC@tB\\ 72	aR,#!j6 |=NFt<\rL5 *>k:+dnbQéj0IYa\r';eHmjIIN_}\"F=\0k2f۩4Ʃ&ånap0i݈*mMqza͸C^m6>㞄;n7F,px(Ea\\\"F\n%:iPn:lنhA7*bn%#\rCz8\nZ#Sl:c٨&0p*R'(BJm@0@L7E^ԥ+G	#zJ:%#`#N	K`!\nBKJI ҕ#\$;<`2PI<c\\53D C93I\rM'&Hز&,	!`@~M\0G4C(CBh8a^\\Q˘\\7C8^J; ^(aض\r`x!R+#;#l@'λ@݌S!r52^ر\r&\\[O:x:8fMU*2II++-æ`\nH	p7hiX =j3LY=ݬ3?B#P\nN9+Mb*q?`Ұvidd5XT(j{!s#cm%2Nl=ݼ6İ;?s1{>Sܠ\"	* [ÁNfqV#\rZ`c<vSYZ3\r#:Z2#\nRu=[!r׳w5[d<,<˻E;\03L(Z@3B/[4=\nZ7h:E7(EMNB%šELOM@D7`ĔsyBCdI!\0\$\r S_,<5\0{ <\"&vX<Pl!4PcfCa\$AH`u\r(!{z(ŊKb\"Ԃhº_,rX%e^B6kP\rq-DTj\$x\$\$œOn?H/_6ػ\$H|C-TIDD\"mq\"hպWj_lyk)fP|Cr(+MjDhHx`%DSjŵV\0v^hjJIb#L![!CRsaFk3	T2<Ɇ@]!Q`eO( &/Oa&qb9)/@\n=fxqHţH\nM!z+IAC*FW\r2% (,\\&T'B LЦ%DLɩ[iU\":E'\$*e3g2!mM@rZE;j#\rճNc&) ZyEH\0[죌?IHnAݒIw\$ůdk3|aƄHvW-!ρ,½G,ぇH/e)Z\rhGM=bP4%H(jJPͱ\\=`l}!<x3S(x #(8Ԗ+R`cﬤ`00T2.A,Ix\"HH[.l.l8@BD!P\"P@(LfA9sqjEdf@e-\\rm \"va\ry8 Q<Tܚ`ۊhk37ZΪ%SmptL̠tM+ǆ+%[49C1\$9_9/ YBH*1665\$f/&fԎb̒T?ӣ#4^.qIq6 (\$EXR!gS&bl_cv,jH:v#BcC (!S\ro4\${!RgAL\r()D^WH@K	6M;@CnIF^IZP<Mg~?Fāx al4Ȇ#.hi1/׸LA!LKGD,N_U頃ۀo7dwǹM)bKө\n;_EPFXJ1\rIn8_pc8_G:OɋVb6l\"I1k'̤𴈫ILW[l؁A@mkF;t<}jaA SpZh'øfS0G%5T9f^{(dlj_܄G4lgA% e\"ˊnB(b	\"KdF|&tM`dX٭R( @8jmɼ426Jjk@LOdrSpvKVƾͯPwL\0>ܰxШnpR&K\nD\nTΔbN@ᰶ\"BU0η\r4l:or6ʞԑ!\nKd'@ԃ&JbQ\"3\$\n,q\$NzBj\$@ڐ^FKT\rP8SL=c1B6qLr/O-1L&&,^mOѺG\nQ6\$1P qH1n)	O0G\00051 (`\n	<\n	D%Bh\r#\"̝0p\nR:	\0fl9RN!+2'RU&e)%lx#%!bc`jB&(`(\$)CC]C.1p:(c4\"R\nЏ(\"%OϦ-RsRݍH\0\0Epd!\"R\0RF2.r.\rs/pm> %2\0@3ލ)Q\$7ApSBQr#@߱ -4\nF n(=cU,\nr-A~U-O`?\0#]'\$n_).P!hCB1!sA(&dhG)s<\$odIJ&(sQ#IcnWt0D8F;I\n	0t~P%\$A41	rdR(&&f^̒8c\no̲TdBnP]\n\"5@¿sY+1+STJdZUnblz-6W\0#D\$44JbgLC})񚪐Pp䉤e`@KljM%\0N,#_QbXqn1̠\"\"hU#neTMfNϟKwST!Io=Mi#&*RCF\$jsdЭ3M\""; break;
		case "fr": $compressed = "%(m8g3IeAt2c4c\"Q0 :M&xcC);fSF %9ȄzA\"Oqo:0,X\nFC1l7AL4T`-;T&8̦(2DQ4E&zdA:Φ脦\$&̆fn9',vnG3RtBpv262S'I\$6N\r@ 5T#VMKxrrB@c7iXȃ%:{=_SL\n|Tns\r<36΄3P\"Ln7;N15h#s\$88!(V֣p7FP2Z\$\r;C(2 (\n)`Ep6L\n\"(ê(c@a\"\n!/L\nL0PI쒜B8CVʲ).qT7326l9ϴKdXPT2C0\n˴J\r@-Z20hʿj \"ҵ\$A`B9}42OH\"N420z\r\r9Ax^;܁pXn,3(C,2`\"cp̋%@|/pŔTa;^ılj51Hؓx'*#(#c\$Pv6h @7h 6C5|<K~hKbL9A6}B8\"e-Dh?Q!:3;4Ϭ)M*)y\\ΣlH!vnj&\"^&;02D|/)vlUƨN8~eT)\"bN>vI#p-dS8A2h' E]]a׌\nR>\"8ֹM^!V6ۚ\0ھ^Q#0\n	!0bI\0>iK!6)dC6~7,\0n}wPϙfFx%w\rX	L0BTBGDvT&0<oqP5	lU#\0`J'u\\U5FEC\$^H0-)ꗃpk&\nnQhF [m-żI5ֻWzoKTxO\$N)\$2Z!Eb\rdv:EZ3EiNX:fMպ\\ksLb]%C2k+._rh]<f2حVgaFBhDV1!	JS<<DjXxukA2Ml4JMk7&r!ɜTj28\\H\n\n\\u\n\nr4(Ň2i\$*jɵ|D6i9JWmpZ&Ѷ(1HYxyk`SX[\"C%-HՌuJkX\r4ՃoD͝}պUBo\rk02*HD^IdVe*TE:(y\$Nq2KHYhh(uTae64++i)JQC|63x9-Be꫊Ihii6\n<)BQwY?*\\5NZ}Kz*\rn\\KIۛ'=XӲG`HBhͱF7wf\n&@A\0F\nBb˃W't]H\"KT2uDRShgPic4A<'\0 A\n%ЈB`El'n\\qR*\r%s'qDu6՞ԼS\0T+nk^mY>BN?)GM&EաǶl4&d*FaSRͶ7'R!D5SD\"E\$4\njEʟv\$SC1`ʣS;(JJm;dՇG4Oj&\n9jau;2.#[1ͽ\r\nS&=gS,ޔ4;m'2.@!Rbh\0Sapqtpb\\,lL3s쮊#4DCUuȧҳa<Rɷ%C=sQ.R'7SW)ѹt:\" DeayՏWriuBݠC	tiWQ&:(%Ou;ch\0C9h&\$nVGX:QIXS iCq!ubo4*zrR=ތ ;V!x\$zd<%FO<\n=ʼoY̏G*a5R/9?Ol7\nB[e%n\$`s\$@\nO6'\$3J_2AC1+;3kF \"l^ϣ\$jOB:f&fR!R(pfF\$q\rKj)fAҚJu~VW\$Ф#\nrfPu	2OFLe^YlBLPqfPw\0B\n<-N=h|6h\rI\$Xgp؃RfBƘ?:\r-2{9w\r0YqHijQPxJQDX0&%V[lPyGzP~O1FD\\%k@B6#B0)o1Əq\$M.N[.^.e\nup_*R0b6:Ő'e0l^̊RCr%bVCCN\rz.:Gl/\$R@\rVJO8k\0ΒYrF5\rd:d!2\"0.\"NN,>00dR%q\$d-!,p\"S\r.b\r_-2Nѡѕ?/',{.S/21Ҵ/S>'1fjB\\ZUeK2t#Fj@!	a-K0yC3ds\r-`Vr.2'V_s;4ղkO8E2=9DT\n#70s/k,;7S3&* %;C:S>AS>.0[70X11Δjz4di4D(6(O\0ȡ\rt\"&&d݀C3D:X)(OH#<;tX~B5/i@,Β4o04iG@+@b~tqkG+HnO,n0L	t0ArєKQGH0ODLԋ\"s?qUjoҲo#1mh3Gd!hn	5lx;#hkR!POa	M/CQmE5-v`)l\r0P5C9.AclGO,F\$8\"H2L1J\npog~J7XZg!B\nd.='m~N3'KfdUx:L#\nO^ Nq@㮡!VK|4M1(K2kq2dM63>8~UӲަvHmc[Pq6:Sd̌bdaU0d}gc\$C1h<%d)f,B\noo5LLkk'4Ai|:*ϤM2Mi\n\n(lr\$E(\0P#1dc:\rdk_Ek*'n`>sQ`	,BD洣G>g; "; break;
		case "gl": $compressed = "%(o7js4Q9'!@f4SI.iXjZ<dH\$RI44r6N\$z 2U:c@59\0(`1ƃQp9k38!uF#N\n73Sue7[ƃfb7eS%\n6\n\$s-]BNFS z;bsX|670·[VpL>&PG1\n9llhE]PӒq^k0&uQT*uC&&9JӐ: @9c2%#&:¸M22CIYJP#\n*4*\r?hҬ\r!)!:C*p(V҇4@7(j6#ç#B`%*~ԨJ0\\6<Z(Co9+dǊ	[@i@1@#\"@M:64L0ȠG8As\n43c0z\r8a^] i\\{\0UlJX|6.3/)jx%ʉcүn ڏz8J#d=hƧMRW\rB=<,\"q?B7.0((J28+T6jJv='#7O\rU'j\nˬ&)(G<ws!x\r\\\$#;631lzqjRzp9.~R2BdbxDN\\8\"y`lNOC☢&U\"7j\r78!OYu \$܎#4l۪9صto-Rn(5\rT_(6ǈ#.޽ u/c'b\r	YZLf9!/\rB9<ͻoP	aL*[T	먒)~\\>\"DOXgqnF(jҐ0h4p鈃Q4uғQ{JVQnE\n0))6fHD	>@A`\$zM I\"\r`S'Q#z!\$-vHa[q*\"C@dW*^:d,)}[qS%	\0002/(KlH\rd*JC7dQS/ӝûU0:b\nWJ_,#Y-o~>R:Z~m-t(S>(\ri;tD1B5n b5F9H	\\S)3fSFP0bqגaμUlj8ap6?\nxD4aOe`\r\0PRI`E!^DH{!\n VG\"a%7ԸoSx7tɀ\\QdĜJ/@\"FNv&a.Oα)D赃U5Њ`i劑@6D50F}(PB(nB=H#ӈ'IoM#ipM	jPlBDrC!b8YcDˆFj	7DLzإW%Y|Hu̾q`BO\naQ\"qky\$nGŔDZ8fvq)/F_P\nBM}(BJ<#&9ki	-%zP(veZGrM=@_7x>y(ɽ@)i 0BL	!h *,p \n@\"pA%A' q>)&\\_<oY<_A\0P#d(fuIa<8]^Z;G*z.{^ELǹC<:f,uə&;\n*Xbjy\rɹTй@#sjM)\ryXjQ\$L9<(T@qIDzH[\$Zii\$3\rT-}El\rGAB%PaSsBPYX洊CuU&P?Pڛs\$x\0(aӞHX`q)u<dEd=IH'.`}NOC\$a5W{_\rh⎚ۦxVZ^zZzZaͮeu\0)trV*N@B@ t\0#4ܴabٞ#@)@X^PlvW]iaVk~(Pth;\$-P?\r:%_@{%k!z{)\n4UFyKOn;AeF%J9S*g?FrSpFI`G)bDާ2v]Q]%<}Ԃ7nABBgDe雹BI`+4%㖜px9xa(*%\"\$d,Ro`^\"\$0\"7I7g4FPr\0`ZMrL\r\$͂ \$D+ZЊ\\Mp\$* BozېRZ\n#>'B(51<Įl۬0XI\"F/\r\rwbѧ;\ru8K}\r3egLM\$0.-n5NΎ1Z .\r\"/N92BPqbi\r@dOvkJ!,G3%pV	9d4!AZ@\nGk\0\0@V\rw&{cc\rdp2q*\rwBxȥu\r#C p3rd_M	r(#2,j#067v KO\rͪ#\"7KH		P\$vf5!Rs(P[(>؀@h̙Lcq'R\$.*4'*zdr˵FzB>\r&te-N-|=rsҟ.r\$.RF;R/.u \n(b-0ȝ.91RLE\rdx`,B2s*Ϡʄb2N)3ʄf\\wµ3|QBvo1\n1H	*1Y(F1j7ts`8NM16/&HsM\$\"xW)sd8%:Ӷ@AbFm]'dI\$,5*#bp(cB5*&<Lkh4OU&4R,\$A+4`J `ƩƢ@3io,fv1Ev'\0t47*FހT\npq&B*j,!(xy+?g1l\\cb:D&TM묩RD/r䧙/r^)CH\0qڴķށt3mIQbfy	l)!lMODD5\nPGR(.UQPϕ&tGr @4NmmUV\$\"Q&A@&:\r>~RUʕ.6xobL.)'XJM\$^ʐNUZ~dj;0*w9\n#!\nB\r"; break;
		case "he": $compressed = "%)kƺAAvUkb*m(]'mu]2וC!ɘ2\nAB)̅E\"ш6\\׎%b1I|:\nh5\r;*bJu<UBk0i]?F'1eTk&G~_&0EAd4U¤MBi~ŕ\"U hn2\\+][vGbҥE(ŷMƳqnNG#y\\\n\"Ne\rStN/c2<\$\rC6\"iJ\$\"k'*V*Z9гw3rk(@s5K%L-LRk{0ͬ<Z\$\$3iH/4v-ry0b>%zZHiR[!1S3i|ľ# 4\01o<=s;#7?\0yK3:xW<<As3(42z\r+<(\r#x}z6\"ݡRT#Ap+\$4ɲ#J9\rB\nr!MrO!B&hBHJ9f9/s4(%z	|dkw!S#\$),\$ˋ5E\nH0P,ȢZL]JJdlN]6ͧ7R,H(L7mJ{\"#\rs[c78R!\$2Kf\"z^2_8q9CjI[t\rI@6Ou:{ChZO?@93\r#?H22O\"\r#6><gp+,:\rk\$f#ۿT飏,|*\"9Z8\$e:Q%(d\"'.		0hBp!\$4Ȋ	GI\$dradTjSVUxwV*`/>O\$:,v!,F0NX%F\$2M3@PZ3mNSʁQ*ELRUC̭ʻyD7=8~i1)7rDI}\$ԉ3rXMKkkcQ:d).VlGQ%@|=YHd WaF00f6T\nH<)^LqvTk3`E|(A=2%`ryp7(r\r!4@,>-)@̛_\$<x9RWxx&%\nUL  h\r!49jA!0ʁ_(@8_N,t\$E4CN8 x fZ48 2(@tLqbAT7zb-N='nIy\"hNLC\\%ҎIam-	?\\\$\n<)JDf12\"`|DJfi2`38d\"}	\"a8:\$8KHCnoq|%t95HfHQ!MJ395ceI+6	P7TZ0NB&3N jHRa1#Br!o)۲A]˛xL\"240DQ!7UԔm35n\"`QJrXA3|g2}QỀd3nf꛳~Ғ_, 07)`u8!\"!Ҍ+rG0e+ޒ%%Y.aMu8	\"\ru#)\"\"E9h4@Kja\$>+Okr|1\n>2~Ҝ|	_dNIL9MłtcH\n,;'.2\\!.\rg`5s8DT\n!U8%Dy)8EGh5ifv!60E,~HI;2dIµS\"D\n9NQ~ݥ1%;ACJkT&yJ\n.jv[>!crID3\\~!*y	bM]*Q	(Տ^0{,JM0&iKבFB6% .w/,[z4I#p%,Ɋܷsh]w;-3e\$RG%hͱy-=Ǆqn?Mxm\$K<G2Igc-xq}ۿX`kw2Fk[k_ȗgei+.{^[`~yGldp|>'@RJ83W`7A9Hٛҷ48LTG(r^g\"njk%pL\$(D4Jˑ\"V)&&@LB.\r\\ĊNlrOefq8n+f\$o3	^\\0\n\"-	ie|]\$j諢\nl\"\$1`lp|88]m~Ng&HnXcldx^BB,:l\\J/\$\"Dep3lqSr8^9	#k/nԬ\"#рօ#ܥ%t>/\0d	m+S)c Q ؖ@nfFd(fA oq%@ Ȕ\nIz\"Kn:p!\rw!qtd:r͐⯨6\ncBB0DZi`#Ps-G!:G%^0D:`Vk8А|7*b(*\0KRq&\r0q\rgC({)>`%,\$M3D\$	21FTnK!CKvDDK0*`CEJb\0<a,'#3.¹n2cOP4Br`TɌr0v&NeКq2N,bjG2ې&h~lD67l+&Npf<֧.V\"F(k³&s%5/o/@z&q3Bȯl!?"; break;
		case "hu": $compressed = "%k\rBs7SN2DC3MF6e7DjD!iMNlNFSK5!Je@n\r5Iz4B\0Pb2ar\n!OGC| L5\nLL<n1c*))`k56Lԁ:'Td2ɼ 4NZ9@p9NƓfKNC\r:&hD7, *msw&kLxtl<7cVAgb=U\n*GNTT<;16B5x73Ð7IP޸oX6*z9C;\"TʑR&XҧLlR*\nh\" Ȣ\$\r##9EV/BحCacz*.6.51*e,\$HZ8x-\nձ2RYBR4{{93\"=A\0 mk\rI1l(\$t 1BA\0\r5L\02\0x\r	C@:t㽜4ґ8^!xDl.46H7x%BR#b/5c')zh˯n/&	aaCR'@Py#pη#ΓNC.OD\rC'װsk28>]6V- 3#6ר2B[dì12֕:v3>9V\0V@zl %pbP5+ktr80&W0LEL󛃇\njeЂK1GH_#-juD:=(6-3#ű#mHkx<zxZGJjw ܕ7@g\$ڞvPG0CvWC]a4pN>EMh0\"kl}<ܪXs%A0Fi4H%MYF(r4@!P3N)AAzC#NK?9\0T[` E/RɪPU@W8BXd,ZD0-Uy	u} K'dJA=6*Uze\rRjT1t?\\-tȠp\r\$+*Y+-fc~[rB	\0HH̉!&Hd:96H	e@hEETr+\$ðS\\Z#HL|#r+0؎dQȲ \$>U+n ȉU7@a\r+yFMhE̔I#E1:^K,\0(0gTVȉ*AO5XY\r{6ّ=2jpU^ܹ\"C&IFz>\nhjIU01bW(Y\0\$XǜVjiNJTRČ=.ZŘ.,ȏ#Ɵ0+%3 S+2tSk\0p886:(\$0iI5]NËSd3 %c 1jqoţhP	L*UszA\0C`پ?<zYEssRݹ0tP76IjP`#J`0zs<eB.s)	g5g8\\Rhݹp&4ct1[/f!8pj:b1pPze6&щ3%Je>(&4L]TJe}שᨒhPNzsCȵ&N̹=PF:Gڲ,(Y1 g\r-?W(i0p5z{*\\xld1 EgXCfS,MR=(̬.Y`\"^	ٯʅJ\$aDədF2r\\{gx\$&մq0[R!cw;[	8lE>M]Z!xC'fV˿ρl3YNo\$#ztU&ah LR0\n\nLpN\ryxMq252c @BHo-ڡC(n]ڤw2yTd 92fE:ASrX-֩6g|YkQWN\nRBS8.9T5xYh#;Q>vx{]W}߰w\ngf5=0q䪏{-5\$pY?Ezb6׺EtH]*^6&䔍bܲrCyKX\$k)&ag\0WU8Q\n1?k\r[ֵNֶd/ak5~>8AklC}oC.:\rVF\n#L*`}=L\0n\$4ذ2%a`Ɯ0Agw({PD?R?+H`/42&'H9#l`Alj'\nhF]f&0DJ\r,Ѭs\n0p| RtG&M;D|ۭbjaKr\"V,xϙ8\$Н̌8,Wlb\0wP2v'wdv'q1(vgXCRf\r.?.FhDtQD.\nN1JQqXg%1VQ\\61`-6:\",8آ;5Q\$lM9iFd(,k\n@2`tB)\rez\$\$sλzjgQW0eLN\"ԞU\rѦ1\"\rQƎbGp%``QLhS%P;P%Ub;&.w\$*NS?'P2W(&X.N\"2;ҟ(1\n|	b<F	s-@##(o,V8BE҅&r%m.\$)Zkl/K*Ka@\r(Jc%rr2'03R\0;9%.9BE\$&10?2#oSJrAsM/5@Rw+>0-7sP37Q0;ӄh~ߍʮ!dDM?~>u9cQ;ު 	:pj@N7	=9?=nS >6O?SO7\rIA\"QnT(nZL=On\\J5AS'q?C3RcCo-2U	Fd;\r\n@ʾ\"/Emh1F0ft[FF(fO\nB:ABp҆EsvFOI3J.Jts_ k\nK]eZj-Tr! ȒET\0\np%QZCcGTU&U	T \"B(\"\$},g0\$BI#;MNDaC\n`0HOH:8A8/.l ~mLdEī;WC7\0g`\0o:PR`f~;nhbBB-G/Bp4b![o\\SE8(.T5AhhHK{u<RR&+B \\!B\re`s n	Hr܅jT\$V6\nNu^?찓2D\"4H	CTaX\rcd\\]e`-ag4Fˆ\rb)5b8)Db,3-afF@\r"; break;
		case "id": $compressed = "%(i2MbItL9(g0#)a9D#)rc1M'I>na&ȀJs!H\0Na2)b2ar\n 2T~\n5f*@l4цa\$E8S4'	ldu'c(oFe3htƝ\ry/s4aUU/l'Q!7nS>S/W95&n/x\n\$NX)\n3 Щx(6ǝӑ\"\"CiߚyӇ!9c\$9:A*7;#I0X\r|iR(ڑ+#:>%:068!\0AmhɬjBS;87QZ%\"m N}kZ(H)\"8m	\05Rڗj6>1`3XƎKDâs?`@-@2D49Ax^;ru4\r\\z&o2~\r34ƣ^0\r(	Er%\n5+Ldtt+sx\"7?9X6Gzį%	)w\$H@'B%0b ;@%׃@#\":Z1q,MXJ(Q\n:}i#.#ㄖ4)\"`1Knb׵4J\rr+!mni%~(2l/=BKTZs%jh6_C'苨Q]c\rO\05po|!0Ҋ|0}		&/+/<'1; ,3]V%0BNƍ0̴SP7r\"Ϝ:AtCEcrrܵaJKID*1IYXx3<9PRMTCEѴ}\"ԪS*mU@7*}rE̒cʆ|H%^i\"Kƀ*SvQgCRYLu4T\nWNK\"Um8u䜢9\rq/0\\#CЖiOhH1XHf|Ɔ1DpȑDau'CYoy>\" a\"@1¸P::bRdHE{\0\n0JA,\0POL/@8ܧc;Hc0Wk&C8yFIDc#/B84J73H\"Dƌ1ԁ)41{Q') (L%&^!7\\êya+P@ c1S?JD̚y\n\r!v@ɇ&+7'\",C<MS,5О,!d@xS\n9JI1睌dNRL\$r9,!9E6utr@	W*[0T	EʶpD&G\$)Ȱc^ԲxxNT(@(\n \"Pl@\n	Wd\$2D\rRe*H%]\r9vH02X:gUzʏ/I(V+5Q\"R1V]0OQ#jAky<lc\\*!R\$OӔ\02%?Kh/\r	K\"c\nf`_%n=0wjG2Xw[^I\"F\\FT2t@M Zkiɮ%SڂV\$7WrC5C;K145iCmGm62O#jbL&x\n\nFؚCkJWK\rta Aa R6xf\r8Vt:XdA@&1%!e1ق_Im,TLYI?裏ni\$Q-DwV\nh\";S%l00tY+&*ΉkrI(T͒Q4Bh=c(HT5lT9eQTGBiDp2tZϽ=֪3H\0Եx\r-ĤpޣEЧfW% 겟>84BHnm('k\$Dt\"P'{2;yGas)Ze?Q%7[@YyI#ۅ-J3Tc-zuSɢZVFqj]Ff{q1	x=,YƳ-fW@RDZ8.-\"i!&[:^cZ;_wWỿ}]=s>y%5\r]:E\"ֽR뷴&@9@l\$G8v:mh.ןXYx\$\$dOlF\rS_/ٵ\$,5\"o\0O\\bDi\"L`+ZBK(gj:@Ƭf2-/rPEpLK#G\"i\$,I`d0\"aP|\$|H0B扆f0T_^_\0io:]N\0n	BYnO@.0_tx1\"#lGP%0@H!q\nZA`DԚozvɐ¬\r1-\rN*[Pq<[E\0\$I@.B\\H1adlʨUٌ9\rmJ&`zJM/+JE`j],`4'zD\$@줨r\npsk(\$M 0p\nbb7\$LDoxz4	Fj^>m,@*2L-(.xp\"N[x^jUV'C:\"xx@4dg,fqRe(DLwBS\0 \r(\r\$Ol܏lV	*5N6T#-fR&Hi몌E+D/9L+-b\\҈CúWr\$\"2\$ˠI 2cA\nF@DdDT E#"; break;
		case "it": $compressed = "%(a9Lfit7S`i6DyA	:fL0č0qL'9t%F#L5@Js!I1Xf7e3M&FC1l7AE8QoS|@o&dN&(fLM7\r1xX(-2dF}(uG&s4M\"vZgZ-(J.WCa[;fʒ 1Ņƭg<	 gJerKDSd׳&ZQT\"H&9:oS!W3G#sѩ8Lg{AL%,BRP%&J\"tjh@e:H\"=@7c4PBʦB87f*\r#&rI`Nb(?\rã2#^7D`޵#Ll2\r[: #1Ȍ*\"=%/i(`@%#CH3:xK7Ar3 _A?o^'laL3-	x\$³޺\0=%tíM|:+͚/K0Y.5Kى 숢,7/ġ(CʚWeܥ x؎Th^jXK(\\1+-B4֒\\Y\"30@1J!+ޱ4J'N\$Hr'5;3lX&1<IcME ^Z9%-]~#pȈb,ﶶl={YĒ0Yl,I̻2	#k9.NˢMkf\"J#8(٧\0ڐ8T`i,05nQHIWt0.hH35=#\\LJLȀS9HL	ޓ7l\\3YTȗ%A2P{K-=xͣs\"g(}-~ŗZ	cɾbl!k'˭|wL3\"MH0<`dCpkt9\\8 *m0%yq (&TzRjUKu3\nJ[\rT#P	3&T0aKF셇'^yd@fHn\$g1SC\$W:pRaM>؂t<\$:n!4!)\".0&͘АJq'#G)n8֝W(IAc\$!(Pe;nr߁F괸30eM&i0α^bDw80È)4@D	A\0P	A86{('क7t˱'3r4OmdNGtL׷^,AntfoPUf)EU5'Y!5P7S\nAL@*-f`كIoɐ	o0_KH(ș:vOqqxPVhȍB	Cd|:*OS%CѬ:y8BxS\n曕\r3=I8^ՀЬf\0PPF%rC;DiDh+\$h#ILG{+*&u~4)o'@BD!P\"z E	a3[>V4VsY(f052e1XL9[椵PZ0qSZs>\$ՃX_mw魷!4S!27tHww'b-nHniMZS!cFɛ7?nBo*x\rpbD	`\"u\nn\nmN0%\$ɡՆp\r0҈QkEWFVѼ8HLY\"@3dBM?V3\r7htva\rsV@PFV:`)I`c-\n!@ɈIJA̎\$'C<L*7^iÖ|^ \0ZC\n]e7`ì@sX@Z,*LM./7gNfl)8P&12e㺊Tz\rX8[˽x><3Gi1qƒiaL.j~RpJArKFVȀpN%']LVEBgW<*D)GUf5\\d\$o3\$F>I8.Q~yO8jWƂ5IfieaXE3;pD B~XR{tX\nY	UΒg3Orn5\"%3\"Pn!Rsz#QzoKlڻ{1{wlFӺ~f[L7nUpv`7Yb'yA{I?kۇp@t)D%\rֈn\"b:P\\	<IփTCn\$iN@*#C`tG¢`6GQjL\r0L% ͅ  vcfM06e5/Z0		タ^5/ŦnFκy\"0ν=Nk\$\\mob\r2Mˌ2\"ҽ<\$\r\nȐ5MϚ/\0c_\rR.1\$&1'a0aV+<3dm\n5%N	AcOGF=ZGPol)f |iHw&aGMM#ѷϭ\rhj&e^ad2'N.^L?#9c:Ѐ@3gKK\0q!'&IFP2=RG rNI֌`&dHJqI<ͱH[^'4v.]rv0ʬI\0\0]h\"-hb1M,B'ݥ`(r\n+e+2J`9`:0逡o0ac;%+Xr>3jZ\r%mpͶS1BbL[0~B@&qH5˿%( ۢ*:#%*o<L)\"K*kVU fC>eR%cg*#\"#,M0	4։RZ\$b,s\0mj ^ :s3:ӛ;#rFӹ9,w<0J˳;	,vMʛ5>d,;@\0g\\q&:E/&\r16	>s+&~X˸(7Ӣn;gPz@Z<րhDmW\n\nGLbnI.4 B8"; break;
		case "ja": $compressed = "%:\$\nqҮ4(b*JqTl}!Mn4N I*ADq\$]HU)̄ )dt'*0N*\$1)AJ堡`(`1ƃQp99UB[Hi[x9+AFCw@~UM^_PPU! F^!UМR<I'2mhK,/P[PtRW^XEvu:kL[&|	W~G*)Am4TO;%~sC\\10G\$%ReK8myCd~\\#%{A	Vr_L(Ce\$\$i	\\se	^1Re&r@I	Fd		\n@ư'HFĺ-:´@򘱜ĩ`y.R\\DNK,U1	)dDK)<EpAF%U%J!1<AMSOG\rp<A'9PW%\0'H1f[Du_5l3XVPNAiy`\\<g95Hk'VȂ2\rH2QMD-KM]Y@@GC1R1l_͒x2\n@4C(C@:tL7Evx3(9pJ(|t%Z^0MW9tze,2I&dKzZ1\rnܧI7QP97{ҁ8sV)\"EA(ȤW'JhM7ѻ1\0Q2U9@?GN\0sPKSa3.sQ;%vs}+'WF[hd\"^KOi/?*[KJOHMeH)\0P(gO!<E2w9\"zηJNO}[,IW=n;\nPn+nspX!>􍻚re0@Cs2r\r0°a-	C>a7`HeAa9\"cB\$aLO 2HII0^TtWBL!J,Jr^-6@TB0sl9+)I8EEC@)qxǓ4|R\nb'9EEbFLiJVK	i!i D;xFƹJID*P\"k90@ D!1KV& b`1V.Xc셑VNe,et\r0`HIm(.C &	>Eܖ%#cj5fKz\ndJd\0DP)DLĶk14 dLvPʥ-eřPsl͞ÐP\\_S\$M&0sp(h͹+M@AOe>Lr3DW\r䱬S!5%'(D9jSaF.:\"j5*+[N	#:lĘl@耠B6-z_AE/fzu9\\j+@%Ut\rq.\$\\d%!!`)\r0j:@=F~æebEH+ZAD)Oi)W`]q2LDyTĆ9Dpn6dmGI*S\nAF)-CiwNU]ĳ4\"hSc6IICv:ؔn(#B	pVՆ|`<*8\\.wWZ:\";؃GʍK,fQ,T_K>JqIdV(!|OI(8XFE\0VwӛkَJ=k{\$بvKF\n@sQI\"Wՙs¾9eB`Y.aLİo>\n)P*P\0D0\"\01q	,U:^ЊҞ{ϛx>jP^0>kUQ_s,춎;\roEfDaa5\\(\"LFZZ3)9qlnx\0\n*C*^}=&LV\\(YclEZkSJ*hsP\\\"<Y=4_A'\$\rkEJ	wp[dDȜsЃGQI,KS!-XZ1qӤq*`ElW?CRY+Mye^5w@!Ԍ!_}PX^1r@G/dw\$%RH[K\\Eu9_z'TuHi`K=l[0*\nGU\$X #o[ LR&M.l0I0{gD9Bb\0/Vk.%\0B!4쒃dvr1*FkRJ1Box\rNLn܌ꐎD\0@W\r*L.u\n1qE*By&Pp!~.PH0\$lb%;43<lmm?N3mCf7f8,bq	@hk7Fb!Q8CqF6Ƥ-O.pMhyb\"л,{JrG(1\rpL/b/DD*yRZWћqprn/\rfa\ngpmx.% >GV'EQ'BJ\$,g͑&J !n* 5}\nI}j\$Otg#,~3}\"<!x+cŃn\n`\$)Ƒ\0@	R;'rh/\0q(Pr%RYO()/:ToXdRPИ.y\0* CE>Y#!.9:R!,0,U\$5\nBr\$|*C.C.P6k)θͮ\"rJ\0o3**4j]4/!44R%*'O4aO161671c4-50!jAC<^@L	D;GoRST*o;-7&'Q;l932VnXSت\"mU>NV!35TTmBg9*AegArK5T(!-\";CSCt3-+Ds\\T/T\\UU-gB3?aXBTl^Rs0;lTYEVRtKFWG4Z?E!AF1=s*u%IW84JKTpDSiBR#MMR;C4NOG6s^ޏEEF!JBԃQSQQ!	l\r\0jDpdCTyp<#*Ho9iRP\0\\E|['D(9Vm9S&x7!4l)4~HkPx7`\np)3;ьG.vpv3(6ᢔ<s:e;cb(I\n/'/VW#+} rHC\rnc.~#cO>s%bӬ< <M^Ȭg\"5\nax@{B/WQk2AMd2nJrVh֝B#CqtOaqsAX[2fNL\nL&*>0^gOj!ja,G\0`XT0#+\0p.r`wgs'pk7XMDT2R1w5V3EDF\\"; break;
		case "ka": $compressed = "%)RAt5BPt2'K¢:R>5-%A(:<PSsE,I5AdNi=	  2i?cXM\")v@\nFC1l7fIɥ	'\"1UdJ	.eiJ\"|:\r]GR1tYg0<SWµK{!feMs'Im&K=e\"r'Q+˿}-<^}nnZ,:K<թ;SV\"zq=o۳*#\0LDζS:-JsL\"4Mi(N\".@9Z7BŴϻ&Vl7RRrF\nKt-Y(˰KpDLΣ*x#	Sj2S!RL,*ʐiDO/ۊj\r1ЧK(N#VJsR(TOTS)HHE:1	%iRՎM%jtfG,>C*^͵LYP\\t6\$\$5;b6#p9J:'TtYRoe\\]J[@4C(C@:tD7{x3(9PJ|(W||MQjxTӧ dwmN	=kRzM;cڮ@CMHh<McJrө*m\"+q(!1lmNQ/O.U/]03a\$?VͭsDj6j7ZNC-]%:o;먿yoGں(Ḩt=I,	Bh!f ĥiEJV{5fоꚋtzܥBb-<u:ގ!?<+\$L\$j(6TUs샇jfnYVS^!+q5|I2#%	^7&	[Z-,̦!8mFX0eD 7\nQ4dBT\0m:1\0Θt7\0CrA<LX:a7`e1ꘒI4Q<%mi+L>CX]¢+EeV`tNбh}d(](G1 !4xfH&D^1>R8\"N+K%gܕe T\n6|CWƄ*fW *N	Dā+\nro)an}\n8i)vJ{Q^K{/3l?ӭ`JNDM\n>LEV.Xc셑YS,e2G@Ltg· mm6=c%b-Q4MSiz،vB+pޥ!4,%zSz4h\$\"F(Lގ\061F,ćvHɧ)elPps,:HީC=b&Rby-ԙZ鳒SYgfI@IũBCT4˧g&E&dN#xYqX,-)xPxv=\$LIM\$3H	4Gt㐶F`2O\0L_bmۻCN뙪V\"+M׼uwj.\ninzs\0L'[H?`[VOv܉QI>&jiAC\naH#/T {#6H܋'A7W^%iڒx)U15ymE\"K?v2X7P[PGB@%{R-Yp\$0^xʽ&l{	Ph\$ַs? ¢4(\0ɭjUt,bKtSWdJ_MQ@e%Oe+sձTS`a^mD9:\\?JD3M<&B5oM7MA(`ޫֹ\\Sa%lIKcc+zI4nQkQw ʽ뮮2oHS>E	2Y\"p'%\$x2.~DJК)>t<0KְiU#XZE|P6@>0SH'2Xu;^઄w5Qqߖ0uBHo;nV]B+j](epA6PνH,ᙫwZů^&(ܐ4Sq8@Ga>3Rm+2W\nCɻyq<x])Zi6QzN~ rt|t{oI߀Nl!YlVP@7&\\|O4g؛)\0 4.	ΜFթF&P`zo\nTp`a|uX(Ī%ئ@\n 	´CVyIĤnZ+n;ֲ	G@DhG\0^-Em֐8@Z:2)J&Sp7,'T2h~koK>AS@'lR|i=|hlh|BNnb<LV6dz4D)Zf¸!Cht\$&-78Qjq&hͦ(Fla 䰘r&΁ȆD&Dqݏx!\rdрl04'b4(-C\rr@F0\$jr<Nr m#.UL%gfUl#u+:Ƣx`1ڗ0G K\r\"i#I&p:tq)-tR0M4φNش#rD~/#Tb=4.\rZ&.;ƈ+,.qd	,F,b.O-n(*+,uO`	XM)'!1oD-M`VW2\0,nr:.!00̚Q\0r{/o%6	6B/6OB-7u)31n,6ajOB`Q7Qx\$B#2zl}nP+):?:\");lꥒ̂*жTONhp\nmSxo=:e=:b}ѿ6rʰq.22#L)Dǰ/\r8sZTHmDE3LR%B˲TMsF+>G`a@g6X&TH>aS1W5MJp<MԚsIKKQt'̦?Ep@m&S&NozqZ1kDI4t\$45]48SsRS@:QӁ\"1b?}C0qS>-U(pp|:HwfVNeUuU@s\$:Z0ys2Sp%/\0L4I0Bv5?.PvYu//52[L%uR䵾Z(ENU+rFi,(Q[STu0Uq;#S`9`\rf&f@@@e@\r&6 &v+bГ%c6\$\r)b0oد\0& e\0X(QZtk\\<94US4ևH[uKXV\n֋T2s]W`uTr+%pVrrj7X,5i֑\"PVTiӖ4`:h5U8EZ1lBK=ĽZ/U(4ǷM\"~^	OK_PC)6U&5aT{*&La@qklzb.q_㌩uI:KE>*pGĀ\n\rrM4O\"\" rbol)A]>	r,P~S]眻&KvZ&sFiKMskW]sZ/jX&zx֪l%m|WDV1OZ@aK|GOpp|7Fs\"\"5F鐶#yW/ ~l mǦON&J-E{-:w WrD^X荈?IlDNΑ,?vl:R;NP*͕&YF/\\)~{qǥ]ٌe8(l6lSRrסr,\$»oE#*\\.̹T*U\r*]Y%/%jE!at	4`"; break;
		case "ko": $compressed = "%b\nv%Ю\nq֓NU)ЈT2;db4V:\0BapbZ;aا;O)Cf4)؋R;RȘVN:J\n\\ZKRSȈb2̛H:kBuY\r֯h!a/\"]dێri؆&XQ]n:[##i.-(Y\nRO)igC#cYNw	NL-\0S0&>yZP',l<VR\np7\r7IX00c(@2\r(A@9DC09Ƞ\$aHHAGE)xPv	RX3bW#gaUD̸=\"V3d bSYa6'0JI`SA\0<7D!`uj*FRO+9:e/T-M4[Di0t#ZvBk*u:I	Zv(d# ;1KQ1p\0\r1\rH4\r㭤ECY\0ym3:x\rdB0p_tcI|63Bl(4px!WR5=S !@vdE\$:aB/i;<\0kh:eeiU/!NF\"\$:n@#X6ewZ-E:BQG(!LNpy#:WQ+1NHpdJUY@V,ĻD?OÈJ\\Lb@VS*δ`P\r,6CC0x&<R&P<\\年KU4MXEQVgiRe97{UCOVo:.;!Z^̓RjM\0-/]qʙb'6P戗{^A:\$!史WC>lt3C,%4hHsvj	4L2VlEÑfI#JAQ5G%`_؃AP7( -E \r30Cr!3m\r>k2\nYV02E.hw X7 Pi%wת_+~X`Ƀ^\"\0tc(Ϥ2%p<͈Rk`mPH/%J%Gu\"/f/ܕ1E9W/]y/E_JD3aQ#ɆI\r!4ˌ7kbȥJ4ܘ͢@HB*m1nL46;PĆr9:tn]Q8\nO᥿DNi5dDRiSX(jʁ*iP°)C>3\0Oᕯ-\0 \rJ2:Bu	\\b%red>Ú*Z6,]uHc\r3pAOwpX2ً5g, aL)b4!X7\"xp]7\$h&@Aԑf9HȘ>>\"#\$Țb7Gqf/\$|dR:@HS axRI+	\$<T7*\"C![>3 (ɵ,143T*d(\"yAJa\"]vHɹ9'x\nBkGPP@(/!XN<B. \nnذM0T\n2ia\"dTĈ%LREK5#y`3bc861f\"ukssxo 0<	: u)ڰ\"	i!aER=:Q:#u,r+R#]W(ͮݝXT>V:73\">i#/h[8@|̛clګf(L\nzN 1QP|k=iRB7uzWTNoDbo<9RQr%ʹcbdBP\$[pbv=Dǵ\nӢݞ8ޢ'>Yf-҄=zzBhb5@=Z!`'\0PC^HRvB\r':O;S=vURb,(sgQQ\"vIRmiZY0C	\0҇\nY\".Y\$\0d/\rp22Z=hْZ>Fլ`O#ޓDZ9|ዚ~[]_L~k}ARizgE~ͬj)Nt cOvoR-ˤ%JVidn҈{>B4gIL49n?'&r\"=;pj6M#Fi0#\"epVpRR@LӃ%\nV6lhO	oIP2'k090i4R.2dN1cHAF(O.7)IqAbӍP\"FNVd[<ٍǟEMyol7\rgی4%\nXܱJ̏z|\"P1PڑcIg,!FCO,	TC6Ѱ44Эפ0Nzqn|\$1G&\$\"\")!:&\"\\8lc%Q0\"\r`qzrgR΢	eg!6|\n))*HO Q\nQz'1NP5rNͣ!gҝan'#2+2*QW ұ\r6/6lnt\$:D\"iH\"b#A66:/))0Q-q#g҂Kc9rp.\$H94(\n/`\$%#4\09#.a6Q&661y!D83)7-4bVNp23P2s*Ns:.G0ZG4ޕ0a\$7EC=o<ΓS=lHh>\0oeS3jS*4Qr(4 r:pB3C.(O8?LT148e,@F)2EUFTY*ƫ,n@C\nl18K<#Aohֳ3sfGFC T)16Tz1Aj<OGKKKM/w! lJ \nt[fo4(\\BVw(Dl\nZ~\0[E7NB#'-6meϾԣLjh,>s:unsG#]Wq)VJg\"=I\0/\r[ oWL܁({'\$i*c~Lh|6OycȻ/\rb6.U^p60!6h4H3+U:1BƇd>@CSA`	X>=k<`b܇G 0)[-R5fLxko,d>nkTm/aj}u)\$_M)f%,\0=#bn]l&ƓG7tCt!*JM\$"; break;
		case "lt": $compressed = "%(e8NǓY@W̦á@f0Mp(a5&	scb!iDS\n:Fe)zQ: #!j6 rT&*4AFi7IgPf\"^ 6MHC	1Պ\0NE\r:Y7DQ@n,h(:C@t4L4:I'S9P춛hb&NqQ}H؈PVuof,k49`\$gYnfQ.JbfM(n5rGHt=. 9c2#P;\r389aPCbڊ˱fir'5*?o4ߍ`*B 2C+&\n5((2lP0MB5.8҄2!,,\")#bz_ r.\nH5\0('MTkX2\rC\r1p4#ϤN@?#AxXD49Ax^;pJRÐ\\zu2h\r#,\"Hpx!: \$S]bc/x2_>.+&͚1c\nB(=t239&?\rKX(O갂#B	#pOC\"`(2We(1c~x3F*#*:9B4\"I:-RF(BbGG#K<2ÚH`յ1#s(\0:Id?J9:T6罸ϸ8h4Lا}d1k\"uS?Gg2,T.+8խ9l,b*Cq.Ly%	@6Msmr507=Ҏ|(f,2|&!-.6Ct\rĉrSraDQ\$o6)e	\0*Ipyrl:%HYp\r\ruAh\$͏Pσ((`J1PhCLhC(|	B7䜴p@qV.LWa,EYg-y-+D }\"߀ȸϵP!iDR\$ǊAp yQhWL#yDuCIkaUV\\qh81[a%BR5	r@K2PHbHJ9*ÚwrE8B\$\0cV@ejJL|0bo>Aő=q)^a\rxd\"7,e#\r%J]\n (!>(&P@\0O'm5^R%Mü F,fmԳs1[`GB\r+4,w5.e=Qr2TN:) 5HiD!9Z\\r\"v\$]q5S,:H\"HIAqsаԺىuKa	B	y/L,s\rN&>j\rQ-<sc|R26Q!4._\r	L*Xcm%k#\rdAglo)!uVND3ma5~N5r\\%d-i)qLוȒ\$	N\$Q:\"\\i]|6`̤ʳ`4:Yi!Ü3<F;wY9+r+*\"(Ed.ID1xZxYt'~xe˜dP>,jnb5Y2~|\n70i,ش* Bȱ>C:jQR]u&GEJ_Z&P(--*yf'ȅ&=2ASzjE\\g{y_暛^H#\$t\$vSB+0bAv+'47F37p1E%D	\$XSmՎz<xaX]l!@Hi?ˇY3y<>z~~{op˔3_1Hn.%RAtb\"v!d=ݒD;a*@@ |eBxCةA.q,'(AXl;	g9dD9-?k'@^Ix&J.bf_9\0tsKDˤu-Ɣ7/zΏ/\0)lEzoCc0ѓEC_7*Owq%nE'i.bk(\ni6!(VXmr\"}Ƞ2緂ZR[T,FႆB'Pl0HBld<I0#L\0}E:Ч.2Vm.bo--f#.a.@(p<LLxȦ*Z *ºwǊVi0*2nG|9?\0DE|8ebJ;@D(E	D#2!LD,ݍ\$,B!0D\"^ނBnj I%mO\rGVl;80@P&[&:DPOa@J.L!E+ort.b/k/\"C1>eOy>0ACQBtҜpB%bp#ĸ,FQ4=Dx|Dp~\"eilp5G	\"~!\"*@B*@pδ&\$dΕ\$&M-6J7\0ONy'm:1-mlQɢe<\$\rF`M*S\$\\,`H;nl!%8r;'U&0\0+--)B Յ?,C ~kQF6c#[M'p;qsU-R(:uk.@]_Ù).&,qY/\rDLa,K%)2r2#(B;/Uk0RvD|;bAs)-\n\rq-&CK4d&@'fD p297s8\$.\"(R.w\n;R\n;3R=6<>Ң	t-rMD&&BpMs!\n.R-@ܤKj.<\0/-°{L/LTb}&>3R Px+3&KC(]g/NDrOD#365?JDj:	}FQ56x&/\$12e\rV9Tiv[^t;\"nظv`\npH@#f(\",/kMNB)\"pJ%d?\"/\n	Am +48@,/*8`cBw3(\$98[J,Ț[P?[#\nNN\$Am\ndԁic{0J@줳rҬXZL40-rMsYʰ2\rom f\$Dl&`tMpdI1G\r\nh2pI3&%/Z`@]e+X#Q\"҅5GgB\nD>է)B3^vYO]J=M\0,2G1*GNN0\$M\nB=\0"; break;
		case "lv": $compressed = "%(e4SsLq:I :SHaa@m0fl:ZiBf3AĀJ2WYCf4(#Y9\"F3It9GCF\"67C8'ab:ǥ%#)DdHobٸuN21	i@ S0Mө_ni2|9q#{o5Mat5_6̆Q32b)V,HC%À9\rRR\$I7Lsu		jCj\$6C\"\nbf*\r40mZ 	d\r#֥ Pbc\\7(轶O5Lh׷r.7\"LL(	²l:& H̢H`7Gb)CAЂL#Nb\\4C(C@:t\\4Zڒp_	cxDò89!Z\"7j@ x!.=!(PNP+(#/p于(U/\n	ݲ%.RrK!YPa(A؂8:\r8!\n%ʞ@c]C,07C̡9)\\x[0%NPhsPv|)C(5˻-!P3A	CV*FLp@3qx90(#UȶaJ~(8Mvn3_qGxCW4LC@I4 <ѐ#h`*e睥~B ϢК\"=bT+ď6c|!`3/E\\[}5AsA>猣IEİTv@H1\\fkm?7B=9Crihܸ)Z4rڇ8e	p	46C|/A(HD'\"FJ^1lV/Dў\rND'!^\$!4T,pDEcm^\n Is&Dj;(A\"a\0Әl2ltz\n)FE8K:\";\nTTZՌJ9+ur	caePb}d)@Ҵ	&DEG)`%sS!\r%̧JW\$\$Sh[	n\0ND2UJWu`l\0U|NVa5PZ}d7dZ	>sLDI9#dϙiy/\nlH!k|3ځPuAa)IPR\\J>7ٙ!\"Q3rk\n%(0 \n (uF4\"DL*+\0c&a<R!J!wKQ}yUUc\0`&8xBѴKP#`h4(܋oҧ\rY`/\n\ru23\0F77yÜ7C49B4\r}P/\rm5MͣhG@ޥ(&rHC%,Z*18tXݰ8a,6 #C{谡<)J	E]eq,RX*Eu&@OC3k+B_ۤWP@b1H \nn>HI%\$jlɍ\rbCG]#%bF\\DM9%{2c.9ܬ1KN&efk\0PGn9'L6\$L3wEFp6M(p:zD!2ha.XGzQ}0PB9,BH*/(V\$\"tc5KZՄx\n\"w. u	XRGaP(5*ӰN7R!@WJIza nqM76{A\$ӵABKLddd͓v4nN3PZrEA!CT&\r>=q{PdW,͓ͦjA|OsqzrfUw7Ԁ6gc/\rc#08Aa\"n+L-Um\0@GujISƍ'6}g~~}\"ݩȃ\"PC-ۯep'=&|S=,TǓQxiD<cCpkKӒ+>gPb=hhsa<Ǹ]>H/\".\nGSjA:31!e_wĲ*/<(\r}\n:}9N=w#\$l F@fnE&\nEr,@0-\"zĚffza#42,LN;Jv4,BB\"ڐJOR\r' OTf&%h%^Pp\n~A~) \"0U,fn#zgdZ.C>b(rfxJz캸L\nШvF,J^d%К ]l0'2A\nXʃ*GA0`bzc\nuQӱ0vpp\ng9FML3ospͮ̼1+(J/R[ n\$kzHu!Vˑwz~xbg#H%!pǬऐLΣbR	LIE<2*Ee8T 2*\0Bg\"Z.s&VYpEq-̆_o~rͮ\\NDm	x8?##\r'. tP,u2Dr\\N]4f'2Peh%'ft@Y&OȍtR|r#O.'\$1\n@PQ]?&+wTs4)~tR_g+n+҅\rƏ-r%2+ib\$B3H+S,Hs1;zKc0Ρ(c2ĜdRfBU1RO'КtQ?%3JN:\rRr\\D(\02S5n@\\\"`{\$m5NO5^)cnF% 7X'<\$22l% ;q42M2n1/.a:bc&LM\rSMΪ2u=u0V8	\rf\r+RP\ned!/cja,AcOB\"fq,·C&>\"#\rhh&/N1;7E,mn3,gb7@K`\0V,&u\"*9*B\nZ.R<D.#\\E&gLyM=Mn3~DE\$&ECc1D<gvB>X2#BOP<L,rh-8\$\$HcK\np\"N&!0.]UHUoHcM3N\00ECWU`ƍcj,rxc\"/'>#C7JI&bKCLi\"=̵ޔH`/O-U8jܓJ0͵dMG>U#Up:%1'\$feĤqɄk\""; break;
		case "ms": $compressed = "%(u0	 3CM9*lpB\$ 6Mg3ImL&8i1a#\\@a2M@Js!FHs;MGS\$dX\nFC1l7ADt@p0Qs7VaT4\"TLS5ki9k-@e6Q@k2()6ɝ/fBk4S%A4Jr[gNMC	œofs6!e9NyCdy`#h(<H>Tk7r!&.7Np|+8zc*v<vhH7lH\"p=xÍit<(íBSV3#ÁBRd+3*BʝLޮc\"!P	؄;QjiꉃzZT3{1/c Ժ?ì&\$bn>o;#7T@X90z\r8a^(\\Ncs=8^%\"9xDk#3ސHx!2(\r+lL#\n&5CtF'@P0\nT (C˞ׅJ|R\n%L!`FPdt6HI\$H0lI|P)L\rë		Ĭ2X9K|rZ9<\rئ(S~I#p/Vka\rB!{K<ǙI+:ϰY9P{\\[m\"HJŪݦ<v*3aLLr|=>c=1u]A(\"ģX&\rZot0̶HhE*:&|9n,Cɩj*i\$xp')PwSD1-w]IQzrރHo!FrRolͽn	RK6TTj;x UBBCIiVCH]	?FXka6>7iJ鍇?0,=[l@\\SJqO*D2*9*`ú̞2`] kmA*_0'P\$9%\\\\ViC7j 62c!r30c uO\0003R&3Q\n((LrS7RtrB@{/\n (fA?2GZcI Ce1cy~٘+ی(Dg#܂F9tx?w+U[(IHc'3+dtr'H2(L\0C\naH#N;bAa9@|>|FQr`&=*(bbfZ.9]hVBNbG88BRɱPN_U=Q0M=\n>Q0(%k'O\\\"894'Ot9=1WL*\r\nGM;iA#(2*LTNm呭fY)\$Wq&3ƾܒA<'\0 A\nЈB`El)(uIJHq5Fkqd,%ޛO#HM=1[4f{3!%Jv`P2h0oKj-ohD蹰!ڏ7;tctT(V:efٖ\0L9OSMbe'w˽\"ܕ\0/HzJVV=QkO:ځV13#XeԪ	ugI]\$OhlΙA:ǭƸ=\rv@rpcgA\rL@.z@%h-)C㤳Ti6MBwFW\$%jXz, jU\n!nNr2J+V /wc^\0vHY\"%jMzc'Pe.TVRv=?D9lH.\r;xmv*0 bb1!D3K*>( 2GaI5SY9##Z`j|N=&bku%iļ:Wԇ`j\rQ!RG\roFFz:ߐ2+E\\9[P0wIXyn7⠇2vۢ&{ͪ\n?b\$}>a\\{XUud%w׮_\$9GO;bk>\"_&}ɾބtCw6Uvx53?_o?:>Ɔζh|C'B9T&g2zJ&5/&9z.b)b)IJϲ,5B,;egxLl*PE:ڜ0XIMD(u\rF{pr:O	P L:bB\nl\rJ~CuBF:`B\$l*|Z=0tNh\",-В,XeýJ]	5_\nMC\n\\/1:&:R̥F߱2czvO	%b\r\r!8?bb\0=\0ł3d 'I?p1`,'\0ϣA x\$\rЭ:PCM1\$hQ ߱\"B-C	f0%wɫ\rlSbi<GLl~^,@(^fzKcm&BN\$V\rVbp!o)j4o\0@C+3PhM,,N\nZ2ڭ0nNk7n\"kn\$:x8IL%D%R+T\"pK*b\n\"pmŸ\$J\0Z˒m\"ǣ#%2LkL|\nx{.\"y0qB1N\r3bO2.Djxc7h\rTKL*BVfZLԨ0@\n 	2-ppt1\r;-}/j '/@zK;Rd86y\r:\00!/l|Y~>~n"; break;
		case "nl": $compressed = "%(n6Sak3do0p(a<MSlde1tF'#yNb)̅%!MуqtBK%FC1l7AEs->8 4YFSY?,pQi3MS`(ebF˔I;ۍ`0߰\n*͍\nmm0K`-Z&.O8Qh6w5m9[Mֿ5!uYqoEkqȕ5u4.T@f7NR\$Y8C)6,ûBю)ϛ\$=b6h9ØtjBȣ^K(HȾX8- 21b(ïC,7 r1kN,+rt2C24e[Qkc2P8cs_2Ѥ1?\0P\rbDHhԁ=jԜ?ʂ:,3ʀO\0@=Q4Np0´D0z4crx[\rAr3^ٶ,J(}>ы94\r |FːƍD[ o+)û\\X!xx\"r:J585_Ӱ8B#˂Xޏ̢%m<37rY>r)YZD	p#?dLK0p2\"̏h2nLj5\n3Sm.f#V1.H \nbF 9.K\"7tN?mUP2;\r{;80BR2\"H8hks\"\"^mkEͻ4\0Pי(VTKIaͩk\$Hc#t3VC\nj¶Ш.ް/#zүI(?Xs3ט3hZKOdw)􌜂\$u@\"TfgW/EᑅvO	\nXⴟOك?@{8}\0PȘs:&3(gIB\rBgN\0AQ*,9*UU*4*^UV\\õxXA0U@>2gfL`#[AV\0q6.?rNrN3(I\"XºB)傅W%۸<	-|JCh\rӠM*dm^	)Hc{\rF\"SU-rhq\"_2d18M\n!S~qԚ	;/`f=-\$B@[MI_~o(\$@zI\0((ȼ&|Ci.E䲠sdA(fvURE)&!uĘ[f>[d\"dDLсfi<)LTʡmMN&^D\n\0F%(	EI-%AgÄp\$ZsTg}G\"S|qnD)>pPìח]U\\:Wj0%\$r&/čd_SfeO\naP+D;'^h1𴆒1e[֏LIx]QX2S2́D6g)#xL0TW\rwV54Je/Զ \$V*'\0 A\n\0B`ElzB+2_p	TIFiPКX1Xk0m7e7j!rucH}'\nC[m?7r<\n	fW^}\"Tܟ,ӶȇҗVABeJ\\6W2,f#Y5RY[L*wܖu7zLux!Pn8'9rxr!//{Ew3ІZCPA\r@ݵ	.ᜅ-T^5n2!@J2Fl33Ǧ\n!GzAB*?/&]\n!@Cy3\$EUX\"^[\nan0eXn!D(f tT0*1;to2λ%,D:	ll༸L܂L'\n_x}옸'Kit,)o2jDk=)(>HL;e<VtkkzlIcCp+Ka4cIA aR)	B,eKVPȒDqgIBO|P3x{>Rӈ^CA^4_K`\$a㗛њ(u\$dL`_gmN,D <eBroi|&--.ߡ<X|\n?}>E7OWp-%{v()-دzT2l܊\r~Pp )-o\rdM`m\$\rk\r6p\"^ 6\n-`؛K\n>ψKe>!UB2bήFش\"BֻW^Ë.#4C!-+b:\"]D\rEC0sl^픦P⤐90)3Qэ,rpzehЮD@KMÄbn\r`̈́GđJ؏G	qW\0Q\\PBAFh\rjY7M/rO\0001[gupQ6a*anqbF-Q#oQ\"&lJKB-h&{\$\rjmJ#N`@1`mc/##`Q^r=\$irDՒPұv\\RN8\$<\"TIQ3m#ʣZ1a'&RSF@1\$n-za0z\"؁2_+.|\r(P-ܥ\"i2_\rHa\r\$\n\\k<=&ȥ\"jH5	n1C.	A*\0\nq~8pN\"9\$!#a^.ͲCI5TO\" bke([ҍqBdi҅0\" \r`NQ5SKB\$\nV/Bd}m(kG\"Dc;f&.L42&/83J\"&>c-=<K838GT0d?`0s@b+bTh5%>C w)l@LjOIM\rjN\n*q.l]\$*!m-r\\&MLT1\0;	?ff5XIT7m\rD2~L\$i.Cz	##B\$`"; break;
		case "no": $compressed = "%(u7I6NgHYp&p(a5&ө@tNHn&\\FSae92t2Y	'8C!X0cAn8!	\r࣡\n7&sIlMzb'ґkfY\\2qNF%DL7;g+0Y'qH16:]40gۈHr:Mqt醡B傽JG\n!n7S:D0LQ(Ye93^;#\":+(#pصa\0\rmH@0j&i#M|:	((@\$H-L܉ ;'2\"B	<<;9Gp7B7Nc|p!Cs69h jڤ@ c\$K&ϚlH4\r㫞0`@RBf3ЕtL19H!}%JC ^'aڴkh̴I@7x@|2KP :52հ6Cb'.+\n7-:\np7 X65P\$2WS z5x ǌ0:x췎6P\$#U%#n	b&;-,&x5R0Xc&\"'h\$2c\$\\Ah1l&^SN8!@R\"[[B̂\rcPBz.˿{n0˯9ۀ[K#l0\"*Ci!gvgI/jL&#͋cv43ڣ/NC48˺ڴtqZs(7\$VF&(ȣy`Z4&r,ѽ*3Z0\rxA4\\C?P6(MgKF7+n0쪔peЪ7qT\rѫrT	N\n)`qIS\0M1:âTjSr\r>v2:0k986\0sh z)&t\r\$j@SpP*%HN0V%^^+y0\\P^q8vhISoݬ4Ơ3`kj\0LLXt&ӈo2/iODBHrȝxRo6?DPJ\$9R4\r%\"Pr1Հb\riBJG-0f!PNL\\Eq]5ጔ(\$TwKOFoMܿ@`CaPI*ȓ\"\r҄FOEr@HZCh}fd痉w/ؙ6Fz`TPoX0B!MV(3bӃ14e(P&>/BsZˉ/tZ8i&p ƛ=b4ht1JD*6,O .bR̜r^LI݊&\\Z֭	yE)`Ly^\rntVEЕx'!M5-VҐP(\"Ĉ.PB3\rPgAjea<'\0 A\nЈB`El	~&WTyY,_Vqoc4~\rRӇ}R/͹u?	kP7\\'!PvHUck-f܅zlY\n˪aĔlVD9Рtm fm̵N!t9鐂l_`o1H(GX8Vw£B86vqiLָhzXKkw#}L4׊[9mЩA\$.%ola1h4ӗrgzf.\nYpfh1əS(R\r։ĵ4D~Ov)(^VP uT!\$\nN5}=3ǃ)!IbM9{ȽW0!.}9	-Kmdr!@FH {p2s'.dvg&ɚI85htu]0o,A117DESAIl\"vS9b)%n\rJ+PaX%skцLkƤhϙ\$<Ҷ0\$`	`!K!sՠ솺qdx#˩IR*vAlvk-~/7+VZQI]A(QcKG\0P%E^WA\nmr[W&\0د;F	}ᲅv\r?xB{]~)ʘ{y'93E_}<72t)zsĊv~;AKsq-ӿ44\nNo\r\r-me.pѠ\"p&\$#GЫ,PbdNc`,0N5	0Fn(B'kPT\r,E4&d8\")f4m(&b}-<УrM`+MEl%['02@PBׄ[/l\"#0pO\rlXr0\"8Q	Ÿk\"PbҜPMmoN>~C\"=l`'n2di|;</vZI#(CQI\rњ\rqRpϢmGG!1_0_\\mqyN&*\$5l.ӑ\0ʙ*)\rr.4 r1ӭJ)\rÑ1_=#QϭH!%\$f\n\\hC&Jb\r&Ѻdc4aD` \r1n\"pc\n:B'>f^ш*pPB_^ʨ2ô0n1\0.+@db\rV\rd!~.+)	8ܚ*c?蔓*\npz<ޥ;-\$g2f8\\(1aJî8'#65\n5^oX#lANb:0HB` ДP05\$6QEƴq	:]O;2qE3N2\"[;fCp#<\n&=B,%\$.:k 4p<	,n0\$4cllN&/?(ĔmMܾm\0/:;C\nc'l4XaZ-Ծ1E\"\"\$8ؒZE\r "; break;
		case "pl": $compressed = "%(g9MƓ(l4΢劂7!fSi̢Q4k9Ma ;\rmD\"BdJs!I\n0@i9#f(@\nFC1l7AD35/8Nxp:L =M0Q\nkm!y:M@!aݤhr20gy&*u8Blp*@do3Qxe5^of!hp[73qiy7pB\rHL>\ry\r+ry;¡\\b@t0.\"D)*a=KS拎;A*7N@@n) 2Mt'5B:p6n3޵藴򂊌r7KҗP)#|h:K*#\n0	65 P?-H6FN?.Ȉ[\$AH޺\rP7H4¹92cU\n 69?;+CMpAcXxƌD49Ax^;ׁpH\\7C8^P;؃ ^'bmB7\"V\r!^00#)ʃ-Pʈ6J,#n\n-^L6Pt1x	LH\\\0:8\$⸌ωC%dfJ?)a#z̫z#zb@6[0B07\rm\" çIcC8=!ؘ]iX>&\r+@yrɹ2V3)\rètH9#_K;Hh(P:V/]38	?s 9;4}Va%Q@0̌,sx/NX\$.tB*Qlo2YW\n!9>~\rKMV#92CGc 1}|)0ҁ~2aI W\"	ȃ0>bވC#.2X\"@Sl)&6%C£<R37CP\$B\\gС#aҪC2d2HiUH~,h(^Ŕ.9\0pO<Tg#r^\$D\n1EF(;r2E:˜b4;.	ő-!\"5ÐDc@\nǂ#UZՂV\\+xeK\rbp.3|h0&Køyp\nUdpiE>N0r.\\!3X\nV\\{*VXdPCrAC)K~yq|D\$OYzKe%޾0CIqGd\\ByL)*Y7^a@\$HWB#0M!)4ZJt;DM%1?Cr)2A4j\r\nYA	>HK4|l5YP(A?*VE6/C@ЧރBtp*5u?2ΌHc&!;%CjIs3JDD`ad4BKݩ/T/OQ>[\$l\$q@R8jf\nBd9>PK|w:9%d!AAe\"\$2H-\"\nV`ɻE!SuPIR3&'LE?!5ROT\r\rqFM\\v|;\nY8L[pWи/&@\$1yAІ\"iL|zb B\n49&fxF\nq1=`!EY,l#iI;\n~N͚{ ٱ,&ZZ\\5}Ʉ'#6r\$n	TÃT0Ԓ8u\rPp|y.'yi\",ȹԂF\\v{'q[e\"/<*yIo9'^\$[%!`޵Cs5sAB#8!}'/kYb[	PM3!6[4-i8ОJ>9GĔwt歷jYR^052!K[^ߝ6\$_w\"B6M\$~cy+ع;5י95ހq'7=g[`&h[Y&pԯZ8+и\"a+iih^4Ҙy:\$a#%_L&8Aa P\"6׹V	<<(i)Ř*c&0,	FLjnbi2RL#?}r!{LZC^,<ĵwǴDcfS:jƧ\\|_M\$W\$/~D3`#mL|\"B!BtK*BMi4|lDH;?c\\J%kΌvC(x\"h>Hl	\rXKro/&t\".LBc!e\n\nB.X\r.@ObHиCtH%riĤmt\"]\r[%\\Y+x^~%q@pgBj}Ü\nB~,g&br.OgQNq*qePlK%KѰ\$D.w-\nqіlOC1Bg0XMq\ngjѕбN2v\rQ\n@	`JQo  r!b2#\n.Z+P!23!9#2l\nzİƱ\rQ6c'rg'C\$4@Z\r\"|\"l~8#*F\n7Uf&\"6\r\\)D'lHB9ҤH*X|tkق6B!`DfƬn)t6]eՍ&:.R1K\$sPs0oO2(\0uΞ!P\nNh\r\"3|nC:~5NhӠ85z9|><@#`<z8fM\0AC:k\nf͊De\$O7\$r\r3Q7A\$G%>Hs䛳.6uit33e:\r@A`z?_gjd Ic\$@4qQ1CDDE?24_E4%:'aEBĆJ>Gk,Nz;nDsIMIΙJIWHA<.6;.6RkHP᲍(4LbYM@1FAcNBAN<6M*xi/MH\nx#M;PQj49EQ!g:)?øRJ _03UFL-DMb;\"h2&5eUkU\nxf:0'Gb<; f&+\r%\0Ś;%MZ5R`+d\rV\"a\"*C\n8z;\"nˣL{XRPb(Bp-\$v&+-\np\$Y,̊\n2il\ra5;a\n8bvkx(ȴNTfB/ȶ^.y&Hv8]q!BC-:ZRؠݐ;eAI\nR<ЃP+;sl~+ވk(\r\"{6JlV h6l.[lv!&ԥnnQo`7V9^B#7F=	vTVjd!^DĂg1rFB\rGb-fG[o<Њgok瘹4te_Gϫ%ukUb6C@Z6LBط36d`y\\;CoGN, `w+HG;b6"; break;
		case "pt": $compressed = "%(Q5Ho9jӱ 2Ɠ	A\n3Lf)oihXj\n2H\$RI4* R4K',t2D\0d3\rFqTe6\"P=Gࣱi7#In0 Ը:a:LQc	RM7\r2tI7k&i#-ڟMQHٳ:e9al])#cs+Î,qX̸q9W|=:IE==\n\"&|q'o<qTk7N9%\"#p0(@\rH6z·0H3O@:;\nZ*\n'\0ԙRCj̈P&cȒ錮0\n8\r({c!#pҜ#,9RҜCfHa\03o.<k272 ҄#Lƹ)|6M3|p\"ʰ.sӒS j@|ccƲ/20#;#`:Ӱ#	C XD49Ax^;؁tI\r0\\z2`2&@˚Z9^0HMp)21xՃ|1Sj||i<7'\r%͘+3(Cʠ4Hx:l1pe-@ Տfk2CsS#ӥ(w8CfJ\r{	2Hy{`h	RX&dZH؋ԭrƍB\\&L_hP7r`Ha\"過|~\"#<7rV4b	#l(۱8T\$r!=/N3lpl;ֶ8:^\\?q\\O9(\r,}y1oߊ=nuVֿ+?80f=K\$B̲I\"'\\*bK2!O\0 kȫ3Cg/Ր((`sAT6,@A*4ح<UYY\"Ԣ4]+|H4\0pn.d7X&ehp\$04PۚP԰aa®BHP#8U^*ǋ,9,՞byZ]ڇ\\_;yD1vdKCZU\$:2	HA60H5EKQg8ШIcT70rً a?^P>زHy^Gc X4#	0DN#!:G9a\n ( xm͈(( yD`bKDT6LTLջ\0k9!q0X:dMQwxIѼQDPn\nA-9n.c͒PQ%ATFԌPRP!9\$o3:\\#,(#&DЛ:c]TSK)K!r<QDmMKh(Ln͉ke=r?ToB<)EIYk]3ܐKԴzKIy\n5gPI,7>FH+ڵԄup\rT\0F\n҃Xf9T\0\r16,QAY{g0P*[{ E	YK\$G\"`GFFǄ`L/hpj\r44eCdԛ>,]!0i	)D]ꓺ}DjVR꒻K`C\rZ*98ʧ}:GVנ8\n\\S0L	9AXNPAԢ7t%z>]m9&XX^TQ|A0`!\ni&'mE*\$o#xԜq84c#a;0txCFrD#OW*](mG,bcv9!kK2؋5W<Mg\$5qDC}¹	mrj9SDͻy[ี<b\"5 ,:yOg@@ kekgo?Gt6:Lc_s8;)VGEg!S9(\\h?!,R9O,A2\r:<pf6	KȂ/<t9g^_ҹ>02ϧi\r9+kRH,r'T1wϡNM=̣֤}%t@\$(4QI}n^Cnw]z+,&6 'rx)ANABGEќlc)SJdEYxӾ<Loa'^:XRmU=`_;IG>vu\0ny.h\$դq'rH8a2u#׼k8 X֤&#@dp@h¤./DLv|*C6#\$P(r'>̉o@'LD00N\\x5\0Tmm.\$΢6Nz匲%aDPyCc,͊(0AcP'R_o^-^\0.dL\">29N94;F&dJVd\$>,00b\rF/z9BdW#%Դq\n\nΎoq\$?,=\0f#d\"\rmL_yO1q\r0\0@_=\rPcb)^!Q|m~#DZ\"fo\"lHuNEZf:bmr-mhh2&SB<1w3 #Qah\$0M'\0(N&aTe9#?q\$\$Aq1mk%7!(#%`#Z &!E\"2d9O!(\rcJDc`DBْ62z2m*f22Җ=%-2+\n]5k0QD1%3LS.R	\n\r\r*\rG-8; ̢nj(6\"*`kF;f0\"cR3PӳW7\"n{ < k(\r& D\0[B\"\n0@Efy0j\\\0Zg#bnT	GԺB'jF	 `/jЏ8&ы\\r.C\"/(p#R	buC<\\&hHGF~&XGQ\"C0b`\\\0@l164Dh%|Nl~\\60+#NOF2.tuHI25\r9%:RH+1djG6]w\0AL<,bu\$\"x:m`3y>j@,.\"Q2It,8ddĥ(<E,]?A>mK'2N#+OE\0004MaJ"; break;
		case "pt-br": $compressed = "%(Q5Ho9jӱ 2Ɠ	A\nN\\\n*MqmaOl( 9Hm4\r3x\\4Js!I3@nB3'h5\rQXcach>#Qz4Fi7MjblLQc	NE2PcI>41e!',΢A+O_cfkNC\rZhbL[I9OvqŸnD,\\(ǵGM7k]3c/_4Iț`&U7094ÒN\"7S`: 9A9ȓ@35˄V72k(RRbγ:]\r @j\\9. \0Ф2(#ھ\"҇h(h7#\$/.<H\"|12KPĐ@+ 3B`޿~#* \0ܟD|Ʒ/@@0Đ(<\n\\:(t'\nC X\0D49Ax^;tG'r3_	\"5`J8|N3/Z燁x8`Ƽ(y>Bj :T6*\0@HMzݭ-y	z%\"+t @1*lcx:)Dc1ˡ2/LdC bkt\\n{&#cOb`ӡh(w&0I 6,'ը'&IbhJV<eT^}hb.n1!B&LS>[& މ;mCtq<jO*rc+ sd=]ZH4b#m0Ɉ^r=1Ps	q\\\rXMxe?\0#ͻo@c6x|xl=L|h&L5r|ƁW4Եh0f\r\0c2Jy/%`Ͱs5Ey䁁3U0RqC	7< dM'\nuGQ9*beV&Z+epXa(:\r>=DL)Se=^#Y@~˳)4ƯUTPfeUWj_0rY)єҎC.a̩wJ\n_P<St_\r2kq5\"oASQ\\*V%7G,80h옠d^>%K&Mv%GuӠGc p4#9yNP\"S@\$0((ऌTfdC*Pw'PmXr6J)a'>\n\$4QQ.VHHiR<sC/`IP\nOFZlPL:*a3VHl#DuwI2&伡BO:\$zG'Dy5HFd6扩T6il;D6\$BA\0¢:䕿4B:@\r4r#%>QRSJsgib[qjJ߂b܎\0'q;ywDU1ZEQ1%22Vbܢ	8PT^@- IP6X\n9O31E&+f%+TYg1fLɪ`JAMU%u\r!%&f\\zQJP .;`T<=\";v-<b.A_qAvG\$T;vB>_\$(oR)ePM;'qGJ\nFbMrbPI&7[0A;F5\nZ%&,6?s!*)Z^23NϪKQR'lhF(w,xFaN/Gԭ`Իzr:Ll;uv븼Ά\rdˆS{5%c\rd{+FgQ!|>l&\n\rqRAdޤt_E\"AEюod,*@@ jE?쥔w\0`AbCDM3a=3msE*zTFU_Í\$p\\^ˡsy!4zbTmU\$\rH'b@EMʁK?Cw59gJߦYL,PKsqݴ- K(I#ta|QhCHw{K'DlΌki?u]<ғ`h\$&|_Yc&j`}L\rm9q&jˈTloN.RݰND0;cAgY=}T9~}-R_p5eFk\$5CDmАdM\$`ˠP#[*b̾\0/O/.	n-TAHn%OqNncmRD:J/OQlTǬr`,dpqCD?drpA,ZfN#QPU&H\n<-4@PLLp~Do=0OMr%C&\nM0V^|b>r~Pz6j'Ц5xM/Qkʌq0yJpNcF81/Z?F*bO\"*-rbj-f>U䚖D!a[0OukX9fh'2\\k/ú*d3)p^OC\"Mn\0Ӄ9Q	10)spi9PNP}O}Q1plEH:1݃(fz@&0!LΒnJ#̈6p=\n\r\"pmQ#&*ȂIi \rCf%rg%e\"J.qyBoMBD- Q#rC(RI@A)ґ)Q\$\$(Rvһ)H/c'lϞc2rQ<!چ>̈́51b2D'//\r/Co/35\nk0-f#`Ov\n-bO,2R33R43MTP4 f-N\"3pĩC6-	\r\0#:{(^%&4bp϶^\$^\"0V/%:бja&T:\0a-b2jH\rl~\rVjB\rZ'4:\$}\nq&b2-|POn֖i46!B! \"I\".Ʀ|τj*31 `=(O?#2R\$ǈ\ro27aŨOC`OcbPat^hîT\"#oU;cDN/\"-jMGAQ	0ÜnNv1-:61\r(BNNC,D?Jr)NL\$m=DVo5-HI:BF!FEH0,4Dj4΃&\"X0(>4It3S1N&]L1AKN]BqFc H>Ѐ++d~Ă"; break;
		case "ro": $compressed = "%(uM0r1DcK!2i2a	!;HE4v?!\ra2M'1\0@%9dtˤ!eұ`(`1ƃQp9Φ㡕4\r&sQFsyo9Z&\rن7Fh&2lAH:LFSaVE2lH(n9Lf;̄+,o^NƜ :nN,h2YYYN)Xy3XAK׬eNZ>A#\ry۳qLYN[Q2lBz2B5x#𕈌S\$0!\07J;\"V#.x掭/qp6JҠDR`*	0P.B,Դ?JD229#\nH/q),#x2h2J`¸+#j\$-4.ύ/\0P!03@ΉxÂB*Έ)y\r <9)\n9oBL*;CCe\\H9`@#CC3:xouTAr3P_זXJ\0|6	23/	퇁x91:d#*:E1-7(*c=UOɇ#N&2cbF/XKҊܽ(J2~:9k,H!#H럼hƍ(t3 3j#b;/)X##,c{#;Qy,T?[ƴ3cѳ-	\r#\"1G-^\"`@8Ðo#n\r8cuVP	Ixu*=t&|\$ǝ`O&J|4vSBO@&Cg7ƇLS\0~]yhr8<|O1H	@L:/RڪEgl9Q#Z@(ؐHC0fOʤGhT\rT\\hcWa3بzN04c\0!0RP \$`3O1+\nS30\nqv)@-&V[kuou«\"?k7._u@;`V&&4|ʀpKF)g Av:сfL*%a5\nQ*:zIhVh[in-z\\tyyʧ22Nײ T	\nu`u/'BSi;`b9IWNl0i<cC9ECI}AE-(,o/eA{5wa\0уr*c1`n4yYrPTQL	F K!5B3jhwxQ<#AX`N:\r5,BC&Eky%a) p|0)Y%2lNiԒQ\$DRȓ&5蝡_[1_?	M'Qx1&M\r,Z1NA<<nrB䱫1t*qJ!7lLBO\naR,቙PZIi!TNzE)^`l5+4x	)Gݗp8W>G72RQɻ#A*gSC(kNIÑ)\\\"P!ڧE,c}a~mj8UK>IҞ]dYLpn\0ƔP	X&i.cZ\\rGD0ùxŗ%E4)B,YHiĆ `ˮSPf[K)Hi3beVi&I4\\< (PI/i^l	8<1Ob\n_jlZ0l)2L	Ab|2sHOSwX\$ڜ/%?3vCkF5v3<|XCg5.\r1SM*j8BA_iop1\nb^݁(0:|n;j0o\nq{:ʙViR^\rB(g%26[99\"_/O̎'qT\n!6IJu[!dh4a+H44-襳aqP`m:,u9&[y5AOW\$=d<u#IR5].a-Ff1œ7Zݼ;q;^+HAKHlW)b85zb7Q/GMR٭`\nGneYG/,Y阮J2`)e2f×0?=A5ۣ ̊M&/\0C\0B6oN\$`çzDdF|JKbjL.M0l\\ÉHfgg&L<m/\$rj0|Ob/m\0dCp,LdE.,б0LSK\n̲0`6#Pc{gjWdnMd{gPp\0p\n%L20n4<Mp1lq&.\\8l\nl\$&=0ҨxQ\\?QcЅ1nlp#\rql\nhìL&!L;\nt<qþ*#uZc\\\rc;^\$>%R[\n1^#JBcۍܑ\r3؆1!&/!g\ro4qj cF\r1p:+\"-mJUG\n,*#.R!!1}N3%p'T`q|;`zu(db@䉏j`f9(8|eJm28Ly,RR!rR#ь@-(#2vB=(j\"q!,q\$-b/&|AV4 !r\02\$s)2ъɓ513?2U=374T25I23^b-6D0.r]6cLnmLӁngnݨBUD-LD``9x2s/:ӛ;#i;`:Q\0Dhݤ/.\0Hs{%2uS>g8Q-dy)@Q>7A_BP-rAnR40	({[-3To\\c	?EѦu/H+F4TQ\"ˠR:&J\"\$fNvUi!ìc-GCT{5~NkZcӲ#pL,DWY&G |2j\0\npC\0hKfW0P.;Pn2!B! {*0̬@{bs|C3FʛL#3&oB\"H)4rh- Ȃ^pf\$BFJfN]5Xcdq`PG'Ή=5)Qp\rY[,>QWC[7̻*'h6Ӹ0-X{:՚G1\$r4c+g梣ly9\0hDQpx;Qb>,\$CdLPL*@c\$.:\";!\$nS6p!Ί}ƴul\n^y\\G\r3:e+!O8-P\$c 	\0@	t\n`"; break;
		case "ru": $compressed = "%)h-D\rAhX4mEFxAf@C#mE#i{a2fAZH^GWqh.ahhh)-IhyL%0q )̅9h(HRDLD)Cf4%Gf\nb֬{R\r%m5!s,kPtv_hn]#Pօ'[\$!&chK'FAIE\$e6jllѬ2\"\\횩mKV7ťs6PhNCh@zP<l:\n,c;jA0p9m#)Đ~ZĎc(1^Ӕ07Ϛ8ŪGHE *8C`*c	..80	9\"\\ҫZH8M\"?>jRʴvȚkKLd ģEQc*\$|z2qR*JC<h|5J~͑o\"ء(SςZ9Ԫ#A	 Y*Wzi8(vI>6\r.רOOJ/=N9w#П4# 4(B5'k֢D_ERs/C\rS,[ӅG\0x0@2D49Ax^;pÇb\\7C8^2x0cxDMF*+~%H2~\\x-|%H&RũD	pv,Q׊zֳ9@7&IK.QiP/U(ɑ	0v\nKb=՟-OzhKb?IxNp&RӋ3yRRFrhq̷\$J+d)#;R\$Iz{>uYfj[9漱J(nd<JK\\J\$P+l\rR6`Q[01#Yd+]\"pv؋AL(Zu\\I\"zHDZK')9U[A/J8pLDhMstrm4jSiLK-7g\r3Yv 0I\0IE.^l\$@)yYe:0̃\\f: !9[y9fmp!HgX塀Dh'@)y\n\$*\n&Q=sA3l#BRq0\$&J\$̥wP IU8\n&RE\"cb&\$E;M[#P\0E<2|7[>eLqJݞ:i?\rf\0d7຅qRJ{OЏYdw0xQHP#|r?QǠ1Җ\rLhHwNY1KFj%S.R2VUV&e3U?R*hX|S\raD93EɔyVac*=6j;g6[Ji9-t\r2٭5].^!AZ|39V'9P]3f%4!]]y-D#A|Z	\\ig8gL4Fi1֞C(x@96Û]7ZXL%!|N\"}J Q\">9x}(P٣ h*&h~eB7\rdvF}2:;Laf12cURFKFE;TZ\"4\r<CNq>IQSrɢkq\nKD\0!heeyaL2\$rjmo)}CX(8Ĺ'J)vxWs5ɦǤ5UTDrmH-\\zZOf\\f,s00-hչNn+Ċ#x3+ZC'k.!nc4Rdk|>	XfvIi͏rCM)@O Pg#'7qf OF~IQ\0¤?^fvkQ(́E\"K\$I	s8AtrD`Rߧ>d-~2\\pXɐ-|ٔ؝y})nV!>\$&0T\n\n갭%N'l n14Jfv-@)Wwj.==gk{Ҭry{Iz\rRHG}-ycC;lib8(si3SZuT\nxX8nAZa\0i\$T^O/2I)0qꏾA)ohފLjB(mK&GzVrrJ36|E2\n\0\n\rlDvʜjatI~@\r^\0b\n<kzFeN(LqXŔ?g20+H	AnG쮁(D0>AZ*'EGz\$7O\n/e\\Upr-P\\B^uhJ(PF*6\"G\"GqŪ\nfTϤ/߯D`\rb\nfn \01g'-Q5tEcq\$\$Îs\"\"fpbR-mL J:*pp,NEo~)G\rPTȱj~\$g<Շ \n`>BC-.L[\\\$.o+@ Ghv\\[b wnI%Jo{í!O06*.04GB.>A&\rcN`PD*B'Y|#a(;(\$2(sdtGH)*##٦6fLJթus,R)--r(Bf|#a--\0l#/s!(j2ǈ)0ID*D 9RV2^#\$B8RC\"sB\"3F!9#SN&=R;/4+8)5'5Re#6LNI\"dG6{3r)\"\$_	-餲xzī#~ǲY&\0L.Cq:\"#5D9f/JB*Ҁ\nzqķ>\"H,+>1+}\":504.E(Lyps=T'02tǔC4 mT8*}18t0\"Fq?4x%yF5FSBM;\rP|؂WЯЎO#\"3!7*bICTXIROE%eeӈp)vxELc,!BwELLLQA+\rB^Ot(h\\KOPt:.h(C0/RL=T?+POQqR'mD?\"J@#Ss!V(qn\"c/P=+KjCV02WH]+WOWaX3*HEu;XY5q-05y5Zұ@c|)UU=1%G3'ЍѫTD)o;^O>QXЧ_\r[_2^`'Rt@e0ģ.P.AD̄g򎋜`VBǶS5HBȻ_- \"˝S\"f\\ )O2ŚSzMV:aU`-\r&V\$EeE3eFGϣ׭tLq;[2MlqAa}HlQ	lߕb/Kb3n@lM2sn(\np\rYvЋquKp_o4Qodm3M1,ro-spobx&a\rub:uQ;A&QdL}(\$ZGBUpwCZ5@vSD7B\$7xtKYu%s4Usw%3BS;44znЌwCs{\"}V}D||o\"I#4@WAV)֓\$f<a@1}4T.J\nď(݁\rbs=xuX1|QH%*/R#}C8^cg:tFqT{R?#WMWe{d\rfjj@@j\0@\r|'8x\0F%_\rNqn`' 'xŏuBu(v-1Y8S`-E2	#q'|qїtBP͒VP3\nt ]33obk	#HYovPOk{\\-K2QC=wyo30*xPYOt7q&Zrkvw)2Yʟ2Y|=xjE&oc9\", >q\\6\rB'GM[Z)[	!r#Մ}ri\rVS%q%X\"lbOGP,Nʟd`RPc,>6i>5b\n qsh%Ba\$<V9\\ZqZ	3M\nC#g=zٹݤb%3=gFIְᰰr78MyԚIQwWV\"G5'&d\$gV@Ay34W\$oq({<pBGԇD}v4[\n\$SfdOAbsLOC\"eK%(D[ho)rO74 [+p0]؛`mÎhd	}*%gOt/*o1dF,-:ЀEC3Gh;ZpH[\r5j\\\$\rн9]Oа걛ƭyY>u\rd,'Wy8d88XoIj>d [IET0ˡ"; break;
		case "sk": $compressed = "%(](!@n2\rC	l7&P\rl25q\$\"r:\rFQ\0B0y%990cAn8yj)AB&sLR\nbM&}a1f̄k01QZ0_bԷ  _0qN:Q\rA n4%b	a6OR5#7\n\n*8	!\"Fo;GA#v8.D8ܞ1*͗ə\n-L6la+y5O&(3:=.ϐ@1xȂ\$2\"J\r(\$\"<jhBz=	1\rH֢jJ|)J	F<\"%\n<9\n\n)1P),`2h:3. -\nn9fR<ʣ3\r4B@P7[0\$Be\n;\"@ؔnC\nEXEQR# ں*lRVR\"(C,Q\n`@!3:xc>Ar30;c ^(0a3!ljx\$\"UdC{,:BKMBx\r	SIx'&h5*-C\0ĳN9dEˠ!t8!5B\\8:A\0LqDDr`B7x107\rm /o\n_/5*r.5##\$IҤ8.bL4\\bB櫮YЉc*`5M S<r,&.Z-t=`4t	8aw2'VTp]Ķ͂{HP\$\"3n wxwvn0ۺ\n)5sX9#u5ź!p̹F̣\$\"ZJ#{f2ty7Dhl=SMADd?YIXmM.2D1\"'E0(TiCG؞RhI`o\ra9\0@FT'jh2dB\n~#(QXg\0V+ah8jR.C|EAM\rCJM16'\"4Dwb@Dl\"FhRGhx(eQDf,!!<FDr\$E`c&)ՖCWQ|X!e*zdV>R͘4bkC\0­vDgS!ZNǬ:4vtQ&BáEwPr=HX+\rbpTYh- ^Ɍ\rmn@L	R-PJ20M.FIZ\\Rac:\"{нCGLĄ 25*b\$f 3S\$VC0t	Ze~-Ni:>HK̏22\n tBrT#e\"(Td-!z)\r<Xp@\n\n0)C\$X8\"Ա&:rBL*Lhf0ժ]MG&63Jq9DBih2*:q&dZaIΙ8AUfa\0FQ rvpMB|9\$Fz	!&%#;Fաy_Ed=k/d܂Gq1l\$&NYv|ɟd{!B\"b.)S\0P	L*/#@\rǖbk C7PQS0wۚ\nOtRdP\0SrD<0(KfRCz,(!*I3'DfCvNnB.jɴQ5يjTEJu͌r[aQe1\$i44t	6f(xOl92^gh:ThwDKu^;LCA)(j(Bҳwjևr5NAT%!L̰ĘB)FO&\"2uTu^	\\q';L\0%Ժ\nem\rG78<u\n;l&vA8MFhLl=0`ȝ{_`S}(xMh^S\"\r@(!n1gOrG9KcsG!tpÔsFmgi(\"i\"nX5R2H*#\$U#G8\$.](i16\nP 0Cs!f<b9cRZ^/*̌@ď0ΓܴV.6If.Gv mԖ=qr{Mjm&'{Uɑ\nHa\r\r\"#|t*/PVj(j錵70>q6_J#AYjO\0n ]\0lFS#4eb#'_\"ðᢈP*R#(vRI@ f!5`CJNh#H1F\"D^x\r/&@MFrd]X:Kbr&!`,C\"^&cdCP{p;⭰#zψ!dKiBB^0q\$>yǞ'!LhȻJDpLM.\$`- ,ʅ\rqD'nOig^ml\nNRaPMPmtA=Leը|`0e]'5PѵOkni1+gQzƌlD4_,.l%/wQc2Rl! rIB;!%r\"֪r(/|΂D\$,T%&RRl9]'c \rF Ljo)+de!'/U l%f;)}+jF^Ϫ{тxl挬\\)c毤l/*JJFBA%i'΄jO Qbpn^2fdNDeg 2],\r&.A3R(M3(v2M:p36Wqysbby\r}(bP0Ш'6IL7>š/ct?&l&d	b<\$n@Ud~.@0o2.5]	<23#p'Sz1!suxƦpf>RCs#Mz>k??r?:p+\rbjEjK53l\"64H!=CcCSi\"PtRv4ABT	8wBB6fUR<=dÑAJyfB_=Cq=sj\$47?DRԔs7H̘pK%C'30R!C7sGKaT8mD Gd53DODgK<NOdO]=tOa}(4C#C/<M\nNmL18U;r\nc%_TB\$T\rE\0u\\I1.dd\rrN T3WIclψ##x*@dKc\r 5F5ka'+@7BLIGG>FP+*RֵLf@\nZ6q!\0_E/\05T0\\G_\0RHc!D\"b*j?D&\n6M7th2B6eLJidҀ0òER#y0\"9^)&ABizA[j^=&\"K2-j(ec7G~-kpі8gl#}\"(i/tiv 4pl>N5b/!n9.:l\0 2hq;4b߂q6vqr)ZJDS,7@\nOEp\n6:-鴕08%`0lG}gF]ֺ-B,ӰV^U)wcfަ=E44mhc#\$ \""; break;
		case "sl": $compressed = "%(eM#)@n0\rUi'Cyk2QF\"	1k7Αv?5B25fA2dB\0Pb2ar\n)Ǆep(0#Upz7PI6AC	laCH(H;_Iуdi1ȋ&aCl2̧1p@u8FGCA9t1f\$E3A}kB|<6?&Ʒ_7K08ʱDы*PIFSԼU8Bҩi;L#.}Np!7c2\$B9#hXϿ2:V7(@	T<ˌ R~:sj Kx9,@P\"Ȏ2h:IDr<C\rk86\r2<+1|\rn%\r2c'T~9QJT\rH)52H2\r{>Ki1l7V>/@;CA+p9Xx(C@:tT6ʈ9X{90^'a926cH7x\$NʫpBޡ\"3 ĵ\nj#!6mB	|5qO+ĒM_R_\0ӄ`#`2`P'B\\.,ɼ)2P2\nsS6&yH:6s־ë1S2CnPi%4+n\r8&A(R\"\re9(*FXy630k7&-c\"n4spb1Wr B\nSho<;|:+;\rF|HŮk\\}\r.n'hHzEpdid93-8ÑSM'LBz}h<#tcm[O Va1lln2\\\$Bx5,7v <0fe缂#(Hڼ*S*[bg` c1XAϨuqAaC	َ!}P5JJUX+%hy\"]c}*yrJl)@Y-!ش\"	ȏBEnT	@b!TJS*Tbx9+5j߈\n!#H\$B4ޒ\"a DkՃE8rTg`!(Faє?\n hHC;+aH@18HF0SDH1FBa\nCHhB]yIب 0\r+Nf\\̆Y^YO)?%G*HXXECND!8A*1\$S1:kdpFX,B%/F pt\$v8i\"HZx8	'І2('T@^,\"N#XY<3m\rTQBYǴByFHc\"J:f0%hD4XP	L*FQح5P7\"+BD澚XrM6T1@܂` x'IaB\$2-nmDX1cvFYVʓ	^Z!K*:<^	B	]\r̳kqEMuDҹ1\"̍RM\rI9Nǐ3<`LT1ȦrD=2v@tU,!#vFP7nj_Y݉=uc&%vBsΒv`G\$L{`o68ABxs\$djlXTtF`:<L!LB<BѐH΅S jČDm0eVS.R:ul{!\ro͐j]Mr_HgM &iw!fjt/RK_,{VlKVK9zY:I\nF^]\nKmhiy1*4	%,'`C	\03s3Pbu\$oc0E0\0002-7_z'zޑI=U\\O\\JޓnCyƫ?RЮ1qxZ[~#C\rsY/5Kr8B8w602r-0\rWgۄȜyyԹ&E.xyq<.i`}ê[^ku2j;V}\$ 'vIHfB3=M^Fݏ@M_[XSrH\\G	dIy`nL;1tMn (ɑ\r϶FpKl\"Ho\r%z8xnCH[ñp{1ÀbnU\0Jsne̅X2+C\n=yBZ\\FB%ŢАK,\$ږ\n	-gn,H<IILF;L̈\0L.L\\Db\0l1	0!	\0Xr	}0XuPpԉB\\%9(8b`Nm!q#	fh1n\0\rpNbbANWnQN߈<-Fan7r%\r~KNw\r0G06x71*b&&b<Chў0(ID\"\"5І9l2KbFxCj(%Q̔ 2hf=d HH!%p&E,`o<7\\Q!;\"#qt\"k#-\00020'LBM\"Q#I%S'm6,%m%#LBJK,dc/Vbg&i'\rc\rf{%-20#Ŭ[#k#pgdpNK'=\$\rfy2xZ?B5]0̀\$mO\nS/ i#1!1+1Đ~)0\"bC	ү#9\rp,4>Iff&@LbR18.ɶ\"(ݳS6W6ʛi6@fhmbB^@#-`\$r_:9104;'<d>M&Dۓ):N/\$;;3>\r#,&6B+i/|4=~,B7EC{BlZ3q3B\"18\$PX@)7( \$oTR2Hnk,5ET~.CCt4|䞄\rV3nIxDDʤ!\0\rT6\npz4//'1\0TzN0WfQP\\'Qj\n_cX\$¾@.M@4/5j+Tl`\$9*b-P3`hj줁>Vn:& 6L6E .A٭ͱJ \$4mu]l\n!Zn&RΉ[0323p91@\"e[J2\\hBdZpaRu8\rb`v@Zf_K\"9ZD2ŋ)lP`E	]SF/ȹ\"cfZ0\"Y24t%mh\0p/ercUJ@2=0L\"d4#Oz`L\$"; break;
		case "sr": $compressed = "%)h.i4	|Ez\\4S֊\rh/PHPnv0G h\r\n)EȄ:%9>/M}H`(`1ƃQp9WhtuO`J\re;ьF\rgKB`X42]nG<^PdeCRFtɼ 4NQ 8'cIg2N9d08CAt0ոD1%Co-'3Do8eAZA)@{b0*;p&\0\r#p΃4\rY] s(>X7\rn07(9\r\\\";/9 踣x:Äk!;ƣ\"N\"\\:C*	zE<E-¶-н\"#JҐ+d*{^@51DK0jF9AhuPڬXD**L@2^@-8R6U45z'QƎT8ЧVG3RD=Oi1 l+Hc#1#*3ݷ,r1Gn 40T9`@`@2D49Ax^;ppqj3(#&\r\nѨ#x|2\rGYA,¯*77ҰeýM:	+YJ\"oVˡMƦۢZS:'Oږx0Cu悄\"\"ѽHiѻM=5(T2_Mz01 *jSO1a=b&0d;#`26fH#IKCDj?3N%Ѱib\"ˢpOI2k+Cf\$L4#L[/_bE-Fu2٣פ6D*zk0Z&JS/k*n\nTFyY9j))I\$/L*/#O ПhM`k!pt^ck&jǶvԂ B ЉB-S]:E`el,:VڻQlA<tKs\r`eD+C7;ǄG\0@Qn&GC\rɛG~\0\nsfq<D\n0A\0uawA\0l\r\n6!0nh0RV(%F&O.\n&b\nBpKd\r!r0kb,Mv2L}5d`NhY[ycE5HlKeW\$Brji?kJ\$vv%dXZN`&p0 Ęcݍ1܌sd,@H)	<XI\rD#\\vHC[/IOSVidSiDblZ\niuh!(4E<Y _a6\0q2F\$dҞʩX:u|!emZC.jz_FL,	@\$dR\0e99in̒xc3NsNxenI9Cw\n;\"L\n2gRs!\rZdFL0\0W q.&`m;\"lgb@#WI F)bkb!\0\$xFE _*Н#M	\n6M=k#u@(4!*J\$(uW\$& ӤË\03#@5)r伈1ʤy`BI:UJTGP	L*Yc1JJ;5y,q&-(&DԂn/	cE\"Q#\r2`,p%t`uS^3\0e1`innA2vCG\r*q&]<(^H܄P*[WB^4M\nm0*Lg! +c\0B`Eٻ>Tdmn\nXdba4\r!N-Ւ5aY\n`k-w:TBc/f]nxflFJAjLfsMǋrmNo,=7<OgO˄\r-Mΰ^Gubn W4OΑhO\n\rIp2ZlOV-CJ!:\"\09Q'{zl32g0\$dr0s{!S1HEN2S7#CЪ(Z8o&QC#!%|F)fRWmG	>ᠨvI8\nu1\n_5i/!IJ8OT7F\$s/,0TTJ\" @rxϰwbKPSZlm\n`Dm];Dzdn&@fj60vfʂ0dJ*LBh.Whb`j(?[\0\"+\nc\r		20	/Q\ncK0)P,RF0\r1)K+2.XD-ïh*ȟ]\n+hK0W-neq9	q=	E1.m3\0QWT%GLXh!\n',%Wtk0%?MXhVqZ*M#b#AbV5C|N.\"P,xexN@x0QGL<C'ox̀>f1	lk27n#\nq AC>DމV7,|tj2F&ŬI'HN0]&|{CD (U)gAA\$4%`xwޢmK\0h1rho\"/\$c^Βx%,.g(YPTW*h hXeQ&Rr')+U\$ܮE,LL^6D!(R'Y52sM(!0Z+oP9'16Ȇd|F6A82'cP?U(.'ĢK6F\$R>q%FЯ3,bƣ;0<Gl<1_==S*>0=3s2^w6o)2U3)yA{:QBBW)wzH\0Q/\n(\rm,Nfx\r>R!>p`n4}>,TbrM4H,TO=g.\"Hqyk7**ET|t'It*x Ϥ\$4X{'/G#\$=gpg.kC6[DO8nkB4-|dD3h4-Q2?PR>5/7%Rb)UDSPu*yƇ\rh#sSL6D@\$\rʼ[X#'i\02tS5\0Y?CSAZT=rQt(R\"uez7	6ua]ΉDo5֓lN/#Sq^3՞V\r%^OaBwYAU&>cd_QcbhpR@T]HO`ХJrUOGkdE-jofpt*0gghnʏB\\'<\\iP-jbUP)vdӗQR\$\0vOmD k\n+YSDV1buk2HK¶DO īpdGl5akTZ5>;BtƖUsU_N\$2oaBPwthݓ4hK]tWS?XPB=_iuF[%w7~LuK>\r\n&@<n`\rVV\0A10D̖F\0\np)-P10\r7WutmAB\".0fY(և	\06CSpMK/q\"1|R+&b0`#Q+9*u}BX),(~4+)GuoYQ+)RQ(.B.sD282yd2Kk8'C;42%'XuUNW{x{yM#bJa++@tҺ{\${n4SK\$T1@-4f,%p ]st=rĉAJNȃvzxfF';%\rz#ΫS0W`ր9C:L\\YX+ՉHx"; break;
		case "sv": $compressed = "%(e:5)@i7	 6ELp&)\\\n\$0s8t!CtrZo9I\rb%9iC7,X\nFC1l7AL4\$8uOMfSt7ASI a6&<b2\$)9Hd7#qu]D(ND0(r4\$U0!1n%(Ɖ:]xId3O\r3Dpt9tQN!ݾr#-+/5&d~hI':4Td5gb(ī7'\"N+<c7\"#̋죦E#μj(\n\$Crů\nL	è63C7M@=9<˫!\"\rh8C*҄3	#c<H<*)C&p&?,5ñH(,lD(4\r̫2\r:/I8LD9]!>JU\r?\0\0x\r0CCD8a^(\\#szJ;!xDlSr`7^0z6\rMK\nHFc:¸򭐖.\"p/-7`Aw\"H(7в&WO8]B\r6rvF :R\\c\$95Ve5B07ZcM#8.)O\nU+.dv)3X2o0<d(腧F4E6c\\E9BkL,mn[Tʂ\0 M4m%9t3I#8	#hᘹ\"(Yz!9[,Z6m,05A(6`\$C=20\"f:FQMv}ϐDKre '3@RN&M\r\0چ.Y\rǩ5J\\gXÓ['p;t6Rp	υWr}O\$C}ւKCX\rh uvC~P)=AA4\$,lVgM		RLU6TTj;xhUEVZ(bl*c d-?ޚPB\$ :`#:b\"\"5#a\"q#Tʢ褫)1[+	\"()1ؼRAo@>G\rQ	I7fi\r+ 93;R77/RviA\re(aJƝE ȫȉ~T9B9'.mzȌIAƤ\0l-82}d)@kH.0sơ3<Q\r0˙6LqBD0\"AГˋl\rѽM?7	u҃#!*ca#,d% nQ\$QFL)C\naH#OX0 qahzP!^9Xšdʍ!x&)t'+51*TBPՒOK\\&l+RT9aD3U&tPAOMI`>HL(eDa@'0Tb3 <*HQX-s1}G Rd=?`I\nmX \rIP(\"Ӝ%=\"Y\$3	N@2v1dő;`\0U\n@\"BI	f^p@xR\nP p|_(x0PD[R,.vL?cGL1hp˾\"FF#|9dB񕦖おRƭ'g[= @9h2w\$1+<X/gtXdH4gmʹu/&n))|ҿWfay:L݄aq%?RKxnCy΢0meqs5ACPD4Xd\\V0xb<Cv\r,YgGvFp,ɆT9jAhd֛ei	hG*E|18W()(hj*>\"-8^)a!gҟwhS\ny#xTDȸ/\0őUEd=|~b1\rPs~k˄`s0XN34K0cxi	Jz߭[!`\nzrPIYlᄔ9p~m#1Fi-CY6̮亩X%Rq5529qqY=yJql0emÎC4r\\qJЖ\"wY8rijcPӍ'\r3\$|3zhoy=1F'0F8!%\0&L3n-^%yylv=縌lYS	4<MIЃ9\"\\.>(W>\\w1?y~ߢ{#ߛ7\$@:R	nۢ`jKH4LG~0vi(,N\"j\"mf}mƏc0HN,oDϔNMig~`ZibA	p?\$lBIؼ\$L.9@=ÌaJ*X\\'svԭN5#Ռ5ϒ%T\\oF[MBo1N\0N1/\"0#\rENРPv\\?,\r\"5Ji4\"mX1cqO%g\0001,[Tf8d:5.¾;#cf:X6nQbZhĪAjJIlDJBPK0i~lLigB\n\nj\$(.Ep^`Ql#\$nT2O!.O\"r\"#\"mx\"HN&\\3~< @؃1m%jRc\0t2m%\"rR\nr}'\r&\r\n=\$dr̚oyKR!Ҫ3BRNH[(@>c\$=x;\"b&dH*Dld/@Nf-\\%1^0eY+W,Ns0-*2U\"f\rV\rd\nO^\"kH/*Ǐ	\"&> p4\"O.ƀҺ6z򳊢c1x\0rq/(\08fR<A;%(^8o!DP+Ŗ	R0ij%Q;D@\\v3<^s.P@k\$15ǎƇď@B \n!9&&2g\$*&B43[De0wJBpnϠD&fK	͸'6=(8dMH<bJd/í<7>P\\AAl@,q@odrG1AF\0U@"; break;
		case "ta": $compressed = "%)J::u>8@#\"\0p6&ALQ\\!_ FKh3Xҽ.B!Pt9_`\$RTmq?5MN%urι@WDS\n4;(pP0cAn8U_\\dj?&JGFMSIXrJ΢_'JuC^ʽp i4=xS/Q*Adu'c(oFe3NbNd0;CAt0l,WKɨNCR,H\0k7S*RޢjMY`,#esrʢ\rBB4;2)(|\nD@\0P7\rn7(9\r㒐\">/9 ;x\$9X;̣#wI@k6G\"I uW(R0,d\rØ7j*+]!1%n,Lk\n.uHY3V7drڱĪ\\)Kz0\\W+q1ezwv櫖J)ӮdBH=Ͷ\nZ̫kF87-8l2=u@)uLWbDh:a	;@@<orR\nh)R_9dMtFa@6f\nMilƪl\"֫\n@ۓa۷J*4I+qj8Jښ#A5kEy# \"LA8;:\01oU=\"t1Mn4039`@q@2D49Ax^;rW]2x7=ϡxDlȃ464}?MV=*bZv±SXUU+c0_Y䀫F@2x2y.lP*V-	VdҐW!7\$@AA(dE͍v#޸zi4W[U5\"jW`=sͅKDav!:WƵ\"8/8WunaT4̉` p;TAAe+`:i-qlRH'oƵ\r]t]R}KU\$x{l9b\0a<1ܐBL%z3YHFF\"HsP2ƞB*GP+\"N~ZCꗏ_FygF:c\0rq	GdL+Y03|.cCW)xuHt+402VP(	&;Hi3!ѡtAO-_RMו\\,A!4Ltŝ3V#zHL\\xavL:XݓA<Rj3O-FŬJϴ;e2V d2V\0s7pb\nco66vfD@@xn (6ރ4?3@v5ZAwHO(`',[rn(Тm6ҵi	H\ni.tc{Di\r!ܫs.mιBbfu&yޤZ\$f,3b4ץ\0ٕE@-\$+54UUW!\nyH	\\\\ba44Nn{ra9<mWeӇ'RUfS;JJ\rJh w'bq\"ekqeVy&\n;Bzsva\"A{o!K쌘YCٗ a` 1ldbj3%j[\n+*gDW(bCz:F.Kq,iQ\na8{<X\$ti.þ?5r(2>\rG5961;yo Gpq. }ChU9f3\$7 !0qU9@xf\$k/U).Qʿ,Fh\$p{	WNI&yAq5Kȝˎŧ:Ԫ{Fxi{񲍤+9Z*N\\錒)*_6Z}-QZ<eHy:=nL8Zp m\r)M8n<F!UH%O\naQV8ՇAJJ4lgg~ȋe~k	hRhE\r6<;Few:Diru0rRJb)ASdq*i=M;q9\0Asr(*ŧz%ţSUWO	*\0BE_@/HB=a<nRا(vpbabꀍ\nFLM\"f摥p_/ҧ|knxGD*X/^[C>!Za.)jRjnJ^0:ㄘhj	fȮpx潃y̢biHtc	%o4<wԌlG\0iқ\n<hd)\nb(Z+K޵/g m\0?CdVj\$\ny<?\nz`鍖 t#L2*wʨ`fɹ%b@%K舲:Î9NV(\"J\nG(r`\rHp'p ,	 \r ʖ4k@Ίmm)P6\0jV\ny,J\rȄ\$\nm4@´l\n(\$<)qP7i5gL-\$\r˘[\"P.Щ\"m6dI')\"i%2N)0Z\"70J^`ˀ\rb\ng&>#rV1 7'/@Dj\nRf`3(bc7+g\nD)!AFm\r%frv>&HQ9)Ҽ1(b	rQ`P,am\0PDj:F#2bO\n`L@N\n\rp>Dh:O1*\"B2\$la\$!%00I0\0]0O1\$nе.L'RtG|жz- \\-_99E:S:~d<92НӣS9pURB\\2=r2Sn,3-5\r-oBs>H43Cs`I\"DT@.OE,gD3nQ#	p@L\$ViJ(&u#IRE\nOz}`\$%`Rxo%,G)\\bߴ}\$#7\r)k\$S\"U6`3dZ1UMn\\bv|`\\M-L'R\nS=KnfU\$|Sb;iH\"`Pc΁fH&Ady\$79\0+jrKGIK@dIOtN.QSMuBu7s&RXFDԨiUsH4vxռtu;E5cE~VE__TU_w:\0Tu>@TwU9OPVrЉ&/(x|XuG\nFLઃ0AR+U|tv6e0u848*v6l\r+_I]?\0,'h䰩\$Sb[v;ij.5m#26\0,2LGt3t=q0\n]՟buo,vi62,?lpu<W꒣DZYpV_yopqs&s`'tw-4ab5`acTY(#vbW+ZWvԚ5<Tb(	/cbl*N!zp5TmN78>u?3kN6OU<#{\nYwk,EtY=82]}_?T+KrT*lt|er`&Q+vrXb贯J17&MpaJsӌ8+vիw3yTЪ])7v&aO6##4^5rQs\$V5_p\\ͪMl-\"+I֤	]TT&b_)uB!\\3Piы\r=I?)lSxs}5@jc3V \"xkAA_z5QYMS/yn؁+Rw+G]'!\r!Nk[z6\r|]oՕpcx[)!7ۙUsgySly{yJCbjAR\$&P\r5krudC0y՝e\$LXwI\$P9h9#\\92ل;+K%ʔd:3V7VupFՑ\\d-I㏦5NIiEmVU].xyuq/ZuW#8Q)?tYpOO=G3y2SYJZA:IDk!ZsRMk:Ԑl{^|?8WfGZŰ%u	39Y;;3\$[K#AjIX%\$cKn85xm;yn\0ˤ٭9:ϛ\$UV{;J-m[s( (cMmuff5;=lɼR\r{ý˼5 )@)S~M:yV3Gk|GҫM[q{;)<-hE;]yXNěl}[7__VyADdk a\\=^*Jѡ+1\$ka1sPl\\B	ˀ):2z5x`%0YiEEGWx8Vq}3eqQ/\nZ|)NBB7aQ\\pTD#g(,(9:	=HsDmuN[.u~0[[Ųh97W֒ޅ\rV Ӽq`@phJnNlۄ\nZ\n=HEkgɽH\\^|>]s;/7סZ9Gjܥ״]zcʚ\\-\r0UxC&	Zwg1\rSɥpwOWRUO|{f\$x{w&T[|;E)rlrf?qH܉nI~sL~m-Ch{O;]\nJ\\(QV}5\r6{¹S#,SKU\"x̤|>a|e=c;L\04 %^~?qBIQ/l>\"𜌯)(dH،5ϰ7PDvج5jFM @&W׸Q`ZwCUZɉJ	Hmqf=)\0e\\XN[' fHd*H#zȗY)uB&!΢&s`\$oKx06Ɂ\r)hR50%z	j&r!j]·\$\rh\0&4@S"; break;
		case "th": $compressed = "%OZAS0U/Z\$CDAUPȴqp*\n*\nW	lM1\"T!R4\\K3umpPUq\\-c8UR\n%bh9\\EY*uq2[S\ny8\\E1BH#'\0Pb2as=UW	8{#+&\\K#[[=-O5,%&ݶ\\&TJ}'[AC\\k%'TLWȽg+!'MbC ɼ 4NQ 8'cI3@:>2#:\rL:#-ڀ EM˘a9~NsL^\\.-R\\\"CCEÚΩMR:()E<)CH3srR7!pbLB5ø7I#|@9ÄC;\$(θ(34#mSAJsت,pA\0b)>֪m/:\$JR\n;~&uU*9l\\S,?#NDN\\MGR\\Ə6\nH#\nj&4ŵ̝{8R!*L1	pNY52-SR<+/օ\\f)i_H.!؜ϊ8؅P'VŶeJ)7z)zx4/cWzF7ȢR2\rP4CQ9P1#3>S;0cݶu 9`@n@2D49Ax^;pðlS3(ѣH\r-#x|:RJ3+|ΩɃX\\铎TKS{a2I7=nzfLÑnW쳺\\;`P0Cv9A(A\r!씂#ȗF1H%	#ʜ#.IGgu/4∅*d!hO	݄ GClP0W]>[iLd&h4bh#2IN9ZdAҬ0WEᄣv|(MJ~D׼P(gtJX')2@\nI(#C؀ЉP0 A`ND0ߍmDGoedd[k__;f^S\"Q\"@evd.%aJ\\5`x-	<YxBB\"tQS;Ch}*\$^hBsa1tt:5[Ta.Cv4yh >X@-,\$II:3D(oGdan,cbBUSC*py֓VC417t0tU(0pEhih:PM;t)BP~/\$ŚR)'PCKnIl5t4Fq=ȹ7*˛9B\rт }h][\$)H FtW>+pRr?MjѪI)s7it:\\Zp&8 䜣s9烓tN:*Oi]XI\rD(IC[Q֤_u]Ya0<̩䭮v\\Phݰ oC#0ksptFCJ@xT9پҒv)Cm*jȱ#\\+<eTvd@@P\0`V%N2yeQ(Pb0s@g+ma po\nL~l;\$HƠlzuu	5(sA7\n]3J܀ H]n8=cJ\0C\naH#@\rj[@3џKrZ.0\\LNINF [я@[,[\nBKBd69#; -ja\$MDEA\$d,nPÌ73'pao3ci1D@<DeD;\$f=\nE-\r4\n2\$K(\0ުdAkPrDtsq'恪nY/3 ?#L#_ij.(nzG<a2E~{ ŉU&p \n@\"|?&_YA<@/Qu</xތ#6hYl\"W~ɓxdd'vlCYg-?047ZZl#j^)mZ<lO'ߗfRz̏Fh&6/|e5)jEjiTD@MGa2NawD\"~(XI4h*ȴc'@\r6#^RWƜ͞\0FD/\n`HpK欨pKx@昉͞JMGM*+A\$Ǌs?POȾ\"\")`|IF^dCO\rv-n_DdH|讏tdNx0ͩr`qp.\rbz+3V&2,@c\0dT\"JHog,xhPZ*1xb:ͺirZHeeEE4 	\0@0Pg>QmΔڎ7mDHp;eDd&QڒgV.g:()\0>9OFN /4a1dDeaJ';\"\"BlZd)\$JF	o@NfC'E6[5D4.~/!b&WvB~9c4NdN	i,D*D{#\$yR+H;hd\$pG*H7I'(00''c^dOE#%z7ض)6I)FO6&Ԍ&G4XJ2k5zX}KhѣVST87U3xU3nS:bBd'48-0p87:BO0\$g;#d]Q2.xg\0NX@~'+Hc.'甍̑jaG1<C6\0gW)B/Ue@XӼZ=0\r @==As=orZяB;.;2-e/gqD\$(et٥~sDQ\nTz|KԕHOI}lp*o5e7\0\nLSI4LnԏctM> DlQVƀA8#sN21!\0003MKd&jZBb%FGHOtBYPCCR̛/]m<J;Uc0%<lDNnF	:Y-IXPTKyT{:d_31CCncSFI:IW\0HYéENO*EJudT7Vu_X3W3F8D<{OZ4H4UU\rNKUr5{'RT4cE5-uyHIRu\rSe72dCW\\\0h)+Ntc6-KDagd2udv}vq^/h1V+gR(8{U1Vc)ֳ0T:p^Mbgdp^\0e(/Ή֮VS\nFlS%ӏWN,VCopFe69TVJ[N	\n\rvw]sc\rkSssrc?tS6& +X/\\4󰢵sd\neWr5?iWtM1q5tW5Vzsj73hTJ\${{֘_|lcsyqeWޏ{}i;a~!UM,i1bR4Fbq>>@tF>@RQ,1(x@׃%r\0F\n0@_yġfUjhuX|{W+DXuR9Wr~A_o8hN'eȏ֏o6C'Q8|71e\\5Ed FY)	\raȴͺ\"`ļTyQrR*{׃i6M/dpa=,CpqLW4C#EJ95R2\"*~Wѥj\rVӃor_/4E]x!xj*\nZ\n:n\0KrQ!Q-%Koi.NLE`3&L@\r\0EB<;éY|L7{hOU9uA63ay|t*E'ZGZBYٮK~&mg܉x35g}<D,W[\rY\\-L'hN:-H=OKU6ofדj\$>#\rfLi:fFF5bOe=We\\u\0%H0)^V<ݜ{?YVP3\r@meh7}TgafCť\\cBX3>/04b^NթfFC<Oߨ:E@\rAW{ZrgĽ{S#T3<4B\0	\0t	@\n`"; break;
		case "tr": $compressed = "%(o9L\";\rln2NFai<BS`z4hP\"2B!Bu:`Ehr2r	LcAb'\0(`1ƃQp9b(Bi=R*|4&`(a1\rɮ|^ZɮK0fK\n!Lx7̦ȭ 4k|\"tit3-z7eLlDa63ڜI7FӺAE=FqH7PuMn7Q#j|aŘ'=sx03=g3hȎ'a\0=;Ch6)j2;I`҉\0A	j%H\\:\$&0@A#H :#\04B\n(ޡS\n;IƜB9Ãk:ê!0XB7\0P{GxҲ	;4=	 \$σ>#\$9p!pc:\rzT#9`@!c@2D49Ax^;Ձp:(\\zNүI|6MjB3#Qx4!^0Ɖ+4#Dym(\"Q92ڼ(*5<O31,U7BL!PTL\"XM-@08+tj#C'4PH{DmnvN֯#-L,b43%8Ô)fd &\r(P&&:X,Pܽ+@Ÿqx9b	Gh0c[Y=B-xQO\"x];纄\0x>ӫ)b,.#cn z,Jsrbp8!n[^=660b9\$Pl6USH|>vIc[\0Myꌣ87+h\ra&6>qp}	|;a \"y^.YbpNWr#ɡ#A3PPy]hGvzM,M3Fq8Q\"\$L)@TT*XrmZ+`KX^\rŉea^*B8nCJ=Tl/3!9IMQ-t.^)AObEE⟁Q;@`bF')jg\"4HT*RuRjJ9+Un^݊9Vs|wJ	\$F)\"+qs>g\$l&8Eсm6s.`g\$NN'бD,7)k1r>f<\"S\$Py\n7 g]H1V Ƞ&>A;1u)]-P//+?1hJc),pOR.Ffj=rAȯ)5tٛa@DDn\\u٠cEd\\3BXkj 2u<i`) DQ8!K2P ԀzdM	-zO@}#Qt@#}\$HJPLAA11\nP *9zCH|C(G<<Ìh2ca~/1^P	L*&W5#s9@&@D8<2DIČhLpG0T5@z/-)P,U9&l3~}EAӈЭ6>\"	_m9X:I	8PT@\$A90`\".fQ +\r.\"Ę3b.\"NF\\f˓v>2\\J:ȉ<[\$ Yrʳadrce}s!#̱ZMp\"Jc+˩dn@8w@\\ΤX#uWoᬽЉ0W^t'-OԞq6oM'! b%π7TOqpƻ'<Y/5davG*{\"	:ߍIKpm8(.!6̞uм+ؔm#ۈDh0BGl5?+\nQhQʓi\n\"\"O 	\"q\"JmD\"|BC	V0'@\"9fLّ#`zu2b\0C+/^1'Ϡ's޵8!O]}t{جv!QtWpL!u@8g f0]c`S5~5!ܼvYzrӎıG~JFtHem{/Y\r*%nw>`Zˋ el7fPH\\̲6QI\"558Rj']6qB.PT\\Q*o;>{CcY2dԬFN.j6)\0'<j;	+\0BUX\"pF4c*n\$|lyˌM:/|pRL70(KZ)0:r`mNo)\0-\" L/\0@liŠТ:	p؇>\"MЛ\n6ʇC\n\nLpг	\0000֜paoN07\$b\"h\npArt_0\$ТO*\nkϏ\r@5mP#C~lD&D# \"-VE(SQdTh8\$LqX;r7Kl/(3G,ߑjn\0D8.2[qI\0ܭ\\Q:jHWp#Nc0p҄nYY2'0v^R!\r@j-a,Jz\\FE7akJL\\K\0002S\"CHdYe!\"%i&cer!&Z!vӄb8NyA&*zP\0 hC'*I# 1*y'z n>\0:>Ü#..vʩ%.C900Rn1*b%\0ⅎ2&S<\r)3n/3R<;rKr|SM \\KPoبZrR^i8C@c]*8~S\"#	9*FR\r!Db-7%賞2Ws\n g++12t3bD5.5H i/8O^!`\npl\0A	:Apbq:3:1:v+CgE&^9\r?in,D_D4QEd_⒤rb&NK/\n:\$NDgu,e,|1Ql1l,ӹKg;MSc\0:\"*!.Ql7QN[ 	:GCc	>i~{,:P	#\rm/0e\nÈa	X55Z\0/!o7qU52s'1&t&LE`gΦ\"d9*'-ƊFR2˦fc"; break;
		case "uk": $compressed = "%)h-ZƂh. h-mhĆ& h#˘.(.<h#v_Ps94R\\h%p	NmcL4PҒ\0(`1ƃQp9\$&;dH5}Q\$C˩ZB	D8(iyA~Gt(ygY1~Қ(BdׯKmJI\r.(VV1>#\$:-r%CǴ)/tp^\r>[73'6SP5dZ{h>/Ѡz0)28?v(P|\"oKB\"i{* 5ϲ:㹉вH8ޣ\"JBZ薉(F)ZY(\$&Y6,X\\Nz#DZ9)ĵ+;DLh1(3 (1@ݷlhQɠMH>K X Ě!аqQ&1d3WH\\C%PnTxH\$D-hU͋^5OR\"\"9#:hƁGQ8mn#NO*# 4(&Ԥ!rްX_ܥ0\\kUsɐ;(~\r3:x\r`Ap9x79cv2Fn=,@Mn;GB^0jXƐ/Q Ѭ+\rbe^V8<\nvxZnz	\nO[D_qNliRI!7`PJ2biZBʫ\rs5hZ~H(5h|\$`K ąDUDΐ':)Y:%<N3u^j\"Se#Yj/pf<Ǒr\rAS\0Zޖ|)zP8FFVTz\$,bdÛV.5'EJ,4V@PS\n!11hA>&IF_:НnB\\1%!DBIUġ35b<NX+x[*S]HaM2[\n׍*o\$+\$-hj\"((k@Cs4Lr\r0Ȱa-i\nC\r3\"C,0K ia1) gWK(x96{L6@)4*^N䙉:6ڦ#9\$Ђ](0\\F!'O<wUXRn@18̲+P		s^E͉=oBN;:Dg5=~O4|a&ͲoB\rN*Cb8e7QŠ(sّK8'؝hY%Wfo	j&3ORS6,(D¿WIfi\"Iө2&HəC*e3&hͫK9gl&t\04,`3gEyb/c	M2\$Q1j;bRXUztIbQ*\n0e\rT1N+ha:`WTϒXfޓX2,=vdZً3՛֦t@27K6ZX4*еWWNDWj8ND>O<Ռ,9vRe:q,ҫwGdj\$#%FoOCqyPPɦj:܅m;RlŲr=gEPϒG+eN*}\\@ \n\rTH\n\05	 hy(\nah#\n \04_L^A\\G`;@')}<]1,JhyԆ9bj90x[+Jҋ4V	Ih6:-CwBPR) z[;d\"i\nmaSZZ5FjC&5qH&.@1fvv2:0\"tLH2jZ+¾4_Э\rT 4&\$5B3f\"<v&MGnNd-)^j<^\0T~q]tQ8g7~_\\l=5wc`ԲGk\\<6je\";( Dθ9т<\$wR>0RfR`BI愓F\nBh#bYBFJ\nTsn]2\nodۢ3}KpӉ0uI_!R0^MdoͻW5Ʈ|e\\r/o)r3%} QVnZϹ%ǓQnD]5XCY1YꤽJ%<@8^:66(X|tM0:h}*JTh+@t\$,ɬBO7:+I5<Su\0p	fs\0\r\0 FJ4	X.#y\\=H><,)*\0eDl8df\nn\n)g#b&〼#LD';\"zSVp\\CHVH,|^B(5\$HBhfoЄObC/22[@\r\0` duȴ|JShIBk¢ƿMYN҄,,3h֤^!G&jn4cPJ|ORI&Ȉ̍:%t2%P \n`~nqf\$`JGfHGJr0ڤob~GB7QSQVX\$#,.z>pf>1a|&eb Krr\re2!d	GW?\".\"g\"bQpX@a<QL7b\\s!-#\$,C#q%'ri \$Б'2`tf'ҕ\$m\rqS1\rQH+dCޡeJG -:!F&Z!o(*>Ȁ8V%JNdW)E4~VA<Rk`6ʼoA%e-)*\$%7x)7lQhcBpR;оpn9rovC#sHHP[;yLo8#;\nS=m>A甞lzJ!@zpA8Jp@0KTB0bOzEo2S[5Ab>\$w\r3k<Y:s `/B\$v\"VFo=PY6osRGsGLc3~:cK!G-Tz!Htԧ\$:gHG9sJ=4MTCMMy%D!.nF(g7D()ܤ<\0p{u\ntRKӽ:O'u\$ORS\n)a* uK\0000Q[SH:hkPp2uQTФ~5sKJՏD^(#)Lj\ndPǢq\\bIj<1<ZW2C)\rt\\2Ula,sګ5ѶxZ ѥ\n-D(kYqA]7=^}P#R^N/4F'Vout3E4o0;YH5ZOLuWeh+UhtUH6Pfg5{eK2pwN[tZjJuC1ërxxT|?Pe֏ooSe4oo1tVgfV6{}'q25.*\$&W(6y^W3R}`EJq@QWWtiHbjW;J30svx5_pd+j{ya,tAGD^0MEBo)f7uY֧qr\\7Q6V\rfHh`@@g@\r6%Ww)7W\rq`%F7¥NvahpfAE˃|ׇXԍ8襡}<8fo%Lyi%du*WUQ'HsXԃEaxч}Op;tSF2?cMrmQӒOrOUu(DΟ0 ql#BixݏAHNvJPfEpC\r-8,4QiSUXT\"/P|Y;yQSK~)\rV#2\0r4)Ô8Cme,.QmJꕄB\nq&#vVoIjڌD_I盓A~:%1UmZb h'3c7+jɠ\"74S@BDU6Ht}Bj8`3ث嘘Bq3C7,0,FyOZSPhaCPJxlS3[7r0pV>XH:ڑe%B9Gowv8n!6{=pgHQY[2c^Rl;\\eѯDd6>x&(RT%\n`0aosyMHySpxg+SXMژ\"j=	om;\\ ڧjN1=5x3CKul/0@"; break;
		case "uz": $compressed = "%(a<\rƑk6LBNl6Lp(a51`u<'Ai6&%4MF`B\"u2Kc'80cAn8'3Ac4MsI@k7#**''3`(;M6,q&ƙ}ģ+77:B:\rW.3b\r4q/|\0(a8ۍ :`*{VvN-o)dgxiw7MX('b%Iyawu:D50K82cz(\n2#ؗCX:\$VL[<&{♺\nn*2M47csX߯#%ct\$n5L P2)s\n</s&c\$+*F6ј1Lr4BCc7+@S ޑ#;Ȟ\rhAPsS@t\rAID3Њ.P\"07c2D49cx_)6̈ġ|9c\"2|-8L4x@/O2l%:2=&KZTȳ\n,=&Cw;˸ȼ~눲!(2,'xa8&L<N:D\09=9+C,#79;Wи:CF`3C*d:\$R(('#j2FRn9L\r!(:!* 7@]m;f	\$	\r67H&J(<8O<\rJ:?9%!;'!`=F/S	)s6%##}.K.u<!uHsQcii\0004۶I@Ea9Øݴ|а:*D́reYcG(`S,3Y#R\rb1M\\jM39¸_#5l8fAc{BL}0NnD+,CRa}Ƣk\r\$DzxRJ ='R:},¢bϑ2%bhC@V\nɦUnUڽW`2ı2\r2VJ4zW{t\$3PNHS2KҐjze&J@;8*T{TS* \nEϏ\\40é3lW`,)(rX\$2IZOY,~RQXBʰsL#g\"mI?dm9L!Т\$#sv)bȉ#-(kjhIT*/	\r4( (i.r|\nMhe,\$£tR]L+NF\n(Ю*\"NJIYM\":}Y'qH\0a>2Ql#2J9-\$@vu_PZFDwLNH{\$4:Y@(^g*E\\W_Ep43) 	(|bm,ɸq'ہ~7-@kKa]!\r\$P-b!%r((k{@:dd?z@n3n%Ie]5ēb\0F\nXCx9q#^FR)b0'Erch,HL@y8C	\$)bXp\nd줚'bxLKO;ݏ\n\nhHuTZ0^di!\"ǥP\\yA<3M`|VUT\n\\5ӷKe\"Ip	Pg#ˉwlUt2	\rOӒtXR\0HAĭ`H֛*MN|A:/N'(,ZȉdѺц`LLB\r4-nb>ff#j9x㽯bx\00ۋtC#J&0Ү:,TTM<sC,ć=6#L}e\" ('H6{\n\neP{t]'=NAVLb)bt׳p}05dFs#92\"̊@(ME!Ta@lz/yR\nD{E\$vnc.q'^\\I΁2pѮ٢/vG囁0yL3B(UCn7E	e,	YnxrmEy(2K:\0MyA`>9>%Sw/RR\r!ὠtӵ\\؈n%!'=/fzMp/;'\"_\0pȌ~`l\nfts.&H`A@'FF\"e͡\n\r?L|O`eݫko:Cv\$\$MIL>~l/Ф l|;	PvN{\0pPPt@vQPCup}\rh6'&|ˮX^lOnmu.VP콰J,8^>p\r\rp1-,<'X&d7X#-#CHe{JbJ1l%Dj:DqPVǂ}\"ݐHZG-66I:\"\\J(=*J\r\rN|oqtv4zʎ87^lЛ-ɕlB6%P\rR\0q D;!GI\rR\rB=`i0x\rqZD04G(7QRS\r%](&0jl\0'q8yF\rƎKB^i%rQhP\\# Z qu*]*o*R* r2@(Q12-2\"\nLұ ..~x2.D!.oqSᲛ0T(cN30&Re2(R3,S3+üd\$Kp%;5s#6Nsa1	FcRDO0'J\"2,(yFF+À\"ϱ/<)3уd::	`ش\rCBz :9ʐ3b8[=\0pnh!	4\\&b*; F\nZj\"Ѱ^nIZw91=*T37eЯD\$#\"6S\$;4`\"Pryp3;?o?CG?8#NBbD\$hmZƻ%\0,H<c6&\r03x|LK@Pxd0Cƶc\nHܑ6Tؐz@/CE,Mj:q_M!SBgƬy@.akb'\"vvU,]Uf7<] OBGO1sF0c\$=c'mSr^-ϐslX	ɟN%ҊfwG+R,LeF4#@䊵 "; break;
		case "vi": $compressed = "%(ha\rq]Ҍ]c\rTnAjhc,\"	b5H؉q	Nd)	R!/5!PäA&n&0cAn8QE\r Y\$Eyt9D0Q(Vh<&b-[no\n(U`+~daH8iD\\Pnpu<4k{C3	2Rum]/tUږ[]7;qqwN(a;m{\rB\n'ٻ_2[aTk7)o9HH0c+767 88@@ \\j L+@ƻl7)vOIvL:I枧fakjc]/P!\0d! K P k<M\0\r@h4AN!c3(7\$Xb,(R-2j]2<!iJ NA1[(Rf1B\"\rAZ8B<&u=SI#qtI>(0P2\r<9ph#n\0@C҇\r㭘B%\n\0x3:x{\ra?/p_pcI|63?k4^02T.́BED\",9e9):&Y^\";\n7ZH(J2/C2S)cs2R̩JVJ\"!7\"]q:V6qJćeJZ7k,2 JGV\n5Hƈ3Q7tWcéVI~;U6Ê4J4YfBΓF\n#TĮ@-џR|[46'h(PCcn\r)+uߢ\$\n)C4{au!FTz~ص%WhUH*]RTs/5ux %JaK:V~>91C>h3,t~J6A@	8nZ(C)\r\$\0`\r3*Ih;\\F BnEP.)],\\ Qz^Qfi.Hn2T	3ŕx3@ @WK1@u޼W^|4\0`A4N\rL¢B&~KQAC:G4ċY3oČEғKuK\$t.ػ^|/2}er_D	&BIfmOd9SCNa\"[ȹ{Hq1B@X-IfhMd, \r'C:FԘ.@fԡ*.]Sצ8u['p&i	wD	%|D\"xw8 vag hb>TycHMCY\\E	j:(뾐Ӓč	!KitA5zaH#X)f1&4Ыec6i\0(;(v(Msrsj-DȚ6BI>腦mRu\$\$0\$74B]Q@\$3s&9+\0դ+2| M/\n<)Hws\$LpjĘ\\XK݀:M+C3\$<R3~Gr\"RH(!p*ζ3dҜE2ti-b^f,+N ^EGB\$yQ+ jgF\$X\rE)w8I)>12Y;'zzdqv\"m(YhDӱ^h?#taL_(#/L\00*:G04Uzm(bz\rPM=5 0\$\\NPT'řȆmQn>F%_ԹmDil=W3Cfڞ[5(ⲳ\\8+>p!\$Kf|7>]jO1CoLS:556>p\rs	_L4忓LAIn֔\n}#*\"eUDn5\n5GC	V|Y(:;R鑅a/\0*\$Z\$=S#+8\$Ġ\"	H8(,Aa<By/(	!_I)c)Q؄;rLWYk~ҰPIdKb-Ź\\\$ԮTb~u#7a%#vo)x#V]e/>*2KrNJ\09Zlbp\"KT K#lЛ IcbL#>06{H.!J&klX+<o6A~#-*s,1Nn#btz!D+@5(\"ؘ)TCPLj]\$dmn6EÍ\\`C6d\"Fk2GKp@.;4H0RLӱdq!QH-T|ge%\")UK/\"O>XE<{B.c\"coNQqjq0^8\\\$\"̸P'¡f\"9|pqNRQ鶶Ed:jDŋ4GF<\$mB0(nHяɭYRq^z.БBifBeJbfpPG '!qNDx|*!!Q'l&GO@9Q%fe%nbkrpvN78\$JR!w\0L֛n:T\\~KTU \"/+%&F.+\0{'zW8F2'Re7MQ*9T<+pF\"\$ 0Z-I-sI.G2,'lD,3w&\\)'sv\riF@\nB`#	jFBTZ@`lV朗5so56le{x1++s\$,33SO;yWNfxR}9E:&ZO:Q	%>nFméQoMd645chA%z6:w!Oq#c:UtwCh-KHCJ)\0-\n@GIF\"SG9.S\nHJ,)\0\nq0.\0JOʊG\".\$x/JLg2nS/Ѹ/kG1-tm.3bn6cГJdJWd+X\"m*K vp.(W0-P#FA/V\$5!:F8v`Z!u0#& @6J/F_#%p4TDPݬJlPx7j<E,JOOlXXV%;Dp35ĎHLnm\\!roUcvMKL\00\n4jPa^0Rd~%P#SZD!_4K#S@h1)n\nf:WWm&mƺ`4B"; break;
		case "zh": $compressed = "%:\$\nr.r/dȻ[8 S8rNT*Ю\\9HHZ1!SVJ@%9Ql]m	FU*qQ;Cf4usUUt w:t\nrU:.:PǑ.\r7d^%u)cxU`Fjrs'PnA̛ZEf]EvitUλSծ{Pg5	EPN1	V\nW]\n!zsΟRRVI:(s#.UzΠ@:w'_T\$pVLD')bJ\$pŢ[MZ\n.>sKAZKaLHAtF3D!zHC*re^K#sXg)<v׬hE')2Anrj\n:1'+ֲ2izJsͲ h7]	9HN_esK?	RY4=DF@4C(C@:tT3>x3(9I|1B:LΝ\$=0!tIE'5(RMy&s#SE͒CH]K:KC%um0KV)\"EA(CGAFpܗ&fTYCG)\0DGSW)\0^cTewa D#8s*.]\"h^9zW#s\0]c9a Dj<V]2gC\$CCm8)\"entI4}|=3Q''1Q,g^nKsSL̋&eȜ:ϿC34;.Ml`#@6S6::=9CZ\\u|!p4Z)M=?pA\$_Kz<O66ҙt_No-6MHX:+3.~ɻFbt\"\$r!bJՄP~T]'@Gr[\0 @\$\$C1v+ʛS}P5JJUX+%hxdx:2°_:X%PQ'Ea/1ir\"&B\rBoM ~pOrTJS*Tb*YUnCtV'+uABV2>,E\$\$W1,\"r,E4H=I>ŊH9kdvd\$?M#9# 9t,Ǩ4zZ) @@Pz\"MQ-DV':HIAØG\nTf,8o&Hs_;0hDM#bʹ{mR NAr@a(3WT&(8LZ((-I\0PP\n\"\0C\naH#.)(#g8/I\$V&.FR:E=9)%hhs\nj;?ca@+\\p ZлНW*RdȱQ4G4\\р\0£RAOXcHUt*!Y?l 	EL#@CȈ&	8E@OIy\"9+1dϣR	c(YP*[s E	a/WE`LSA/DMBsFQB.QwEߢ)0t7gʅ2Ş\rna#/-/)&dF@&g@DCOA^|\n|JIaޫC+:t3DtP\\39Pq\\' `ܡpNN\\i mNtᜈM@!6%\0C\r1xL1)hAttΛKeM&HQ(9}DC\raƐN	s8M9]@BHǀ\n#~qrǈLhE6&\\\n)v]!0fmt\n&\"f)ȗf;Cm	Xj/2\\-*ޚs;F,NV&*XӀ(+Pm!|ЎQa k`fB6-9vZ9Ĳeƺ:D,b\0003kg,뢹_΋ôd*\"5m#)#PL`]1pikS&Bk;ʜG<a~6+`AhL\$Xyߗ.I/R\$__gۻm2M~?Z;[ͮ/wx{C\\i~yHSC>\"7dd%(ٙ3%v[ͤ.(a\"&pQ6;jCv!!(Ik\07D.+so-6Oψc|o4lҏ/rPat\"2oj^\rp'\\^~#fAu	mJv\\a:J^o\\h.L\"4Ψb֌6Ͼ^-LώЌ'*eN֭zem<p3\"a<Oh\rϝxK'=0αmрSet\0l\0@\0'GM^'OlQq qxv`)\$~1~M(n\nӐ]1QmD4FG!N.FFDQ:o,?	t`i	#\$-t>F-j^.tFʈ,w! 8m֟>,g\rv~p 3F) PC\nZho0lC!d*H.!6ì6-VNdgʳT3\n-Rˋ䊪#\\JΐQ1\$\npX-4*2fL.nҽ+,NAt,FE \r!(,#G/El\$\$039{1#\nJ::9L>784Co50\0>(+DD"; break;
		case "zh-tw": $compressed = "%:\$\ns.eUȸE9PK72(Ph)ʅ@:i	aJe R)ܫ{	Nd(vQDCѮUjaʜTOABPb2ar\nr/TuʮM9Rz?Tא9>SNeI̜Dhw2Y2PcмW*=s7B9J\"XQ2M/J2@\"WrTD{utspS\\=\0Vp\"R )ЪOHΔT\\ӊ:}JF+JV*rEZs!ZyV꽯yPA.yZ6YI)\ns	Z̢[2̒KdJ12A\$&Y+;ZY+\$j[GAn%Jst)P)<?\0Uw*x].2ft+<Kd(A2]*X!rB\n# 4(tE\rl	Tr{:OpbJBO:F@4C(C@:t㽄4)Kx3(9PI|t(B1֯B|\$	q/9H]D̸et\\K6?஋LQ\\1H@PJ2:@ea	&s2So1Qd0ם3Mewd:<C)xGdrBHreBi^1I@\"ZP@fg1pM	j^B<JLf*83(ڰ:s (&^)Dr\n淘1(\\մ	_ܾ7&]>Tt734ҥr</hCHHs,o7|u+wxbØ@6XU:Cp@09#uW97H}U1	~1KTNFQ8)HTKPs3cM#& )FGpXa*%)FG4B',a\na\$\n0Cո)Ȉ%)(pfD N*R:a\0(&D	`\$UV\\z\nXId,xd|:ζV\"@XCt	e/0ʴE#Ȫ髃dpO­WJ_,\"Ɗk%e՞nrEl-b(f\0CG@e}!]>]w/U\$fEhC51QB'!!&Ѱ9sa\"CQ>-0d\$TD2(2xDv\0 \r\$BGu\"#䄑Fօ*DM?!PD*\$G@&d͘tl!E*U)\r\$8lG9i^*hЃL\nf#@A\n/8F[(4ZjI\0C\naH#\0Z')E\$&ra\\E^0\nG0\\Ck=&ȵ4tXEP,S!'BbS*r?QXʼW~9KĂ\"G4N\"xS\nVEFLIfdΜ;&/Uiਪ\r@50T\n;&8L%DޜG*sp,96ԹBLpO	*\0BEX@.	d,^1 #G9##=V0sn黳ba:XVAEX/OD堔qhTػޛE\0*0H P\"`\$ӣR\"\0\0R]aeP\0ֈYEhD4nAj07PlJ5&LY*>`9&\$͢*j+Ģ]h\\uYk:xnֺ(8\"EL`eK1T30/'0Q-BKm΋oE滿#]	Ñ rYI}`(!^j:P@<\\W2aܛ9u2<A/D9`]4H2uEضֆU:T!2S`BHn[/O*+\01f1@H<c\0\"&b:D3&FӉ.%P\0`8<7ly`Kyx!yuq)(=|9\$DNM/i((\rNs+Ee<k\"Q徽?'qbuE\\%ra1#t7*{\rOy:05Ű_0aB{07Egȇ\rk__ʛl~cX_ts6SBV:yP?<-FLGY.qu5ƺKb&#ϨGVo^π>uobn\rG<!W\nNx02Ԣ0\$s\rFu<h-lFZ#..N2^pOlrAkrx+n\rp|ب\$9\r^Bʛ0>\$fNU풁\nɴpJ-8\"tLUzK,m1JpB`&6,1--d6\nw<pjm\r{.5F0pCQ2A\\rB.GB^h+B^~(7p/(!.溓!h\re0%^hqcyL\"'\$-߱21pt({ 1ڭ\refZF2\0@\0!!R\${(R4ڍ!\"A\"`ۆ82FM\$Vܡ20rv'-C(ݰR'Hq]rK}*\r+\$#*\n)@ZMM.aLwAD<L,i	+N?H^!Fb:!k/D:V.r3!z\0:6:èb^6&rIj\$YP+@\np9'POvB!gJ0Fe涡@9dÔ1\ru{5E\nh\\^S\r8Ht:Ӽq/Bl-/#Nl/qTa`F'	{%uai#͂/Ă\rdnO, &ʨ\n`ڑS++@\\N0k>xRw3>,nD0Hlj0c&rqBM#*\0L"; break;
	}
	$translations = array();
	foreach (explode("\n", lzw_decompress($compressed)) as $val) {
		$translations[] = (strpos($val, "\t") ? explode("\t", $val) : $val);
	}
	return $translations;
}

if (!$translations) {
	$translations = get_translations($LANG);
	$_SESSION["translations"] = $translations;
}

?>
<?php
// PDO can be used in several database drivers
if (extension_loaded('pdo')) {
	abstract class PdoDb {
		public $flavor = '', $server_info, $affected_rows, $errno, $error;
		protected $pdo;
		private $result;

		function dsn($dsn, $username, $password, $options = array()) {
			$options[\PDO::ATTR_ERRMODE] = \PDO::ERRMODE_SILENT;
			$options[\PDO::ATTR_STATEMENT_CLASS] = array('Adminer\PdoDbStatement');
			try {
				$this->pdo = new \PDO($dsn, $username, $password, $options);
			} catch (\Exception $ex) {
				auth_error(h($ex->getMessage()));
			}
			$this->server_info = @$this->pdo->getAttribute(\PDO::ATTR_SERVER_VERSION);
		}

		abstract function select_db($database);

		function quote($string) {
			return $this->pdo->quote($string);
		}

		function query($query, $unbuffered = false) {
			$result = $this->pdo->query($query);
			$this->error = "";
			if (!$result) {
				list(, $this->errno, $this->error) = $this->pdo->errorInfo();
				if (!$this->error) {
					$this->error = lang(21);
				}
				return false;
			}
			$this->store_result($result);
			return $result;
		}

		function multi_query($query) {
			return $this->result = $this->query($query);
		}

		function store_result($result = null) {
			if (!$result) {
				$result = $this->result;
				if (!$result) {
					return false;
				}
			}
			if ($result->columnCount()) {
				$result->num_rows = $result->rowCount(); // is not guaranteed to work with all drivers
				return $result;
			}
			$this->affected_rows = $result->rowCount();
			return true;
		}

		function next_result() {
			if (!$this->result) {
				return false;
			}
			$this->result->_offset = 0;
			return @$this->result->nextRowset(); // @ - PDO_PgSQL doesn't support it
		}

		function result($query, $field = 0) {
			$result = $this->query($query);
			if (!$result) {
				return false;
			}
			$row = $result->fetch();
			return $row ? $row[$field] : false;
		}
	}

	class PdoDbStatement extends \PDOStatement {
		public $_offset = 0, $num_rows;

		function fetch_assoc() {
			return $this->fetch(\PDO::FETCH_ASSOC);
		}

		function fetch_row() {
			return $this->fetch(\PDO::FETCH_NUM);
		}

		function fetch_column($field) {
			return $this->fetchColumn($field);
		}

		function fetch_field() {
			$row = (object) $this->getColumnMeta($this->_offset++);
			$type = $row->pdo_type;
			$row->type = ($type == \PDO::PARAM_INT ? 0 : 15);
			$row->charsetnr = ($type == \PDO::PARAM_LOB || (isset($row->flags) && in_array("blob", (array) $row->flags)) ? 63 : 0);
			return $row;
		}

		function seek($offset) {
			for ($i=0; $i < $offset; $i++) {
				$this->fetch();
			}
		}
	}
}

?>
<?php
$drivers = array();

/** Add a driver
* @param string
* @param string
* @return null
*/
function add_driver($id, $name) {
	global $drivers;
	$drivers[$id] = $name;
}

/** Get driver name
* @param string
* @return string
*/
function get_driver($id) {
	global $drivers;
	return $drivers[$id];
}

abstract class SqlDriver {
	static $possibleDrivers = array();
	static $jush; ///< @var string JUSH identifier

	protected $conn;
	protected $types = array(); ///< @var array [$description => [$type => $maximum_unsigned_length, ...], ...]
	public $editFunctions = array(); ///< @var array of ["$type|$type2" => "$function/$function2"] functions used in editing, [0] - edit and insert, [1] - edit only
	public $unsigned = array(); ///< @var array number variants
	public $operators = array(); ///< @var array operators used in select
	public $functions = array(); ///< @var array functions used in select
	public $grouping = array(); ///< @var array grouping functions used in select
	public $onActions = "RESTRICT|NO ACTION|CASCADE|SET NULL|SET DEFAULT"; ///< @var string used in foreign_keys()
	public $inout = "IN|OUT|INOUT"; ///< @var string used in routines
	public $enumLength = "'(?:''|[^'\\\\]|\\\\.)*'"; ///< @var string regular expression for parsing enum lengths
	public $generated = array(); ///< @var array allowed types of generated columns

	/** Create object for performing database operations
	* @param Db
	*/
	function __construct($connection) {
		$this->conn = $connection;
	}

	/** Get all types
	* @return array [$type => $maximum_unsigned_length, ...]
	*/
	function types() {
		return call_user_func_array('array_merge', array_values($this->types));
	}

	/** Get structured types
	* @return array [$description => [$type, ...], ...]
	*/
	function structuredTypes() {
		return array_map('array_keys', $this->types);
	}

	/** Get enum values
	* @param array
	* @return string or null
	*/
	function enumLength($field) {
	}

	/** Function used to convert the value inputted by user
	* @param array
	* @return string or null
	*/
	function unconvertFunction($field) {
	}

	/** Select data from table
	* @param string
	* @param array result of $adminer->selectColumnsProcess()[0]
	* @param array result of $adminer->selectSearchProcess()
	* @param array result of $adminer->selectColumnsProcess()[1]
	* @param array result of $adminer->selectOrderProcess()
	* @param int result of $adminer->selectLimitProcess()
	* @param int index of page starting at zero
	* @param bool whether to print the query
	* @return Result
	*/
	function select($table, $select, $where, $group, $order = array(), $limit = 1, $page = 0, $print = false) {
		global $adminer;
		$is_group = (count($group) < count($select));
		$query = $adminer->selectQueryBuild($select, $where, $group, $order, $limit, $page);
		if (!$query) {
			$query = "SELECT" . limit(
				($_GET["page"] != "last" && $limit != "" && $group && $is_group && JUSH == "sql" ? "SQL_CALC_FOUND_ROWS " : "") . implode(", ", $select) . "\nFROM " . table($table),
				($where ? "\nWHERE " . implode(" AND ", $where) : "") . ($group && $is_group ? "\nGROUP BY " . implode(", ", $group) : "") . ($order ? "\nORDER BY " . implode(", ", $order) : ""),
				($limit != "" ? +$limit : null),
				($page ? $limit * $page : 0),
				"\n"
			);
		}
		$start = microtime(true);
		$return = $this->conn->query($query);
		if ($print) {
			echo $adminer->selectQuery($query, $start, !$return);
		}
		return $return;
	}

	/** Delete data from table
	* @param string
	* @param string " WHERE ..."
	* @param int 0 or 1
	* @return bool
	*/
	function delete($table, $queryWhere, $limit = 0) {
		$query = "FROM " . table($table);
		return queries("DELETE" . ($limit ? limit1($table, $query, $queryWhere) : " $query$queryWhere"));
	}

	/** Update data in table
	* @param string
	* @param array escaped columns in keys, quoted data in values
	* @param string " WHERE ..."
	* @param int 0 or 1
	* @param string
	* @return bool
	*/
	function update($table, $set, $queryWhere, $limit = 0, $separator = "\n") {
		$values = array();
		foreach ($set as $key => $val) {
			$values[] = "$key = $val";
		}
		$query = table($table) . " SET$separator" . implode(",$separator", $values);
		return queries("UPDATE" . ($limit ? limit1($table, $query, $queryWhere, $separator) : " $query$queryWhere"));
	}

	/** Insert data into table
	* @param string
	* @param array escaped columns in keys, quoted data in values
	* @return bool
	*/
	function insert($table, $set) {
		return queries("INSERT INTO " . table($table) . ($set
			? " (" . implode(", ", array_keys($set)) . ")\nVALUES (" . implode(", ", $set) . ")"
			: " DEFAULT VALUES"
		) . $this->insertReturning($table));
	}

	/** Get RETURNING clause for INSERT queries, PostgreSQL specific
	* @param string
	* @return string
	*/
	function insertReturning($table) {
		return "";
	}

	/** Insert or update data in table
	* @param string
	* @param array
	* @param array of arrays with escaped columns in keys and quoted data in values
	* @return bool
	*/
	function insertUpdate($table, $rows, $primary) {
		return false;
	}

	/** Begin transaction
	* @return bool
	*/
	function begin() {
		return queries("BEGIN");
	}

	/** Commit transaction
	* @return bool
	*/
	function commit() {
		return queries("COMMIT");
	}

	/** Rollback transaction
	* @return bool
	*/
	function rollback() {
		return queries("ROLLBACK");
	}

	/** Return query with a timeout
	* @param string
	* @param int seconds
	* @return string or null if the driver doesn't support query timeouts
	*/
	function slowQuery($query, $timeout) {
	}

	/** Convert column to be searchable
	* @param string escaped column name
	* @param array ["op" => , "val" => ]
	* @param array
	* @return string
	*/
	function convertSearch($idf, $val, $field) {
		return $idf;
	}

	/** Convert operator so it can be used in search
	* @param string $operator
	* @return string
	*/
	function convertOperator($operator) {
		return $operator;
	}

	/** Convert value returned by database to actual value
	* @param string
	* @param array
	* @return string
	*/
	function value($val, $field) {
		return (method_exists($this->conn, 'value')
			? $this->conn->value($val, $field)
			: (is_resource($val) ? stream_get_contents($val) : $val)
		);
	}

	/** Quote binary string
	* @param string
	* @return string
	*/
	function quoteBinary($s) {
		return q($s);
	}

	/** Get warnings about the last command
	* @return string HTML
	*/
	function warnings() {
		return '';
	}

	/** Get help link for table
	* @param string
	* @param bool
	* @return string relative URL or null
	*/
	function tableHelp($name, $is_view = false) {
	}

	/** Check if C-style escapes are supported
	* @return bool
	*/
	function hasCStyleEscapes() {
		return false;
	}

	/** Get supported engines
	* @return array
	*/
	function engines() {
		return array();
	}

	/** Check whether table supports indexes
	* @param array result of table_status()
	* @return bool
	*/
	function supportsIndex($table_status) {
		return !is_view($table_status);
	}

	/** Get defined check constraints
	* @param string
	* @return array [$name => $clause]
	*/
	function checkConstraints($table) {
		// MariaDB contains CHECK_CONSTRAINTS.TABLE_NAME, MySQL and PostrgreSQL not
		return get_key_vals("SELECT c.CONSTRAINT_NAME, CHECK_CLAUSE
FROM INFORMATION_SCHEMA.CHECK_CONSTRAINTS c
JOIN INFORMATION_SCHEMA.TABLE_CONSTRAINTS t ON c.CONSTRAINT_SCHEMA = t.CONSTRAINT_SCHEMA AND c.CONSTRAINT_NAME = t.CONSTRAINT_NAME
WHERE c.CONSTRAINT_SCHEMA = " . q($_GET["ns"] != "" ? $_GET["ns"] : DB) . "
AND t.TABLE_NAME = " . q($table) . "
AND CHECK_CLAUSE NOT LIKE '% IS NOT NULL'"); // ignore default IS NOT NULL checks in PostrgreSQL
	}
}

?>
<?php
$drivers["sqlite"] = "SQLite";

if (isset($_GET["sqlite"])) {
	define('Adminer\DRIVER', "sqlite");
	if (class_exists("SQLite3") && $_GET["ext"] != "pdo") {

		class SqliteDb {
			public $extension = "SQLite3", $server_info, $affected_rows, $errno, $error;
			private $link;

			function __construct($filename) {
				$this->link = new \SQLite3($filename);
				$version = $this->link->version();
				$this->server_info = $version["versionString"];
			}

			function query($query) {
				$result = @$this->link->query($query);
				$this->error = "";
				if (!$result) {
					$this->errno = $this->link->lastErrorCode();
					$this->error = $this->link->lastErrorMsg();
					return false;
				} elseif ($result->numColumns()) {
					return new Result($result);
				}
				$this->affected_rows = $this->link->changes();
				return true;
			}

			function quote($string) {
				return (is_utf8($string)
					? "'" . $this->link->escapeString($string) . "'"
					: "x'" . first(unpack('H*', $string)) . "'"
				);
			}

			function store_result() {
				return $this->result;
			}

			function result($query, $field = 0) {
				$result = $this->query($query);
				if (!is_object($result)) {
					return false;
				}
				$row = $result->fetch_row();
				return $row ? $row[$field] : false;
			}
		}

		class Result {
			public $num_rows;
			private $result, $offset = 0;

			function __construct($result) {
				$this->result = $result;
			}

			function fetch_assoc() {
				return $this->result->fetchArray(SQLITE3_ASSOC);
			}

			function fetch_row() {
				return $this->result->fetchArray(SQLITE3_NUM);
			}

			function fetch_field() {
				$column = $this->offset++;
				$type = $this->result->columnType($column);
				return (object) array(
					"name" => $this->result->columnName($column),
					"type" => ($type == SQLITE3_TEXT ? 15 : 0),
					"charsetnr" => ($type == SQLITE3_BLOB ? 63 : 0), // 63 - binary
				);
			}

			function __destruct() {
				return $this->result->finalize();
			}
		}

	} elseif (extension_loaded("pdo_sqlite")) {
		class SqliteDb extends PdoDb {
			public $extension = "PDO_SQLite";

			function __construct($filename) {
				$this->dsn(DRIVER . ":$filename", "", "");
			}

			function select_db($db) {
				return false;
			}
		}

	}

	if (class_exists('Adminer\SqliteDb')) {
		class Db extends SqliteDb {
			public $flavor = '';

			function __construct() {
				parent::__construct(":memory:");
				$this->query("PRAGMA foreign_keys = 1");
			}

			function select_db($filename) {
				if (is_readable($filename) && $this->query("ATTACH " . $this->quote(preg_match("~(^[/\\\\]|:)~", $filename) ? $filename : dirname($_SERVER["SCRIPT_FILENAME"]) . "/$filename") . " AS a")) { // is_readable - SQLite 3
					parent::__construct($filename);
					$this->query("PRAGMA foreign_keys = 1");
					$this->query("PRAGMA busy_timeout = 500");
					return true;
				}
				return false;
			}
		}
	}



	class Driver extends SqlDriver {
		static $possibleDrivers = array("SQLite3", "PDO_SQLite");
		static $jush = "sqlite";

		protected $types = array(array("integer" => 0, "real" => 0, "numeric" => 0, "text" => 0, "blob" => 0));

		public $editFunctions = array(
			array(
				// "text" => "date('now')/time('now')/datetime('now')",
			), array(
				"integer|real|numeric" => "+/-",
				// "text" => "date/time/datetime",
				"text" => "||",
			)
		);

		public $operators = array("=", "<", ">", "<=", ">=", "!=", "LIKE", "LIKE %%", "IN", "IS NULL", "NOT LIKE", "NOT IN", "IS NOT NULL", "SQL"); // REGEXP can be user defined function
		public $functions = array("hex", "length", "lower", "round", "unixepoch", "upper");
		public $grouping = array("avg", "count", "count distinct", "group_concat", "max", "min", "sum");

		function __construct($connection) {
			parent::__construct($connection);
			if (min_version(3.31, 0, $connection)) {
				$this->generated = array("STORED", "VIRTUAL");
			}
		}

		function structuredTypes() {
			return array_keys($this->types[0]);
		}

		function insertUpdate($table, $rows, $primary) {
			$values = array();
			foreach ($rows as $set) {
				$values[] = "(" . implode(", ", $set) . ")";
			}
			return queries("REPLACE INTO " . table($table) . " (" . implode(", ", array_keys(reset($rows))) . ") VALUES\n" . implode(",\n", $values));
		}

		function tableHelp($name, $is_view = false) {
			if ($name == "sqlite_sequence") {
				return "fileformat2.html#seqtab";
			}
			if ($name == "sqlite_master") {
				return "fileformat2.html#$name";
			}
		}

		function checkConstraints($table) {
			preg_match_all('~ CHECK *(\( *(((?>[^()]*[^() ])|(?1))*) *\))~', $this->conn->result("SELECT sql FROM sqlite_master WHERE type = 'table' AND name = " . q($table)), $matches); //! could be inside a comment
			return array_combine($matches[2], $matches[2]);
		}
	}



	function idf_escape($idf) {
		return '"' . str_replace('"', '""', $idf) . '"';
	}

	function table($idf) {
		return idf_escape($idf);
	}

	function connect($credentials) {
		list(, , $password) = $credentials;
		if ($password != "") {
			return lang(22);
		}
		return new Db;
	}

	function get_databases() {
		return array();
	}

	function limit($query, $where, $limit, $offset = 0, $separator = " ") {
		return " $query$where" . ($limit !== null ? $separator . "LIMIT $limit" . ($offset ? " OFFSET $offset" : "") : "");
	}

	function limit1($table, $query, $where, $separator = "\n") {
		return (preg_match('~^INTO~', $query) || get_val("SELECT sqlite_compileoption_used('ENABLE_UPDATE_DELETE_LIMIT')")
			? limit($query, $where, 1, 0, $separator)
			: " $query WHERE rowid = (SELECT rowid FROM " . table($table) . $where . $separator . "LIMIT 1)" //! use primary key in tables with WITHOUT rowid
		);
	}

	function db_collation($db, $collations) {
		return get_val("PRAGMA encoding"); // there is no database list so $db == DB
	}

	function logged_user() {
		return get_current_user(); // should return effective user
	}

	function tables_list() {
		return get_key_vals("SELECT name, type FROM sqlite_master WHERE type IN ('table', 'view') ORDER BY (name = 'sqlite_sequence'), name");
	}

	function count_tables($databases) {
		return array();
	}

	function table_status($name = "") {
		$return = array();
		foreach (get_rows("SELECT name AS Name, type AS Engine, 'rowid' AS Oid, '' AS Auto_increment FROM sqlite_master WHERE type IN ('table', 'view') " . ($name != "" ? "AND name = " . q($name) : "ORDER BY name")) as $row) {
			$row["Rows"] = get_val("SELECT COUNT(*) FROM " . idf_escape($row["Name"]));
			$return[$row["Name"]] = $row;
		}
		foreach (get_rows("SELECT * FROM sqlite_sequence", null, "") as $row) {
			$return[$row["name"]]["Auto_increment"] = $row["seq"];
		}
		return ($name != "" ? $return[$name] : $return);
	}

	function is_view($table_status) {
		return $table_status["Engine"] == "view";
	}

	function fk_support($table_status) {
		return !get_val("SELECT sqlite_compileoption_used('OMIT_FOREIGN_KEY')");
	}

	function fields($table) {
		$return = array();
		$primary = "";
		foreach (get_rows("PRAGMA table_" . (min_version(3.31) ? "x" : "") . "info(" . table($table) . ")") as $row) {
			$name = $row["name"];
			$type = strtolower($row["type"]);
			$default = $row["dflt_value"];
			$return[$name] = array(
				"field" => $name,
				"type" => (preg_match('~int~i', $type) ? "integer" : (preg_match('~char|clob|text~i', $type) ? "text" : (preg_match('~blob~i', $type) ? "blob" : (preg_match('~real|floa|doub~i', $type) ? "real" : "numeric")))),
				"full_type" => $type,
				"default" => (preg_match("~^'(.*)'$~", $default, $match) ? str_replace("''", "'", $match[1]) : ($default == "NULL" ? null : $default)),
				"null" => !$row["notnull"],
				"privileges" => array("select" => 1, "insert" => 1, "update" => 1, "where" => 1, "order" => 1),
				"primary" => $row["pk"],
			);
			if ($row["pk"]) {
				if ($primary != "") {
					$return[$primary]["auto_increment"] = false;
				} elseif (preg_match('~^integer$~i', $type)) {
					$return[$name]["auto_increment"] = true;
				}
				$primary = $name;
			}
		}
		$sql = get_val("SELECT sql FROM sqlite_master WHERE type = 'table' AND name = " . q($table));
		$idf = '(("[^"]*+")+|[a-z0-9_]+)';
		preg_match_all('~' . $idf . '\s+text\s+COLLATE\s+(\'[^\']+\'|\S+)~i', $sql, $matches, PREG_SET_ORDER);
		foreach ($matches as $match) {
			$name = str_replace('""', '"', preg_replace('~^"|"$~', '', $match[1]));
			if ($return[$name]) {
				$return[$name]["collation"] = trim($match[3], "'");
			}
		}
		preg_match_all('~' . $idf . '\s.*GENERATED ALWAYS AS \((.+)\) (STORED|VIRTUAL)~i', $sql, $matches, PREG_SET_ORDER);
		foreach ($matches as $match) {
			$name = str_replace('""', '"', preg_replace('~^"|"$~', '', $match[1]));
			$return[$name]["default"] = $match[3];
			$return[$name]["generated"] = strtoupper($match[4]);
		}
		return $return;
	}

	function indexes($table, $connection2 = null) {
		global $connection;
		if (!is_object($connection2)) {
			$connection2 = $connection;
		}
		$return = array();
		$sql = $connection2->result("SELECT sql FROM sqlite_master WHERE type = 'table' AND name = " . q($table));
		if (preg_match('~\bPRIMARY\s+KEY\s*\((([^)"]+|"[^"]*"|`[^`]*`)++)~i', $sql, $match)) {
			$return[""] = array("type" => "PRIMARY", "columns" => array(), "lengths" => array(), "descs" => array());
			preg_match_all('~((("[^"]*+")+|(?:`[^`]*+`)+)|(\S+))(\s+(ASC|DESC))?(,\s*|$)~i', $match[1], $matches, PREG_SET_ORDER);
			foreach ($matches as $match) {
				$return[""]["columns"][] = idf_unescape($match[2]) . $match[4];
				$return[""]["descs"][] = (preg_match('~DESC~i', $match[5]) ? '1' : null);
			}
		}
		if (!$return) {
			foreach (fields($table) as $name => $field) {
				if ($field["primary"]) {
					$return[""] = array("type" => "PRIMARY", "columns" => array($name), "lengths" => array(), "descs" => array(null));
				}
			}
		}
		$sqls = get_key_vals("SELECT name, sql FROM sqlite_master WHERE type = 'index' AND tbl_name = " . q($table), $connection2);
		foreach (get_rows("PRAGMA index_list(" . table($table) . ")", $connection2) as $row) {
			$name = $row["name"];
			$index = array("type" => ($row["unique"] ? "UNIQUE" : "INDEX"));
			$index["lengths"] = array();
			$index["descs"] = array();
			foreach (get_rows("PRAGMA index_info(" . idf_escape($name) . ")", $connection2) as $row1) {
				$index["columns"][] = $row1["name"];
				$index["descs"][] = null;
			}
			if (preg_match('~^CREATE( UNIQUE)? INDEX ' . preg_quote(idf_escape($name) . ' ON ' . idf_escape($table), '~') . ' \((.*)\)$~i', $sqls[$name], $regs)) {
				preg_match_all('/("[^"]*+")+( DESC)?/', $regs[2], $matches);
				foreach ($matches[2] as $key => $val) {
					if ($val) {
						$index["descs"][$key] = '1';
					}
				}
			}
			if (!$return[""] || $index["type"] != "UNIQUE" || $index["columns"] != $return[""]["columns"] || $index["descs"] != $return[""]["descs"] || !preg_match("~^sqlite_~", $name)) {
				$return[$name] = $index;
			}
		}
		return $return;
	}

	function foreign_keys($table) {
		$return = array();
		foreach (get_rows("PRAGMA foreign_key_list(" . table($table) . ")") as $row) {
			$foreign_key = &$return[$row["id"]];
			if (!$foreign_key) {
				$foreign_key = $row;
			}
			$foreign_key["source"][] = $row["from"];
			$foreign_key["target"][] = $row["to"];
		}
		return $return;
	}

	function view($name) {
		return array("select" => preg_replace('~^(?:[^`"[]+|`[^`]*`|"[^"]*")* AS\s+~iU', '', get_val("SELECT sql FROM sqlite_master WHERE type = 'view' AND name = " . q($name)))); //! identifiers may be inside []
	}

	function collations() {
		return (isset($_GET["create"]) ? get_vals("PRAGMA collation_list", 1) : array());
	}

	function information_schema($db) {
		return false;
	}

	function error() {
		global $connection;
		return h($connection->error);
	}

	function check_sqlite_name($name) {
		// avoid creating PHP files on unsecured servers
		global $connection;
		$extensions = "db|sdb|sqlite";
		if (!preg_match("~^[^\\0]*\\.($extensions)\$~", $name)) {
			$connection->error = lang(23, str_replace("|", ", ", $extensions));
			return false;
		}
		return true;
	}

	function create_database($db, $collation) {
		global $connection;
		if (file_exists($db)) {
			$connection->error = lang(24);
			return false;
		}
		if (!check_sqlite_name($db)) {
			return false;
		}
		try {
			$link = new SqliteDb($db);
		} catch (\Exception $ex) {
			$connection->error = $ex->getMessage();
			return false;
		}
		$link->query('PRAGMA encoding = "UTF-8"');
		$link->query('CREATE TABLE adminer (i)'); // otherwise creates empty file
		$link->query('DROP TABLE adminer');
		return true;
	}

	function drop_databases($databases) {
		global $connection;
		$connection->__construct(":memory:"); // to unlock file, doesn't work in PDO on Windows
		foreach ($databases as $db) {
			if (!@unlink($db)) {
				$connection->error = lang(24);
				return false;
			}
		}
		return true;
	}

	function rename_database($name, $collation) {
		global $connection;
		if (!check_sqlite_name($name)) {
			return false;
		}
		$connection->__construct(":memory:");
		$connection->error = lang(24);
		return @rename(DB, $name);
	}

	function auto_increment() {
		return " PRIMARY KEY AUTOINCREMENT";
	}

	function alter_table($table, $name, $fields, $foreign, $comment, $engine, $collation, $auto_increment, $partitioning) {
		global $connection;
		$use_all_fields = ($table == "" || $foreign);
		foreach ($fields as $field) {
			if ($field[0] != "" || !$field[1] || $field[2]) {
				$use_all_fields = true;
				break;
			}
		}
		$alter = array();
		$originals = array();
		foreach ($fields as $field) {
			if ($field[1]) {
				$alter[] = ($use_all_fields ? $field[1] : "ADD " . implode($field[1]));
				if ($field[0] != "") {
					$originals[$field[0]] = $field[1][0];
				}
			}
		}
		if (!$use_all_fields) {
			foreach ($alter as $val) {
				if (!queries("ALTER TABLE " . table($table) . " $val")) {
					return false;
				}
			}
			if ($table != $name && !queries("ALTER TABLE " . table($table) . " RENAME TO " . table($name))) {
				return false;
			}
		} elseif (!recreate_table($table, $name, $alter, $originals, $foreign, $auto_increment)) {
			return false;
		}
		if ($auto_increment) {
			queries("BEGIN");
			queries("UPDATE sqlite_sequence SET seq = $auto_increment WHERE name = " . q($name)); // ignores error
			if (!$connection->affected_rows) {
				queries("INSERT INTO sqlite_sequence (name, seq) VALUES (" . q($name) . ", $auto_increment)");
			}
			queries("COMMIT");
		}
		return true;
	}

	/** Recreate table
	* @param string original name
	* @param string new name
	* @param array [process_field()], empty to preserve
	* @param array [$original => idf_escape($new_column)], empty to preserve
	* @param string [format_foreign_key()], empty to preserve
	* @param int set auto_increment to this value, 0 to preserve
	* @param array [[$type, $name, $columns]], empty to preserve
	* @param string CHECK constraint to drop
	* @param string CHECK constraint to add
	* @return bool
	*/
	function recreate_table($table, $name, $fields, $originals, $foreign, $auto_increment = 0, $indexes = array(), $drop_check = "", $add_check = "") {
		global $driver;
		if ($table != "") {
			if (!$fields) {
				foreach (fields($table) as $key => $field) {
					if ($indexes) {
						$field["auto_increment"] = 0;
					}
					$fields[] = process_field($field, $field);
					$originals[$key] = idf_escape($key);
				}
			}
			$primary_key = false;
			foreach ($fields as $field) {
				if ($field[6]) {
					$primary_key = true;
				}
			}
			$drop_indexes = array();
			foreach ($indexes as $key => $val) {
				if ($val[2] == "DROP") {
					$drop_indexes[$val[1]] = true;
					unset($indexes[$key]);
				}
			}
			foreach (indexes($table) as $key_name => $index) {
				$columns = array();
				foreach ($index["columns"] as $key => $column) {
					if (!$originals[$column]) {
						continue 2;
					}
					$columns[] = $originals[$column] . ($index["descs"][$key] ? " DESC" : "");
				}
				if (!$drop_indexes[$key_name]) {
					if ($index["type"] != "PRIMARY" || !$primary_key) {
						$indexes[] = array($index["type"], $key_name, $columns);
					}
				}
			}
			foreach ($indexes as $key => $val) {
				if ($val[0] == "PRIMARY") {
					unset($indexes[$key]);
					$foreign[] = "  PRIMARY KEY (" . implode(", ", $val[2]) . ")";
				}
			}
			foreach (foreign_keys($table) as $key_name => $foreign_key) {
				foreach ($foreign_key["source"] as $key => $column) {
					if (!$originals[$column]) {
						continue 2;
					}
					$foreign_key["source"][$key] = idf_unescape($originals[$column]);
				}
				if (!isset($foreign[" $key_name"])) {
					$foreign[] = " " . format_foreign_key($foreign_key);
				}
			}
			queries("BEGIN");
		}
		foreach ($fields as $key => $field) {
			if (preg_match('~GENERATED~', $field[3])) {
				unset($originals[array_search($field[0], $originals)]);
			}
			$fields[$key] = "  " . implode($field);
		}
		$fields = array_merge($fields, array_filter($foreign));
		foreach ($driver->checkConstraints($table) as $check) {
			if ($check != $drop_check) {
				$fields[] = "  CHECK ($check)";
			}
		}
		if ($add_check) {
			$fields[] = "  CHECK ($add_check)";
		}
		$temp_name = ($table == $name ? "adminer_$name" : $name);
		if (!queries("CREATE TABLE " . table($temp_name) . " (\n" . implode(",\n", $fields) . "\n)")) {
			// implicit ROLLBACK to not overwrite $connection->error
			return false;
		}
		if ($table != "") {
			if ($originals && !queries("INSERT INTO " . table($temp_name) . " (" . implode(", ", $originals) . ") SELECT " . implode(", ", array_map('Adminer\idf_escape', array_keys($originals))) . " FROM " . table($table))) {
				return false;
			}
			$triggers = array();
			foreach (triggers($table) as $trigger_name => $timing_event) {
				$trigger = trigger($trigger_name);
				$triggers[] = "CREATE TRIGGER " . idf_escape($trigger_name) . " " . implode(" ", $timing_event) . " ON " . table($name) . "\n$trigger[Statement]";
			}
			$auto_increment = $auto_increment ? 0 : get_val("SELECT seq FROM sqlite_sequence WHERE name = " . q($table)); // if $auto_increment is set then it will be updated later
			if (
				!queries("DROP TABLE " . table($table)) // drop before creating indexes and triggers to allow using old names
				|| ($table == $name && !queries("ALTER TABLE " . table($temp_name) . " RENAME TO " . table($name)))
				|| !alter_indexes($name, $indexes)
			) {
				return false;
			}
			if ($auto_increment) {
				queries("UPDATE sqlite_sequence SET seq = $auto_increment WHERE name = " . q($name)); // ignores error
			}
			foreach ($triggers as $trigger) {
				if (!queries($trigger)) {
					return false;
				}
			}
			queries("COMMIT");
		}
		return true;
	}

	function index_sql($table, $type, $name, $columns) {
		return "CREATE $type " . ($type != "INDEX" ? "INDEX " : "")
			. idf_escape($name != "" ? $name : uniqid($table . "_"))
			. " ON " . table($table)
			. " $columns"
		;
	}

	function alter_indexes($table, $alter) {
		foreach ($alter as $primary) {
			if ($primary[0] == "PRIMARY") {
				return recreate_table($table, $table, array(), array(), array(), 0, $alter);
			}
		}
		foreach (array_reverse($alter) as $val) {
			if (
				!queries($val[2] == "DROP"
				? "DROP INDEX " . idf_escape($val[1])
				: index_sql($table, $val[0], $val[1], "(" . implode(", ", $val[2]) . ")"))
			) {
				return false;
			}
		}
		return true;
	}

	function truncate_tables($tables) {
		return apply_queries("DELETE FROM", $tables);
	}

	function drop_views($views) {
		return apply_queries("DROP VIEW", $views);
	}

	function drop_tables($tables) {
		return apply_queries("DROP TABLE", $tables);
	}

	function move_tables($tables, $views, $target) {
		return false;
	}

	function trigger($name) {
		if ($name == "") {
			return array("Statement" => "BEGIN\n\t;\nEND");
		}
		$idf = '(?:[^`"\s]+|`[^`]*`|"[^"]*")+';
		$trigger_options = trigger_options();
		preg_match(
			"~^CREATE\\s+TRIGGER\\s*$idf\\s*(" . implode("|", $trigger_options["Timing"]) . ")\\s+([a-z]+)(?:\\s+OF\\s+($idf))?\\s+ON\\s*$idf\\s*(?:FOR\\s+EACH\\s+ROW\\s)?(.*)~is",
			get_val("SELECT sql FROM sqlite_master WHERE type = 'trigger' AND name = " . q($name)),
			$match
		);
		$of = $match[3];
		return array(
			"Timing" => strtoupper($match[1]),
			"Event" => strtoupper($match[2]) . ($of ? " OF" : ""),
			"Of" => idf_unescape($of),
			"Trigger" => $name,
			"Statement" => $match[4],
		);
	}

	function triggers($table) {
		$return = array();
		$trigger_options = trigger_options();
		foreach (get_rows("SELECT * FROM sqlite_master WHERE type = 'trigger' AND tbl_name = " . q($table)) as $row) {
			preg_match('~^CREATE\s+TRIGGER\s*(?:[^`"\s]+|`[^`]*`|"[^"]*")+\s*(' . implode("|", $trigger_options["Timing"]) . ')\s*(.*?)\s+ON\b~i', $row["sql"], $match);
			$return[$row["name"]] = array($match[1], $match[2]);
		}
		return $return;
	}

	function trigger_options() {
		return array(
			"Timing" => array("BEFORE", "AFTER", "INSTEAD OF"),
			"Event" => array("INSERT", "UPDATE", "UPDATE OF", "DELETE"),
			"Type" => array("FOR EACH ROW"),
		);
	}

	function begin() {
		return queries("BEGIN");
	}

	function last_id($result) {
		return get_val("SELECT LAST_INSERT_ROWID()");
	}

	function explain($connection, $query) {
		return $connection->query("EXPLAIN QUERY PLAN $query");
	}

	function found_rows($table_status, $where) {
	}

	function types() {
		return array();
	}

	function create_sql($table, $auto_increment, $style) {
		$return = get_val("SELECT sql FROM sqlite_master WHERE type IN ('table', 'view') AND name = " . q($table));
		foreach (indexes($table) as $name => $index) {
			if ($name == '') {
				continue;
			}
			$return .= ";\n\n" . index_sql($table, $index['type'], $name, "(" . implode(", ", array_map('Adminer\idf_escape', $index['columns'])) . ")");
		}
		return $return;
	}

	function truncate_sql($table) {
		return "DELETE FROM " . table($table);
	}

	function use_sql($database) {
	}

	function trigger_sql($table) {
		return implode(get_vals("SELECT sql || ';;\n' FROM sqlite_master WHERE type = 'trigger' AND tbl_name = " . q($table)));
	}

	function show_variables() {
		$return = array();
		foreach (get_rows("PRAGMA pragma_list") as $row) {
			$name = $row["name"];
			if ($name != "pragma_list" && $name != "compile_options") {
				$return[$name] = array($name, '');
				foreach (get_rows("PRAGMA $name") as $row) {
					$return[$name][1] .= implode(", ", $row) . "\n";
				}
			}
		}
		return $return;
	}

	function show_status() {
		$return = array();
		foreach (get_vals("PRAGMA compile_options") as $option) {
			$return[] = explode("=", $option, 2);
		}
		return $return;
	}

	function convert_field($field) {
	}

	function unconvert_field($field, $return) {
		return $return;
	}

	function support($feature) {
		return preg_match('~^(check|columns|database|drop_col|dump|indexes|descidx|move_col|sql|status|table|trigger|variables|view|view_trigger)$~', $feature);
	}
}

?>
<?php
$drivers["pgsql"] = "PostgreSQL";

if (isset($_GET["pgsql"])) {
	define('Adminer\DRIVER', "pgsql");
	if (extension_loaded("pgsql") && $_GET["ext"] != "pdo") {
		class Db {
			public $extension = "PgSQL", $flavor = '', $server_info, $affected_rows, $error, $timeout;
			private $link, $result, $string, $database = true;

			function _error($errno, $error) {
				if (ini_bool("html_errors")) {
					$error = html_entity_decode(strip_tags($error));
				}
				$error = preg_replace('~^[^:]*: ~', '', $error);
				$this->error = $error;
			}

			function connect($server, $username, $password) {
				global $adminer;
				$db = $adminer->database();
				set_error_handler(array($this, '_error'));
				$this->string = "host='" . str_replace(":", "' port='", addcslashes($server, "'\\")) . "' user='" . addcslashes($username, "'\\") . "' password='" . addcslashes($password, "'\\") . "'";
				$ssl = $adminer->connectSsl();
				if (isset($ssl["mode"])) {
					$this->string .= " sslmode='" . $ssl["mode"] . "'";
				}
				$this->link = @pg_connect("$this->string dbname='" . ($db != "" ? addcslashes($db, "'\\") : "postgres") . "'", PGSQL_CONNECT_FORCE_NEW);
				if (!$this->link && $db != "") {
					// try to connect directly with database for performance
					$this->database = false;
					$this->link = @pg_connect("$this->string dbname='postgres'", PGSQL_CONNECT_FORCE_NEW);
				}
				restore_error_handler();
				if ($this->link) {
					pg_set_client_encoding($this->link, "UTF8");
				}
				return (bool) $this->link;
			}

			function quote($string) {
				return (function_exists('pg_escape_literal')
					? pg_escape_literal($this->link, $string) // available since PHP 5.4.4
					: "'" . pg_escape_string($this->link, $string) . "'"
				);
			}

			function value($val, $field) {
				return ($field["type"] == "bytea" && $val !== null ? pg_unescape_bytea($val) : $val);
			}

			function select_db($database) {
				global $adminer;
				if ($database == $adminer->database()) {
					return $this->database;
				}
				$return = @pg_connect("$this->string dbname='" . addcslashes($database, "'\\") . "'", PGSQL_CONNECT_FORCE_NEW);
				if ($return) {
					$this->link = $return;
				}
				return $return;
			}

			function close() {
				$this->link = @pg_connect("$this->string dbname='postgres'");
			}

			function query($query, $unbuffered = false) {
				$result = @pg_query($this->link, $query);
				$this->error = "";
				if (!$result) {
					$this->error = pg_last_error($this->link);
					$return = false;
				} elseif (!pg_num_fields($result)) {
					$this->affected_rows = pg_affected_rows($result);
					$return = true;
				} else {
					$return = new Result($result);
				}
				if ($this->timeout) {
					$this->timeout = 0;
					$this->query("RESET statement_timeout");
				}
				return $return;
			}

			function multi_query($query) {
				return $this->result = $this->query($query);
			}

			function store_result() {
				return $this->result;
			}

			function next_result() {
				// PgSQL extension doesn't support multiple results
				return false;
			}

			function result($query, $field = 0) {
				$result = $this->query($query);
				return ($result ? $result->fetch_column($field) : false);
			}

			function warnings() {
				return h(pg_last_notice($this->link)); // second parameter is available since PHP 7.1.0
			}
		}

		class Result {
			public $num_rows;
			private $result, $offset = 0;

			function __construct($result) {
				$this->result = $result;
				$this->num_rows = pg_num_rows($result);
			}

			function fetch_assoc() {
				return pg_fetch_assoc($this->result);
			}

			function fetch_row() {
				return pg_fetch_row($this->result);
			}

			function fetch_column($field) {
				return ($this->num_rows ? pg_fetch_result($this->result, 0, $field) : false);
			}

			function fetch_field() {
				$column = $this->offset++;
				$return = new \stdClass;
				$return->orgtable = pg_field_table($this->result, $column);
				$return->name = pg_field_name($this->result, $column);
				$return->type = pg_field_type($this->result, $column); //! map to MySQL numbers
				$return->charsetnr = ($return->type == "bytea" ? 63 : 0); // 63 - binary
				return $return;
			}

			function __destruct() {
				pg_free_result($this->result);
			}
		}

	} elseif (extension_loaded("pdo_pgsql")) {
		class Db extends PdoDb {
			public $extension = "PDO_PgSQL", $timeout;

			function connect($server, $username, $password) {
				global $adminer;
				$db = $adminer->database();
				//! client_encoding is supported since 9.1, but we can't yet use min_version here
				$dsn = "pgsql:host='" . str_replace(":", "' port='", addcslashes($server, "'\\")) . "' client_encoding=utf8 dbname='" . ($db != "" ? addcslashes($db, "'\\") : "postgres") . "'";
				$ssl = $adminer->connectSsl();
				if (isset($ssl["mode"])) {
					$dsn .= " sslmode='" . $ssl["mode"] . "'";
				}
				$this->dsn($dsn, $username, $password);
				return true;
			}

			function select_db($database) {
				global $adminer;
				return ($adminer->database() == $database);
			}

			function query($query, $unbuffered = false) {
				$return = parent::query($query, $unbuffered);
				if ($this->timeout) {
					$this->timeout = 0;
					parent::query("RESET statement_timeout");
				}
				return $return;
			}

			function warnings() {
				return ''; // not implemented in PDO_PgSQL as of PHP 7.2.1
			}

			function close() {
			}
		}

	}



	class Driver extends SqlDriver {
		static $possibleDrivers = array("PgSQL", "PDO_PgSQL");
		static $jush = "pgsql";

		public $operators = array("=", "<", ">", "<=", ">=", "!=", "~", "!~", "LIKE", "LIKE %%", "ILIKE", "ILIKE %%", "IN", "IS NULL", "NOT LIKE", "NOT IN", "IS NOT NULL"); // no "SQL" to avoid CSRF
		public $functions = array("char_length", "lower", "round", "to_hex", "to_timestamp", "upper");
		public $grouping = array("avg", "count", "count distinct", "max", "min", "sum");

		function __construct($connection) {
			parent::__construct($connection);
			$this->types = array( //! arrays
				lang(25) => array("smallint" => 5, "integer" => 10, "bigint" => 19, "boolean" => 1, "numeric" => 0, "real" => 7, "double precision" => 16, "money" => 20),
				lang(26) => array("date" => 13, "time" => 17, "timestamp" => 20, "timestamptz" => 21, "interval" => 0),
				lang(27) => array("character" => 0, "character varying" => 0, "text" => 0, "tsquery" => 0, "tsvector" => 0, "uuid" => 0, "xml" => 0),
				lang(28) => array("bit" => 0, "bit varying" => 0, "bytea" => 0),
				lang(29) => array("cidr" => 43, "inet" => 43, "macaddr" => 17, "macaddr8" => 23, "txid_snapshot" => 0),
				lang(30) => array("box" => 0, "circle" => 0, "line" => 0, "lseg" => 0, "path" => 0, "point" => 0, "polygon" => 0),
			);
			if (min_version(9.2, 0, $connection)) {
				$this->types[lang(27)]["json"] = 4294967295;
				if (min_version(9.4, 0, $connection)) {
					$this->types[lang(27)]["jsonb"] = 4294967295;
				}
			}
			$this->editFunctions = array(
				array(
					"char" => "md5",
					"date|time" => "now",
				), array(
					number_type() => "+/-",
					"date|time" => "+ interval/- interval", //! escape
					"char|text" => "||",
				)
			);
			if (min_version(12, 0, $connection)) {
				$this->generated = array("STORED");
			}
		}

		function enumLength($field) {
			$enum = $this->types[lang(31)][$field["type"]];
			return ($enum ? type_values($enum) : "");
		}

		function setUserTypes($types) {
			$this->types[lang(31)] = array_flip($types);
		}

		function insertReturning($table) {
			$auto_increment = array_filter(fields($table), function ($field) {
				return $field['auto_increment'];
			});
			return (count($auto_increment) == 1 ? " RETURNING " . idf_escape(key($auto_increment)) : "");
		}

		function insertUpdate($table, $rows, $primary) {
			global $connection;
			foreach ($rows as $set) {
				$update = array();
				$where = array();
				foreach ($set as $key => $val) {
					$update[] = "$key = $val";
					if (isset($primary[idf_unescape($key)])) {
						$where[] = "$key = $val";
					}
				}
				if (
					!(($where && queries("UPDATE " . table($table) . " SET " . implode(", ", $update) . " WHERE " . implode(" AND ", $where)) && $connection->affected_rows)
					|| queries("INSERT INTO " . table($table) . " (" . implode(", ", array_keys($set)) . ") VALUES (" . implode(", ", $set) . ")"))
				) {
					return false;
				}
			}
			return true;
		}

		function slowQuery($query, $timeout) {
			$this->conn->query("SET statement_timeout = " . (1000 * $timeout));
			$this->conn->timeout = 1000 * $timeout;
			return $query;
		}

		function convertSearch($idf, $val, $field) {
			$textTypes = "char|text";
			if (strpos($val["op"], "LIKE") === false) {
				$textTypes .= "|date|time(stamp)?|boolean|uuid|inet|cidr|macaddr|" . number_type();
			}

			return (preg_match("~$textTypes~", $field["type"]) ? $idf : "CAST($idf AS text)");
		}

		function quoteBinary($s) {
			return "'\\x" . bin2hex($s) . "'"; // available since PostgreSQL 8.1
		}

		function warnings() {
			return $this->conn->warnings();
		}

		function tableHelp($name, $is_view = false) {
			$links = array(
				"information_schema" => "infoschema",
				"pg_catalog" => ($is_view ? "view" : "catalog"),
			);
			$link = $links[$_GET["ns"]];
			if ($link) {
				return "$link-" . str_replace("_", "-", $name) . ".html";
			}
		}

		function supportsIndex($table_status) {
			// returns true for "materialized view"
			return $table_status["Engine"] != "view";
		}

		function hasCStyleEscapes() {
			static $c_style;
			if ($c_style === null) {
				$c_style = ($this->conn->result("SHOW standard_conforming_strings") == "off");
			}
			return $c_style;
		}
	}



	function idf_escape($idf) {
		return '"' . str_replace('"', '""', $idf) . '"';
	}

	function table($idf) {
		return idf_escape($idf);
	}

	function connect($credentials) {
		global $drivers;
		$connection = new Db;
		if ($connection->connect($credentials[0], $credentials[1], $credentials[2])) {
			if (min_version(9, 0, $connection)) {
				$connection->query("SET application_name = 'Adminer'");
			}
			$version = $connection->result("SELECT version()");
			$connection->flavor = (preg_match('~CockroachDB~', $version) ? 'cockroach' : '');
			$connection->server_info = preg_replace('~^\D*([\d.]+[-\w]*).*~', '\1', $version);
			if ($connection->flavor == 'cockroach') { // we don't use "PostgreSQL / CockroachDB" by default because it's too long
				$drivers[DRIVER] = "CockroachDB";
			}
			return $connection;
		}
		return $connection->error;
	}

	function get_databases() {
		return get_vals("SELECT datname FROM pg_database
WHERE datallowconn = TRUE AND has_database_privilege(datname, 'CONNECT')
ORDER BY datname");
	}

	function limit($query, $where, $limit, $offset = 0, $separator = " ") {
		return " $query$where" . ($limit !== null ? $separator . "LIMIT $limit" . ($offset ? " OFFSET $offset" : "") : "");
	}

	function limit1($table, $query, $where, $separator = "\n") {
		return (preg_match('~^INTO~', $query)
			? limit($query, $where, 1, 0, $separator)
			: " $query" . (is_view(table_status1($table)) ? $where : $separator . "WHERE ctid = (SELECT ctid FROM " . table($table) . $where . $separator . "LIMIT 1)")
		);
	}

	function db_collation($db, $collations) {
		return get_val("SELECT datcollate FROM pg_database WHERE datname = " . q($db));
	}

	function logged_user() {
		return get_val("SELECT user");
	}

	function tables_list() {
		$query = "SELECT table_name, table_type FROM information_schema.tables WHERE table_schema = current_schema()";
		if (support("materializedview")) {
			$query .= "
UNION ALL
SELECT matviewname, 'MATERIALIZED VIEW'
FROM pg_matviews
WHERE schemaname = current_schema()";
		}
		$query .= "
ORDER BY 1";
		return get_key_vals($query);
	}

	function count_tables($databases) {
		global $connection;
		$return = array();
		foreach ($databases as $db) {
			if ($connection->select_db($db)) {
				$return[$db] = count(tables_list());
			}
		}
		return $return;
	}

	function table_status($name = "") {
		static $has_size;
		if ($has_size === null) {
			// https://github.com/cockroachdb/cockroach/issues/40391
			$has_size = get_val("SELECT 'pg_table_size'::regproc");
		}
		$return = array();
		foreach (
			get_rows("SELECT
	c.relname AS \"Name\",
	CASE c.relkind WHEN 'r' THEN 'table' WHEN 'm' THEN 'materialized view' ELSE 'view' END AS \"Engine\"" . ($has_size ? ",
	pg_table_size(c.oid) AS \"Data_length\",
	pg_indexes_size(c.oid) AS \"Index_length\"" : "") . ",
	obj_description(c.oid, 'pg_class') AS \"Comment\",
	" . (min_version(12) ? "''" : "CASE WHEN c.relhasoids THEN 'oid' ELSE '' END") . " AS \"Oid\",
	c.reltuples as \"Rows\",
	n.nspname
FROM pg_class c
JOIN pg_namespace n ON(n.nspname = current_schema() AND n.oid = c.relnamespace)
WHERE relkind IN ('r', 'm', 'v', 'f', 'p')
" . ($name != "" ? "AND relname = " . q($name) : "ORDER BY relname")) as $row //! Index_length, Auto_increment
		) {
			$return[$row["Name"]] = $row;
		}
		return ($name != "" ? $return[$name] : $return);
	}

	function is_view($table_status) {
		return in_array($table_status["Engine"], array("view", "materialized view"));
	}

	function fk_support($table_status) {
		return true;
	}

	function fields($table) {
		$return = array();
		$aliases = array(
			'timestamp without time zone' => 'timestamp',
			'timestamp with time zone' => 'timestamptz',
		);
		foreach (
			get_rows("SELECT
	a.attname AS field,
	format_type(a.atttypid, a.atttypmod) AS full_type,
	pg_get_expr(d.adbin, d.adrelid) AS default,
	a.attnotnull::int,
	col_description(c.oid, a.attnum) AS comment" . (min_version(10) ? ",
	a.attidentity" . (min_version(12) ? ",
	a.attgenerated" : "") : "") . "
FROM pg_class c
JOIN pg_namespace n ON c.relnamespace = n.oid
JOIN pg_attribute a ON c.oid = a.attrelid
LEFT JOIN pg_attrdef d ON c.oid = d.adrelid AND a.attnum = d.adnum
WHERE c.relname = " . q($table) . "
AND n.nspname = current_schema()
AND NOT a.attisdropped
AND a.attnum > 0
ORDER BY a.attnum") as $row
		) {
			//! collation, primary
			preg_match('~([^([]+)(\((.*)\))?([a-z ]+)?((\[[0-9]*])*)$~', $row["full_type"], $match);
			list(, $type, $length, $row["length"], $addon, $array) = $match;
			$row["length"] .= $array;
			$check_type = $type . $addon;
			if (isset($aliases[$check_type])) {
				$row["type"] = $aliases[$check_type];
				$row["full_type"] = $row["type"] . $length . $array;
			} else {
				$row["type"] = $type;
				$row["full_type"] = $row["type"] . $length . $addon . $array;
			}
			if (in_array($row['attidentity'], array('a', 'd'))) {
				$row['default'] = 'GENERATED ' . ($row['attidentity'] == 'd' ? 'BY DEFAULT' : 'ALWAYS') . ' AS IDENTITY';
			}
			$row["generated"] = ($row["attgenerated"] == "s" ? "STORED" : "");
			$row["null"] = !$row["attnotnull"];
			$row["auto_increment"] = $row['attidentity'] || preg_match('~^nextval\(~i', $row["default"])
				|| preg_match('~^unique_rowid\(~', $row["default"]); // CockroachDB
			$row["privileges"] = array("insert" => 1, "select" => 1, "update" => 1, "where" => 1, "order" => 1);
			if (preg_match('~(.+)::[^,)]+(.*)~', $row["default"], $match)) {
				$row["default"] = ($match[1] == "NULL" ? null : idf_unescape($match[1]) . $match[2]);
			}
			$return[$row["field"]] = $row;
		}
		return $return;
	}

	function indexes($table, $connection2 = null) {
		global $connection;
		if (!is_object($connection2)) {
			$connection2 = $connection;
		}
		$return = array();
		$table_oid = $connection2->result("SELECT oid FROM pg_class WHERE relnamespace = (SELECT oid FROM pg_namespace WHERE nspname = current_schema()) AND relname = " . q($table));
		$columns = get_key_vals("SELECT attnum, attname FROM pg_attribute WHERE attrelid = $table_oid AND attnum > 0", $connection2);
		foreach (
			get_rows("SELECT relname, indisunique::int, indisprimary::int, indkey, indoption, (indpred IS NOT NULL)::int as indispartial
FROM pg_index i, pg_class ci
WHERE i.indrelid = $table_oid AND ci.oid = i.indexrelid
ORDER BY indisprimary DESC, indisunique DESC", $connection2) as $row
		) {
			$relname = $row["relname"];
			$return[$relname]["type"] = ($row["indispartial"] ? "INDEX" : ($row["indisprimary"] ? "PRIMARY" : ($row["indisunique"] ? "UNIQUE" : "INDEX")));
			$return[$relname]["columns"] = array();
			$return[$relname]["descs"] = array();
			if ($row["indkey"]) {
				foreach (explode(" ", $row["indkey"]) as $indkey) {
					$return[$relname]["columns"][] = $columns[$indkey];
				}
				foreach (explode(" ", $row["indoption"]) as $indoption) {
					$return[$relname]["descs"][] = ($indoption & 1 ? '1' : null); // 1 - INDOPTION_DESC
				}
			}
			$return[$relname]["lengths"] = array();
		}
		return $return;
	}

	function foreign_keys($table) {
		global $driver;
		$return = array();
		foreach (
			get_rows("SELECT conname, condeferrable::int AS deferrable, pg_get_constraintdef(oid) AS definition
FROM pg_constraint
WHERE conrelid = (SELECT pc.oid FROM pg_class AS pc INNER JOIN pg_namespace AS pn ON (pn.oid = pc.relnamespace) WHERE pc.relname = " . q($table) . " AND pn.nspname = current_schema())
AND contype = 'f'::char
ORDER BY conkey, conname") as $row
		) {
			if (preg_match('~FOREIGN KEY\s*\((.+)\)\s*REFERENCES (.+)\((.+)\)(.*)$~iA', $row['definition'], $match)) {
				$row['source'] = array_map('Adminer\idf_unescape', array_map('trim', explode(',', $match[1])));
				if (preg_match('~^(("([^"]|"")+"|[^"]+)\.)?"?("([^"]|"")+"|[^"]+)$~', $match[2], $match2)) {
					$row['ns'] = idf_unescape($match2[2]);
					$row['table'] = idf_unescape($match2[4]);
				}
				$row['target'] = array_map('Adminer\idf_unescape', array_map('trim', explode(',', $match[3])));
				$row['on_delete'] = (preg_match("~ON DELETE ($driver->onActions)~", $match[4], $match2) ? $match2[1] : 'NO ACTION');
				$row['on_update'] = (preg_match("~ON UPDATE ($driver->onActions)~", $match[4], $match2) ? $match2[1] : 'NO ACTION');
				$return[$row['conname']] = $row;
			}
		}
		return $return;
	}

	function view($name) {
		return array("select" => trim(get_val("SELECT pg_get_viewdef(" . get_val("SELECT oid FROM pg_class WHERE relnamespace = (SELECT oid FROM pg_namespace WHERE nspname = current_schema()) AND relname = " . q($name)) . ")")));
	}

	function collations() {
		//! supported in CREATE DATABASE
		return array();
	}

	function information_schema($db) {
		return get_schema() == "information_schema";
	}

	function error() {
		global $connection;
		$return = h($connection->error);
		if (preg_match('~^(.*\n)?([^\n]*)\n( *)\^(\n.*)?$~s', $return, $match)) {
			$return = $match[1] . preg_replace('~((?:[^&]|&[^;]*;){' . strlen($match[3]) . '})(.*)~', '\1<b>\2</b>', $match[2]) . $match[4];
		}
		return nl_br($return);
	}

	function create_database($db, $collation) {
		return queries("CREATE DATABASE " . idf_escape($db) . ($collation ? " ENCODING " . idf_escape($collation) : ""));
	}

	function drop_databases($databases) {
		global $connection;
		$connection->close();
		return apply_queries("DROP DATABASE", $databases, 'Adminer\idf_escape');
	}

	function rename_database($name, $collation) {
		global $connection;
		$connection->close();
		return queries("ALTER DATABASE " . idf_escape(DB) . " RENAME TO " . idf_escape($name));
	}

	function auto_increment() {
		return "";
	}

	function alter_table($table, $name, $fields, $foreign, $comment, $engine, $collation, $auto_increment, $partitioning) {
		$alter = array();
		$queries = array();
		if ($table != "" && $table != $name) {
			$queries[] = "ALTER TABLE " . table($table) . " RENAME TO " . table($name);
		}
		$sequence = "";
		foreach ($fields as $field) {
			$column = idf_escape($field[0]);
			$val = $field[1];
			if (!$val) {
				$alter[] = "DROP $column";
			} else {
				$val5 = $val[5];
				unset($val[5]);
				if ($field[0] == "") {
					if (isset($val[6])) { // auto_increment
						$val[1] = ($val[1] == " bigint" ? " big" : ($val[1] == " smallint" ? " small" : " ")) . "serial";
					}
					$alter[] = ($table != "" ? "ADD " : "  ") . implode($val);
					if (isset($val[6])) {
						$alter[] = ($table != "" ? "ADD" : " ") . " PRIMARY KEY ($val[0])";
					}
				} else {
					if ($column != $val[0]) {
						$queries[] = "ALTER TABLE " . table($name) . " RENAME $column TO $val[0]";
					}
					$alter[] = "ALTER $column TYPE$val[1]";
					$sequence_name = $table . "_" . idf_unescape($val[0]) . "_seq";
					$alter[] = "ALTER $column " . ($val[3] ? "SET" . preg_replace('~GENERATED ALWAYS(.*) STORED~', 'EXPRESSION\1', $val[3])
						: (isset($val[6]) ? "SET DEFAULT nextval(" . q($sequence_name) . ")"
						: "DROP DEFAULT" //! change to DROP EXPRESSION with generated columns
					));
					if (isset($val[6])) {
						$sequence = "CREATE SEQUENCE IF NOT EXISTS " . idf_escape($sequence_name) . " OWNED BY " . idf_escape($table) . ".$val[0]";
					}
					$alter[] = "ALTER $column " . ($val[2] == " NULL" ? "DROP NOT" : "SET") . $val[2];
				}
				if ($field[0] != "" || $val5 != "") {
					$queries[] = "COMMENT ON COLUMN " . table($name) . ".$val[0] IS " . ($val5 != "" ? substr($val5, 9) : "''");
				}
			}
		}
		$alter = array_merge($alter, $foreign);
		if ($table == "") {
			array_unshift($queries, "CREATE TABLE " . table($name) . " (\n" . implode(",\n", $alter) . "\n)");
		} elseif ($alter) {
			array_unshift($queries, "ALTER TABLE " . table($table) . "\n" . implode(",\n", $alter));
		}
		if ($sequence) {
			array_unshift($queries, $sequence);
		}
		if ($comment !== null) {
			$queries[] = "COMMENT ON TABLE " . table($name) . " IS " . q($comment);
		}
		// if ($auto_increment != "") {
			//! $queries[] = "SELECT setval(pg_get_serial_sequence(" . q($name) . ", ), $auto_increment)";
		// }
		foreach ($queries as $query) {
			if (!queries($query)) {
				return false;
			}
		}
		return true;
	}

	function alter_indexes($table, $alter) {
		$create = array();
		$drop = array();
		$queries = array();
		foreach ($alter as $val) {
			if ($val[0] != "INDEX") {
				//! descending UNIQUE indexes results in syntax error
				$create[] = ($val[2] == "DROP"
					? "\nDROP CONSTRAINT " . idf_escape($val[1])
					: "\nADD" . ($val[1] != "" ? " CONSTRAINT " . idf_escape($val[1]) : "") . " $val[0] " . ($val[0] == "PRIMARY" ? "KEY " : "") . "(" . implode(", ", $val[2]) . ")"
				);
			} elseif ($val[2] == "DROP") {
				$drop[] = idf_escape($val[1]);
			} else {
				$queries[] = "CREATE INDEX " . idf_escape($val[1] != "" ? $val[1] : uniqid($table . "_")) . " ON " . table($table) . " (" . implode(", ", $val[2]) . ")";
			}
		}
		if ($create) {
			array_unshift($queries, "ALTER TABLE " . table($table) . implode(",", $create));
		}
		if ($drop) {
			array_unshift($queries, "DROP INDEX " . implode(", ", $drop));
		}
		foreach ($queries as $query) {
			if (!queries($query)) {
				return false;
			}
		}
		return true;
	}

	function truncate_tables($tables) {
		return queries("TRUNCATE " . implode(", ", array_map('Adminer\table', $tables)));
	}

	function drop_views($views) {
		return drop_tables($views);
	}

	function drop_tables($tables) {
		foreach ($tables as $table) {
			$status = table_status($table);
			if (!queries("DROP " . strtoupper($status["Engine"]) . " " . table($table))) {
				return false;
			}
		}
		return true;
	}

	function move_tables($tables, $views, $target) {
		foreach (array_merge($tables, $views) as $table) {
			$status = table_status($table);
			if (!queries("ALTER " . strtoupper($status["Engine"]) . " " . table($table) . " SET SCHEMA " . idf_escape($target))) {
				return false;
			}
		}
		return true;
	}

	function trigger($name, $table) {
		if ($name == "") {
			return array("Statement" => "EXECUTE PROCEDURE ()");
		}
		$columns = array();
		$where = "WHERE trigger_schema = current_schema() AND event_object_table = " . q($table) . " AND trigger_name = " . q($name);
		foreach (get_rows("SELECT * FROM information_schema.triggered_update_columns $where") as $row) {
			$columns[] = $row["event_object_column"];
		}
		$return = array();
		foreach (
			get_rows('SELECT trigger_name AS "Trigger", action_timing AS "Timing", event_manipulation AS "Event", \'FOR EACH \' || action_orientation AS "Type", action_statement AS "Statement"
FROM information_schema.triggers' . "
$where
ORDER BY event_manipulation DESC") as $row
		) {
			if ($columns && $row["Event"] == "UPDATE") {
				$row["Event"] .= " OF";
			}
			$row["Of"] = implode(", ", $columns);
			if ($return) {
				$row["Event"] .= " OR $return[Event]";
			}
			$return = $row;
		}
		return $return;
	}

	function triggers($table) {
		$return = array();
		foreach (get_rows("SELECT * FROM information_schema.triggers WHERE trigger_schema = current_schema() AND event_object_table = " . q($table)) as $row) {
			$trigger = trigger($row["trigger_name"], $table);
			$return[$trigger["Trigger"]] = array($trigger["Timing"], $trigger["Event"]);
		}
		return $return;
	}

	function trigger_options() {
		return array(
			"Timing" => array("BEFORE", "AFTER"),
			"Event" => array("INSERT", "UPDATE", "UPDATE OF", "DELETE", "INSERT OR UPDATE", "INSERT OR UPDATE OF", "DELETE OR INSERT", "DELETE OR UPDATE", "DELETE OR UPDATE OF", "DELETE OR INSERT OR UPDATE", "DELETE OR INSERT OR UPDATE OF"),
			"Type" => array("FOR EACH ROW", "FOR EACH STATEMENT"),
		);
	}

	function routine($name, $type) {
		$rows = get_rows('SELECT routine_definition AS definition, LOWER(external_language) AS language, *
FROM information_schema.routines
WHERE routine_schema = current_schema() AND specific_name = ' . q($name));
		$return = $rows[0];
		$return["returns"] = array("type" => $return["type_udt_name"]);
		$return["fields"] = get_rows('SELECT parameter_name AS field, data_type AS type, character_maximum_length AS length, parameter_mode AS inout
FROM information_schema.parameters
WHERE specific_schema = current_schema() AND specific_name = ' . q($name) . '
ORDER BY ordinal_position');
		return $return;
	}

	function routines() {
		return get_rows('SELECT specific_name AS "SPECIFIC_NAME", routine_type AS "ROUTINE_TYPE", routine_name AS "ROUTINE_NAME", type_udt_name AS "DTD_IDENTIFIER"
FROM information_schema.routines
WHERE routine_schema = current_schema()
ORDER BY SPECIFIC_NAME');
	}

	function routine_languages() {
		return get_vals("SELECT LOWER(lanname) FROM pg_catalog.pg_language");
	}

	function routine_id($name, $row) {
		$return = array();
		foreach ($row["fields"] as $field) {
			$length = $field["length"];
			$return[] = $field["type"] . ($length ? "($length)" : "");
		}
		return idf_escape($name) . "(" . implode(", ", $return) . ")";
	}

	function last_id($result) {
		return (is_object($result) && $result->num_rows ? $result->fetch_column(0) : 0);
	}

	function explain($connection, $query) {
		return $connection->query("EXPLAIN $query");
	}

	function found_rows($table_status, $where) {
		if (preg_match("~ rows=([0-9]+)~", get_val("EXPLAIN SELECT * FROM " . idf_escape($table_status["Name"]) . ($where ? " WHERE " . implode(" AND ", $where) : "")), $regs)) {
			return $regs[1];
		}
		return false;
	}

	function types() {
		return get_key_vals(
			"SELECT oid, typname
FROM pg_type
WHERE typnamespace = (SELECT oid FROM pg_namespace WHERE nspname = current_schema())
AND typtype IN ('b','d','e')
AND typelem = 0"
		);
	}

	function type_values($id) {
		// to get values from type string: unnest(enum_range(NULL::"$type"))
		$enums = get_vals("SELECT enumlabel FROM pg_enum WHERE enumtypid = $id ORDER BY enumsortorder");
		return ($enums ? "'" . implode("', '", array_map('addslashes', $enums)) . "'" : "");
	}

	function schemas() {
		return get_vals("SELECT nspname FROM pg_namespace ORDER BY nspname");
	}

	function get_schema() {
		return get_val("SELECT current_schema()");
	}

	function set_schema($schema, $connection2 = null) {
		global $connection, $driver;
		if (!$connection2) {
			$connection2 = $connection;
		}
		$return = $connection2->query("SET search_path TO " . idf_escape($schema));
		$driver->setUserTypes(types()); //! get types from current_schemas('t')
		return $return;
	}

	// create_sql() produces CREATE TABLE without FK CONSTRAINTs
	// foreign_keys_sql() produces all FK CONSTRAINTs as ALTER TABLE ... ADD CONSTRAINT
	// so that all FKs can be added after all tables have been created, avoiding any need to reorder CREATE TABLE statements in order of their FK dependencies
	function foreign_keys_sql($table) {
		$return = "";

		$status = table_status($table);
		$fkeys = foreign_keys($table);
		ksort($fkeys);

		foreach ($fkeys as $fkey_name => $fkey) {
			$return .= "ALTER TABLE ONLY " . idf_escape($status['nspname']) . "." . idf_escape($status['Name']) . " ADD CONSTRAINT " . idf_escape($fkey_name) . " $fkey[definition] " . ($fkey['deferrable'] ? 'DEFERRABLE' : 'NOT DEFERRABLE') . ";\n";
		}

		return ($return ? "$return\n" : $return);
	}

	function create_sql($table, $auto_increment, $style) {
		global $driver;
		$return_parts = array();
		$sequences = array();

		$status = table_status($table);
		if (is_view($status)) {
			$view = view($table);
			return rtrim("CREATE VIEW " . idf_escape($table) . " AS $view[select]", ";");
		}
		$fields = fields($table);

		if (!$status || empty($fields)) {
			return false;
		}

		$return = "CREATE TABLE " . idf_escape($status['nspname']) . "." . idf_escape($status['Name']) . " (\n    ";

		// fields' definitions
		foreach ($fields as $field) {
			$part = idf_escape($field['field']) . ' ' . $field['full_type']
				. default_value($field)
				. ($field['attnotnull'] ? " NOT NULL" : "");
			$return_parts[] = $part;

			// sequences for fields
			if (preg_match('~nextval\(\'([^\']+)\'\)~', $field['default'], $matches)) {
				$sequence_name = $matches[1];
				$sq = first(get_rows((min_version(10)
					? "SELECT *, cache_size AS cache_value FROM pg_sequences WHERE schemaname = current_schema() AND sequencename = " . q(idf_unescape($sequence_name))
					: "SELECT * FROM $sequence_name"
				), null, "-- "));
				$sequences[] = ($style == "DROP+CREATE" ? "DROP SEQUENCE IF EXISTS $sequence_name;\n" : "")
					. "CREATE SEQUENCE $sequence_name INCREMENT $sq[increment_by] MINVALUE $sq[min_value] MAXVALUE $sq[max_value]"
					. ($auto_increment && $sq['last_value'] ? " START " . ($sq["last_value"] + 1) : "")
					. " CACHE $sq[cache_value];"
				;
			}
		}

		// adding sequences before table definition
		if (!empty($sequences)) {
			$return = implode("\n\n", $sequences) . "\n\n$return";
		}

		$primary = "";
		foreach (indexes($table) as $index_name => $index) {
			if ($index['type'] == 'PRIMARY') {
				$primary = $index_name;
				$return_parts[] = "CONSTRAINT " . idf_escape($index_name) . " PRIMARY KEY (" . implode(', ', array_map('Adminer\idf_escape', $index['columns'])) . ")";
			}
		}

		foreach ($driver->checkConstraints($table) as $conname => $consrc) {
			$return_parts[] = "CONSTRAINT " . idf_escape($conname) . " CHECK $consrc";
		}

		$return .= implode(",\n    ", $return_parts) . "\n) WITH (oids = " . ($status['Oid'] ? 'true' : 'false') . ");";

		// comments for table & fields
		if ($status['Comment']) {
			$return .= "\n\nCOMMENT ON TABLE " . idf_escape($status['nspname']) . "." . idf_escape($status['Name']) . " IS " . q($status['Comment']) . ";";
		}

		foreach ($fields as $field_name => $field) {
			if ($field['comment']) {
				$return .= "\n\nCOMMENT ON COLUMN " . idf_escape($status['nspname']) . "." . idf_escape($status['Name']) . "." . idf_escape($field_name) . " IS " . q($field['comment']) . ";";
			}
		}

		foreach (get_rows("SELECT indexdef FROM pg_catalog.pg_indexes WHERE schemaname = current_schema() AND tablename = " . q($table) . ($primary ? " AND indexname != " . q($primary) : ""), null, "-- ") as $row) {
			$return .= "\n\n$row[indexdef];";
		}

		return rtrim($return, ';');
	}

	function truncate_sql($table) {
		return "TRUNCATE " . table($table);
	}

	function trigger_sql($table) {
		$status = table_status($table);
		$return = "";
		foreach (triggers($table) as $trg_id => $trg) {
			$trigger = trigger($trg_id, $status['Name']);
			$return .= "\nCREATE TRIGGER " . idf_escape($trigger['Trigger']) . " $trigger[Timing] $trigger[Event] ON " . idf_escape($status["nspname"]) . "." . idf_escape($status['Name']) . " $trigger[Type] $trigger[Statement];;\n";
		}
		return $return;
	}


	function use_sql($database) {
		return "\connect " . idf_escape($database);
	}

	function show_variables() {
		return get_rows("SHOW ALL");
	}

	function process_list() {
		return get_rows("SELECT * FROM pg_stat_activity ORDER BY " . (min_version(9.2) ? "pid" : "procpid"));
	}

	function convert_field($field) {
	}

	function unconvert_field($field, $return) {
		return $return;
	}

	function support($feature) {
		global $connection;
		return preg_match('~^(check|database|table|columns|sql|indexes|descidx|comment|view|' . (min_version(9.3) ? 'materializedview|' : '') . 'scheme|' . (min_version(11) ? 'procedure|' : '') . 'routine|sequence|trigger|type|variables|drop_col'
			. ($connection->flavor == 'cockroach' ? '' : '|processlist') // https://github.com/cockroachdb/cockroach/issues/24745
			. '|kill|dump)$~', $feature)
		;
	}

	function kill_process($val) {
		return queries("SELECT pg_terminate_backend(" . number($val) . ")");
	}

	function connection_id() {
		return "SELECT pg_backend_pid()";
	}

	function max_connections() {
		return get_val("SHOW max_connections");
	}
}

?>
<?php
$drivers["oracle"] = "Oracle (beta)";

if (isset($_GET["oracle"])) {
	define('Adminer\DRIVER', "oracle");
	if (extension_loaded("oci8") && $_GET["ext"] != "pdo") {
		class Db {
			public $extension = "oci8", $flavor = '', $server_info, $affected_rows, $errno, $error;
			public $_current_db;
			private $link, $result;

			function _error($errno, $error) {
				if (ini_bool("html_errors")) {
					$error = html_entity_decode(strip_tags($error));
				}
				$error = preg_replace('~^[^:]*: ~', '', $error);
				$this->error = $error;
			}

			function connect($server, $username, $password) {
				$this->link = @oci_new_connect($username, $password, $server, "AL32UTF8");
				if ($this->link) {
					$this->server_info = oci_server_version($this->link);
					return true;
				}
				$error = oci_error();
				$this->error = $error["message"];
				return false;
			}

			function quote($string) {
				return "'" . str_replace("'", "''", $string) . "'";
			}

			function select_db($database) {
				$this->_current_db = $database;
				return true;
			}

			function query($query, $unbuffered = false) {
				$result = oci_parse($this->link, $query);
				$this->error = "";
				if (!$result) {
					$error = oci_error($this->link);
					$this->errno = $error["code"];
					$this->error = $error["message"];
					return false;
				}
				set_error_handler(array($this, '_error'));
				$return = @oci_execute($result);
				restore_error_handler();
				if ($return) {
					if (oci_num_fields($result)) {
						return new Result($result);
					}
					$this->affected_rows = oci_num_rows($result);
					oci_free_statement($result);
				}
				return $return;
			}

			function multi_query($query) {
				return $this->result = $this->query($query);
			}

			function store_result() {
				return $this->result;
			}

			function next_result() {
				return false;
			}

			function result($query, $field = 0) {
				$result = $this->query($query);
				return (is_object($result) ? $result->fetch_column($field) : false);
			}
		}

		class Result {
			public $num_rows;
			private $result, $offset = 1;

			function __construct($result) {
				$this->result = $result;
			}

			private function convert($row) {
				foreach ((array) $row as $key => $val) {
					if (is_a($val, 'OCI-Lob')) {
						$row[$key] = $val->load();
					}
				}
				return $row;
			}

			function fetch_assoc() {
				return $this->convert(oci_fetch_assoc($this->result));
			}

			function fetch_row() {
				return $this->convert(oci_fetch_row($this->result));
			}

			function fetch_column($field) {
				return (oci_fetch($this->result) ? oci_result($this->result, $field + 1) : false);
			}

			function fetch_field() {
				$column = $this->offset++;
				$return = new \stdClass;
				$return->name = oci_field_name($this->result, $column);
				$return->type = oci_field_type($this->result, $column); //! map to MySQL numbers
				$return->charsetnr = (preg_match("~raw|blob|bfile~", $return->type) ? 63 : 0); // 63 - binary
				return $return;
			}

			function __destruct() {
				oci_free_statement($this->result);
			}
		}

	} elseif (extension_loaded("pdo_oci")) {
		class Db extends PdoDb {
			public $extension = "PDO_OCI";
			public $_current_db;

			function connect($server, $username, $password) {
				$this->dsn("oci:dbname=//$server;charset=AL32UTF8", $username, $password);
				return true;
			}

			function select_db($database) {
				$this->_current_db = $database;
				return true;
			}
		}

	}



	class Driver extends SqlDriver {
		static $possibleDrivers = array("OCI8", "PDO_OCI");
		static $jush = "oracle";

		public $editFunctions = array(
			array( //! no parentheses
				"date" => "current_date",
				"timestamp" => "current_timestamp",
			), array(
				"number|float|double" => "+/-",
				"date|timestamp" => "+ interval/- interval",
				"char|clob" => "||",
			)
		);

		public $operators = array("=", "<", ">", "<=", ">=", "!=", "LIKE", "LIKE %%", "IN", "IS NULL", "NOT LIKE", "NOT IN", "IS NOT NULL", "SQL");
		public $functions = array("length", "lower", "round", "upper");
		public $grouping = array("avg", "count", "count distinct", "max", "min", "sum");

		function __construct($connection) {
			parent::__construct($connection);
			$this->types = array(
				lang(25) => array("number" => 38, "binary_float" => 12, "binary_double" => 21),
				lang(26) => array("date" => 10, "timestamp" => 29, "interval year" => 12, "interval day" => 28), //! year(), day() to second()
				lang(27) => array("char" => 2000, "varchar2" => 4000, "nchar" => 2000, "nvarchar2" => 4000, "clob" => 4294967295, "nclob" => 4294967295),
				lang(28) => array("raw" => 2000, "long raw" => 2147483648, "blob" => 4294967295, "bfile" => 4294967296),
			);
		}

		//! support empty $set in insert()

		function begin() {
			return true; // automatic start
		}

		function insertUpdate($table, $rows, $primary) {
			global $connection;
			foreach ($rows as $set) {
				$update = array();
				$where = array();
				foreach ($set as $key => $val) {
					$update[] = "$key = $val";
					if (isset($primary[idf_unescape($key)])) {
						$where[] = "$key = $val";
					}
				}
				if (
					!(($where && queries("UPDATE " . table($table) . " SET " . implode(", ", $update) . " WHERE " . implode(" AND ", $where)) && $connection->affected_rows)
					|| queries("INSERT INTO " . table($table) . " (" . implode(", ", array_keys($set)) . ") VALUES (" . implode(", ", $set) . ")"))
				) {
					return false;
				}
			}
			return true;
		}

		function hasCStyleEscapes() {
			return true;
		}
	}



	function idf_escape($idf) {
		return '"' . str_replace('"', '""', $idf) . '"';
	}

	function table($idf) {
		return idf_escape($idf);
	}

	function connect($credentials) {
		$connection = new Db;
		if ($connection->connect($credentials[0], $credentials[1], $credentials[2])) {
			return $connection;
		}
		return $connection->error;
	}

	function get_databases() {
		return get_vals(
			"SELECT DISTINCT tablespace_name FROM (
SELECT tablespace_name FROM user_tablespaces
UNION SELECT tablespace_name FROM all_tables WHERE tablespace_name IS NOT NULL
)
ORDER BY 1"
		);
	}

	function limit($query, $where, $limit, $offset = 0, $separator = " ") {
		return ($offset ? " * FROM (SELECT t.*, rownum AS rnum FROM (SELECT $query$where) t WHERE rownum <= " . ($limit + $offset) . ") WHERE rnum > $offset"
			: ($limit !== null ? " * FROM (SELECT $query$where) WHERE rownum <= " . ($limit + $offset)
			: " $query$where"
		));
	}

	function limit1($table, $query, $where, $separator = "\n") {
		return " $query$where"; //! limit
	}

	function db_collation($db, $collations) {
		return get_val("SELECT value FROM nls_database_parameters WHERE parameter = 'NLS_CHARACTERSET'"); //! respect $db
	}

	function logged_user() {
		return get_val("SELECT USER FROM DUAL");
	}

	function get_current_db() {
		global $connection;
		$db = $connection->_current_db ?: DB;
		unset($connection->_current_db);
		return $db;
	}

	function where_owner($prefix, $owner = "owner") {
		if (!$_GET["ns"]) {
			return '';
		}
		return "$prefix$owner = sys_context('USERENV', 'CURRENT_SCHEMA')";
	}

	function views_table($columns) {
		$owner = where_owner('');
		return "(SELECT $columns FROM all_views WHERE " . ($owner ?: "rownum < 0") . ")";
	}

	function tables_list() {
		$view = views_table("view_name");
		$owner = where_owner(" AND ");
		return get_key_vals(
			"SELECT table_name, 'table' FROM all_tables WHERE tablespace_name = " . q(DB) . "$owner
UNION SELECT view_name, 'view' FROM $view
ORDER BY 1"
		); //! views don't have schema
	}

	function count_tables($databases) {
		$return = array();
		foreach ($databases as $db) {
			$return[$db] = get_val("SELECT COUNT(*) FROM all_tables WHERE tablespace_name = " . q($db));
		}
		return $return;
	}

	function table_status($name = "") {
		$return = array();
		$search = q($name);
		$db = get_current_db();
		$view = views_table("view_name");
		$owner = where_owner(" AND ");
		foreach (
			get_rows('SELECT table_name "Name", \'table\' "Engine", avg_row_len * num_rows "Data_length", num_rows "Rows" FROM all_tables WHERE tablespace_name = ' . q($db) . $owner . ($name != "" ? " AND table_name = $search" : "") . "
UNION SELECT view_name, 'view', 0, 0 FROM $view" . ($name != "" ? " WHERE view_name = $search" : "") . "
ORDER BY 1") as $row
		) {
			if ($name != "") {
				return $row;
			}
			$return[$row["Name"]] = $row;
		}
		return $return;
	}

	function is_view($table_status) {
		return $table_status["Engine"] == "view";
	}

	function fk_support($table_status) {
		return true;
	}

	function fields($table) {
		$return = array();
		$owner = where_owner(" AND ");
		foreach (get_rows("SELECT * FROM all_tab_columns WHERE table_name = " . q($table) . "$owner ORDER BY column_id") as $row) {
			$type = $row["DATA_TYPE"];
			$length = "$row[DATA_PRECISION],$row[DATA_SCALE]";
			if ($length == ",") {
				$length = $row["CHAR_COL_DECL_LENGTH"];
			} //! int
			$return[$row["COLUMN_NAME"]] = array(
				"field" => $row["COLUMN_NAME"],
				"full_type" => $type . ($length ? "($length)" : ""),
				"type" => strtolower($type),
				"length" => $length,
				"default" => $row["DATA_DEFAULT"],
				"null" => ($row["NULLABLE"] == "Y"),
				//! "auto_increment" => false,
				//! "collation" => $row["CHARACTER_SET_NAME"],
				"privileges" => array("insert" => 1, "select" => 1, "update" => 1, "where" => 1, "order" => 1),
				//! "comment" => $row["Comment"],
				//! "primary" => ($row["Key"] == "PRI"),
			);
		}
		return $return;
	}

	function indexes($table, $connection2 = null) {
		$return = array();
		$owner = where_owner(" AND ", "aic.table_owner");
		foreach (
			get_rows("SELECT aic.*, ac.constraint_type, atc.data_default
FROM all_ind_columns aic
LEFT JOIN all_constraints ac ON aic.index_name = ac.constraint_name AND aic.table_name = ac.table_name AND aic.index_owner = ac.owner
LEFT JOIN all_tab_cols atc ON aic.column_name = atc.column_name AND aic.table_name = atc.table_name AND aic.index_owner = atc.owner
WHERE aic.table_name = " . q($table) . "$owner
ORDER BY ac.constraint_type, aic.column_position", $connection2) as $row
		) {
			$index_name = $row["INDEX_NAME"];
			$column_name = $row["DATA_DEFAULT"];
			$column_name = ($column_name ? trim($column_name, '"') : $row["COLUMN_NAME"]); // trim - possibly wrapped in quotes but never contains quotes inside
			$return[$index_name]["type"] = ($row["CONSTRAINT_TYPE"] == "P" ? "PRIMARY" : ($row["CONSTRAINT_TYPE"] == "U" ? "UNIQUE" : "INDEX"));
			$return[$index_name]["columns"][] = $column_name;
			$return[$index_name]["lengths"][] = ($row["CHAR_LENGTH"] && $row["CHAR_LENGTH"] != $row["COLUMN_LENGTH"] ? $row["CHAR_LENGTH"] : null);
			$return[$index_name]["descs"][] = ($row["DESCEND"] && $row["DESCEND"] == "DESC" ? '1' : null);
		}
		return $return;
	}

	function view($name) {
		$view = views_table("view_name, text");
		$rows = get_rows('SELECT text "select" FROM ' . $view . ' WHERE view_name = ' . q($name));
		return reset($rows);
	}

	function collations() {
		return array(); //!
	}

	function information_schema($db) {
		return get_schema() == "INFORMATION_SCHEMA";
	}

	function error() {
		global $connection;
		return h($connection->error); //! highlight sqltext from offset
	}

	function explain($connection, $query) {
		$connection->query("EXPLAIN PLAN FOR $query");
		return $connection->query("SELECT * FROM plan_table");
	}

	function found_rows($table_status, $where) {
	}

	function auto_increment() {
		return "";
	}

	function alter_table($table, $name, $fields, $foreign, $comment, $engine, $collation, $auto_increment, $partitioning) {
		$alter = $drop = array();
		$orig_fields = ($table ? fields($table) : array());
		foreach ($fields as $field) {
			$val = $field[1];
			if ($val && $field[0] != "" && idf_escape($field[0]) != $val[0]) {
				queries("ALTER TABLE " . table($table) . " RENAME COLUMN " . idf_escape($field[0]) . " TO $val[0]");
			}
			$orig_field = $orig_fields[$field[0]];
			if ($val && $orig_field) {
				$old = process_field($orig_field, $orig_field);
				if ($val[2] == $old[2]) {
					$val[2] = "";
				}
			}
			if ($val) {
				$alter[] = ($table != "" ? ($field[0] != "" ? "MODIFY (" : "ADD (") : "  ") . implode($val) . ($table != "" ? ")" : ""); //! error with name change only
			} else {
				$drop[] = idf_escape($field[0]);
			}
		}
		if ($table == "") {
			return queries("CREATE TABLE " . table($name) . " (\n" . implode(",\n", $alter) . "\n)");
		}
		return (!$alter || queries("ALTER TABLE " . table($table) . "\n" . implode("\n", $alter)))
			&& (!$drop || queries("ALTER TABLE " . table($table) . " DROP (" . implode(", ", $drop) . ")"))
			&& ($table == $name || queries("ALTER TABLE " . table($table) . " RENAME TO " . table($name)))
		;
	}

	function alter_indexes($table, $alter) {
		$drop = array();
		$queries = array();
		foreach ($alter as $val) {
			if ($val[0] != "INDEX") {
				//! descending UNIQUE indexes results in syntax error
				$val[2] = preg_replace('~ DESC$~', '', $val[2]);
				$create = ($val[2] == "DROP"
					? "\nDROP CONSTRAINT " . idf_escape($val[1])
					: "\nADD" . ($val[1] != "" ? " CONSTRAINT " . idf_escape($val[1]) : "") . " $val[0] " . ($val[0] == "PRIMARY" ? "KEY " : "") . "(" . implode(", ", $val[2]) . ")"
				);
				array_unshift($queries, "ALTER TABLE " . table($table) . $create);
			} elseif ($val[2] == "DROP") {
				$drop[] = idf_escape($val[1]);
			} else {
				$queries[] = "CREATE INDEX " . idf_escape($val[1] != "" ? $val[1] : uniqid($table . "_")) . " ON " . table($table) . " (" . implode(", ", $val[2]) . ")";
			}
		}
		if ($drop) {
			array_unshift($queries, "DROP INDEX " . implode(", ", $drop));
		}
		foreach ($queries as $query) {
			if (!queries($query)) {
				return false;
			}
		}
		return true;
	}

	function foreign_keys($table) {
		$return = array();
		$query = "SELECT c_list.CONSTRAINT_NAME as NAME,
c_src.COLUMN_NAME as SRC_COLUMN,
c_dest.OWNER as DEST_DB,
c_dest.TABLE_NAME as DEST_TABLE,
c_dest.COLUMN_NAME as DEST_COLUMN,
c_list.DELETE_RULE as ON_DELETE
FROM ALL_CONSTRAINTS c_list, ALL_CONS_COLUMNS c_src, ALL_CONS_COLUMNS c_dest
WHERE c_list.CONSTRAINT_NAME = c_src.CONSTRAINT_NAME
AND c_list.R_CONSTRAINT_NAME = c_dest.CONSTRAINT_NAME
AND c_list.CONSTRAINT_TYPE = 'R'
AND c_src.TABLE_NAME = " . q($table);
		foreach (get_rows($query) as $row) {
			$return[$row['NAME']] = array(
				"db" => $row['DEST_DB'],
				"table" => $row['DEST_TABLE'],
				"source" => array($row['SRC_COLUMN']),
				"target" => array($row['DEST_COLUMN']),
				"on_delete" => $row['ON_DELETE'],
				"on_update" => null,
			);
		}
		return $return;
	}

	function truncate_tables($tables) {
		return apply_queries("TRUNCATE TABLE", $tables);
	}

	function drop_views($views) {
		return apply_queries("DROP VIEW", $views);
	}

	function drop_tables($tables) {
		return apply_queries("DROP TABLE", $tables);
	}

	function last_id($result) {
		return 0; //!
	}

	function schemas() {
		$return = get_vals("SELECT DISTINCT owner FROM dba_segments WHERE owner IN (SELECT username FROM dba_users WHERE default_tablespace NOT IN ('SYSTEM','SYSAUX')) ORDER BY 1");
		return ($return ?: get_vals("SELECT DISTINCT owner FROM all_tables WHERE tablespace_name = " . q(DB) . " ORDER BY 1"));
	}

	function get_schema() {
		return get_val("SELECT sys_context('USERENV', 'SESSION_USER') FROM dual");
	}

	function set_schema($scheme, $connection2 = null) {
		global $connection;
		if (!$connection2) {
			$connection2 = $connection;
		}
		return $connection2->query("ALTER SESSION SET CURRENT_SCHEMA = " . idf_escape($scheme));
	}

	function show_variables() {
		return get_rows('SELECT name, display_value FROM v$parameter');
	}

	function show_status() {
		$return = array();
		$rows = get_rows('SELECT * FROM v$instance');
		foreach (reset($rows) as $key => $val) {
			$return[] = array($key, $val);
		}
		return $return;
	}

	function process_list() {
		return get_rows('SELECT
	sess.process AS "process",
	sess.username AS "user",
	sess.schemaname AS "schema",
	sess.status AS "status",
	sess.wait_class AS "wait_class",
	sess.seconds_in_wait AS "seconds_in_wait",
	sql.sql_text AS "sql_text",
	sess.machine AS "machine",
	sess.port AS "port"
FROM v$session sess LEFT OUTER JOIN v$sql sql
ON sql.sql_id = sess.sql_id
WHERE sess.type = \'USER\'
ORDER BY PROCESS
');
	}

	function convert_field($field) {
	}

	function unconvert_field($field, $return) {
		return $return;
	}

	function support($feature) {
		return preg_match('~^(columns|database|drop_col|indexes|descidx|processlist|scheme|sql|status|table|variables|view)$~', $feature); //!
	}
}

?>
<?php
/**
* @author Jakub Cernohuby
* @author Vladimir Stastka
* @author Jakub Vrana
*/

$drivers["mssql"] = "MS SQL";

if (isset($_GET["mssql"])) {
	define('Adminer\DRIVER', "mssql");
	if (extension_loaded("sqlsrv") && $_GET["ext"] != "pdo") {
		class Db {
			public $extension = "sqlsrv", $flavor = '', $server_info, $affected_rows, $errno, $error;
			private $link, $result;

			private function get_error() {
				$this->error = "";
				foreach (sqlsrv_errors() as $error) {
					$this->errno = $error["code"];
					$this->error .= "$error[message]\n";
				}
				$this->error = rtrim($this->error);
			}

			function connect($server, $username, $password) {
				global $adminer;
				$connection_info = array("UID" => $username, "PWD" => $password, "CharacterSet" => "UTF-8");
				$ssl = $adminer->connectSsl();
				if (isset($ssl["Encrypt"])) {
					$connection_info["Encrypt"] = $ssl["Encrypt"];
				}
				if (isset($ssl["TrustServerCertificate"])) {
					$connection_info["TrustServerCertificate"] = $ssl["TrustServerCertificate"];
				}
				$db = $adminer->database();
				if ($db != "") {
					$connection_info["Database"] = $db;
				}
				$this->link = @sqlsrv_connect(preg_replace('~:~', ',', $server), $connection_info);
				if ($this->link) {
					$info = sqlsrv_server_info($this->link);
					$this->server_info = $info['SQLServerVersion'];
				} else {
					$this->get_error();
				}
				return (bool) $this->link;
			}

			function quote($string) {
				$unicode = strlen($string) != strlen(utf8_decode($string));
				return ($unicode ? "N" : "") . "'" . str_replace("'", "''", $string) . "'";
			}

			function select_db($database) {
				return $this->query(use_sql($database));
			}

			function query($query, $unbuffered = false) {
				$result = sqlsrv_query($this->link, $query); //! , array(), ($unbuffered ? array() : array("Scrollable" => "keyset"))
				$this->error = "";
				if (!$result) {
					$this->get_error();
					return false;
				}
				return $this->store_result($result);
			}

			function multi_query($query) {
				$this->result = sqlsrv_query($this->link, $query);
				$this->error = "";
				if (!$this->result) {
					$this->get_error();
					return false;
				}
				return true;
			}

			function store_result($result = null) {
				if (!$result) {
					$result = $this->result;
				}
				if (!$result) {
					return false;
				}
				if (sqlsrv_field_metadata($result)) {
					return new Result($result);
				}
				$this->affected_rows = sqlsrv_rows_affected($result);
				return true;
			}

			function next_result() {
				return $this->result ? sqlsrv_next_result($this->result) : null;
			}

			function result($query, $field = 0) {
				$result = $this->query($query);
				if (!is_object($result)) {
					return false;
				}
				$row = $result->fetch_row();
				return $row[$field];
			}
		}

		class Result {
			public $num_rows;
			private $result, $offset = 0, $fields;

			function __construct($result) {
				$this->result = $result;
				// $this->num_rows = sqlsrv_num_rows($result); // available only in scrollable results
			}

			private function convert($row) {
				foreach ((array) $row as $key => $val) {
					if (is_a($val, 'DateTime')) {
						$row[$key] = $val->format("Y-m-d H:i:s");
					}
					//! stream
				}
				return $row;
			}

			function fetch_assoc() {
				return $this->convert(sqlsrv_fetch_array($this->result, SQLSRV_FETCH_ASSOC));
			}

			function fetch_row() {
				return $this->convert(sqlsrv_fetch_array($this->result, SQLSRV_FETCH_NUMERIC));
			}

			function fetch_field() {
				if (!$this->fields) {
					$this->fields = sqlsrv_field_metadata($this->result);
				}
				$field = $this->fields[$this->offset++];
				$return = new \stdClass;
				$return->name = $field["Name"];
				$return->type = ($field["Type"] == 1 ? 254 : 15);
				$return->charsetnr = 0;
				return $return;
			}

			function seek($offset) {
				for ($i=0; $i < $offset; $i++) {
					sqlsrv_fetch($this->result); // SQLSRV_SCROLL_ABSOLUTE added in sqlsrv 1.1
				}
			}

			function __destruct() {
				sqlsrv_free_stmt($this->result);
			}
		}

		function last_id($result) {
			return get_val("SELECT SCOPE_IDENTITY()"); // @@IDENTITY can return trigger INSERT
		}

		function explain($connection, $query) {
			$connection->query("SET SHOWPLAN_ALL ON");
			$return = $connection->query($query);
			$connection->query("SET SHOWPLAN_ALL OFF"); // connection is used also for indexes
			return $return;
		}

	} else {
		class MssqlDb extends PdoDb {
			function select_db($database) {
				// database selection is separated from the connection so dbname in DSN can't be used
				return $this->query(use_sql($database));
			}

			function lastInsertId() {
				return $this->pdo->lastInsertId();
			}
		}

		function last_id($result) {
			global $connection;
			return $connection->lastInsertId();
		}

		function explain($connection, $query) {
		}

		if (extension_loaded("pdo_sqlsrv")) {
			class Db extends MssqlDb {
				public $extension = "PDO_SQLSRV";

				function connect($server, $username, $password) {
					$this->dsn("sqlsrv:Server=" . str_replace(":", ",", $server), $username, $password);
					return true;
				}
			}

		} elseif (extension_loaded("pdo_dblib")) {
			class Db extends MssqlDb {
				public $extension = "PDO_DBLIB";

				function connect($server, $username, $password) {
					$this->dsn("dblib:charset=utf8;host=" . str_replace(":", ";unix_socket=", preg_replace('~:(\d)~', ';port=\1', $server)), $username, $password);
					return true;
				}
			}
		}
	}


	class Driver extends SqlDriver {
		static $possibleDrivers = array("SQLSRV", "PDO_SQLSRV", "PDO_DBLIB");
		static $jush = "mssql";

		public $editFunctions = array(
			array(
				"date|time" => "getdate",
			), array(
				"int|decimal|real|float|money|datetime" => "+/-",
				"char|text" => "+",
			)
		);

		public $operators = array("=", "<", ">", "<=", ">=", "!=", "LIKE", "LIKE %%", "IN", "IS NULL", "NOT LIKE", "NOT IN", "IS NOT NULL");
		public $functions = array("len", "lower", "round", "upper");
		public $grouping = array("avg", "count", "count distinct", "max", "min", "sum");
		public $onActions = "NO ACTION|CASCADE|SET NULL|SET DEFAULT";
		public $generated = array("PERSISTED", "VIRTUAL");

		function __construct($connection) {
			parent::__construct($connection);
			$this->types = array( //! use sys.types
				lang(25) => array("tinyint" => 3, "smallint" => 5, "int" => 10, "bigint" => 20, "bit" => 1, "decimal" => 0, "real" => 12, "float" => 53, "smallmoney" => 10, "money" => 20),
				lang(26) => array("date" => 10, "smalldatetime" => 19, "datetime" => 19, "datetime2" => 19, "time" => 8, "datetimeoffset" => 10),
				lang(27) => array("char" => 8000, "varchar" => 8000, "text" => 2147483647, "nchar" => 4000, "nvarchar" => 4000, "ntext" => 1073741823),
				lang(28) => array("binary" => 8000, "varbinary" => 8000, "image" => 2147483647),
			);
		}

		function insertUpdate($table, $rows, $primary) {
			$fields = fields($table);
			$update = array();
			$where = array();
			$set = reset($rows);
			$columns = "c" . implode(", c", range(1, count($set)));
			$c = 0;
			$insert = array();
			foreach ($set as $key => $val) {
				$c++;
				$name = idf_unescape($key);
				if (!$fields[$name]["auto_increment"]) {
					$insert[$key] = "c$c";
				}
				if (isset($primary[$name])) {
					$where[] = "$key = c$c";
				} else {
					$update[] = "$key = c$c";
				}
			}
			$values = array();
			foreach ($rows as $set) {
				$values[] = "(" . implode(", ", $set) . ")";
			}
			if ($where) {
				$identity = queries("SET IDENTITY_INSERT " . table($table) . " ON");
				$return = queries(
					"MERGE " . table($table) . " USING (VALUES\n\t" . implode(",\n\t", $values) . "\n) AS source ($columns) ON " . implode(" AND ", $where) //! source, c1 - possible conflict
					. ($update ? "\nWHEN MATCHED THEN UPDATE SET " . implode(", ", $update) : "")
					. "\nWHEN NOT MATCHED THEN INSERT (" . implode(", ", array_keys($identity ? $set : $insert)) . ") VALUES (" . ($identity ? $columns : implode(", ", $insert)) . ");" // ; is mandatory
				);
				if ($identity) {
					queries("SET IDENTITY_INSERT " . table($table) . " OFF");
				}
			} else {
				$return = queries("INSERT INTO " . table($table) . " (" . implode(", ", array_keys($set)) . ") VALUES\n" . implode(",\n", $values));
			}
			return $return;
		}

		function begin() {
			return queries("BEGIN TRANSACTION");
		}

		function tableHelp($name, $is_view = false) {
			$links = array(
				"sys" => "catalog-views/sys-",
				"INFORMATION_SCHEMA" => "information-schema-views/",
			);
			$link = $links[get_schema()];
			if ($link) {
				return "relational-databases/system-$link" . preg_replace('~_~', '-', strtolower($name)) . "-transact-sql";
			}
		}
	}



	function idf_escape($idf) {
		return "[" . str_replace("]", "]]", $idf) . "]";
	}

	function table($idf) {
		return ($_GET["ns"] != "" ? idf_escape($_GET["ns"]) . "." : "") . idf_escape($idf);
	}

	function connect($credentials) {
		$connection = new Db;
		if ($credentials[0] == "") {
			$credentials[0] = "localhost:1433";
		}
		if ($connection->connect($credentials[0], $credentials[1], $credentials[2])) {
			return $connection;
		}
		return $connection->error;
	}

	function get_databases() {
		return get_vals("SELECT name FROM sys.databases WHERE name NOT IN ('master', 'tempdb', 'model', 'msdb')");
	}

	function limit($query, $where, $limit, $offset = 0, $separator = " ") {
		return ($limit !== null ? " TOP (" . ($limit + $offset) . ")" : "") . " $query$where"; // seek later
	}

	function limit1($table, $query, $where, $separator = "\n") {
		return limit($query, $where, 1, 0, $separator);
	}

	function db_collation($db, $collations) {
		return get_val("SELECT collation_name FROM sys.databases WHERE name = " . q($db));
	}

	function logged_user() {
		return get_val("SELECT SUSER_NAME()");
	}

	function tables_list() {
		return get_key_vals("SELECT name, type_desc FROM sys.all_objects WHERE schema_id = SCHEMA_ID(" . q(get_schema()) . ") AND type IN ('S', 'U', 'V') ORDER BY name");
	}

	function count_tables($databases) {
		global $connection;
		$return = array();
		foreach ($databases as $db) {
			$connection->select_db($db);
			$return[$db] = get_val("SELECT COUNT(*) FROM INFORMATION_SCHEMA.TABLES");
		}
		return $return;
	}

	function table_status($name = "") {
		$return = array();
		foreach (
			get_rows("SELECT ao.name AS Name, ao.type_desc AS Engine, (SELECT value FROM fn_listextendedproperty(default, 'SCHEMA', schema_name(schema_id), 'TABLE', ao.name, null, null)) AS Comment
FROM sys.all_objects AS ao
WHERE schema_id = SCHEMA_ID(" . q(get_schema()) . ") AND type IN ('S', 'U', 'V') " . ($name != "" ? "AND name = " . q($name) : "ORDER BY name")) as $row
		) {
			if ($name != "") {
				return $row;
			}
			$return[$row["Name"]] = $row;
		}
		return $return;
	}

	function is_view($table_status) {
		return $table_status["Engine"] == "VIEW";
	}

	function fk_support($table_status) {
		return true;
	}

	function fields($table) {
		$comments = get_key_vals("SELECT objname, cast(value as varchar(max)) FROM fn_listextendedproperty('MS_DESCRIPTION', 'schema', " . q(get_schema()) . ", 'table', " . q($table) . ", 'column', NULL)");
		$return = array();
		$table_id = get_val("SELECT object_id FROM sys.all_objects WHERE schema_id = SCHEMA_ID(" . q(get_schema()) . ") AND type IN ('S', 'U', 'V') AND name = " . q($table));
		foreach (
			get_rows("SELECT c.max_length, c.precision, c.scale, c.name, c.is_nullable, c.is_identity, c.collation_name, t.name type, d.definition [default], d.name default_constraint, i.is_primary_key
FROM sys.all_columns c
JOIN sys.types t ON c.user_type_id = t.user_type_id
LEFT JOIN sys.default_constraints d ON c.default_object_id = d.object_id
LEFT JOIN sys.index_columns ic ON c.object_id = ic.object_id AND c.column_id = ic.column_id
LEFT JOIN sys.indexes i ON ic.object_id = i.object_id AND ic.index_id = i.index_id
WHERE c.object_id = " . q($table_id)) as $row
		) {
			$type = $row["type"];
			$length = (preg_match("~char|binary~", $type)
				? $row["max_length"] / ($type[0] == 'n' ? 2 : 1)
				: ($type == "decimal" ? "$row[precision],$row[scale]" : "")
			);
			$return[$row["name"]] = array(
				"field" => $row["name"],
				"full_type" => $type . ($length ? "($length)" : ""),
				"type" => $type,
				"length" => $length,
				"default" => (preg_match("~^\('(.*)'\)$~", $row["default"], $match) ? str_replace("''", "'", $match[1]) : $row["default"]),
				"default_constraint" => $row["default_constraint"],
				"null" => $row["is_nullable"],
				"auto_increment" => $row["is_identity"],
				"collation" => $row["collation_name"],
				"privileges" => array("insert" => 1, "select" => 1, "update" => 1, "where" => 1, "order" => 1),
				"primary" => $row["is_primary_key"],
				"comment" => $comments[$row["name"]],
			);
		}
		foreach (get_rows("SELECT * FROM sys.computed_columns WHERE object_id = " . q($table_id)) as $row) {
			$return[$row["name"]]["generated"] = ($row["is_persisted"] ? "PERSISTED" : "VIRTUAL");
			$return[$row["name"]]["default"] = $row["definition"];
		}
		return $return;
	}

	function indexes($table, $connection2 = null) {
		$return = array();
		// sp_statistics doesn't return information about primary key
		foreach (
			get_rows("SELECT i.name, key_ordinal, is_unique, is_primary_key, c.name AS column_name, is_descending_key
FROM sys.indexes i
INNER JOIN sys.index_columns ic ON i.object_id = ic.object_id AND i.index_id = ic.index_id
INNER JOIN sys.columns c ON ic.object_id = c.object_id AND ic.column_id = c.column_id
WHERE OBJECT_NAME(i.object_id) = " . q($table), $connection2) as $row
		) {
			$name = $row["name"];
			$return[$name]["type"] = ($row["is_primary_key"] ? "PRIMARY" : ($row["is_unique"] ? "UNIQUE" : "INDEX"));
			$return[$name]["lengths"] = array();
			$return[$name]["columns"][$row["key_ordinal"]] = $row["column_name"];
			$return[$name]["descs"][$row["key_ordinal"]] = ($row["is_descending_key"] ? '1' : null);
		}
		return $return;
	}

	function view($name) {
		return array("select" => preg_replace('~^(?:[^[]|\[[^]]*])*\s+AS\s+~isU', '', get_val("SELECT VIEW_DEFINITION FROM INFORMATION_SCHEMA.VIEWS WHERE TABLE_SCHEMA = SCHEMA_NAME() AND TABLE_NAME = " . q($name))));
	}

	function collations() {
		$return = array();
		foreach (get_vals("SELECT name FROM fn_helpcollations()") as $collation) {
			$return[preg_replace('~_.*~', '', $collation)][] = $collation;
		}
		return $return;
	}

	function information_schema($db) {
		return get_schema() == "INFORMATION_SCHEMA";
	}

	function error() {
		global $connection;
		return nl_br(h(preg_replace('~^(\[[^]]*])+~m', '', $connection->error)));
	}

	function create_database($db, $collation) {
		return queries("CREATE DATABASE " . idf_escape($db) . (preg_match('~^[a-z0-9_]+$~i', $collation) ? " COLLATE $collation" : ""));
	}

	function drop_databases($databases) {
		return queries("DROP DATABASE " . implode(", ", array_map('Adminer\idf_escape', $databases)));
	}

	function rename_database($name, $collation) {
		if (preg_match('~^[a-z0-9_]+$~i', $collation)) {
			queries("ALTER DATABASE " . idf_escape(DB) . " COLLATE $collation");
		}
		queries("ALTER DATABASE " . idf_escape(DB) . " MODIFY NAME = " . idf_escape($name));
		return true; //! false negative "The database name 'test2' has been set."
	}

	function auto_increment() {
		return " IDENTITY" . ($_POST["Auto_increment"] != "" ? "(" . number($_POST["Auto_increment"]) . ",1)" : "") . " PRIMARY KEY";
	}

	function alter_table($table, $name, $fields, $foreign, $comment, $engine, $collation, $auto_increment, $partitioning) {
		$alter = array();
		$comments = array();
		$orig_fields = fields($table);
		foreach ($fields as $field) {
			$column = idf_escape($field[0]);
			$val = $field[1];
			if (!$val) {
				$alter["DROP"][] = " COLUMN $column";
			} else {
				$val[1] = preg_replace("~( COLLATE )'(\\w+)'~", '\1\2', $val[1]);
				$comments[$field[0]] = $val[5];
				unset($val[5]);
				if (preg_match('~ AS ~', $val[3])) {
					unset($val[1], $val[2]);
				}
				if ($field[0] == "") {
					$alter["ADD"][] = "\n  " . implode("", $val) . ($table == "" ? substr($foreign[$val[0]], 16 + strlen($val[0])) : ""); // 16 - strlen("  FOREIGN KEY ()")
				} else {
					$default = $val[3];
					unset($val[3]); // default values are set separately
					unset($val[6]); //! identity can't be removed
					if ($column != $val[0]) {
						queries("EXEC sp_rename " . q(table($table) . ".$column") . ", " . q(idf_unescape($val[0])) . ", 'COLUMN'");
					}
					$alter["ALTER COLUMN " . implode("", $val)][] = "";
					$orig_field = $orig_fields[$field[0]];
					if (default_value($orig_field) != $default) {
						if ($orig_field["default"] !== null) {
							$alter["DROP"][] = " " . idf_escape($orig_field["default_constraint"]);
						}
						if ($default) {
							$alter["ADD"][] = "\n $default FOR $column";
						}
					}
				}
			}
		}
		if ($table == "") {
			return queries("CREATE TABLE " . table($name) . " (" . implode(",", (array) $alter["ADD"]) . "\n)");
		}
		if ($table != $name) {
			queries("EXEC sp_rename " . q(table($table)) . ", " . q($name));
		}
		if ($foreign) {
			$alter[""] = $foreign;
		}
		foreach ($alter as $key => $val) {
			if (!queries("ALTER TABLE " . table($name) . " $key" . implode(",", $val))) {
				return false;
			}
		}
		foreach ($comments as $key => $val) {
			$comment = substr($val, 9); // 9 - strlen(" COMMENT ")
			queries("EXEC sp_dropextendedproperty @name = N'MS_Description', @level0type = N'Schema', @level0name = " . q(get_schema()) . ", @level1type = N'Table', @level1name = " . q($name) . ", @level2type = N'Column', @level2name = " . q($key));
			queries("EXEC sp_addextendedproperty
@name = N'MS_Description',
@value = $comment,
@level0type = N'Schema',
@level0name = " . q(get_schema()) . ",
@level1type = N'Table',
@level1name = " . q($name) . ",
@level2type = N'Column',
@level2name = " . q($key))
			;
		}
		return true;
	}

	function alter_indexes($table, $alter) {
		$index = array();
		$drop = array();
		foreach ($alter as $val) {
			if ($val[2] == "DROP") {
				if ($val[0] == "PRIMARY") { //! sometimes used also for UNIQUE
					$drop[] = idf_escape($val[1]);
				} else {
					$index[] = idf_escape($val[1]) . " ON " . table($table);
				}
			} elseif (
				!queries(($val[0] != "PRIMARY"
					? "CREATE $val[0] " . ($val[0] != "INDEX" ? "INDEX " : "") . idf_escape($val[1] != "" ? $val[1] : uniqid($table . "_")) . " ON " . table($table)
					: "ALTER TABLE " . table($table) . " ADD PRIMARY KEY"
				) . " (" . implode(", ", $val[2]) . ")")
			) {
				return false;
			}
		}
		return (!$index || queries("DROP INDEX " . implode(", ", $index)))
			&& (!$drop || queries("ALTER TABLE " . table($table) . " DROP " . implode(", ", $drop)))
		;
	}

	function found_rows($table_status, $where) {
	}

	function foreign_keys($table) {
		$return = array();
		$on_actions = array("CASCADE", "NO ACTION", "SET NULL", "SET DEFAULT");
		foreach (get_rows("EXEC sp_fkeys @fktable_name = " . q($table) . ", @fktable_owner = " . q(get_schema())) as $row) {
			$foreign_key = &$return[$row["FK_NAME"]];
			$foreign_key["db"] = $row["PKTABLE_QUALIFIER"];
			$foreign_key["ns"] = $row["PKTABLE_OWNER"];
			$foreign_key["table"] = $row["PKTABLE_NAME"];
			$foreign_key["on_update"] = $on_actions[$row["UPDATE_RULE"]];
			$foreign_key["on_delete"] = $on_actions[$row["DELETE_RULE"]];
			$foreign_key["source"][] = $row["FKCOLUMN_NAME"];
			$foreign_key["target"][] = $row["PKCOLUMN_NAME"];
		}
		return $return;
	}

	function truncate_tables($tables) {
		return apply_queries("TRUNCATE TABLE", $tables);
	}

	function drop_views($views) {
		return queries("DROP VIEW " . implode(", ", array_map('Adminer\table', $views)));
	}

	function drop_tables($tables) {
		return queries("DROP TABLE " . implode(", ", array_map('Adminer\table', $tables)));
	}

	function move_tables($tables, $views, $target) {
		return apply_queries("ALTER SCHEMA " . idf_escape($target) . " TRANSFER", array_merge($tables, $views));
	}

	function trigger($name) {
		if ($name == "") {
			return array();
		}
		$rows = get_rows(
			"SELECT s.name [Trigger],
CASE WHEN OBJECTPROPERTY(s.id, 'ExecIsInsertTrigger') = 1 THEN 'INSERT' WHEN OBJECTPROPERTY(s.id, 'ExecIsUpdateTrigger') = 1 THEN 'UPDATE' WHEN OBJECTPROPERTY(s.id, 'ExecIsDeleteTrigger') = 1 THEN 'DELETE' END [Event],
CASE WHEN OBJECTPROPERTY(s.id, 'ExecIsInsteadOfTrigger') = 1 THEN 'INSTEAD OF' ELSE 'AFTER' END [Timing],
c.text
FROM sysobjects s
JOIN syscomments c ON s.id = c.id
WHERE s.xtype = 'TR' AND s.name = " . q($name)
		); // triggers are not schema-scoped
		$return = reset($rows);
		if ($return) {
			$return["Statement"] = preg_replace('~^.+\s+AS\s+~isU', '', $return["text"]); //! identifiers, comments
		}
		return $return;
	}

	function triggers($table) {
		$return = array();
		foreach (
			get_rows("SELECT sys1.name,
CASE WHEN OBJECTPROPERTY(sys1.id, 'ExecIsInsertTrigger') = 1 THEN 'INSERT' WHEN OBJECTPROPERTY(sys1.id, 'ExecIsUpdateTrigger') = 1 THEN 'UPDATE' WHEN OBJECTPROPERTY(sys1.id, 'ExecIsDeleteTrigger') = 1 THEN 'DELETE' END [Event],
CASE WHEN OBJECTPROPERTY(sys1.id, 'ExecIsInsteadOfTrigger') = 1 THEN 'INSTEAD OF' ELSE 'AFTER' END [Timing]
FROM sysobjects sys1
JOIN sysobjects sys2 ON sys1.parent_obj = sys2.id
WHERE sys1.xtype = 'TR' AND sys2.name = " . q($table)) as $row
		) { // triggers are not schema-scoped
			$return[$row["name"]] = array($row["Timing"], $row["Event"]);
		}
		return $return;
	}

	function trigger_options() {
		return array(
			"Timing" => array("AFTER", "INSTEAD OF"),
			"Event" => array("INSERT", "UPDATE", "DELETE"),
			"Type" => array("AS"),
		);
	}

	function schemas() {
		return get_vals("SELECT name FROM sys.schemas");
	}

	function get_schema() {
		if ($_GET["ns"] != "") {
			return $_GET["ns"];
		}
		return get_val("SELECT SCHEMA_NAME()");
	}

	function set_schema($schema) {
		$_GET["ns"] = $schema;
		return true; // ALTER USER is permanent
	}

	function create_sql($table, $auto_increment, $style) {
		global $driver;
		if (is_view(table_status($table))) {
			$view = view($table);
			return "CREATE VIEW " . table($table) . " AS $view[select]";
		}
		$fields = array();
		$primary = false;
		foreach (fields($table) as $name => $field) {
			$val = process_field($field, $field);
			if ($val[6]) {
				$primary = true;
			}
			$fields[] = implode("", $val);
		}
		foreach (indexes($table) as $name => $index) {
			if (!$primary || $index["type"] != "PRIMARY") {
				$columns = array();
				foreach ($index["columns"] as $key => $val) {
					$columns[] = idf_escape($val) . ($index["descs"][$key] ? " DESC" : "");
				}
				$name = idf_escape($name);
				$fields[] = ($index["type"] == "INDEX" ? "INDEX $name" : "CONSTRAINT $name " . ($index["type"] == "UNIQUE" ? "UNIQUE" : "PRIMARY KEY")) . " (" . implode(", ", $columns) . ")";
			}
		}
		foreach ($driver->checkConstraints($table) as $name => $check) {
			$fields[] = "CONSTRAINT " . idf_escape($name) . " CHECK ($check)";
		}
		return "CREATE TABLE " . table($table) . " (\n\t" . implode(",\n\t", $fields) . "\n)";
	}

	function foreign_keys_sql($table) {
		$fields = array();
		foreach (foreign_keys($table) as $foreign) {
			$fields[] = ltrim(format_foreign_key($foreign));
		}
		return ($fields ? "ALTER TABLE " . table($table) . " ADD\n\t" . implode(",\n\t", $fields) . ";\n\n" : "");
	}

	function truncate_sql($table) {
		return "TRUNCATE TABLE " . table($table);
	}

	function use_sql($database) {
		return "USE " . idf_escape($database);
	}

	function trigger_sql($table) {
		$return = "";
		foreach (triggers($table) as $name => $trigger) {
			$return .= create_trigger(" ON " . table($table), trigger($name)) . ";";
		}
		return $return;
	}

	function convert_field($field) {
	}

	function unconvert_field($field, $return) {
		return $return;
	}

	function support($feature) {
		return preg_match('~^(check|comment|columns|database|drop_col|dump|indexes|descidx|scheme|sql|table|trigger|view|view_trigger)$~', $feature); //! routine|
	}
}

?>
<?php
// any method change in this file should be transferred to editor/include/adminer.inc.php and plugins/plugin.php

class Adminer {
	public $operators; ///< @var array operators used in select, null for all operators
	public $error = ''; ///< @var protected(set) string HTML

	/** Name in title and navigation
	* @return string HTML code
	*/
	function name() {
		return "<a href='https://www.adminer.org/'" . target_blank() . " id='h1'>Adminer</a>";
	}

	/** Connection parameters
	* @return array [$server, $username, $password]
	*/
	function credentials() {
		return array(SERVER, $_GET["username"], get_password());
	}

	/** Get SSL connection options
	* @return array ["key" => filename, "cert" => filename, "ca" => filename] or null
	*/
	function connectSsl() {
	}

	/** Get key used for permanent login
	* @param bool
	* @return string cryptic string which gets combined with password or false in case of an error
	*/
	function permanentLogin($create = false) {
		return password_file($create);
	}

	/** Return key used to group brute force attacks; behind a reverse proxy, you want to return the last part of X-Forwarded-For
	* @return string
	*/
	function bruteForceKey() {
		return $_SERVER["REMOTE_ADDR"];
	}

	/** Get server name displayed in breadcrumbs
	* @param string
	* @return string HTML code or null
	*/
	function serverName($server) {
		return h($server);
	}

	/** Identifier of selected database
	* @return string
	*/
	function database() {
		// should be used everywhere instead of DB
		return DB;
	}

	/** Get cached list of databases
	* @param bool
	* @return array
	*/
	function databases($flush = true) {
		return get_databases($flush);
	}

	/** Get list of schemas
	* @return array
	*/
	function schemas() {
		return schemas();
	}

	/** Specify limit for waiting on some slow queries like DB list
	* @return float number of seconds
	*/
	function queryTimeout() {
		return 2;
	}

	/** Headers to send before HTML output
	* @return null
	*/
	function headers() {
	}

	/** Get Content Security Policy headers
	* @return array of arrays with directive name in key, allowed sources in value
	*/
	function csp() {
		return csp();
	}

	/** Print HTML code inside <head>
	* @param bool dark CSS: false to disable, true to force, null to base on user preferences
	* @return bool true to link favicon.ico
	*/
	function head($dark = null) {
		// this is matched by compile.php
		
		
		return true;
	}

	/** Get URLs of the CSS files
	* @return array of strings
	*/
	function css() {
		$return = array();
		foreach (array("", "-dark") as $mode) {
			$filename = "adminer$mode.css";
			if (file_exists($filename)) {
				$return[] = "$filename?v=" . crc32(file_get_contents($filename));
			}
		}
		return $return;
	}

	/** Print login form
	* @return null
	*/
	function loginForm() {
		global $drivers;
		echo "<table class='layout'>\n";
		// this is matched by compile.php
		echo $this->loginFormField('driver', '<tr><th>' . lang(32) . '<td>', html_select("auth[driver]", $drivers, DRIVER, "loginDriver(this);"));
		echo $this->loginFormField('server', '<tr><th>' . lang(33) . '<td>', '<input name="auth[server]" value="' . h(SERVER) . '" title="hostname[:port]" placeholder="localhost" autocapitalize="off">');
		// this is matched by compile.php
		echo $this->loginFormField('username', '<tr><th>' . lang(34) . '<td>', '<input name="auth[username]" id="username" autofocus value="' . h($_GET["username"]) . '" autocomplete="username" autocapitalize="off">' . script("qs('#username').form['auth[driver]'].onchange();"));
		echo $this->loginFormField('password', '<tr><th>' . lang(35) . '<td>', '<input type="password" name="auth[password]" autocomplete="current-password">');
		echo $this->loginFormField('db', '<tr><th>' . lang(36) . '<td>', '<input name="auth[db]" value="' . h($_GET["db"]) . '" autocapitalize="off">');
		echo "</table>\n";
		echo "<p><input type='submit' value='" . lang(37) . "'>\n";
		echo checkbox("auth[permanent]", 1, $_COOKIE["adminer_permanent"], lang(38)) . "\n";
	}

	/** Get login form field
	* @param string
	* @param string HTML
	* @param string HTML
	* @return string
	*/
	function loginFormField($name, $heading, $value) {
		return $heading . $value . "\n";
	}

	/** Authorize the user
	* @param string
	* @param string
	* @return mixed true for success, string for error message, false for unknown error
	*/
	function login($login, $password) {
		if ($password == "") {
			return lang(39, target_blank());
		}
		return true;
	}

	/** Table caption used in navigation and headings
	* @param array result of SHOW TABLE STATUS
	* @return string HTML code, "" to ignore table
	*/
	function tableName($tableStatus) {
		return h($tableStatus["Name"]);
	}

	/** Field caption used in select and edit
	* @param array single field returned from fields()
	* @param int order of column in select
	* @return string HTML code, "" to ignore field
	*/
	function fieldName($field, $order = 0) {
		$type = $field["full_type"];
		$comment = $field["comment"];
		return '<span title="' . h($type . ($comment != "" ? ($type ? ": " : "") . $comment : '')) . '">' . h($field["field"]) . '</span>';
	}

	/** Print links after select heading
	* @param array result of SHOW TABLE STATUS
	* @param string new item options, NULL for no new item
	* @return null
	*/
	function selectLinks($tableStatus, $set = "") {
		global $driver;
		echo '<p class="links">';
		$links = array("select" => lang(40));
		if (support("table") || support("indexes")) {
			$links["table"] = lang(41);
		}
		$is_view = false;
		if (support("table")) {
			$is_view = is_view($tableStatus);
			if ($is_view) {
				$links["view"] = lang(42);
			} else {
				$links["create"] = lang(43);
			}
		}
		if ($set !== null) {
			$links["edit"] = lang(44);
		}
		$name = $tableStatus["Name"];
		foreach ($links as $key => $val) {
			echo " <a href='" . h(ME) . "$key=" . urlencode($name) . ($key == "edit" ? $set : "") . "'" . bold(isset($_GET[$key])) . ">$val</a>";
		}
		echo doc_link(array(JUSH => $driver->tableHelp($name, $is_view)), "?");
		echo "\n";
	}

	/** Get foreign keys for table
	* @param string
	* @return array same format as foreign_keys()
	*/
	function foreignKeys($table) {
		return foreign_keys($table);
	}

	/** Find backward keys for table
	* @param string
	* @param string
	* @return array $return[$target_table]["keys"][$key_name][$target_column] = $source_column; $return[$target_table]["name"] = $this->tableName($target_table);
	*/
	function backwardKeys($table, $tableName) {
		return array();
	}

	/** Print backward keys for row
	* @param array result of $this->backwardKeys()
	* @param array
	* @return null
	*/
	function backwardKeysPrint($backwardKeys, $row) {
	}

	/** Query printed in select before execution
	* @param string query to be executed
	* @param float start time of the query
	* @param bool
	* @return string
	*/
	function selectQuery($query, $start, $failed = false) {
		global $driver;
		$return = "</p>\n"; // required for IE9 inline edit
		if (!$failed && ($warnings = $driver->warnings())) {
			$id = "warnings";
			$return = ", <a href='#$id'>" . lang(45) . "</a>" . script("qsl('a').onclick = partial(toggle, '$id');", "")
				. "$return<div id='$id' class='hidden'>\n$warnings</div>\n"
			;
		}
		return "<p><code class='jush-" . JUSH . "'>" . h(str_replace("\n", " ", $query)) . "</code> <span class='time'>(" . format_time($start) . ")</span>"
			. (support("sql") ? " <a href='" . h(ME) . "sql=" . urlencode($query) . "'>" . lang(10) . "</a>" : "")
			. $return
		;
	}

	/** Query printed in SQL command before execution
	* @param string query to be executed
	* @return string escaped query to be printed
	*/
	function sqlCommandQuery($query) {
		return shorten_utf8(trim($query), 1000);
	}

	/** Print HTML code just before the Execute button in SQL command
	*/
	function sqlPrintAfter() {
	}

	/** Description of a row in a table
	* @param string
	* @return string SQL expression, empty string for no description
	*/
	function rowDescription($table) {
		return "";
	}

	/** Get descriptions of selected data
	* @param array all data to print
	* @param array
	* @return array
	*/
	function rowDescriptions($rows, $foreignKeys) {
		return $rows;
	}

	/** Get a link to use in select table
	* @param string raw value of the field
	* @param array single field returned from fields()
	* @return string or null to create the default link
	*/
	function selectLink($val, $field) {
	}

	/** Value printed in select table
	* @param string HTML-escaped value to print
	* @param string link to foreign key
	* @param array single field returned from fields()
	* @param array original value before applying editVal() and escaping
	* @return string
	*/
	function selectVal($val, $link, $field, $original) {
		$return = ($val === null ? "<i>NULL</i>"
			: (preg_match("~char|binary|boolean~", $field["type"]) && !preg_match("~var~", $field["type"]) ? "<code>$val</code>"
			: (preg_match('~json~', $field["type"]) ? "<code class='jush-js'>$val</code>"
			: $val)
		));
		if (preg_match('~blob|bytea|raw|file~', $field["type"]) && !is_utf8($val)) {
			$return = "<i>" . lang(46, strlen($original)) . "</i>";
		}
		return ($link ? "<a href='" . h($link) . "'" . (is_url($link) ? target_blank() : "") . ">$return</a>" : $return);
	}

	/** Value conversion used in select and edit
	* @param string
	* @param array single field returned from fields()
	* @return string
	*/
	function editVal($val, $field) {
		return $val;
	}

	/** Print table structure in tabular format
	* @param array data about individual fields
	* @param array
	* @return null
	*/
	function tableStructurePrint($fields, $tableStatus = null) {
		global $driver;
		echo "<div class='scrollable'>\n";
		echo "<table class='nowrap odds'>\n";
		echo "<thead><tr><th>" . lang(47) . "<td>" . lang(48) . (support("comment") ? "<td>" . lang(49) : "") . "</thead>\n";
		$structured_types = $driver->structuredTypes();
		foreach ($fields as $field) {
			echo "<tr><th>" . h($field["field"]);
			$type = h($field["full_type"]);
			$collation = h($field["collation"]);
			echo "<td><span title='$collation'>"
				. (in_array($type, (array) $structured_types[lang(31)])
					? "<a href='" . h(ME . 'type=' . urlencode($type)) . "'>$type</a>"
					: $type . ($collation && isset($tableStatus["Collation"]) && $collation != $tableStatus["Collation"] ? " $collation" : ""))
				. "</span>"
			;
			echo ($field["null"] ? " <i>NULL</i>" : "");
			echo ($field["auto_increment"] ? " <i>" . lang(50) . "</i>" : "");
			$default = h($field["default"]);
			echo (isset($field["default"]) ? " <span title='" . lang(51) . "'>[<b>" . ($field["generated"] ? "<code class='jush-" . JUSH . "'>$default</code>" : $default) . "</b>]</span>" : "");
			echo (support("comment") ? "<td>" . h($field["comment"]) : "");
			echo "\n";
		}
		echo "</table>\n";
		echo "</div>\n";
	}

	/** Print list of indexes on table in tabular format
	* @param array data about all indexes on a table
	* @return null
	*/
	function tableIndexesPrint($indexes) {
		echo "<table>\n";
		foreach ($indexes as $name => $index) {
			ksort($index["columns"]); // enforce correct columns order
			$print = array();
			foreach ($index["columns"] as $key => $val) {
				$print[] = "<i>" . h($val) . "</i>"
					. ($index["lengths"][$key] ? "(" . $index["lengths"][$key] . ")" : "")
					. ($index["descs"][$key] ? " DESC" : "")
				;
			}
			echo "<tr title='" . h($name) . "'><th>$index[type]<td>" . implode(", ", $print) . "\n";
		}
		echo "</table>\n";
	}

	/** Print columns box in select
	* @param array result of selectColumnsProcess()[0]
	* @param array selectable columns
	* @return null
	*/
	function selectColumnsPrint($select, $columns) {
		global $driver;
		print_fieldset("select", lang(52), $select);
		$i = 0;
		$select[""] = array();
		foreach ($select as $key => $val) {
			$val = $_GET["columns"][$key];
			$column = select_input(
				" name='columns[$i][col]'",
				$columns,
				$val["col"],
				($key !== "" ? "selectFieldChange" : "selectAddRow")
			);
			echo "<div>" . ($driver->functions || $driver->grouping ? html_select("columns[$i][fun]", array(-1 => "") + array_filter(array(lang(53) => $driver->functions, lang(54) => $driver->grouping)), $val["fun"])
				. on_help("event.target.value && event.target.value.replace(/ |\$/, '(') + ')'", 1)
				. script("qsl('select').onchange = function () { helpClose();" . ($key !== "" ? "" : " qsl('select, input', this.parentNode).onchange();") . " };", "")
				. "($column)" : $column) . "</div>\n";
			$i++;
		}
		echo "</div></fieldset>\n";
	}

	/** Print search box in select
	* @param array result of selectSearchProcess()
	* @param array selectable columns
	* @param array
	* @return null
	*/
	function selectSearchPrint($where, $columns, $indexes) {
		print_fieldset("search", lang(55), $where);
		foreach ($indexes as $i => $index) {
			if ($index["type"] == "FULLTEXT") {
				echo "<div>(<i>" . implode("</i>, <i>", array_map('Adminer\h', $index["columns"])) . "</i>) AGAINST";
				echo " <input type='search' name='fulltext[$i]' value='" . h($_GET["fulltext"][$i]) . "'>";
				echo script("qsl('input').oninput = selectFieldChange;", "");
				echo checkbox("boolean[$i]", 1, isset($_GET["boolean"][$i]), "BOOL");
				echo "</div>\n";
			}
		}
		$change_next = "this.parentNode.firstChild.onchange();";
		foreach (array_merge((array) $_GET["where"], array(array())) as $i => $val) {
			if (!$val || ("$val[col]$val[val]" != "" && in_array($val["op"], $this->operators))) {
				echo "<div>" . select_input(
					" name='where[$i][col]'",
					$columns,
					$val["col"],
					($val ? "selectFieldChange" : "selectAddRow"),
					"(" . lang(56) . ")"
				);
				echo html_select("where[$i][op]", $this->operators, $val["op"], $change_next);
				echo "<input type='search' name='where[$i][val]' value='" . h($val["val"]) . "'>";
				echo script("mixin(qsl('input'), {oninput: function () { $change_next }, onkeydown: selectSearchKeydown, onsearch: selectSearchSearch});", "");
				echo "</div>\n";
			}
		}
		echo "</div></fieldset>\n";
	}

	/** Print order box in select
	* @param array result of selectOrderProcess()
	* @param array selectable columns
	* @param array
	* @return null
	*/
	function selectOrderPrint($order, $columns, $indexes) {
		print_fieldset("sort", lang(57), $order);
		$i = 0;
		foreach ((array) $_GET["order"] as $key => $val) {
			if ($val != "") {
				echo "<div>" . select_input(" name='order[$i]'", $columns, $val, "selectFieldChange");
				echo checkbox("desc[$i]", 1, isset($_GET["desc"][$key]), lang(58)) . "</div>\n";
				$i++;
			}
		}
		echo "<div>" . select_input(" name='order[$i]'", $columns, "", "selectAddRow");
		echo checkbox("desc[$i]", 1, false, lang(58)) . "</div>\n";
		echo "</div></fieldset>\n";
	}

	/** Print limit box in select
	* @param string result of selectLimitProcess()
	* @return null
	*/
	function selectLimitPrint($limit) {
		echo "<fieldset><legend>" . lang(59) . "</legend><div>"; // <div> for easy styling
		echo "<input type='number' name='limit' class='size' value='" . h($limit) . "'>";
		echo script("qsl('input').oninput = selectFieldChange;", "");
		echo "</div></fieldset>\n";
	}

	/** Print text length box in select
	* @param string result of selectLengthProcess()
	* @return null
	*/
	function selectLengthPrint($text_length) {
		if ($text_length !== null) {
			echo "<fieldset><legend>" . lang(60) . "</legend><div>";
			echo "<input type='number' name='text_length' class='size' value='" . h($text_length) . "'>";
			echo "</div></fieldset>\n";
		}
	}

	/** Print action box in select
	* @param array
	* @return null
	*/
	function selectActionPrint($indexes) {
		echo "<fieldset><legend>" . lang(61) . "</legend><div>";
		echo "<input type='submit' value='" . lang(52) . "'>";
		echo " <span id='noindex' title='" . lang(62) . "'></span>";
		echo "<script" . nonce() . ">\n";
		echo "const indexColumns = ";
		$columns = array();
		foreach ($indexes as $index) {
			$current_key = reset($index["columns"]);
			if ($index["type"] != "FULLTEXT" && $current_key) {
				$columns[$current_key] = 1;
			}
		}
		$columns[""] = 1;
		foreach ($columns as $key => $val) {
			json_row($key);
		}
		echo ";\n";
		echo "selectFieldChange.call(qs('#form')['select']);\n";
		echo "</script>\n";
		echo "</div></fieldset>\n";
	}

	/** Print command box in select
	* @return bool whether to print default commands
	*/
	function selectCommandPrint() {
		return !information_schema(DB);
	}

	/** Print import box in select
	* @return bool whether to print default import
	*/
	function selectImportPrint() {
		return !information_schema(DB);
	}

	/** Print extra text in the end of a select form
	* @param array fields holding e-mails
	* @param array selectable columns
	* @return null
	*/
	function selectEmailPrint($emailFields, $columns) {
	}

	/** Process columns box in select
	* @param array selectable columns
	* @param array
	* @return array [[select_expressions], [group_expressions]]
	*/
	function selectColumnsProcess($columns, $indexes) {
		global $driver;
		$select = array(); // select expressions, empty for *
		$group = array(); // expressions without aggregation - will be used for GROUP BY if an aggregation function is used
		foreach ((array) $_GET["columns"] as $key => $val) {
			if ($val["fun"] == "count" || ($val["col"] != "" && (!$val["fun"] || in_array($val["fun"], $driver->functions) || in_array($val["fun"], $driver->grouping)))) {
				$select[$key] = apply_sql_function($val["fun"], ($val["col"] != "" ? idf_escape($val["col"]) : "*"));
				if (!in_array($val["fun"], $driver->grouping)) {
					$group[] = $select[$key];
				}
			}
		}
		return array($select, $group);
	}

	/** Process search box in select
	* @param array
	* @param array
	* @return array expressions to join by AND
	*/
	function selectSearchProcess($fields, $indexes) {
		global $connection, $driver;
		$return = array();
		foreach ($indexes as $i => $index) {
			if ($index["type"] == "FULLTEXT" && $_GET["fulltext"][$i] != "") {
				$return[] = "MATCH (" . implode(", ", array_map('Adminer\idf_escape', $index["columns"])) . ") AGAINST (" . q($_GET["fulltext"][$i]) . (isset($_GET["boolean"][$i]) ? " IN BOOLEAN MODE" : "") . ")";
			}
		}
		foreach ((array) $_GET["where"] as $key => $val) {
			if ("$val[col]$val[val]" != "" && in_array($val["op"], $this->operators)) {
				$prefix = "";
				$cond = " $val[op]";
				if (preg_match('~IN$~', $val["op"])) {
					$in = process_length($val["val"]);
					$cond .= " " . ($in != "" ? $in : "(NULL)");
				} elseif ($val["op"] == "SQL") {
					$cond = " $val[val]"; // SQL injection
				} elseif ($val["op"] == "LIKE %%") {
					$cond = " LIKE " . $this->processInput($fields[$val["col"]], "%$val[val]%");
				} elseif ($val["op"] == "ILIKE %%") {
					$cond = " ILIKE " . $this->processInput($fields[$val["col"]], "%$val[val]%");
				} elseif ($val["op"] == "FIND_IN_SET") {
					$prefix = "$val[op](" . q($val["val"]) . ", ";
					$cond = ")";
				} elseif (!preg_match('~NULL$~', $val["op"])) {
					$cond .= " " . $this->processInput($fields[$val["col"]], $val["val"]);
				}
				if ($val["col"] != "") {
					$return[] = $prefix . $driver->convertSearch(idf_escape($val["col"]), $val, $fields[$val["col"]]) . $cond;
				} else {
					// find anywhere
					$cols = array();
					foreach ($fields as $name => $field) {
						if (
							isset($field["privileges"]["where"])
							&& (preg_match('~^[-\d.' . (preg_match('~IN$~', $val["op"]) ? ',' : '') . ']+$~', $val["val"]) || !preg_match('~' . number_type() . '|bit~', $field["type"]))
							&& (!preg_match("~[\x80-\xFF]~", $val["val"]) || preg_match('~char|text|enum|set~', $field["type"]))
							&& (!preg_match('~date|timestamp~', $field["type"]) || preg_match('~^\d+-\d+-\d+~', $val["val"]))
						) {
							$cols[] = $prefix . $driver->convertSearch(idf_escape($name), $val, $field) . $cond;
						}
					}
					$return[] = ($cols ? "(" . implode(" OR ", $cols) . ")" : "1 = 0");
				}
			}
		}
		return $return;
	}

	/** Process order box in select
	* @param array
	* @param array
	* @return array expressions to join by comma
	*/
	function selectOrderProcess($fields, $indexes) {
		$return = array();
		foreach ((array) $_GET["order"] as $key => $val) {
			if ($val != "") {
				$return[] = (preg_match('~^((COUNT\(DISTINCT |[A-Z0-9_]+\()(`(?:[^`]|``)+`|"(?:[^"]|"")+")\)|COUNT\(\*\))$~', $val) ? $val : idf_escape($val)) //! MS SQL uses []
					. (isset($_GET["desc"][$key]) ? " DESC" : "")
				;
			}
		}
		return $return;
	}

	/** Process limit box in select
	* @return string expression to use in LIMIT, will be escaped
	*/
	function selectLimitProcess() {
		return (isset($_GET["limit"]) ? $_GET["limit"] : "50");
	}

	/** Process length box in select
	* @return string number of characters to shorten texts, will be escaped
	*/
	function selectLengthProcess() {
		return (isset($_GET["text_length"]) ? $_GET["text_length"] : "100");
	}

	/** Process extras in select form
	* @param array AND conditions
	* @param array
	* @return bool true if processed, false to process other parts of form
	*/
	function selectEmailProcess($where, $foreignKeys) {
		return false;
	}

	/** Build SQL query used in select
	* @param array result of selectColumnsProcess()[0]
	* @param array result of selectSearchProcess()
	* @param array result of selectColumnsProcess()[1]
	* @param array result of selectOrderProcess()
	* @param int result of selectLimitProcess()
	* @param int index of page starting at zero
	* @return string empty string to use default query
	*/
	function selectQueryBuild($select, $where, $group, $order, $limit, $page) {
		return "";
	}

	/** Query printed after execution in the message
	* @param string executed query
	* @param string elapsed time
	* @param bool
	* @return string
	*/
	function messageQuery($query, $time, $failed = false) {
		global $driver;
		restart_session();
		$history = &get_session("queries");
		if (!$history[$_GET["db"]]) {
			$history[$_GET["db"]] = array();
		}
		if (strlen($query) > 1e6) {
			$query = preg_replace('~[\x80-\xFF]+$~', '', substr($query, 0, 1e6)) . "\n…"; // [\x80-\xFF] - valid UTF-8, \n - can end by one-line comment
		}
		$history[$_GET["db"]][] = array($query, time(), $time); // not DB - $_GET["db"] is changed in database.inc.php //! respect $_GET["ns"]
		$sql_id = "sql-" . count($history[$_GET["db"]]);
		$return = "<a href='#$sql_id' class='toggle'>" . lang(63) . "</a>\n";
		if (!$failed && ($warnings = $driver->warnings())) {
			$id = "warnings-" . count($history[$_GET["db"]]);
			$return = "<a href='#$id' class='toggle'>" . lang(45) . "</a>, $return<div id='$id' class='hidden'>\n$warnings</div>\n";
		}
		return " <span class='time'>" . @date("H:i:s") . "</span>" // @ - time zone may be not set
			. " $return<div id='$sql_id' class='hidden'><pre><code class='jush-" . JUSH . "'>" . shorten_utf8($query, 1000) . "</code></pre>"
			. ($time ? " <span class='time'>($time)</span>" : '')
			. (support("sql") ? '<p><a href="' . h(str_replace("db=" . urlencode(DB), "db=" . urlencode($_GET["db"]), ME) . 'sql=&history=' . (count($history[$_GET["db"]]) - 1)) . '">' . lang(10) . '</a>' : '')
			. '</div>'
		;
	}

	/** Print before edit form
	* @param string
	* @param array
	* @param mixed
	* @param bool
	* @return null
	*/
	function editRowPrint($table, $fields, $row, $update) {
	}

	/** Functions displayed in edit form
	* @param array single field from fields()
	* @return array
	*/
	function editFunctions($field) {
		global $driver;
		$return = ($field["null"] ? "NULL/" : "");
		$update = isset($_GET["select"]) || where($_GET);
		foreach ($driver->editFunctions as $key => $functions) {
			if (!$key || (!isset($_GET["call"]) && $update)) { // relative functions
				foreach ($functions as $pattern => $val) {
					if (!$pattern || preg_match("~$pattern~", $field["type"])) {
						$return .= "/$val";
					}
				}
			}
			if ($key && !preg_match('~set|blob|bytea|raw|file|bool~', $field["type"])) {
				$return .= "/SQL";
			}
		}
		if ($field["auto_increment"] && !$update) {
			$return = lang(50);
		}
		return explode("/", $return);
	}

	/** Get options to display edit field
	* @param string table name
	* @param array single field from fields()
	* @param string attributes to use inside the tag
	* @param string
	* @return string custom input field or empty string for default
	*/
	function editInput($table, $field, $attrs, $value) {
		if ($field["type"] == "enum") {
			return (isset($_GET["select"]) ? "<label><input type='radio'$attrs value='-1' checked><i>" . lang(8) . "</i></label> " : "")
				. ($field["null"] ? "<label><input type='radio'$attrs value=''" . ($value !== null || isset($_GET["select"]) ? "" : " checked") . "><i>NULL</i></label> " : "")
				. enum_input("radio", $attrs, $field, $value, $value === 0 ? 0 : null) // 0 - empty value
			;
		}
		return "";
	}

	/** Get hint for edit field
	* @param string table name
	* @param array single field from fields()
	* @param string
	* @return string
	*/
	function editHint($table, $field, $value) {
		return "";
	}

	/** Process sent input
	* @param array single field from fields()
	* @param string
	* @param string
	* @return string expression to use in a query
	*/
	function processInput($field, $value, $function = "") {
		if ($function == "SQL") {
			return $value; // SQL injection
		}
		$name = $field["field"];
		$return = q($value);
		if (preg_match('~^(now|getdate|uuid)$~', $function)) {
			$return = "$function()";
		} elseif (preg_match('~^current_(date|timestamp)$~', $function)) {
			$return = $function;
		} elseif (preg_match('~^([+-]|\|\|)$~', $function)) {
			$return = idf_escape($name) . " $function $return";
		} elseif (preg_match('~^[+-] interval$~', $function)) {
			$return = idf_escape($name) . " $function " . (preg_match("~^(\\d+|'[0-9.: -]') [A-Z_]+\$~i", $value) ? $value : $return);
		} elseif (preg_match('~^(addtime|subtime|concat)$~', $function)) {
			$return = "$function(" . idf_escape($name) . ", $return)";
		} elseif (preg_match('~^(md5|sha1|password|encrypt)$~', $function)) {
			$return = "$function($return)";
		}
		return unconvert_field($field, $return);
	}

	/** Return export output options
	* @return array
	*/
	function dumpOutput() {
		$return = array('text' => lang(64), 'file' => lang(65));
		if (function_exists('gzencode')) {
			$return['gz'] = 'gzip';
		}
		return $return;
	}

	/** Return export format options
	* @return array empty to disable export
	*/
	function dumpFormat() {
		return (support("dump") ? array('sql' => 'SQL') : array()) + array('csv' => 'CSV,', 'csv;' => 'CSV;', 'tsv' => 'TSV');
	}

	/** Export database structure
	* @param string
	* @return null prints data
	*/
	function dumpDatabase($db) {
	}

	/** Export table structure
	* @param string
	* @param string
	* @param int 0 table, 1 view, 2 temporary view table
	* @return null prints data
	*/
	function dumpTable($table, $style, $is_view = 0) {
		if ($_POST["format"] != "sql") {
			echo "\xef\xbb\xbf"; // UTF-8 byte order mark
			if ($style) {
				dump_csv(array_keys(fields($table)));
			}
		} else {
			if ($is_view == 2) {
				$fields = array();
				foreach (fields($table) as $name => $field) {
					$fields[] = idf_escape($name) . " $field[full_type]";
				}
				$create = "CREATE TABLE " . table($table) . " (" . implode(", ", $fields) . ")";
			} else {
				$create = create_sql($table, $_POST["auto_increment"], $style);
			}
			set_utf8mb4($create);
			if ($style && $create) {
				if ($style == "DROP+CREATE" || $is_view == 1) {
					echo "DROP " . ($is_view == 2 ? "VIEW" : "TABLE") . " IF EXISTS " . table($table) . ";\n";
				}
				if ($is_view == 1) {
					$create = remove_definer($create);
				}
				echo "$create;\n\n";
			}
		}
	}

	/** Export table data
	* @param string
	* @param string
	* @param string
	* @return null prints data
	*/
	function dumpData($table, $style, $query) {
		global $connection;
		if ($style) {
			$max_packet = (JUSH == "sqlite" ? 0 : 1048576); // default, minimum is 1024
			$fields = array();
			$identity_insert = false;
			if ($_POST["format"] == "sql") {
				if ($style == "TRUNCATE+INSERT") {
					echo truncate_sql($table) . ";\n";
				}
				$fields = fields($table);
				if (JUSH == "mssql") {
					foreach ($fields as $field) {
						if ($field["auto_increment"]) {
							echo "SET IDENTITY_INSERT " . table($table) . " ON;\n";
							$identity_insert = true;
							break;
						}
					}
				}
			}
			$result = $connection->query($query, 1); // 1 - MYSQLI_USE_RESULT //! enum and set as numbers
			if ($result) {
				$insert = "";
				$buffer = "";
				$keys = array();
				$generated = array();
				$suffix = "";
				$fetch_function = ($table != '' ? 'fetch_assoc' : 'fetch_row');
				while ($row = $result->$fetch_function()) {
					if (!$keys) {
						$values = array();
						foreach ($row as $val) {
							$field = $result->fetch_field();
							if ($fields[$field->name]['generated']) {
								$generated[$field->name] = true;
								continue;
							}
							$keys[] = $field->name;
							$key = idf_escape($field->name);
							$values[] = "$key = VALUES($key)";
						}
						$suffix = ($style == "INSERT+UPDATE" ? "\nON DUPLICATE KEY UPDATE " . implode(", ", $values) : "") . ";\n";
					}
					if ($_POST["format"] != "sql") {
						if ($style == "table") {
							dump_csv($keys);
							$style = "INSERT";
						}
						dump_csv($row);
					} else {
						if (!$insert) {
							$insert = "INSERT INTO " . table($table) . " (" . implode(", ", array_map('Adminer\idf_escape', $keys)) . ") VALUES";
						}
						foreach ($row as $key => $val) {
							if ($generated[$key]) {
								unset($row[$key]);
								continue;
							}
							$field = $fields[$key];
							$row[$key] = ($val !== null
								? unconvert_field($field, preg_match(number_type(), $field["type"]) && !preg_match('~\[~', $field["full_type"]) && is_numeric($val) ? $val : q(($val === false ? 0 : $val)))
								: "NULL"
							);
						}
						$s = ($max_packet ? "\n" : " ") . "(" . implode(",\t", $row) . ")";
						if (!$buffer) {
							$buffer = $insert . $s;
						} elseif (strlen($buffer) + 4 + strlen($s) + strlen($suffix) < $max_packet) { // 4 - length specification
							$buffer .= ",$s";
						} else {
							echo $buffer . $suffix;
							$buffer = $insert . $s;
						}
					}
				}
				if ($buffer) {
					echo $buffer . $suffix;
				}
			} elseif ($_POST["format"] == "sql") {
				echo "-- " . str_replace("\n", " ", $connection->error) . "\n";
			}
			if ($identity_insert) {
				echo "SET IDENTITY_INSERT " . table($table) . " OFF;\n";
			}
		}
	}

	/** Set export filename
	* @param string
	* @return string filename without extension
	*/
	function dumpFilename($identifier) {
		return friendly_url($identifier != "" ? $identifier : (SERVER != "" ? SERVER : "localhost"));
	}

	/** Send headers for export
	* @param string
	* @param bool
	* @return string extension
	*/
	function dumpHeaders($identifier, $multi_table = false) {
		$output = $_POST["output"];
		$ext = (preg_match('~sql~', $_POST["format"]) ? "sql" : ($multi_table ? "tar" : "csv")); // multiple CSV packed to TAR
		header("Content-Type: " .
			($output == "gz" ? "application/x-gzip" :
			($ext == "tar" ? "application/x-tar" :
			($ext == "sql" || $output != "file" ? "text/plain" : "text/csv") . "; charset=utf-8"
		)));
		if ($output == "gz") {
			ob_start(function ($string) {
				// ob_start() callback receives an optional parameter $phase but gzencode() accepts optional parameter $level
				return gzencode($string);
			}, 1e6);
		}
		return $ext;
	}

	/** Print text after export
	* @return null prints data
	*/
	function dumpFooter() {
		if ($_POST["format"] == "sql") {
			echo "-- " . gmdate("Y-m-d H:i:s e") . "\n";
		}
	}

	/** Set the path of the file for webserver load
	* @return string path of the sql dump file
	*/
	function importServerPath() {
		return "adminer.sql";
	}

	/** Print homepage
	* @return bool whether to print default homepage
	*/
	function homepage() {
		echo '<p class="links">' . ($_GET["ns"] == "" && support("database") ? '<a href="' . h(ME) . 'database=">' . lang(66) . "</a>\n" : "");
		echo (support("scheme") ? "<a href='" . h(ME) . "scheme='>" . ($_GET["ns"] != "" ? lang(67) : lang(68)) . "</a>\n" : "");
		echo ($_GET["ns"] !== "" ? '<a href="' . h(ME) . 'schema=">' . lang(69) . "</a>\n" : "");
		echo (support("privileges") ? "<a href='" . h(ME) . "privileges='>" . lang(70) . "</a>\n" : "");
		return true;
	}

	/** Print navigation after Adminer title
	* @param string can be "auth" if there is no database connection, "db" if there is no database selected, "ns" with invalid schema
	* @return null
	*/
	function navigation($missing) {
		global $VERSION, $drivers, $connection;
		echo "<h1>" . $this->name() . " <span class='version'>$VERSION";
		$new_version = $_COOKIE["adminer_version"];
		echo " <a href='https://www.adminer.org/#download'" . target_blank() . " id='version'>" . (version_compare($VERSION, $new_version) < 0 ? h($new_version) : "") . "</a>";
		echo "</span></h1>\n";
		// this is matched by compile.php
		switch_lang();
		if ($missing == "auth") {
			$output = "";
			foreach ((array) $_SESSION["pwds"] as $vendor => $servers) {
				foreach ($servers as $server => $usernames) {
					$name = h(get_setting("vendor-$vendor-$server") ?: $drivers[$vendor]);
					foreach ($usernames as $username => $password) {
						if ($password !== null) {
							$dbs = $_SESSION["db"][$vendor][$server][$username];
							foreach (($dbs ? array_keys($dbs) : array("")) as $db) {
								$output .= "<li><a href='" . h(auth_url($vendor, $server, $username, $db)) . "'>($name) " . h($username . ($server != "" ? "@" . $this->serverName($server) : "") . ($db != "" ? " - $db" : "")) . "</a>\n";
							}
						}
					}
				}
			}
			if ($output) {
				echo "<ul id='logins'>\n$output</ul>\n" . script("mixin(qs('#logins'), {onmouseover: menuOver, onmouseout: menuOut});");
			}
		} else {
			$tables = array();
			if ($_GET["ns"] !== "" && !$missing && DB != "") {
				$connection->select_db(DB);
				$tables = table_status('', true);
			}
			$this->syntaxHighlighting($tables);
			$this->databasesPrint($missing);
			$actions = array();
			if (DB == "" || !$missing) {
				if (support("sql")) {
					$actions[] = "<a href='" . h(ME) . "sql='" . bold(isset($_GET["sql"]) && !isset($_GET["import"])) . ">" . lang(63) . "</a>";
					$actions[] = "<a href='" . h(ME) . "import='" . bold(isset($_GET["import"])) . ">" . lang(71) . "</a>";
				}
				$actions[] = "<a href='" . h(ME) . "dump=" . urlencode(isset($_GET["table"]) ? $_GET["table"] : $_GET["select"]) . "' id='dump'" . bold(isset($_GET["dump"])) . ">" . lang(72) . "</a>";
			}
			$in_db = $_GET["ns"] !== "" && !$missing && DB != "";
			if ($in_db) {
				$actions[] = '<a href="' . h(ME) . 'create="' . bold($_GET["create"] === "") . ">" . lang(73) . "</a>";
			}
			echo ($actions ? "<p class='links'>\n" . implode("\n", $actions) . "\n" : "");
			if ($in_db) {
				if ($tables) {
					$this->tablesPrint($tables);
				} else {
					echo "<p class='message'>" . lang(9) . "</p>\n";
				}
			}
		}
	}

	/** Set up syntax highlight for code and <textarea>
	* @param array result of table_status()
	*/
	function syntaxHighlighting($tables) {
		global $connection;
		// this is matched by compile.php
		echo script_src(preg_replace("~\\?.*~", "", ME) . "?file=jush.js&version=5.1.0");
		if (support("sql")) {
			echo "<script" . nonce() . ">\n";
			if ($tables) {
				$links = array();
				foreach ($tables as $table => $type) {
					$links[] = preg_quote($table, '/');
				}
				echo "var jushLinks = { " . JUSH . ": [ '" . js_escape(ME) . (support("table") ? "table=" : "select=") . "\$&', /\\b(" . implode("|", $links) . ")\\b/g ] };\n";
				foreach (array("bac", "bra", "sqlite_quo", "mssql_bra") as $val) {
					echo "jushLinks.$val = jushLinks." . JUSH . ";\n";
				}
			}
			echo "</script>\n";
		}
		echo script("syntaxHighlighting('" . (is_object($connection) ? preg_replace('~^(\d\.?\d).*~s', '\1', $connection->server_info) : "") . "'"
			. ($connection->flavor == 'maria' ? ", 'maria'" : ($connection->flavor == 'cockroach' ? ", 'cockroach'" : "")) . ");"
		);
	}

	/** Print databases list in menu
	* @param string
	* @return null
	*/
	function databasesPrint($missing) {
		global $adminer, $connection;
		$databases = $this->databases();
		if (DB && $databases && !in_array(DB, $databases)) {
			array_unshift($databases, DB);
		}
		echo "<form action=''>\n<p id='dbs'>\n";
		hidden_fields_get();
		$db_events = script("mixin(qsl('select'), {onmousedown: dbMouseDown, onchange: dbChange});");
		echo "<span title='" . lang(36) . "'>" . lang(74) . ":</span> " . ($databases
			? html_select("db", array("" => "") + $databases, DB) . $db_events
			: "<input name='db' value='" . h(DB) . "' autocapitalize='off' size='19'>\n"
		);
		echo "<input type='submit' value='" . lang(20) . "'" . ($databases ? " class='hidden'" : "") . ">\n";
		if (support("scheme")) {
			if ($missing != "db" && DB != "" && $connection->select_db(DB)) {
				echo "<br><span>" . lang(75) . ":</span> " . html_select("ns", array("" => "") + $adminer->schemas(), $_GET["ns"]) . $db_events;
				if ($_GET["ns"] != "") {
					set_schema($_GET["ns"]);
				}
			}
		}
		foreach (array("import", "sql", "schema", "dump", "privileges") as $val) {
			if (isset($_GET[$val])) {
				echo input_hidden($val);
				break;
			}
		}
		echo "</p></form>\n";
	}

	/** Print table list in menu
	* @param array result of table_status('', true)
	* @return null
	*/
	function tablesPrint($tables) {
		echo "<ul id='tables'>" . script("mixin(qs('#tables'), {onmouseover: menuOver, onmouseout: menuOut});");
		foreach ($tables as $table => $status) {
			$name = $this->tableName($status);
			if ($name != "") {
				echo '<li><a href="' . h(ME) . 'select=' . urlencode($table) . '"'
					. bold($_GET["select"] == $table || $_GET["edit"] == $table, "select")
					. " title='" . lang(40) . "'>" . lang(76) . "</a> "
				;
				echo (support("table") || support("indexes")
					? '<a href="' . h(ME) . 'table=' . urlencode($table) . '"'
						. bold(in_array($table, array($_GET["table"], $_GET["create"], $_GET["indexes"], $_GET["foreign"], $_GET["trigger"])), (is_view($status) ? "view" : "structure"))
						. " title='" . lang(41) . "'>$name</a>"
					: "<span>$name</span>"
				) . "\n";
			}
		}
		echo "</ul>\n";
	}
}

?>
<?php
class Plugins extends Adminer {
	public $plugins; ///< @var protected(set) array

	/** Register plugins
	* @param array object instances or null to autoload plugins from adminer-plugins/
	*/
	function __construct($plugins) {
		if ($plugins === null) {
			$plugins = array();
			$basename = "adminer-plugins";
			if (is_dir($basename)) {
				foreach (glob("$basename/*.php") as $filename) {
					$include = include_once "./$filename";
				}
			}
			$help = " href='https://www.adminer.org/plugins/#use'" . target_blank();
			if (file_exists("$basename.php")) {
				$include = include_once "./$basename.php"; // example: return array(new AdminerLoginOtp($secret))
				if (is_array($include)) {
					foreach ($include as $plugin) {
						$plugins[get_class($plugin)] = $plugin;
					}
				} else {
					$this->error .= lang(77, "<b>$basename.php</b>", $help) . "<br>";
				}
			}
			foreach (get_declared_classes() as $class) {
				if (!$plugins[$class] && preg_match('~^Adminer\w~i', $class)) {
					$reflection = new \ReflectionClass($class);
					$constructor = $reflection->getConstructor();
					if ($constructor && $constructor->getNumberOfRequiredParameters()) {
						$this->error .= lang(78, $help, "<b>$class</b>", "<b>$basename.php</b>") . "<br>";
					} else {
						$plugins[$class] = new $class;
					}
				}
			}
		}
		$this->plugins = $plugins;
	}

	private function callParent($function, $args) {
		return call_user_func_array(array('parent', $function), $args);
	}

	private function applyPlugin($function, $params) {
		$args = array();
		foreach ($params as $key => $val) {
			// some plugins accept params by reference - we don't need to propage it outside, just to the other plugins
			$args[] = &$params[$key];
		}
		foreach ($this->plugins as $plugin) {
			if (method_exists($plugin, $function)) {
				$return = call_user_func_array(array($plugin, $function), $args);
				if ($return !== null) {
					return $return;
				}
			}
		}
		return $this->callParent($function, $args);
	}

	private function appendPlugin($function, $args) {
		$return = $this->callParent($function, $args);
		foreach ($this->plugins as $plugin) {
			if (method_exists($plugin, $function)) {
				$value = call_user_func_array(array($plugin, $function), $args);
				if ($value) {
					$return += $value;
				}
			}
		}
		return $return;
	}

	// appendPlugin

	function dumpFormat() {
		$args = func_get_args();
		return $this->appendPlugin(__FUNCTION__, $args);
	}

	function dumpOutput() {
		$args = func_get_args();
		return $this->appendPlugin(__FUNCTION__, $args);
	}

	function editRowPrint($table, $fields, $row, $update) {
		$args = func_get_args();
		return $this->appendPlugin(__FUNCTION__, $args);
	}

	function editFunctions($field) {
		$args = func_get_args();
		return $this->appendPlugin(__FUNCTION__, $args);
	}

	// applyPlugin

	function name() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function credentials() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function connectSsl() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function permanentLogin($create = false) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function bruteForceKey() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function serverName($server) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function database() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function schemas() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function databases($flush = true) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function queryTimeout() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function headers() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function csp() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function head($dark = null) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function css() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function loginForm() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function loginFormField($name, $heading, $value) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function login($login, $password) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function tableName($tableStatus) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function fieldName($field, $order = 0) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectLinks($tableStatus, $set = "") {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function foreignKeys($table) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function backwardKeys($table, $tableName) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function backwardKeysPrint($backwardKeys, $row) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectQuery($query, $start, $failed = false) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function sqlCommandQuery($query) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function sqlPrintAfter() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function rowDescription($table) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function rowDescriptions($rows, $foreignKeys) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectLink($val, $field) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectVal($val, $link, $field, $original) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function editVal($val, $field) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function tableStructurePrint($fields, $tableStatus = null) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function tableIndexesPrint($indexes) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectColumnsPrint($select, $columns) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectSearchPrint($where, $columns, $indexes) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectOrderPrint($order, $columns, $indexes) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectLimitPrint($limit) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectLengthPrint($text_length) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectActionPrint($indexes) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectCommandPrint() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectImportPrint() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectEmailPrint($emailFields, $columns) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectColumnsProcess($columns, $indexes) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectSearchProcess($fields, $indexes) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectOrderProcess($fields, $indexes) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectLimitProcess() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectLengthProcess() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectEmailProcess($where, $foreignKeys) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function selectQueryBuild($select, $where, $group, $order, $limit, $page) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function messageQuery($query, $time, $failed = false) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function editInput($table, $field, $attrs, $value) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function editHint($table, $field, $value) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function processInput($field, $value, $function = "") {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function dumpDatabase($db) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function dumpTable($table, $style, $is_view = 0) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function dumpData($table, $style, $query) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function dumpFilename($identifier) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function dumpHeaders($identifier, $multi_table = false) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function dumpFooter() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function importServerPath() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function homepage() {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function navigation($missing) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function syntaxHighlighting($tables) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function databasesPrint($missing) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}

	function tablesPrint($tables) {
		$args = func_get_args();
		return $this->applyPlugin(__FUNCTION__, $args);
	}
}


if (function_exists('adminer_object')) {
	$adminer = adminer_object();
} elseif (is_dir("adminer-plugins") || file_exists("adminer-plugins.php")) {
	$adminer = new Plugins(null);
} else {
	$adminer = new Adminer;
}

// this is matched by compile.php
?>
<?php
$drivers = array("server" => "MySQL / MariaDB") + $drivers;

if (!defined('Adminer\DRIVER')) {
	define('Adminer\DRIVER', "server"); // server - backwards compatibility
	// MySQLi supports everything, MySQL doesn't support multiple result sets, PDO_MySQL doesn't support orgtable
	if (extension_loaded("mysqli") && $_GET["ext"] != "pdo") {
		class Db extends \MySQLi {
			public $extension = "MySQLi", $flavor = '';

			function __construct() {
				parent::init();
			}

			/** @see https://php.net/mysqli.construct */
			function connect($server = "", $username = "", $password = "", $database = null, $port = null, $socket = null) {
				global $adminer;
				mysqli_report(MYSQLI_REPORT_OFF); // stays between requests, not required since PHP 5.3.4
				list($host, $port) = explode(":", $server, 2); // part after : is used for port or socket
				$ssl = $adminer->connectSsl();
				if ($ssl) {
					$this->ssl_set($ssl['key'], $ssl['cert'], $ssl['ca'], '', '');
				}
				$return = @$this->real_connect(
					($server != "" ? $host : ini_get("mysqli.default_host")),
					($server . $username != "" ? $username : ini_get("mysqli.default_user")),
					($server . $username . $password != "" ? $password : ini_get("mysqli.default_pw")),
					$database,
					(is_numeric($port) ? $port : ini_get("mysqli.default_port")),
					(!is_numeric($port) ? $port : $socket),
					($ssl ? ($ssl['verify'] !== false ? 2048 : 64) : 0) // 2048 - MYSQLI_CLIENT_SSL, 64 - MYSQLI_CLIENT_SSL_DONT_VERIFY_SERVER_CERT (not available before PHP 5.6.16)
				);
				$this->options(MYSQLI_OPT_LOCAL_INFILE, false);
				return $return;
			}

			function set_charset($charset) {
				if (parent::set_charset($charset)) {
					return true;
				}
				// the client library may not support utf8mb4
				parent::set_charset('utf8');
				return $this->query("SET NAMES $charset");
			}

			function result($query, $field = 0) {
				$result = $this->query($query);
				if (!$result) {
					return false;
				}
				$row = $result->fetch_array();
				return ($row ? $row[$field] : false);
			}

			function quote($string) {
				return "'" . $this->escape_string($string) . "'";
			}
		}

	} elseif (extension_loaded("mysql") && !((ini_bool("sql.safe_mode") || ini_bool("mysql.allow_local_infile")) && extension_loaded("pdo_mysql"))) {
		class Db {
			public
				$extension = "MySQL", ///< @var string extension name
				$flavor = '', ///< @var string different vendor with the same API, e.g. MariaDB, usually stays empty
				$server_info, ///< @var string server version
				$affected_rows, ///< @var int number of affected rows
				$info, ///< @var string see https://php.net/mysql_info
				$errno, ///< @var int last error code
				$error ///< @var string last error message
			;
			private $link, $result;

			/** Connect to server
			* @param string
			* @param string
			* @param string
			* @return bool
			*/
			function connect($server, $username, $password) {
				if (ini_bool("mysql.allow_local_infile")) {
					$this->error = lang(79, "'mysql.allow_local_infile'", "MySQLi", "PDO_MySQL");
					return false;
				}
				$this->link = @mysql_connect(
					($server != "" ? $server : ini_get("mysql.default_host")),
					("$server$username" != "" ? $username : ini_get("mysql.default_user")),
					("$server$username$password" != "" ? $password : ini_get("mysql.default_password")),
					true,
					131072 // CLIENT_MULTI_RESULTS for CALL
				);
				if ($this->link) {
					$this->server_info = mysql_get_server_info($this->link);
				} else {
					$this->error = mysql_error();
				}
				return (bool) $this->link;
			}

			/** Set the client character set
			* @param string
			* @return bool
			*/
			function set_charset($charset) {
				if (function_exists('mysql_set_charset')) {
					if (mysql_set_charset($charset, $this->link)) {
						return true;
					}
					// the client library may not support utf8mb4
					mysql_set_charset('utf8', $this->link);
				}
				return $this->query("SET NAMES $charset");
			}

			/** Quote string to use in SQL
			* @param string
			* @return string escaped string enclosed in '
			*/
			function quote($string) {
				return "'" . mysql_real_escape_string($string, $this->link) . "'";
			}

			/** Select database
			* @param string
			* @return bool
			*/
			function select_db($database) {
				return mysql_select_db($database, $this->link);
			}

			/** Send query
			* @param string
			* @param bool
			* @return mixed bool or Result
			*/
			function query($query, $unbuffered = false) {
				$result = @($unbuffered ? mysql_unbuffered_query($query, $this->link) : mysql_query($query, $this->link)); // @ - mute mysql.trace_mode
				$this->error = "";
				if (!$result) {
					$this->errno = mysql_errno($this->link);
					$this->error = mysql_error($this->link);
					return false;
				}
				if ($result === true) {
					$this->affected_rows = mysql_affected_rows($this->link);
					$this->info = mysql_info($this->link);
					return true;
				}
				return new Result($result);
			}

			/** Send query with more resultsets
			* @param string
			* @return bool
			*/
			function multi_query($query) {
				return $this->result = $this->query($query);
			}

			/** Get current resultset
			* @return Result
			*/
			function store_result() {
				return $this->result;
			}

			/** Fetch next resultset
			* @return bool
			*/
			function next_result() {
				// MySQL extension doesn't support multiple results
				return false;
			}

			/** Get single field from result
			* @param string
			* @param int
			* @return string
			*/
			function result($query, $field = 0) {
				$result = $this->query($query);
				return ($result ? $result->fetch_column($field) : false);
			}
		}

		class Result {
			public $num_rows; ///< @var int number of rows in the result
			private $result, $offset = 0;

			/** Constructor
			* @param resource
			*/
			function __construct($result) {
				$this->result = $result;
				$this->num_rows = mysql_num_rows($result);
			}

			/** Fetch next row as associative array
			* @return array
			*/
			function fetch_assoc() {
				return mysql_fetch_assoc($this->result);
			}

			/** Fetch next row as numbered array
			* @return array
			*/
			function fetch_row() {
				return mysql_fetch_row($this->result);
			}

			/** Fetch a single column
			* @param int
			* @return string or false if there are no rows
			*/
			function fetch_column($field) {
				return ($this->num_rows ? mysql_result($this->result, 0, $field) : false);
			}

			/** Fetch next field
			* @return object properties: name, type (0 number, 15 varchar, 254 char), charsetnr (63 binary); optionally: table, orgtable, orgname
			*/
			function fetch_field() {
				$return = mysql_fetch_field($this->result, $this->offset++); // offset required under certain conditions
				$return->orgtable = $return->table;
				$return->charsetnr = ($return->blob ? 63 : 0);
				return $return;
			}

			/** Free result set
			*/
			function __destruct() {
				mysql_free_result($this->result);
			}
		}

	} elseif (extension_loaded("pdo_mysql")) {
		class Db extends PdoDb {
			public $extension = "PDO_MySQL";

			function connect($server, $username, $password) {
				global $adminer;
				$options = array(\PDO::MYSQL_ATTR_LOCAL_INFILE => false);
				$ssl = $adminer->connectSsl();
				if ($ssl) {
					if ($ssl['key']) {
						$options[\PDO::MYSQL_ATTR_SSL_KEY] = $ssl['key'];
					}
					if ($ssl['cert']) {
						$options[\PDO::MYSQL_ATTR_SSL_CERT] = $ssl['cert'];
					}
					if ($ssl['ca']) {
						$options[\PDO::MYSQL_ATTR_SSL_CA] = $ssl['ca'];
					}
					if (isset($ssl['verify'])) {
						$options[\PDO::MYSQL_ATTR_SSL_VERIFY_SERVER_CERT] = $ssl['verify'];
					}
				}
				$this->dsn(
					"mysql:charset=utf8;host=" . str_replace(":", ";unix_socket=", preg_replace('~:(\d)~', ';port=\1', $server)),
					$username,
					$password,
					$options
				);
				return true;
			}

			function set_charset($charset) {
				$this->query("SET NAMES $charset"); // charset in DSN is ignored before PHP 5.3.6
			}

			function select_db($database) {
				// database selection is separated from the connection so dbname in DSN can't be used
				return $this->query("USE " . idf_escape($database));
			}

			function query($query, $unbuffered = false) {
				$this->pdo->setAttribute(\PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, !$unbuffered);
				return parent::query($query, $unbuffered);
			}
		}

	}



	class Driver extends SqlDriver {
		static $possibleDrivers = array("MySQLi", "MySQL", "PDO_MySQL");
		static $jush = "sql"; ///< @var string JUSH identifier

		public $unsigned = array("unsigned", "zerofill", "unsigned zerofill");
		public $operators = array("=", "<", ">", "<=", ">=", "!=", "LIKE", "LIKE %%", "REGEXP", "IN", "FIND_IN_SET", "IS NULL", "NOT LIKE", "NOT REGEXP", "NOT IN", "IS NOT NULL", "SQL");
		public $functions = array("char_length", "date", "from_unixtime", "lower", "round", "floor", "ceil", "sec_to_time", "time_to_sec", "upper");
		public $grouping = array("avg", "count", "count distinct", "group_concat", "max", "min", "sum");

		function __construct($connection) {
			parent::__construct($connection);
			$this->types = array(
				lang(25) => array("tinyint" => 3, "smallint" => 5, "mediumint" => 8, "int" => 10, "bigint" => 20, "decimal" => 66, "float" => 12, "double" => 21),
				lang(26) => array("date" => 10, "datetime" => 19, "timestamp" => 19, "time" => 10, "year" => 4),
				lang(27) => array("char" => 255, "varchar" => 65535, "tinytext" => 255, "text" => 65535, "mediumtext" => 16777215, "longtext" => 4294967295),
				lang(80) => array("enum" => 65535, "set" => 64),
				lang(28) => array("bit" => 20, "binary" => 255, "varbinary" => 65535, "tinyblob" => 255, "blob" => 65535, "mediumblob" => 16777215, "longblob" => 4294967295),
				lang(30) => array("geometry" => 0, "point" => 0, "linestring" => 0, "polygon" => 0, "multipoint" => 0, "multilinestring" => 0, "multipolygon" => 0, "geometrycollection" => 0),
			);
			$this->editFunctions = array(
				array(
					"char" => "md5/sha1/password/encrypt/uuid",
					"binary" => "md5/sha1",
					"date|time" => "now",
				), array(
					number_type() => "+/-",
					"date" => "+ interval/- interval",
					"time" => "addtime/subtime",
					"char|text" => "concat",
				)
			);
			if (min_version('5.7.8', 10.2, $connection)) {
				$this->types[lang(27)]["json"] = 4294967295;
			}
			if (min_version('', 10.7, $connection)) {
				$this->types[lang(27)]["uuid"] = 128;
				$this->editFunctions[0]['uuid'] = 'uuid';
			}
			if (min_version(9, '', $connection)) {
				$this->types[lang(25)]["vector"] = 16383;
				$this->editFunctions[0]['vector'] = 'string_to_vector';
			}
			if (min_version(5.7, 10.2, $connection)) {
				$this->generated = array("STORED", "VIRTUAL");
			}
		}

		function unconvertFunction($field) {
			return (preg_match("~binary~", $field["type"]) ? "<code class='jush-sql'>UNHEX</code>"
				: ($field["type"] == "bit" ? doc_link(array('sql' => 'bit-value-literals.html'), "<code>b''</code>")
				: (preg_match("~geometry|point|linestring|polygon~", $field["type"]) ? "<code class='jush-sql'>GeomFromText</code>"
				: "")));
		}

		function insert($table, $set) {
			return ($set ? parent::insert($table, $set) : queries("INSERT INTO " . table($table) . " ()\nVALUES ()"));
		}

		function insertUpdate($table, $rows, $primary) {
			$columns = array_keys(reset($rows));
			$prefix = "INSERT INTO " . table($table) . " (" . implode(", ", $columns) . ") VALUES\n";
			$values = array();
			foreach ($columns as $key) {
				$values[$key] = "$key = VALUES($key)";
			}
			$suffix = "\nON DUPLICATE KEY UPDATE " . implode(", ", $values);
			$values = array();
			$length = 0;
			foreach ($rows as $set) {
				$value = "(" . implode(", ", $set) . ")";
				if ($values && (strlen($prefix) + $length + strlen($value) + strlen($suffix) > 1e6)) { // 1e6 - default max_allowed_packet
					if (!queries($prefix . implode(",\n", $values) . $suffix)) {
						return false;
					}
					$values = array();
					$length = 0;
				}
				$values[] = $value;
				$length += strlen($value) + 2; // 2 - strlen(",\n")
			}
			return queries($prefix . implode(",\n", $values) . $suffix);
		}

		function slowQuery($query, $timeout) {
			if (min_version('5.7.8', '10.1.2')) {
				if ($this->conn->flavor == 'maria') {
					return "SET STATEMENT max_statement_time=$timeout FOR $query";
				} elseif (preg_match('~^(SELECT\b)(.+)~is', $query, $match)) {
					return "$match[1] /*+ MAX_EXECUTION_TIME(" . ($timeout * 1000) . ") */ $match[2]";
				}
			}
		}

		function convertSearch($idf, $val, $field) {
			return (preg_match('~char|text|enum|set~', $field["type"]) && !preg_match("~^utf8~", $field["collation"]) && preg_match('~[\x80-\xFF]~', $val['val'])
				? "CONVERT($idf USING " . charset($this->conn) . ")"
				: $idf
			);
		}

		function warnings() {
			$result = $this->conn->query("SHOW WARNINGS");
			if ($result && $result->num_rows) {
				ob_start();
				select($result); // select() usually needs to print a big table progressively
				return ob_get_clean();
			}
		}

		function tableHelp($name, $is_view = false) {
			$maria = ($this->conn->flavor == 'maria');
			if (information_schema(DB)) {
				return strtolower("information-schema-" . ($maria ? "$name-table/" : str_replace("_", "-", $name) . "-table.html"));
			}
			if (DB == "mysql") {
				return ($maria ? "mysql$name-table/" : "system-schema.html"); //! more precise link
			}
		}

		function hasCStyleEscapes() {
			static $c_style;
			if ($c_style === null) {
				$sql_mode = $this->conn->result("SHOW VARIABLES LIKE 'sql_mode'", 1);
				$c_style = (strpos($sql_mode, 'NO_BACKSLASH_ESCAPES') === false);
			}
			return $c_style;
		}

		function engines() {
			$return = array();
			foreach (get_rows("SHOW ENGINES") as $row) {
				if (preg_match("~YES|DEFAULT~", $row["Support"])) {
					$return[] = $row["Engine"];
				}
			}
			return $return;
		}
	}



	/** Escape database identifier
	* @param string
	* @return string
	*/
	function idf_escape($idf) {
		return "`" . str_replace("`", "``", $idf) . "`";
	}

	/** Get escaped table name
	* @param string
	* @return string
	*/
	function table($idf) {
		return idf_escape($idf);
	}

	/** Connect to the database
	* @param array [$server, $username, $password]
	* @return mixed Db or string for error
	*/
	function connect($credentials) {
		global $drivers;
		$connection = new Db;
		if ($connection->connect($credentials[0], $credentials[1], $credentials[2])) {
			$connection->set_charset(charset($connection));
			$connection->query("SET sql_quote_show_create = 1, autocommit = 1");
			$connection->flavor = (preg_match('~MariaDB~', $connection->server_info) ? 'maria' : '');
			$drivers[DRIVER] = ($connection->flavor == 'maria' ? "MariaDB" : "MySQL");
			return $connection;
		}
		$return = $connection->error;
		if (function_exists('iconv') && !is_utf8($return) && strlen($s = iconv("windows-1250", "utf-8", $return)) > strlen($return)) { // windows-1250 - most common Windows encoding
			$return = $s;
		}
		return $return;
	}

	/** Get cached list of databases
	* @param bool
	* @return array
	*/
	function get_databases($flush) {
		// SHOW DATABASES can take a very long time so it is cached
		$return = get_session("dbs");
		if ($return === null) {
			$query = "SELECT SCHEMA_NAME FROM information_schema.SCHEMATA ORDER BY SCHEMA_NAME"; // SHOW DATABASES can be disabled by skip_show_database
			$return = ($flush ? slow_query($query) : get_vals($query));
			restart_session();
			set_session("dbs", $return);
			stop_session();
		}
		return $return;
	}

	/** Formulate SQL query with limit
	* @param string everything after SELECT
	* @param string including WHERE
	* @param int
	* @param int
	* @param string
	* @return string
	*/
	function limit($query, $where, $limit, $offset = 0, $separator = " ") {
		return " $query$where" . ($limit !== null ? $separator . "LIMIT $limit" . ($offset ? " OFFSET $offset" : "") : "");
	}

	/** Formulate SQL modification query with limit 1
	* @param string
	* @param string everything after UPDATE or DELETE
	* @param string
	* @param string
	* @return string
	*/
	function limit1($table, $query, $where, $separator = "\n") {
		return limit($query, $where, 1, 0, $separator);
	}

	/** Get database collation
	* @param string
	* @param array result of collations()
	* @return string
	*/
	function db_collation($db, $collations) {
		$return = null;
		$create = get_val("SHOW CREATE DATABASE " . idf_escape($db), 1);
		if (preg_match('~ COLLATE ([^ ]+)~', $create, $match)) {
			$return = $match[1];
		} elseif (preg_match('~ CHARACTER SET ([^ ]+)~', $create, $match)) {
			// default collation
			$return = $collations[$match[1]][-1];
		}
		return $return;
	}

	/** Get logged user
	* @return string
	*/
	function logged_user() {
		return get_val("SELECT USER()");
	}

	/** Get tables list
	* @return array [$name => $type]
	*/
	function tables_list() {
		return get_key_vals("SELECT TABLE_NAME, TABLE_TYPE FROM information_schema.TABLES WHERE TABLE_SCHEMA = DATABASE() ORDER BY TABLE_NAME");
	}

	/** Count tables in all databases
	* @param array
	* @return array [$db => $tables]
	*/
	function count_tables($databases) {
		$return = array();
		foreach ($databases as $db) {
			$return[$db] = count(get_vals("SHOW TABLES IN " . idf_escape($db)));
		}
		return $return;
	}

	/** Get table status
	* @param string
	* @param bool return only "Name", "Engine" and "Comment" fields
	* @return array [$name => ["Name" => , "Engine" => , "Comment" => , "Oid" => , "Rows" => , "Collation" => , "Auto_increment" => , "Data_length" => , "Index_length" => , "Data_free" => ]] or only inner array with $name
	*/
	function table_status($name = "", $fast = false) {
		$return = array();
		foreach (
			get_rows(
				$fast
				? "SELECT TABLE_NAME AS Name, ENGINE AS Engine, TABLE_COMMENT AS Comment FROM information_schema.TABLES WHERE TABLE_SCHEMA = DATABASE() " . ($name != "" ? "AND TABLE_NAME = " . q($name) : "ORDER BY Name")
				: "SHOW TABLE STATUS" . ($name != "" ? " LIKE " . q(addcslashes($name, "%_\\")) : "")
			) as $row
		) {
			if ($row["Engine"] == "InnoDB") {
				// ignore internal comment, unnecessary since MySQL 5.1.21
				$row["Comment"] = preg_replace('~(?:(.+); )?InnoDB free: .*~', '\1', $row["Comment"]);
			}
			if (!isset($row["Engine"])) {
				$row["Comment"] = "";
			}
			if ($name != "") {
				// MariaDB: Table name is returned as lowercase on macOS, so we fix it here.
				$row["Name"] = $name;
				return $row;
			}
			$return[$row["Name"]] = $row;
		}
		return $return;
	}

	/** Find out whether the identifier is view
	* @param array
	* @return bool
	*/
	function is_view($table_status) {
		return $table_status["Engine"] === null;
	}

	/** Check if table supports foreign keys
	* @param array result of table_status
	* @return bool
	*/
	function fk_support($table_status) {
		return preg_match('~InnoDB|IBMDB2I~i', $table_status["Engine"])
			|| (preg_match('~NDB~i', $table_status["Engine"]) && min_version(5.6));
	}

	/** Get information about fields
	* @param string
	* @return array [$name => ["field" =>, "full_type" =>, "type" =>, "length" =>, "unsigned" =>, "default" =>, "null" =>, "auto_increment" =>, "on_update" =>, "collation" =>, "privileges" =>, "comment" =>, "primary" =>, "generated" =>]]
	*/
	function fields($table) {
		global $connection;
		$maria = ($connection->flavor == 'maria');
		$return = array();
		foreach (get_rows("SELECT * FROM information_schema.COLUMNS WHERE TABLE_SCHEMA = DATABASE() AND TABLE_NAME = " . q($table) . " ORDER BY ORDINAL_POSITION") as $row) {
			$field = $row["COLUMN_NAME"];
			$type = $row["COLUMN_TYPE"];
			$generation = $row["GENERATION_EXPRESSION"];
			$extra = $row["EXTRA"];
			// https://mariadb.com/kb/en/library/show-columns/, https://github.com/vrana/adminer/pull/359#pullrequestreview-276677186
			preg_match('~^(VIRTUAL|PERSISTENT|STORED)~', $extra, $generated);
			preg_match('~^([^( ]+)(?:\((.+)\))?( unsigned)?( zerofill)?$~', $type, $match_type);
			$default = $row["COLUMN_DEFAULT"];
			if ($default != "") {
				$is_text = preg_match('~text|json~', $match_type[1]);
				if (!$maria && $is_text) {
					// default value a'b of text column is stored as _utf8mb4\'a\\\'b\' in MySQL
					$default = preg_replace("~^(_\w+)?('.*')$~", '\2', stripslashes($default));
				}
				if ($maria || $is_text) {
					$default = ($default == "NULL" ? null : preg_replace_callback("~^'(.*)'$~", function ($match) {
						return stripslashes(str_replace("''", "'", $match[1]));
					}, $default));
				}
				if (!$maria && preg_match('~binary~', $match_type[1]) && preg_match('~^0x(\w*)$~', $default, $match)) {
					$default = pack("H*", $match[1]);
				}
			}
			$return[$field] = array(
				"field" => $field,
				"full_type" => $type,
				"type" => $match_type[1],
				"length" => $match_type[2],
				"unsigned" => ltrim($match_type[3] . $match_type[4]),
				"default" => ($generated
					? ($maria ? $generation : stripslashes($generation))
					: $default
				),
				"null" => ($row["IS_NULLABLE"] == "YES"),
				"auto_increment" => ($extra == "auto_increment"),
				"on_update" => (preg_match('~\bon update (\w+)~i', $extra, $match) ? $match[1] : ""), //! available since MySQL 5.1.23
				"collation" => $row["COLLATION_NAME"],
				"privileges" => array_flip(explode(",", "$row[PRIVILEGES],where,order")),
				"comment" => $row["COLUMN_COMMENT"],
				"primary" => ($row["COLUMN_KEY"] == "PRI"),
				"generated" => ($generated[1] == "PERSISTENT" ? "STORED" : $generated[1]),
			);
		}
		return $return;
	}

	/** Get table indexes
	* @param string
	* @param string Db to use
	* @return array [$key_name => ["type" => , "columns" => [], "lengths" => [], "descs" => []]]
	*/
	function indexes($table, $connection2 = null) {
		$return = array();
		foreach (get_rows("SHOW INDEX FROM " . table($table), $connection2) as $row) {
			$name = $row["Key_name"];
			$return[$name]["type"] = ($name == "PRIMARY" ? "PRIMARY" : ($row["Index_type"] == "FULLTEXT" ? "FULLTEXT" : ($row["Non_unique"] ? ($row["Index_type"] == "SPATIAL" ? "SPATIAL" : "INDEX") : "UNIQUE")));
			$return[$name]["columns"][] = $row["Column_name"];
			$return[$name]["lengths"][] = ($row["Index_type"] == "SPATIAL" ? null : $row["Sub_part"]);
			$return[$name]["descs"][] = null;
		}
		return $return;
	}

	/** Get foreign keys in table
	* @param string
	* @return array [$name => ["db" => , "ns" => , "table" => , "source" => [], "target" => [], "on_delete" => , "on_update" => ]]
	*/
	function foreign_keys($table) {
		global $driver;
		static $pattern = '(?:`(?:[^`]|``)+`|"(?:[^"]|"")+")';
		$return = array();
		$create_table = get_val("SHOW CREATE TABLE " . table($table), 1);
		if ($create_table) {
			preg_match_all(
				"~CONSTRAINT ($pattern) FOREIGN KEY ?\\(((?:$pattern,? ?)+)\\) REFERENCES ($pattern)(?:\\.($pattern))? \\(((?:$pattern,? ?)+)\\)(?: ON DELETE ($driver->onActions))?(?: ON UPDATE ($driver->onActions))?~",
				$create_table,
				$matches,
				PREG_SET_ORDER
			);
			foreach ($matches as $match) {
				preg_match_all("~$pattern~", $match[2], $source);
				preg_match_all("~$pattern~", $match[5], $target);
				$return[idf_unescape($match[1])] = array(
					"db" => idf_unescape($match[4] != "" ? $match[3] : $match[4]),
					"table" => idf_unescape($match[4] != "" ? $match[4] : $match[3]),
					"source" => array_map('Adminer\idf_unescape', $source[0]),
					"target" => array_map('Adminer\idf_unescape', $target[0]),
					"on_delete" => ($match[6] ?: "RESTRICT"),
					"on_update" => ($match[7] ?: "RESTRICT"),
				);
			}
		}
		return $return;
	}

	/** Get view SELECT
	* @param string
	* @return array ["select" => ]
	*/
	function view($name) {
		return array("select" => preg_replace('~^(?:[^`]|`[^`]*`)*\s+AS\s+~isU', '', get_val("SHOW CREATE VIEW " . table($name), 1)));
	}

	/** Get sorted grouped list of collations
	* @return array
	*/
	function collations() {
		$return = array();
		foreach (get_rows("SHOW COLLATION") as $row) {
			if ($row["Default"]) {
				$return[$row["Charset"]][-1] = $row["Collation"];
			} else {
				$return[$row["Charset"]][] = $row["Collation"];
			}
		}
		ksort($return);
		foreach ($return as $key => $val) {
			asort($return[$key]);
		}
		return $return;
	}

	/** Find out if database is information_schema
	* @param string
	* @return bool
	*/
	function information_schema($db) {
		return ($db == "information_schema")
			|| (min_version(5.5) && $db == "performance_schema");
	}

	/** Get escaped error message
	* @return string
	*/
	function error() {
		global $connection;
		return h(preg_replace('~^You have an error.*syntax to use~U', "Syntax error", $connection->error));
	}

	/** Create database
	* @param string
	* @param string
	* @return string
	*/
	function create_database($db, $collation) {
		return queries("CREATE DATABASE " . idf_escape($db) . ($collation ? " COLLATE " . q($collation) : ""));
	}

	/** Drop databases
	* @param array
	* @return bool
	*/
	function drop_databases($databases) {
		$return = apply_queries("DROP DATABASE", $databases, 'Adminer\idf_escape');
		restart_session();
		set_session("dbs", null);
		return $return;
	}

	/** Rename database from DB
	* @param string new name
	* @param string
	* @return bool
	*/
	function rename_database($name, $collation) {
		$return = false;
		if (create_database($name, $collation)) {
			$tables = array();
			$views = array();
			foreach (tables_list() as $table => $type) {
				if ($type == 'VIEW') {
					$views[] = $table;
				} else {
					$tables[] = $table;
				}
			}
			$return = (!$tables && !$views) || move_tables($tables, $views, $name);
			drop_databases($return ? array(DB) : array());
		}
		return $return;
	}

	/** Generate modifier for auto increment column
	* @return string
	*/
	function auto_increment() {
		$auto_increment_index = " PRIMARY KEY";
		// don't overwrite primary key by auto_increment
		if ($_GET["create"] != "" && $_POST["auto_increment_col"]) {
			foreach (indexes($_GET["create"]) as $index) {
				if (in_array($_POST["fields"][$_POST["auto_increment_col"]]["orig"], $index["columns"], true)) {
					$auto_increment_index = "";
					break;
				}
				if ($index["type"] == "PRIMARY") {
					$auto_increment_index = " UNIQUE";
				}
			}
		}
		return " AUTO_INCREMENT$auto_increment_index";
	}

	/** Run commands to create or alter table
	* @param string "" to create
	* @param string new name
	* @param array of [$orig, $process_field, $after]
	* @param array of strings
	* @param string
	* @param string
	* @param string
	* @param string number
	* @param string
	* @return bool
	*/
	function alter_table($table, $name, $fields, $foreign, $comment, $engine, $collation, $auto_increment, $partitioning) {
		global $connection;
		$alter = array();
		foreach ($fields as $field) {
			if ($field[1]) {
				$default = $field[1][3];
				if (preg_match('~ GENERATED~', $default)) {
					// swap default and null
					$field[1][3] = ($connection->flavor == 'maria' ? "" : $field[1][2]); // MariaDB doesn't support NULL on virtual columns
					$field[1][2] = $default;
				}
				$alter[] = ($table != "" ? ($field[0] != "" ? "CHANGE " . idf_escape($field[0]) : "ADD") : " ") . " " . implode($field[1]) . ($table != "" ? $field[2] : "");
			} else {
				$alter[] = "DROP " . idf_escape($field[0]);
			}
		}
		$alter = array_merge($alter, $foreign);
		$status = ($comment !== null ? " COMMENT=" . q($comment) : "")
			. ($engine ? " ENGINE=" . q($engine) : "")
			. ($collation ? " COLLATE " . q($collation) : "")
			. ($auto_increment != "" ? " AUTO_INCREMENT=$auto_increment" : "")
		;
		if ($table == "") {
			return queries("CREATE TABLE " . table($name) . " (\n" . implode(",\n", $alter) . "\n)$status$partitioning");
		}
		if ($table != $name) {
			$alter[] = "RENAME TO " . table($name);
		}
		if ($status) {
			$alter[] = ltrim($status);
		}
		return ($alter || $partitioning ? queries("ALTER TABLE " . table($table) . "\n" . implode(",\n", $alter) . $partitioning) : true);
	}

	/** Run commands to alter indexes
	* @param string escaped table name
	* @param array of ["index type", "name", ["column definition", ...]] or ["index type", "name", "DROP"]
	* @return bool
	*/
	function alter_indexes($table, $alter) {
		foreach ($alter as $key => $val) {
			$alter[$key] = ($val[2] == "DROP"
				? "\nDROP INDEX " . idf_escape($val[1])
				: "\nADD $val[0] " . ($val[0] == "PRIMARY" ? "KEY " : "") . ($val[1] != "" ? idf_escape($val[1]) . " " : "") . "(" . implode(", ", $val[2]) . ")"
			);
		}
		return queries("ALTER TABLE " . table($table) . implode(",", $alter));
	}

	/** Run commands to truncate tables
	* @param array
	* @return bool
	*/
	function truncate_tables($tables) {
		return apply_queries("TRUNCATE TABLE", $tables);
	}

	/** Drop views
	* @param array
	* @return bool
	*/
	function drop_views($views) {
		return queries("DROP VIEW " . implode(", ", array_map('Adminer\table', $views)));
	}

	/** Drop tables
	* @param array
	* @return bool
	*/
	function drop_tables($tables) {
		return queries("DROP TABLE " . implode(", ", array_map('Adminer\table', $tables)));
	}

	/** Move tables to other schema
	* @param array
	* @param array
	* @param string
	* @return bool
	*/
	function move_tables($tables, $views, $target) {
		global $connection;
		$rename = array();
		foreach ($tables as $table) {
			$rename[] = table($table) . " TO " . idf_escape($target) . "." . table($table);
		}
		if (!$rename || queries("RENAME TABLE " . implode(", ", $rename))) {
			$definitions = array();
			foreach ($views as $table) {
				$definitions[table($table)] = view($table);
			}
			$connection->select_db($target);
			$db = idf_escape(DB);
			foreach ($definitions as $name => $view) {
				if (!queries("CREATE VIEW $name AS " . str_replace(" $db.", " ", $view["select"])) || !queries("DROP VIEW $db.$name")) {
					return false;
				}
			}
			return true;
		}
		//! move triggers
		return false;
	}

	/** Copy tables to other schema
	* @param array
	* @param array
	* @param string
	* @return bool
	*/
	function copy_tables($tables, $views, $target) {
		queries("SET sql_mode = 'NO_AUTO_VALUE_ON_ZERO'");
		foreach ($tables as $table) {
			$name = ($target == DB ? table("copy_$table") : idf_escape($target) . "." . table($table));
			if (
				($_POST["overwrite"] && !queries("\nDROP TABLE IF EXISTS $name"))
				|| !queries("CREATE TABLE $name LIKE " . table($table))
				|| !queries("INSERT INTO $name SELECT * FROM " . table($table))
			) {
				return false;
			}
			foreach (get_rows("SHOW TRIGGERS LIKE " . q(addcslashes($table, "%_\\"))) as $row) {
				$trigger = $row["Trigger"];
				if (!queries("CREATE TRIGGER " . ($target == DB ? idf_escape("copy_$trigger") : idf_escape($target) . "." . idf_escape($trigger)) . " $row[Timing] $row[Event] ON $name FOR EACH ROW\n$row[Statement];")) {
					return false;
				}
			}
		}
		foreach ($views as $table) {
			$name = ($target == DB ? table("copy_$table") : idf_escape($target) . "." . table($table));
			$view = view($table);
			if (
				($_POST["overwrite"] && !queries("DROP VIEW IF EXISTS $name"))
				|| !queries("CREATE VIEW $name AS $view[select]") //! USE to avoid db.table
			) {
				return false;
			}
		}
		return true;
	}

	/** Get information about trigger
	* @param string trigger name
	* @return array ["Trigger" => , "Timing" => , "Event" => , "Of" => , "Type" => , "Statement" => ]
	*/
	function trigger($name) {
		if ($name == "") {
			return array();
		}
		$rows = get_rows("SHOW TRIGGERS WHERE `Trigger` = " . q($name));
		return reset($rows);
	}

	/** Get defined triggers
	* @param string
	* @return array [$name => [$timing, $event]]
	*/
	function triggers($table) {
		$return = array();
		foreach (get_rows("SHOW TRIGGERS LIKE " . q(addcslashes($table, "%_\\"))) as $row) {
			$return[$row["Trigger"]] = array($row["Timing"], $row["Event"]);
		}
		return $return;
	}

	/** Get trigger options
	* @return array ["Timing" => [], "Event" => [], "Type" => []]
	*/
	function trigger_options() {
		return array(
			"Timing" => array("BEFORE", "AFTER"),
			"Event" => array("INSERT", "UPDATE", "DELETE"),
			"Type" => array("FOR EACH ROW"),
		);
	}

	/** Get information about stored routine
	* @param string
	* @param string "FUNCTION" or "PROCEDURE"
	* @return array ["fields" => ["field" => , "type" => , "length" => , "unsigned" => , "inout" => , "collation" => ], "returns" => , "definition" => , "language" => ]
	*/
	function routine($name, $type) {
		global $driver;
		$aliases = array("bool", "boolean", "integer", "double precision", "real", "dec", "numeric", "fixed", "national char", "national varchar");
		$space = "(?:\\s|/\\*[\s\S]*?\\*/|(?:#|-- )[^\n]*\n?|--\r?\n)";
		$enum = $driver->enumLength;
		$type_pattern = "((" . implode("|", array_merge(array_keys($driver->types()), $aliases)) . ")\\b(?:\\s*\\(((?:[^'\")]|$enum)++)\\))?"
			. "\\s*(zerofill\\s*)?(unsigned(?:\\s+zerofill)?)?)(?:\\s*(?:CHARSET|CHARACTER\\s+SET)\\s*['\"]?([^'\"\\s,]+)['\"]?)?";
		$pattern = "$space*(" . ($type == "FUNCTION" ? "" : $driver->inout) . ")?\\s*(?:`((?:[^`]|``)*)`\\s*|\\b(\\S+)\\s+)$type_pattern";
		$create = get_val("SHOW CREATE $type " . idf_escape($name), 2);
		preg_match("~\\(((?:$pattern\\s*,?)*)\\)\\s*" . ($type == "FUNCTION" ? "RETURNS\\s+$type_pattern\\s+" : "") . "(.*)~is", $create, $match);
		$fields = array();
		preg_match_all("~$pattern\\s*,?~is", $match[1], $matches, PREG_SET_ORDER);
		foreach ($matches as $param) {
			$fields[] = array(
				"field" => str_replace("``", "`", $param[2]) . $param[3],
				"type" => strtolower($param[5]),
				"length" => preg_replace_callback("~$enum~s", 'Adminer\normalize_enum', $param[6]),
				"unsigned" => strtolower(preg_replace('~\s+~', ' ', trim("$param[8] $param[7]"))),
				"null" => 1,
				"full_type" => $param[4],
				"inout" => strtoupper($param[1]),
				"collation" => strtolower($param[9]),
			);
		}
		return array(
			"fields" => $fields,
			"comment" => get_val("SELECT ROUTINE_COMMENT FROM information_schema.ROUTINES WHERE ROUTINE_SCHEMA = DATABASE() AND ROUTINE_NAME = " . q($name)),
		) + ($type != "FUNCTION" ? array("definition" => $match[11]) : array(
			"returns" => array("type" => $match[12], "length" => $match[13], "unsigned" => $match[15], "collation" => $match[16]),
			"definition" => $match[17],
			"language" => "SQL", // available in information_schema.ROUTINES.BODY_STYLE
		));
	}

	/** Get list of routines
	* @return array ["SPECIFIC_NAME" => , "ROUTINE_NAME" => , "ROUTINE_TYPE" => , "DTD_IDENTIFIER" => ]
	*/
	function routines() {
		return get_rows("SELECT ROUTINE_NAME AS SPECIFIC_NAME, ROUTINE_NAME, ROUTINE_TYPE, DTD_IDENTIFIER FROM information_schema.ROUTINES WHERE ROUTINE_SCHEMA = DATABASE()");
	}

	/** Get list of available routine languages
	* @return array
	*/
	function routine_languages() {
		return array(); // "SQL" not required
	}

	/** Get routine signature
	* @param string
	* @param array result of routine()
	* @return string
	*/
	function routine_id($name, $row) {
		return idf_escape($name);
	}

	/** Get last auto increment ID
	* @param Result or true
	* @return string
	*/
	function last_id($result) {
		return get_val("SELECT LAST_INSERT_ID()"); // mysql_insert_id() truncates bigint
	}

	/** Explain select
	* @param Db
	* @param string
	* @return Result
	*/
	function explain($connection, $query) {
		return $connection->query("EXPLAIN " . (min_version(5.1) && !min_version(5.7) ? "PARTITIONS " : "") . $query);
	}

	/** Get approximate number of rows
	* @param array
	* @param array
	* @return int or null if approximate number can't be retrieved
	*/
	function found_rows($table_status, $where) {
		return ($where || $table_status["Engine"] != "InnoDB" ? null : $table_status["Rows"]);
	}

	/* Not used is MySQL but checked in compile.php:
	/** Get user defined types
	* @return array [$id => $name]
	function types() {
		return array();
	}

	/** Get values of user defined type
	* @param int
	* @return string
	function type_values($id) {
		return "";
	}

	/** Get existing schemas
	* @return array
	function schemas() {
		return array();
	}

	/** Get current schema
	* @return string
	function get_schema() {
		return "";
	}

	/** Set current schema
	* @param string
	* @param Db
	* @return bool
	function set_schema($schema, $connection2 = null) {
		return true;
	}
	*/

	/** Get SQL command to create table
	* @param string
	* @param bool
	* @param string
	* @return string
	*/
	function create_sql($table, $auto_increment, $style) {
		$return = get_val("SHOW CREATE TABLE " . table($table), 1);
		if (!$auto_increment) {
			$return = preg_replace('~ AUTO_INCREMENT=\d+~', '', $return); //! skip comments
		}
		return $return;
	}

	/** Get SQL command to truncate table
	* @param string
	* @return string
	*/
	function truncate_sql($table) {
		return "TRUNCATE " . table($table);
	}

	/** Get SQL command to change database
	* @param string
	* @return string
	*/
	function use_sql($database) {
		return "USE " . idf_escape($database);
	}

	/** Get SQL commands to create triggers
	* @param string
	* @return string
	*/
	function trigger_sql($table) {
		$return = "";
		foreach (get_rows("SHOW TRIGGERS LIKE " . q(addcslashes($table, "%_\\")), null, "-- ") as $row) {
			$return .= "\nCREATE TRIGGER " . idf_escape($row["Trigger"]) . " $row[Timing] $row[Event] ON " . table($row["Table"]) . " FOR EACH ROW\n$row[Statement];;\n";
		}
		return $return;
	}

	/** Get server variables
	* @return array [[$name, $value]]
	*/
	function show_variables() {
		return get_rows("SHOW VARIABLES");
	}

	/** Get status variables
	* @return array [[$name, $value]]
	*/
	function show_status() {
		return get_rows("SHOW STATUS");
	}

	/** Get process list
	* @return array [$row]
	*/
	function process_list() {
		return get_rows("SHOW FULL PROCESSLIST");
	}

	/** Convert field in select and edit
	* @param array one element from fields()
	* @return string
	*/
	function convert_field($field) {
		if (preg_match("~binary~", $field["type"])) {
			return "HEX(" . idf_escape($field["field"]) . ")";
		}
		if ($field["type"] == "bit") {
			return "BIN(" . idf_escape($field["field"]) . " + 0)"; // + 0 is required outside MySQLnd
		}
		if (preg_match("~geometry|point|linestring|polygon~", $field["type"])) {
			return (min_version(8) ? "ST_" : "") . "AsWKT(" . idf_escape($field["field"]) . ")";
		}
	}

	/** Convert value in edit after applying functions back
	* @param array one element from fields()
	* @param string SQL expression
	* @return string
	*/
	function unconvert_field($field, $return) {
		if (preg_match("~binary~", $field["type"])) {
			$return = "UNHEX($return)";
		}
		if ($field["type"] == "bit") {
			$return = "CONVERT(b$return, UNSIGNED)";
		}
		if (preg_match("~geometry|point|linestring|polygon~", $field["type"])) {
			$prefix = (min_version(8) ? "ST_" : "");
			$return = $prefix . "GeomFromText($return, $prefix" . "SRID($field[field]))";
		}
		return $return;
	}

	/** Check whether a feature is supported
	* @param string "check|comment|copy|database|descidx|drop_col|dump|event|indexes|kill|materializedview|partitioning|privileges|procedure|processlist|routine|scheme|sequence|status|table|trigger|type|variables|view|view_trigger"
	* @return bool
	*/
	function support($feature) {
		return !preg_match("~scheme|sequence|type|view_trigger|materializedview" . (min_version(8) ? "" : "|descidx" . (min_version(5.1) ? "" : "|event|partitioning")) . (min_version('8.0.16', '10.2.1') ? "" : "|check") . "~", $feature);
	}

	/** Kill a process
	* @param int
	* @return bool
	*/
	function kill_process($val) {
		return queries("KILL " . number($val));
	}

	/** Return query to get connection ID
	* @return string
	*/
	function connection_id() {
		return "SELECT CONNECTION_ID()";
	}

	/** Get maximum number of connections
	* @return int
	*/
	function max_connections() {
		return get_val("SELECT @@max_connections");
	}
}
 // must be included as last driver

define('Adminer\JUSH', Driver::$jush);
define('Adminer\SERVER', $_GET[DRIVER]); // read from pgsql=localhost
define('Adminer\DB', $_GET["db"]); // for the sake of speed and size
define(
	'Adminer\ME',
	preg_replace('~\?.*~', '', relative_uri()) . '?'
		. (sid() ? SID . '&' : '')
		. (SERVER !== null ? DRIVER . "=" . urlencode(SERVER) . '&' : '')
		. ($_GET["ext"] ? "ext=" . urlencode($_GET["ext"]) . '&' : '')
		. (isset($_GET["username"]) ? "username=" . urlencode($_GET["username"]) . '&' : '')
		. (DB != "" ? 'db=' . urlencode(DB) . '&' . (isset($_GET["ns"]) ? "ns=" . urlencode($_GET["ns"]) . "&" : "") : '')
);

?>
<?php
/** Print HTML header
* @param string used in title, breadcrumb and heading, should be HTML escaped
* @param string
* @param mixed ["key" => "link", "key2" => ["link", "desc"]], null for nothing, false for driver only, true for driver and server
* @param string used after colon in title and heading, should be HTML escaped
* @return null
*/
function page_header($title, $error = "", $breadcrumb = array(), $title2 = "") {
	global $LANG, $VERSION, $adminer, $drivers;
	page_headers();
	if (is_ajax() && $error) {
		page_messages($error);
		exit;
	}
	if (!ob_get_level()) {
		ob_start(null, 4096);
	}
	$title_all = $title . ($title2 != "" ? ": $title2" : "");
	$title_page = strip_tags($title_all . (SERVER != "" && SERVER != "localhost" ? h(" - " . SERVER) : "") . " - " . $adminer->name());
	// initial-scale=1 is the default but Chrome 134 on iOS is not able to zoom out without it
	?>
<!DOCTYPE html>
<html lang="<?php echo $LANG; ?>" dir="<?php echo lang(81); ?>">
<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
<meta name="robots" content="noindex">
<meta name="viewport" content="width=device-width,initial-scale=1">
<title><?php echo $title_page; ?></title>
<link rel="stylesheet" href="<?php echo h(preg_replace("~\\?.*~", "", ME) . "?file=default.css&version=5.1.0"); ?>">
<?php
	$css = $adminer->css();
	$has_light = false;
	$has_dark = false;
	foreach ($css as $filename) {
		if (strpos($filename, "adminer.css") !== false) {
			$has_light = true;
		}
		if (strpos($filename, "adminer-dark.css") !== false) {
			$has_dark = true;
		}
	}
	$dark = ($has_light
		? ($has_dark ? null : false) // both styles - autoswitching, only adminer.css - light
		: ($has_dark ?: null) // only adminer-dark.css - dark, neither - autoswitching
	);
	$media = " media='(prefers-color-scheme: dark)'";
	if ($dark !== false) {
		echo "<link rel='stylesheet'" . ($dark ? "" : $media) . " href='" . h(preg_replace("~\\?.*~", "", ME) . "?file=dark.css&version=5.1.0") . "'>\n";
	}
	echo "<meta name='color-scheme' content='" . ($dark === null ? "light dark" : ($dark ? "dark" : "light")) . "'>\n";
	// this is matched by compile.php
	echo script_src(preg_replace("~\\?.*~", "", ME) . "?file=functions.js&version=5.1.0");
		if ($adminer->head($dark)) {
		echo "<link rel='shortcut icon' type='image/x-icon' href='" . h(preg_replace("~\\?.*~", "", ME) . "?file=favicon.ico&version=5.1.0") . "'>\n";
		echo "<link rel='apple-touch-icon' href='" . h(preg_replace("~\\?.*~", "", ME) . "?file=favicon.ico&version=5.1.0") . "'>\n";
	}
	foreach ($css as $val) {
		echo "<link rel='stylesheet'" . (preg_match('~-dark~', $val) && !$dark ? $media : "") . " href='" . h($val) . "'>\n";
	}
	echo "\n<body class='" . lang(81) . " nojs'>\n";
	$filename = get_temp_dir() . "/adminer.version";
	if (!$_COOKIE["adminer_version"] && function_exists('openssl_verify') && file_exists($filename) && filemtime($filename) + 86400 > time()) { // 86400 - 1 day in seconds
		$version = unserialize(file_get_contents($filename));
		$public = "-----BEGIN PUBLIC KEY-----
MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAwqWOVuF5uw7/+Z70djoK
RlHIZFZPO0uYRezq90+7Amk+FDNd7KkL5eDve+vHRJBLAszF/7XKXe11xwliIsFs
DFWQlsABVZB3oisKCBEuI71J4kPH8dKGEWR9jDHFw3cWmoH3PmqImX6FISWbG3B8
h7FIx3jEaw5ckVPVTeo5JRm/1DZzJxjyDenXvBQ/6o9DgZKeNDgxwKzH+sw9/YCO
jHnq1cFpOIISzARlrHMa/43YfeNRAm/tsBXjSxembBPo7aQZLAWHmaj5+K19H10B
nCpz9Y++cipkVEiKRGih4ZEvjoFysEOdRLj6WiD/uUNky4xGeA6LaJqh5XpkFkcQ
fQIDAQAB
-----END PUBLIC KEY-----
";
		if (openssl_verify($version["version"], base64_decode($version["signature"]), $public) == 1) {
			$_COOKIE["adminer_version"] = $version["version"]; // doesn't need to send to the browser
		}
	}
	echo script("mixin(document.body, {onkeydown: bodyKeydown, onclick: bodyClick"
		. (isset($_COOKIE["adminer_version"]) ? "" : ", onload: partial(verifyVersion, '$VERSION', '" . js_escape(ME) . "', '" . get_token() . "')") // $token may be empty in auth.inc.php
		. "});
document.body.classList.replace('nojs', 'js');
const offlineMessage = '" . js_escape(lang(82)) . "';
const thousandsSeparator = '" . js_escape(lang(4)) . "';")
	;
	echo "<div id='help' class='jush-" . JUSH . " jsonly hidden'></div>\n";
	echo script("mixin(qs('#help'), {onmouseover: () => { helpOpen = 1; }, onmouseout: helpMouseout});");
	echo "<div id='content'>\n";
	if ($breadcrumb !== null) {
		$link = substr(preg_replace('~\b(username|db|ns)=[^&]*&~', '', ME), 0, -1);
		echo '<p id="breadcrumb"><a href="' . h($link ?: ".") . '">' . $drivers[DRIVER] . '</a> » ';
		$link = substr(preg_replace('~\b(db|ns)=[^&]*&~', '', ME), 0, -1);
		$server = $adminer->serverName(SERVER);
		$server = ($server != "" ? $server : lang(33));
		if ($breadcrumb === false) {
			echo "$server\n";
		} else {
			echo "<a href='" . h($link) . "' accesskey='1' title='Alt+Shift+1'>$server</a> » ";
			if ($_GET["ns"] != "" || (DB != "" && is_array($breadcrumb))) {
				echo '<a href="' . h($link . "&db=" . urlencode(DB) . (support("scheme") ? "&ns=" : "")) . '">' . h(DB) . '</a> » ';
			}
			if (is_array($breadcrumb)) {
				if ($_GET["ns"] != "") {
					echo '<a href="' . h(substr(ME, 0, -1)) . '">' . h($_GET["ns"]) . '</a> » ';
				}
				foreach ($breadcrumb as $key => $val) {
					$desc = (is_array($val) ? $val[1] : h($val));
					if ($desc != "") {
						echo "<a href='" . h(ME . "$key=") . urlencode(is_array($val) ? $val[0] : $val) . "'>$desc</a> » ";
					}
				}
			}
			echo "$title\n";
		}
	}
	echo "<h2>$title_all</h2>\n";
	echo "<div id='ajaxstatus' class='jsonly hidden'></div>\n";
	restart_session();
	page_messages($error);
	$databases = &get_session("dbs");
	if (DB != "" && $databases && !in_array(DB, $databases, true)) {
		$databases = null;
	}
	stop_session();
	define('Adminer\PAGE_HEADER', 1);
}

/** Send HTTP headers
* @return null
*/
function page_headers() {
	global $adminer;
	header("Content-Type: text/html; charset=utf-8");
	header("Cache-Control: no-cache");
	header("X-Frame-Options: deny"); // ClickJacking protection in IE8, Safari 4, Chrome 2, Firefox 3.6.9
	header("X-XSS-Protection: 0"); // prevents introducing XSS in IE8 by removing safe parts of the page
	header("X-Content-Type-Options: nosniff");
	header("Referrer-Policy: origin-when-cross-origin");
	foreach ($adminer->csp() as $csp) {
		$header = array();
		foreach ($csp as $key => $val) {
			$header[] = "$key $val";
		}
		header("Content-Security-Policy: " . implode("; ", $header));
	}
	$adminer->headers();
}

/** Get Content Security Policy headers
* @return array of arrays with directive name in key, allowed sources in value
*/
function csp() {
	return array(
		array(
			"script-src" => "'self' 'unsafe-inline' 'nonce-" . get_nonce() . "' 'strict-dynamic'", // 'self' is a fallback for browsers not supporting 'strict-dynamic', 'unsafe-inline' is a fallback for browsers not supporting 'nonce-'
			"connect-src" => "'self'",
			"frame-src" => "https://www.adminer.org",
			"object-src" => "'none'",
			"base-uri" => "'none'",
			"form-action" => "'self'",
		),
	);
}

/** Get a CSP nonce
* @return string Base64 value
*/
function get_nonce() {
	static $nonce;
	if (!$nonce) {
		$nonce = base64_encode(rand_string());
	}
	return $nonce;
}

/** Print flash and error messages
* @param string
* @return null
*/
function page_messages($error) {
	global $adminer;
	$uri = preg_replace('~^[^?]*~', '', $_SERVER["REQUEST_URI"]);
	$messages = $_SESSION["messages"][$uri];
	if ($messages) {
		echo "<div class='message'>" . implode("</div>\n<div class='message'>", $messages) . "</div>" . script("messagesPrint();");
		unset($_SESSION["messages"][$uri]);
	}
	if ($error) {
		echo "<div class='error'>$error</div>\n";
	}
	if ($adminer->error) { // separate <div>
		echo "<div class='error'>$adminer->error</div>\n";
	}
}

/** Print HTML footer
* @param string "auth", "db", "ns"
* @return null
*/
function page_footer($missing = "") {
	global $adminer;
	echo "</div>\n\n<div id='menu'>\n";
	$adminer->navigation($missing);
	echo "</div>\n\n";
	if ($missing != "auth") {
		?>
<form action="" method="post">
<p class="logout">
<span><?php echo h($_GET["username"]) . "\n"; ?></span>
<input type="submit" name="logout" value="<?php echo lang(83); ?>" id="logout">
<?php echo input_token(); ?>
</p>
</form>
<?php
	}
	echo script("setupSubmitHighlight(document);");
}

?>
<?php
/** PHP implementation of XXTEA encryption algorithm
* @author Ma Bingyao <andot@ujn.edu.cn>
* @link http://www.coolcode.cn/?action=show&id=128
*/

function int32($n) {
	while ($n >= 2147483648) {
		$n -= 4294967296;
	}
	while ($n <= -2147483649) {
		$n += 4294967296;
	}
	return (int) $n;
}

function long2str($v, $w) {
	$s = '';
	foreach ($v as $val) {
		$s .= pack('V', $val);
	}
	if ($w) {
		return substr($s, 0, end($v));
	}
	return $s;
}

function str2long($s, $w) {
	$v = array_values(unpack('V*', str_pad($s, 4 * ceil(strlen($s) / 4), "\0")));
	if ($w) {
		$v[] = strlen($s);
	}
	return $v;
}

function xxtea_mx($z, $y, $sum, $k) {
	return int32((($z >> 5 & 0x7FFFFFF) ^ $y << 2) + (($y >> 3 & 0x1FFFFFFF) ^ $z << 4)) ^ int32(($sum ^ $y) + ($k ^ $z));
}

/** Cipher
* @param string plain-text password
* @param string
* @return string binary cipher
*/
function encrypt_string($str, $key) {
	if ($str == "") {
		return "";
	}
	$key = array_values(unpack("V*", pack("H*", md5($key))));
	$v = str2long($str, true);
	$n = count($v) - 1;
	$z = $v[$n];
	$y = $v[0];
	$q = floor(6 + 52 / ($n + 1));
	$sum = 0;
	while ($q-- > 0) {
		$sum = int32($sum + 0x9E3779B9);
		$e = $sum >> 2 & 3;
		for ($p=0; $p < $n; $p++) {
			$y = $v[$p + 1];
			$mx = xxtea_mx($z, $y, $sum, $key[$p & 3 ^ $e]);
			$z = int32($v[$p] + $mx);
			$v[$p] = $z;
		}
		$y = $v[0];
		$mx = xxtea_mx($z, $y, $sum, $key[$p & 3 ^ $e]);
		$z = int32($v[$n] + $mx);
		$v[$n] = $z;
	}
	return long2str($v, false);
}

/** Decipher
* @param string binary cipher
* @param string
* @return string plain-text password
*/
function decrypt_string($str, $key) {
	if ($str == "") {
		return "";
	}
	if (!$key) {
		return false;
	}
	$key = array_values(unpack("V*", pack("H*", md5($key))));
	$v = str2long($str, false);
	$n = count($v) - 1;
	$z = $v[$n];
	$y = $v[0];
	$q = floor(6 + 52 / ($n + 1));
	$sum = int32($q * 0x9E3779B9);
	while ($sum) {
		$e = $sum >> 2 & 3;
		for ($p=$n; $p > 0; $p--) {
			$z = $v[$p - 1];
			$mx = xxtea_mx($z, $y, $sum, $key[$p & 3 ^ $e]);
			$y = int32($v[$p] - $mx);
			$v[$p] = $y;
		}
		$z = $v[$n];
		$mx = xxtea_mx($z, $y, $sum, $key[$p & 3 ^ $e]);
		$y = int32($v[0] - $mx);
		$v[0] = $y;
		$sum = int32($sum - 0x9E3779B9);
	}
	return long2str($v, true);
}

?>
<?php
$connection = '';

$has_token = $_SESSION["token"];
if (!$has_token) {
	$_SESSION["token"] = rand(1, 1e6); // defense against cross-site request forgery
}
$token = get_token(); ///< @var string CSRF protection

$permanent = array();
if ($_COOKIE["adminer_permanent"]) {
	foreach (explode(" ", $_COOKIE["adminer_permanent"]) as $val) {
		list($key) = explode(":", $val);
		$permanent[$key] = $val;
	}
}

function add_invalid_login() {
	global $adminer;
	$base = get_temp_dir() . "/adminer.invalid";
	// adminer.invalid may not be writable by us, try the files with random suffixes
	foreach (glob("$base*") ?: array($base) as $filename) {
		$fp = file_open_lock($filename);
		if ($fp) {
			break;
		}
	}
	if (!$fp) {
		$fp = file_open_lock("$base-" . rand_string());
	}
	if (!$fp) {
		return;
	}
	$invalids = unserialize(stream_get_contents($fp));
	$time = time();
	if ($invalids) {
		foreach ($invalids as $ip => $val) {
			if ($val[0] < $time) {
				unset($invalids[$ip]);
			}
		}
	}
	$invalid = &$invalids[$adminer->bruteForceKey()];
	if (!$invalid) {
		$invalid = array($time + 30*60, 0); // active for 30 minutes
	}
	$invalid[1]++;
	file_write_unlock($fp, serialize($invalids));
}

function check_invalid_login() {
	global $adminer;
	$invalids = array();
	foreach (glob(get_temp_dir() . "/adminer.invalid*") as $filename) {
		$fp = file_open_lock($filename);
		if ($fp) {
			$invalids = unserialize(stream_get_contents($fp));
			file_unlock($fp);
			break;
		}
	}
	$invalid = ($invalids ? $invalids[$adminer->bruteForceKey()] : array());
	$next_attempt = ($invalid[1] > 29 ? $invalid[0] - time() : 0); // allow 30 invalid attempts
	if ($next_attempt > 0) { //! do the same with permanent login
		auth_error(lang(84, ceil($next_attempt / 60)));
	}
}

$auth = $_POST["auth"];
if ($auth) {
	session_regenerate_id(); // defense against session fixation
	$vendor = $auth["driver"];
	$server = $auth["server"];
	$username = $auth["username"];
	$password = (string) $auth["password"];
	$db = $auth["db"];
	set_password($vendor, $server, $username, $password);
	$_SESSION["db"][$vendor][$server][$username][$db] = true;
	if ($auth["permanent"]) {
		$key = implode("-", array_map('base64_encode', array($vendor, $server, $username, $db)));
		$private = $adminer->permanentLogin(true);
		$permanent[$key] = "$key:" . base64_encode($private ? encrypt_string($password, $private) : "");
		cookie("adminer_permanent", implode(" ", $permanent));
	}
	if (
		count($_POST) == 1 // 1 - auth
		|| DRIVER != $vendor
		|| SERVER != $server
		|| $_GET["username"] !== $username // "0" == "00"
		|| DB != $db
	) {
		redirect(auth_url($vendor, $server, $username, $db));
	}

} elseif ($_POST["logout"] && (!$has_token || verify_token())) {
	foreach (array("pwds", "db", "dbs", "queries") as $key) {
		set_session($key, null);
	}
	unset_permanent();
	redirect(substr(preg_replace('~\b(username|db|ns)=[^&]*&~', '', ME), 0, -1), lang(85) . ' ' . lang(86));

} elseif ($permanent && !$_SESSION["pwds"]) {
	session_regenerate_id();
	$private = $adminer->permanentLogin();
	foreach ($permanent as $key => $val) {
		list(, $cipher) = explode(":", $val);
		list($vendor, $server, $username, $db) = array_map('base64_decode', explode("-", $key));
		set_password($vendor, $server, $username, decrypt_string(base64_decode($cipher), $private));
		$_SESSION["db"][$vendor][$server][$username][$db] = true;
	}
}

function unset_permanent() {
	global $permanent;
	foreach ($permanent as $key => $val) {
		list($vendor, $server, $username, $db) = array_map('base64_decode', explode("-", $key));
		if ($vendor == DRIVER && $server == SERVER && $username == $_GET["username"] && $db == DB) {
			unset($permanent[$key]);
		}
	}
	cookie("adminer_permanent", implode(" ", $permanent));
}

/** Render an error message and a login form
* @param string plain text
* @return null exits
*/
function auth_error($error) {
	global $adminer, $has_token;
	$session_name = session_name();
	if (isset($_GET["username"])) {
		header("HTTP/1.1 403 Forbidden"); // 401 requires sending WWW-Authenticate header
		if (($_COOKIE[$session_name] || $_GET[$session_name]) && !$has_token) {
			$error = lang(87);
		} else {
			restart_session();
			add_invalid_login();
			$password = get_password();
			if ($password !== null) {
				if ($password === false) {
					$error .= ($error ? '<br>' : '') . lang(88, target_blank(), '<code>permanentLogin()</code>');
				}
				set_password(DRIVER, SERVER, $_GET["username"], null);
			}
			unset_permanent();
		}
	}
	if (!$_COOKIE[$session_name] && $_GET[$session_name] && ini_bool("session.use_only_cookies")) {
		$error = lang(89);
	}
	$params = session_get_cookie_params();
	cookie("adminer_key", ($_COOKIE["adminer_key"] ?: rand_string()), $params["lifetime"]);
	page_header(lang(37), $error, null);
	echo "<form action='' method='post'>\n";
	echo "<div>";
	if (hidden_fields($_POST, array("auth"))) { // expired session
		echo "<p class='message'>" . lang(90) . "\n";
	}
	echo "</div>\n";
	$adminer->loginForm();
	echo "</form>\n";
	page_footer("auth");
	exit;
}

if (isset($_GET["username"]) && !class_exists('Adminer\Db')) {
	unset($_SESSION["pwds"][DRIVER]);
	unset_permanent();
	page_header(lang(91), lang(92, implode(", ", Driver::$possibleDrivers)), false);
	page_footer("auth");
	exit;
}

stop_session(true);

if (isset($_GET["username"]) && is_string(get_password())) {
	list($host, $port) = explode(":", SERVER, 2);
	if (preg_match('~^\s*([-+]?\d+)~', $port, $match) && ($match[1] < 1024 || $match[1] > 65535)) { // is_numeric('80#') would still connect to port 80
		auth_error(lang(93));
	}
	check_invalid_login();
	$connection = connect($adminer->credentials());
	if (is_object($connection)) {
		$driver = new Driver($connection);
		if ($adminer->operators === null) {
			$adminer->operators = $driver->operators;
		}
		if (Driver::$jush == 'sql' || $connection->flavor == 'cockroach') {
			save_settings(array("vendor-" . DRIVER . "-" . SERVER => $drivers[DRIVER]));
		}
	}
}

$login = null;
if (!is_object($connection) || ($login = $adminer->login($_GET["username"], get_password())) !== true) {
	$error = (is_string($connection) ? nl_br(h($connection)) : (is_string($login) ? $login : lang(94)));
	auth_error($error . (preg_match('~^ | $~', get_password()) ? '<br>' . lang(95) : ''));
}

if ($_POST["logout"] && $has_token && !verify_token()) {
	page_header(lang(83), lang(96));
	page_footer("db");
	exit;
}

if ($auth && $_POST["token"]) {
	$_POST["token"] = $token; // reset token after explicit login
}

$error = ''; ///< @var string
if ($_POST) {
	if (!verify_token()) {
		$ini = "max_input_vars";
		$max_vars = ini_get($ini);
		if (extension_loaded("suhosin")) {
			foreach (array("suhosin.request.max_vars", "suhosin.post.max_vars") as $key) {
				$val = ini_get($key);
				if ($val && (!$max_vars || $val < $max_vars)) {
					$ini = $key;
					$max_vars = $val;
				}
			}
		}
		$error = (!$_POST["token"] && $max_vars
			? lang(97, "'$ini'")
			: lang(96) . ' ' . lang(98)
		);
	}

} elseif ($_SERVER["REQUEST_METHOD"] == "POST") {
	// posted form with no data means that post_max_size exceeded because Adminer always sends token at least
	$error = lang(99, "'post_max_size'");
	if (isset($_GET["sql"])) {
		$error .= ' ' . lang(100);
	}
}

?>
<?php
// This file is not used in Adminer Editor.

/** Print select result
* @param Result
* @param Db connection to examine indexes
* @param array
* @param int
* @return array $orgtables
*/
function select($result, $connection2 = null, $orgtables = array(), $limit = 0) {
	$links = array(); // colno => orgtable - create links from these columns
	$indexes = array(); // orgtable => array(column => colno) - primary keys
	$columns = array(); // orgtable => array(column => ) - not selected columns in primary key
	$blobs = array(); // colno => bool - display bytes for blobs
	$types = array(); // colno => type - display char in <code>
	$return = array(); // table => orgtable - mapping to use in EXPLAIN
	for ($i=0; (!$limit || $i < $limit) && ($row = $result->fetch_row()); $i++) {
		if (!$i) {
			echo "<div class='scrollable'>\n";
			echo "<table class='nowrap odds'>\n";
			echo "<thead><tr>";
			for ($j=0; $j < count($row); $j++) {
				$field = $result->fetch_field();
				$name = $field->name;
				$orgtable = (isset($field->orgtable) ? $field->orgtable : "");
				$orgname = (isset($field->orgname) ? $field->orgname : $name);
				if ($orgtables && JUSH == "sql") { // MySQL EXPLAIN
					$links[$j] = ($name == "table" ? "table=" : ($name == "possible_keys" ? "indexes=" : null));
				} elseif ($orgtable != "") {
					if (isset($field->table)) {
						$return[$field->table] = $orgtable;
					}
					if (!isset($indexes[$orgtable])) {
						// find primary key in each table
						$indexes[$orgtable] = array();
						foreach (indexes($orgtable, $connection2) as $index) {
							if ($index["type"] == "PRIMARY") {
								$indexes[$orgtable] = array_flip($index["columns"]);
								break;
							}
						}
						$columns[$orgtable] = $indexes[$orgtable];
					}
					if (isset($columns[$orgtable][$orgname])) {
						unset($columns[$orgtable][$orgname]);
						$indexes[$orgtable][$orgname] = $j;
						$links[$j] = $orgtable;
					}
				}
				if ($field->charsetnr == 63) { // 63 - binary
					$blobs[$j] = true;
				}
				$types[$j] = $field->type;
				echo "<th" . ($orgtable != "" || $field->name != $orgname ? " title='" . h(($orgtable != "" ? "$orgtable." : "") . $orgname) . "'" : "") . ">" . h($name)
					. ($orgtables ? doc_link(array(
						'sql' => "explain-output.html#explain_" . strtolower($name),
						'mariadb' => "explain/#the-columns-in-explain-select",
					)) : "")
				;
			}
			echo "</thead>\n";
		}
		echo "<tr>";
		foreach ($row as $key => $val) {
			$link = "";
			if (isset($links[$key]) && !$columns[$links[$key]]) {
				if ($orgtables && JUSH == "sql") { // MySQL EXPLAIN
					$table = $row[array_search("table=", $links)];
					$link = ME . $links[$key] . urlencode($orgtables[$table] != "" ? $orgtables[$table] : $table);
				} else {
					$link = ME . "edit=" . urlencode($links[$key]);
					foreach ($indexes[$links[$key]] as $col => $j) {
						$link .= "&where" . urlencode("[" . bracket_escape($col) . "]") . "=" . urlencode($row[$j]);
					}
				}
			} elseif (is_url($val)) {
				$link = $val;
			}
			if ($val === null) {
				$val = "<i>NULL</i>";
			} elseif ($blobs[$key] && !is_utf8($val)) {
				$val = "<i>" . lang(46, strlen($val)) . "</i>"; //! link to download
			} else {
				$val = h($val);
				if ($types[$key] == 254) { // 254 - char
					$val = "<code>$val</code>";
				}
			}
			if ($link) {
				$val = "<a href='" . h($link) . "'" . (is_url($link) ? target_blank() : '') . ">$val</a>";
			}
			// https://dev.mysql.com/doc/dev/mysql-server/latest/field__types_8h.html
			echo "<td" . ($types[$key] <= 9 || $types[$key] == 246 ? " class='number'" : "") . ">$val";
		}
	}
	echo ($i ? "</table>\n</div>" : "<p class='message'>" . lang(12)) . "\n";
	return $return;
}

/** Get referencable tables with single column primary key except self
* @param string
* @return array [$table_name => $field]
*/
function referencable_primary($self) {
	$return = array(); // table_name => field
	foreach (table_status('', true) as $table_name => $table) {
		if ($table_name != $self && fk_support($table)) {
			foreach (fields($table_name) as $field) {
				if ($field["primary"]) {
					if ($return[$table_name]) { // multi column primary key
						unset($return[$table_name]);
						break;
					}
					$return[$table_name] = $field;
				}
			}
		}
	}
	return $return;
}

/** Print SQL <textarea> tag
* @param string
* @param string or array in which case [0] of every element is used
* @param int
* @param int
* @return null
*/
function textarea($name, $value, $rows = 10, $cols = 80) {
	echo "<textarea name='" . h($name) . "' rows='$rows' cols='$cols' class='sqlarea jush-" . JUSH . "' spellcheck='false' wrap='off'>";
	if (is_array($value)) {
		foreach ($value as $val) { // not implode() to save memory
			echo h($val[0]) . "\n\n\n"; // $val == array($query, $time, $elapsed)
		}
	} else {
		echo h($value);
	}
	echo "</textarea>";
}

/** Generate HTML <select> or <input> if $options are empty
* @param string
* @param array
* @param string
* @param string
* @param string
* @return string
*/
function select_input($attrs, $options, $value = "", $onchange = "", $placeholder = "") {
	$tag = ($options ? "select" : "input");
	return "<$tag$attrs" . ($options
		? "><option value=''>$placeholder" . optionlist($options, $value, true) . "</select>"
		: " size='10' value='" . h($value) . "' placeholder='$placeholder'>"
	) . ($onchange ? script("qsl('$tag').onchange = $onchange;", "") : ""); //! use oninput for input
}

/** Print one row in JSON object
* @param string or "" to close the object
* @param string
* @return null
*/
function json_row($key, $val = null) {
	static $first = true;
	if ($first) {
		echo "{";
	}
	if ($key != "") {
		echo ($first ? "" : ",") . "\n\t\"" . addcslashes($key, "\r\n\t\"\\/") . '": ' . ($val !== null ? '"' . addcslashes($val, "\r\n\"\\/") . '"' : 'null');
		$first = false;
	} else {
		echo "\n}\n";
		$first = true;
	}
}

/** Print table columns for type edit
* @param string
* @param array
* @param array
* @param array returned by referencable_primary()
* @param array extra types to prepend
* @return null
*/
function edit_type($key, $field, $collations, $foreign_keys = array(), $extra_types = array()) {
	global $driver;
	$type = $field["type"];
	echo "<td><select name='" . h($key) . "[type]' class='type' aria-labelledby='label-type'>";
	if ($type && !array_key_exists($type, $driver->types()) && !isset($foreign_keys[$type]) && !in_array($type, $extra_types)) {
		$extra_types[] = $type;
	}
	$structured_types = $driver->structuredTypes();
	if ($foreign_keys) {
		$structured_types[lang(101)] = $foreign_keys;
	}
	echo optionlist(array_merge($extra_types, $structured_types), $type);
	echo "</select><td>";
	echo "<input name='" . h($key) . "[length]' value='" . h($field["length"]) . "' size='3'"
		. (!$field["length"] && preg_match('~var(char|binary)$~', $type) ? " class='required'" : "") //! type="number" with enabled JavaScript
		. " aria-labelledby='label-length'>";
	echo "<td class='options'>";
	echo ($collations
		? "<input list='collations' name='" . h($key) . "[collation]'" . (preg_match('~(char|text|enum|set)$~', $type) ? "" : " class='hidden'") . " value='" . h($field["collation"]) . "' placeholder='(" . lang(102) . ")'>"
		: ''
	);
	echo ($driver->unsigned ? "<select name='" . h($key) . "[unsigned]'" . (!$type || preg_match(number_type(), $type) ? "" : " class='hidden'") . '><option>' . optionlist($driver->unsigned, $field["unsigned"]) . '</select>' : '');
	echo (isset($field['on_update']) ? "<select name='" . h($key) . "[on_update]'" . (preg_match('~timestamp|datetime~', $type) ? "" : " class='hidden'") . '>'
		. optionlist(array("" => "(" . lang(103) . ")", "CURRENT_TIMESTAMP"), (preg_match('~^CURRENT_TIMESTAMP~i', $field["on_update"]) ? "CURRENT_TIMESTAMP" : $field["on_update"]))
		. '</select>' : ''
	);
	echo ($foreign_keys
		? "<select name='" . h($key) . "[on_delete]'" . (preg_match("~`~", $type) ? "" : " class='hidden'") . "><option value=''>(" . lang(104) . ")" . optionlist(explode("|", $driver->onActions), $field["on_delete"]) . "</select> "
		: " " // space for IE
	);
}

/** Get partition info
* @param string
* @return array
*/
function get_partitions_info($table) {
	global $connection;
	$from = "FROM information_schema.PARTITIONS WHERE TABLE_SCHEMA = " . q(DB) . " AND TABLE_NAME = " . q($table);
	$result = $connection->query("SELECT PARTITION_METHOD, PARTITION_EXPRESSION, PARTITION_ORDINAL_POSITION $from ORDER BY PARTITION_ORDINAL_POSITION DESC LIMIT 1");
	$return = array();
	list($return["partition_by"], $return["partition"], $return["partitions"]) = $result->fetch_row();
	$partitions = get_key_vals("SELECT PARTITION_NAME, PARTITION_DESCRIPTION $from AND PARTITION_NAME != '' ORDER BY PARTITION_ORDINAL_POSITION");
	$return["partition_names"] = array_keys($partitions);
	$return["partition_values"] = array_values($partitions);
	return $return;
}

/** Filter length value including enums
* @param string
* @return string
*/
function process_length($length) {
	global $driver;
	$enum_length = $driver->enumLength;
	return (preg_match("~^\\s*\\(?\\s*$enum_length(?:\\s*,\\s*$enum_length)*+\\s*\\)?\\s*\$~", $length) && preg_match_all("~$enum_length~", $length, $matches)
		? "(" . implode(",", $matches[0]) . ")"
		: preg_replace('~^[0-9].*~', '(\0)', preg_replace('~[^-0-9,+()[\]]~', '', $length))
	);
}

/** Create SQL string from field type
* @param array
* @param string
* @return string
*/
function process_type($field, $collate = "COLLATE") {
	global $driver;
	return " $field[type]"
		. process_length($field["length"])
		. (preg_match(number_type(), $field["type"]) && in_array($field["unsigned"], $driver->unsigned) ? " $field[unsigned]" : "")
		. (preg_match('~char|text|enum|set~', $field["type"]) && $field["collation"] ? " $collate " . (JUSH == "mssql" ? $field["collation"] : q($field["collation"])) : "")
	;
}

/** Create SQL string from field
* @param array basic field information
* @param array information about field type
* @return array ["field", "type", "NULL", "DEFAULT", "ON UPDATE", "COMMENT", "AUTO_INCREMENT"]
*/
function process_field($field, $type_field) {
	// MariaDB exports CURRENT_TIMESTAMP as a function.
	if ($field["on_update"]) {
		$field["on_update"] = str_ireplace("current_timestamp()", "CURRENT_TIMESTAMP", $field["on_update"]);
	}
	return array(
		idf_escape(trim($field["field"])),
		process_type($type_field),
		($field["null"] ? " NULL" : " NOT NULL"), // NULL for timestamp
		default_value($field),
		(preg_match('~timestamp|datetime~', $field["type"]) && $field["on_update"] ? " ON UPDATE $field[on_update]" : ""),
		(support("comment") && $field["comment"] != "" ? " COMMENT " . q($field["comment"]) : ""),
		($field["auto_increment"] ? auto_increment() : null),
	);
}

/** Get default value clause
* @param array
* @return string
*/
function default_value($field) {
	global $driver;
	$default = $field["default"];
	$generated = $field["generated"];
	return ($default === null ? "" : (in_array($generated, $driver->generated)
		? (JUSH == "mssql" ? " AS ($default)" . ($generated == "VIRTUAL" ? "" : " $generated") . "" : " GENERATED ALWAYS AS ($default) $generated")
		: " DEFAULT " . (!preg_match('~^GENERATED ~i', $default) && (preg_match('~char|binary|text|json|enum|set~', $field["type"]) || preg_match('~^(?![a-z])~i', $default))
			? (JUSH == "sql" && preg_match('~text|json~', $field["type"]) ? "(" . q($default) . ")" : q($default)) // MySQL requires () around default value of text column
			: str_ireplace("current_timestamp()", "CURRENT_TIMESTAMP", (JUSH == "sqlite" ? "($default)" : $default))
		)
	));
}

/** Get type class to use in CSS
* @param string
* @return string class=''
*/
function type_class($type) {
	foreach (
		array(
			'char' => 'text',
			'date' => 'time|year',
			'binary' => 'blob',
			'enum' => 'set',
		) as $key => $val
	) {
		if (preg_match("~$key|$val~", $type)) {
			return " class='$key'";
		}
	}
}

/** Print table interior for fields editing
* @param array
* @param array
* @param string TABLE or PROCEDURE
* @param array returned by referencable_primary()
* @return null
*/
function edit_fields($fields, $collations, $type = "TABLE", $foreign_keys = array()) {
	global $driver;
	$fields = array_values($fields);
	$default_class = (($_POST ? $_POST["defaults"] : get_setting("defaults")) ? "" : " class='hidden'");
	$comment_class = (($_POST ? $_POST["comments"] : get_setting("comments")) ? "" : " class='hidden'");
	echo "<thead><tr>\n";
	echo ($type == "PROCEDURE" ? "<td>" : "");
	echo "<th id='label-name'>" . ($type == "TABLE" ? lang(105) : lang(106));
	echo "<td id='label-type'>" . lang(48) . "<textarea id='enum-edit' rows='4' cols='12' wrap='off' style='display: none;'></textarea>" . script("qs('#enum-edit').onblur = editingLengthBlur;");
	echo "<td id='label-length'>" . lang(107);
	echo "<td>" . lang(108); // no label required, options have their own label
	if ($type == "TABLE") {
		echo "<td id='label-null'>NULL\n";
		echo "<td><input type='radio' name='auto_increment_col' value=''><abbr id='label-ai' title='" . lang(50) . "'>AI</abbr>";
		echo doc_link(array(
			'sql' => "example-auto-increment.html",
			'mariadb' => "auto_increment/",
			'sqlite' => "autoinc.html",
			'pgsql' => "datatype-numeric.html#DATATYPE-SERIAL",
			'mssql' => "t-sql/statements/create-table-transact-sql-identity-property",
		));
		echo "<td id='label-default'$default_class>" . lang(51);
		echo (support("comment") ? "<td id='label-comment'$comment_class>" . lang(49) : "");
	}
	echo "<td><input type='image' class='icon' name='add[" . (support("move_col") ? 0 : count($fields)) . "]' src='" . h(preg_replace("~\\?.*~", "", ME) . "?file=plus.gif&version=5.1.0") . "' alt='+' title='" . lang(109) . "'>" . script("row_count = " . count($fields) . ";");
	echo "</thead>\n<tbody>\n";
	echo script("mixin(qsl('tbody'), {onclick: editingClick, onkeydown: editingKeydown, oninput: editingInput});");
	foreach ($fields as $i => $field) {
		$i++;
		$orig = $field[($_POST ? "orig" : "field")];
		$display = (isset($_POST["add"][$i-1]) || (isset($field["field"]) && !$_POST["drop_col"][$i])) && (support("drop_col") || $orig == "");
		echo "<tr" . ($display ? "" : " style='display: none;'") . ">\n";
		echo ($type == "PROCEDURE" ? "<td>" . html_select("fields[$i][inout]", explode("|", $driver->inout), $field["inout"]) : "") . "<th>";
		if ($display) {
			echo "<input name='fields[$i][field]' value='" . h($field["field"]) . "' data-maxlength='64' autocapitalize='off' aria-labelledby='label-name'>";
		}
		echo input_hidden("fields[$i][orig]", $orig);
		edit_type("fields[$i]", $field, $collations, $foreign_keys);
		if ($type == "TABLE") {
			echo "<td>" . checkbox("fields[$i][null]", 1, $field["null"], "", "", "block", "label-null");
			echo "<td><label class='block'><input type='radio' name='auto_increment_col' value='$i'" . ($field["auto_increment"] ? " checked" : "") . " aria-labelledby='label-ai'></label>";
			echo "<td$default_class>" . ($driver->generated
				? html_select("fields[$i][generated]", array_merge(array("", "DEFAULT"), $driver->generated), $field["generated"]) . " "
				: checkbox("fields[$i][generated]", 1, $field["generated"], "", "", "", "label-default")
			);
			echo "<input name='fields[$i][default]' value='" . h($field["default"]) . "' aria-labelledby='label-default'>";
			echo (support("comment") ? "<td$comment_class><input name='fields[$i][comment]' value='" . h($field["comment"]) . "' data-maxlength='" . (min_version(5.5) ? 1024 : 255) . "' aria-labelledby='label-comment'>" : "");
		}
		echo "<td>";
		echo (support("move_col") ?
			"<input type='image' class='icon' name='add[$i]' src='" . h(preg_replace("~\\?.*~", "", ME) . "?file=plus.gif&version=5.1.0") . "' alt='+' title='" . lang(109) . "'> "
			. "<input type='image' class='icon' name='up[$i]' src='" . h(preg_replace("~\\?.*~", "", ME) . "?file=up.gif&version=5.1.0") . "' alt='↑' title='" . lang(110) . "'> "
			. "<input type='image' class='icon' name='down[$i]' src='" . h(preg_replace("~\\?.*~", "", ME) . "?file=down.gif&version=5.1.0") . "' alt='↓' title='" . lang(111) . "'> "
		: "");
		echo ($orig == "" || support("drop_col") ? "<input type='image' class='icon' name='drop_col[$i]' src='" . h(preg_replace("~\\?.*~", "", ME) . "?file=cross.gif&version=5.1.0") . "' alt='x' title='" . lang(112) . "'>" : "");
	}
}

/** Move fields up and down or add field
* @param array
* @return bool
*/
function process_fields(&$fields) {
	$offset = 0;
	if ($_POST["up"]) {
		$last = 0;
		foreach ($fields as $key => $field) {
			if (key($_POST["up"]) == $key) {
				unset($fields[$key]);
				array_splice($fields, $last, 0, array($field));
				break;
			}
			if (isset($field["field"])) {
				$last = $offset;
			}
			$offset++;
		}
	} elseif ($_POST["down"]) {
		$found = false;
		foreach ($fields as $key => $field) {
			if (isset($field["field"]) && $found) {
				unset($fields[key($_POST["down"])]);
				array_splice($fields, $offset, 0, array($found));
				break;
			}
			if (key($_POST["down"]) == $key) {
				$found = $field;
			}
			$offset++;
		}
	} elseif ($_POST["add"]) {
		$fields = array_values($fields);
		array_splice($fields, key($_POST["add"]), 0, array(array()));
	} elseif (!$_POST["drop_col"]) {
		return false;
	}
	return true;
}

/** Callback used in routine()
* @param array
* @return string
*/
function normalize_enum($match) {
	return "'" . str_replace("'", "''", addcslashes(stripcslashes(str_replace($match[0][0] . $match[0][0], $match[0][0], substr($match[0], 1, -1))), '\\')) . "'";
}

/** Issue grant or revoke commands
* @param string GRANT or REVOKE
* @param array
* @param string
* @param string
* @return bool
*/
function grant($grant, $privileges, $columns, $on) {
	if (!$privileges) {
		return true;
	}
	if ($privileges == array("ALL PRIVILEGES", "GRANT OPTION")) {
		// can't be granted or revoked together
		return ($grant == "GRANT"
			? queries("$grant ALL PRIVILEGES$on WITH GRANT OPTION")
			: queries("$grant ALL PRIVILEGES$on") && queries("$grant GRANT OPTION$on")
		);
	}
	return queries("$grant " . preg_replace('~(GRANT OPTION)\([^)]*\)~', '\1', implode("$columns, ", $privileges) . $columns) . $on);
}

/** Drop old object and create a new one
* @param string drop old object query
* @param string create new object query
* @param string drop new object query
* @param string create test object query
* @param string drop test object query
* @param string
* @param string
* @param string
* @param string
* @param string
* @param string
* @return null redirect in success
*/
function drop_create($drop, $create, $drop_created, $test, $drop_test, $location, $message_drop, $message_alter, $message_create, $old_name, $new_name) {
	if ($_POST["drop"]) {
		query_redirect($drop, $location, $message_drop);
	} elseif ($old_name == "") {
		query_redirect($create, $location, $message_create);
	} elseif ($old_name != $new_name) {
		$created = queries($create);
		queries_redirect($location, $message_alter, $created && queries($drop));
		if ($created) {
			queries($drop_created);
		}
	} else {
		queries_redirect(
			$location,
			$message_alter,
			queries($test) && queries($drop_test) && queries($drop) && queries($create)
		);
	}
}

/** Generate SQL query for creating trigger
* @param string
* @param array result of trigger()
* @return string
*/
function create_trigger($on, $row) {
	$timing_event = " $row[Timing] $row[Event]" . (preg_match('~ OF~', $row["Event"]) ? " $row[Of]" : ""); // SQL injection
	return "CREATE TRIGGER "
		. idf_escape($row["Trigger"])
		. (JUSH == "mssql" ? $on . $timing_event : $timing_event . $on)
		. rtrim(" $row[Type]\n$row[Statement]", ";")
		. ";"
	;
}

/** Generate SQL query for creating routine
* @param string "PROCEDURE" or "FUNCTION"
* @param array result of routine()
* @return string
*/
function create_routine($routine, $row) {
	global $driver;
	$set = array();
	$fields = (array) $row["fields"];
	ksort($fields); // enforce fields order
	foreach ($fields as $field) {
		if ($field["field"] != "") {
			$set[] = (preg_match("~^($driver->inout)\$~", $field["inout"]) ? "$field[inout] " : "") . idf_escape($field["field"]) . process_type($field, "CHARACTER SET");
		}
	}
	$definition = rtrim($row["definition"], ";");
	return "CREATE $routine "
		. idf_escape(trim($row["name"]))
		. " (" . implode(", ", $set) . ")"
		. ($routine == "FUNCTION" ? " RETURNS" . process_type($row["returns"], "CHARACTER SET") : "")
		. ($row["language"] ? " LANGUAGE $row[language]" : "")
		. (JUSH == "pgsql" ? " AS " . q($definition) : "\n$definition;")
	;
}

/** Remove current user definer from SQL command
* @param string
* @return string
*/
function remove_definer($query) {
	return preg_replace('~^([A-Z =]+) DEFINER=`' . preg_replace('~@(.*)~', '`@`(%|\1)', logged_user()) . '`~', '\1', $query); //! proper escaping of user
}

/** Format foreign key to use in SQL query
* @param array ["db" => string, "ns" => string, "table" => string, "source" => array, "target" => array, "on_delete" => one of $on_actions, "on_update" => one of $on_actions]
* @return string
*/
function format_foreign_key($foreign_key) {
	global $driver;
	$db = $foreign_key["db"];
	$ns = $foreign_key["ns"];
	return " FOREIGN KEY (" . implode(", ", array_map('Adminer\idf_escape', $foreign_key["source"])) . ") REFERENCES "
		. ($db != "" && $db != $_GET["db"] ? idf_escape($db) . "." : "")
		. ($ns != "" && $ns != $_GET["ns"] ? idf_escape($ns) . "." : "")
		. idf_escape($foreign_key["table"])
		. " (" . implode(", ", array_map('Adminer\idf_escape', $foreign_key["target"])) . ")" //! reuse $name - check in older MySQL versions
		. (preg_match("~^($driver->onActions)\$~", $foreign_key["on_delete"]) ? " ON DELETE $foreign_key[on_delete]" : "")
		. (preg_match("~^($driver->onActions)\$~", $foreign_key["on_update"]) ? " ON UPDATE $foreign_key[on_update]" : "")
	;
}

/** Add a file to TAR
* @param string
* @param TmpFile
* @return null prints the output
*/
function tar_file($filename, $tmp_file) {
	$return = pack("a100a8a8a8a12a12", $filename, 644, 0, 0, decoct($tmp_file->size), decoct(time()));
	$checksum = 8*32; // space for checksum itself
	for ($i=0; $i < strlen($return); $i++) {
		$checksum += ord($return[$i]);
	}
	$return .= sprintf("%06o", $checksum) . "\0 ";
	echo $return;
	echo str_repeat("\0", 512 - strlen($return));
	$tmp_file->send();
	echo str_repeat("\0", 511 - ($tmp_file->size + 511) % 512);
}

/** Get INI bytes value
* @param string
* @return int
*/
function ini_bytes($ini) {
	$val = ini_get($ini);
	switch (strtolower(substr($val, -1))) {
		case 'g':
			$val = (int) $val * 1024; // no break
		case 'm':
			$val = (int) $val * 1024; // no break
		case 'k':
			$val = (int) $val * 1024;
	}
	return $val;
}

/** Create link to database documentation
* @param array JUSH => $path
* @param string HTML code
* @return string HTML code
*/
function doc_link($paths, $text = "<sup>?</sup>") {
	global $connection;
	$server_info = $connection->server_info;
	$version = preg_replace('~^(\d\.?\d).*~s', '\1', $server_info); // two most significant digits
	$urls = array(
		'sql' => "https://dev.mysql.com/doc/refman/$version/en/",
		'sqlite' => "https://www.sqlite.org/",
		'pgsql' => "https://www.postgresql.org/docs/" . ($connection->flavor == 'cockroach' ? "current" : $version) . "/",
		'mssql' => "https://learn.microsoft.com/en-us/sql/",
		'oracle' => "https://www.oracle.com/pls/topic/lookup?ctx=db" . preg_replace('~^.* (\d+)\.(\d+)\.\d+\.\d+\.\d+.*~s', '\1\2', $server_info) . "&id=",
	);
	if ($connection->flavor == 'maria') {
		$urls['sql'] = "https://mariadb.com/kb/en/";
		$paths['sql'] = (isset($paths['mariadb']) ? $paths['mariadb'] : str_replace(".html", "/", $paths['sql']));
	}
	return ($paths[JUSH] ? "<a href='" . h($urls[JUSH] . $paths[JUSH] . (JUSH == 'mssql' ? "?view=sql-server-ver$version" : "")) . "'" . target_blank() . ">$text</a>" : "");
}

/** Compute size of database
* @param string
* @return string formatted
*/
function db_size($db) {
	global $connection;
	if (!$connection->select_db($db)) {
		return "?";
	}
	$return = 0;
	foreach (table_status() as $table_status) {
		$return += $table_status["Data_length"] + $table_status["Index_length"];
	}
	return format_number($return);
}

/** Print SET NAMES if utf8mb4 might be needed
* @param string
* @return null
*/
function set_utf8mb4($create) {
	global $connection;
	static $set = false;
	if (!$set && preg_match('~\butf8mb4~i', $create)) { // possible false positive
		$set = true;
		echo "SET NAMES " . charset($connection) . ";\n\n";
	}
}

?>
<?php
if (isset($_GET["status"])) {
	$_GET["variables"] = $_GET["status"];
}
if (isset($_GET["import"])) {
	$_GET["sql"] = $_GET["import"];
}

if (
	!(DB != ""
		? $connection->select_db(DB)
		: isset($_GET["sql"]) || isset($_GET["dump"]) || isset($_GET["database"]) || isset($_GET["processlist"]) || isset($_GET["privileges"]) || isset($_GET["user"]) || isset($_GET["variables"])
			|| $_GET["script"] == "connect" || $_GET["script"] == "kill"
	)
) {
	if (DB != "" || $_GET["refresh"]) {
		restart_session();
		set_session("dbs", null);
	}
	if (DB != "") {
		header("HTTP/1.1 404 Not Found");
		page_header(lang(36) . ": " . h(DB), lang(113), true);
	} else {
		if ($_POST["db"] && !$error) {
			queries_redirect(substr(ME, 0, -1), lang(114), drop_databases($_POST["db"]));
		}

		page_header(lang(115), $error, false);
		echo "<p class='links'>\n";
		foreach (
			array(
				'database' => lang(116),
				'privileges' => lang(70),
				'processlist' => lang(117),
				'variables' => lang(118),
				'status' => lang(119),
			) as $key => $val
		) {
			if (support($key)) {
				echo "<a href='" . h(ME) . "$key='>$val</a>\n";
			}
		}
		echo "<p>" . lang(120, $drivers[DRIVER], "<b>" . h($connection->server_info) . "</b>", "<b>$connection->extension</b>") . "\n";
		echo "<p>" . lang(121, "<b>" . h(logged_user()) . "</b>") . "\n";
		if (isset($adminer->plugins) && is_array($adminer->plugins)) {
			echo "<p>" . lang(122) . ":\n<ul>\n";
			foreach ($adminer->plugins as $plugin) {
				$reflection = new \ReflectionObject($plugin);
				echo "<li><b>" . get_class($plugin) . "</b>" . h(preg_match('~^/[\s*]+(.+)~', $reflection->getDocComment(), $match) ? ": $match[1]" : "") . "\n";
			}
			echo "</ul>\n";
		}
		$databases = $adminer->databases();
		if ($databases) {
			$scheme = support("scheme");
			$collations = collations();
			echo "<form action='' method='post'>\n";
			echo "<table class='checkable odds'>\n";
			echo script("mixin(qsl('table'), {onclick: tableClick, ondblclick: partialArg(tableClick, true)});");
			echo "<thead><tr>"
				. (support("database") ? "<td>" : "")
				. "<th>" . lang(36) . (get_session("dbs") !== null ? " - <a href='" . h(ME) . "refresh=1'>" . lang(123) . "</a>" : "")
				. "<td>" . lang(124)
				. "<td>" . lang(125)
				. "<td>" . lang(126) . " - <a href='" . h(ME) . "dbsize=1'>" . lang(127) . "</a>" . script("qsl('a').onclick = partial(ajaxSetHtml, '" . js_escape(ME) . "script=connect');", "")
				. "</thead>\n"
			;

			$databases = ($_GET["dbsize"] ? count_tables($databases) : array_flip($databases));

			foreach ($databases as $db => $tables) {
				$root = h(ME) . "db=" . urlencode($db);
				$id = h("Db-" . $db);
				echo "<tr>" . (support("database") ? "<td>" . checkbox("db[]", $db, in_array($db, (array) $_POST["db"]), "", "", "", $id) : "");
				echo "<th><a href='$root' id='$id'>" . h($db) . "</a>";
				$collation = h(db_collation($db, $collations));
				echo "<td>" . (support("database") ? "<a href='$root" . ($scheme ? "&amp;ns=" : "") . "&amp;database=' title='" . lang(66) . "'>$collation</a>" : $collation);
				echo "<td align='right'><a href='$root&amp;schema=' id='tables-" . h($db) . "' title='" . lang(69) . "'>" . ($_GET["dbsize"] ? $tables : "?") . "</a>";
				echo "<td align='right' id='size-" . h($db) . "'>" . ($_GET["dbsize"] ? db_size($db) : "?");
				echo "\n";
			}

			echo "</table>\n";
			echo (support("database")
				? "<div class='footer'><div>\n"
					. "<fieldset><legend>" . lang(128) . " <span id='selected'></span></legend><div>\n"
					. input_hidden("all") . script("qsl('input').onclick = function () { selectCount('selected', formChecked(this, /^db/)); };") // used by trCheck()
					. "<input type='submit' name='drop' value='" . lang(129) . "'>" . confirm() . "\n"
					. "</div></fieldset>\n"
					. "</div></div>\n"
				: ""
			);
			echo input_token();
			echo "</form>\n";
			echo script("tableCheck();");
		}
	}

	page_footer("db");
	exit;
}

if (support("scheme")) {
	if (DB != "" && $_GET["ns"] !== "") {
		if (!isset($_GET["ns"])) {
			redirect(preg_replace('~ns=[^&]*&~', '', ME) . "ns=" . get_schema());
		}
		if (!set_schema($_GET["ns"])) {
			header("HTTP/1.1 404 Not Found");
			page_header(lang(75) . ": " . h($_GET["ns"]), lang(130), true);
			page_footer("ns");
			exit;
		}
	}
}


?>
<?php
class TmpFile {
	private $handler, $size;

	function __construct() {
		$this->handler = tmpfile();
	}

	function write($contents) {
		$this->size += strlen($contents);
		fwrite($this->handler, $contents);
	}

	function send() {
		fseek($this->handler, 0);
		fpassthru($this->handler);
		fclose($this->handler);
	}
}


if (isset($_GET["select"]) && ($_POST["edit"] || $_POST["clone"]) && !$_POST["save"]) {
	$_GET["edit"] = $_GET["select"];
}
// this is matched by compile.php
if (isset($_GET["callf"])) {
	$_GET["call"] = $_GET["callf"];
}
if (isset($_GET["function"])) {
	$_GET["procedure"] = $_GET["function"];
}

if (isset($_GET["download"])) {
	?>
<?php
$TABLE = $_GET["download"];
$fields = fields($TABLE);
header("Content-Type: application/octet-stream");
header("Content-Disposition: attachment; filename=" . friendly_url("$TABLE-" . implode("_", $_GET["where"])) . "." . friendly_url($_GET["field"]));
$select = array(idf_escape($_GET["field"]));
$result = $driver->select($TABLE, $select, array(where($_GET, $fields)), $select);
$row = ($result ? $result->fetch_row() : array());
echo $driver->value($row[0], $fields[$_GET["field"]]);
exit; // don't output footer

} elseif (isset($_GET["table"])) {
	?>
<?php
$TABLE = $_GET["table"];
$fields = fields($TABLE);
if (!$fields) {
	$error = error();
}
$table_status = table_status1($TABLE);
$name = $adminer->tableName($table_status);

page_header(($fields && is_view($table_status) ? $table_status['Engine'] == 'materialized view' ? lang(131) : lang(132) : lang(133)) . ": " . ($name != "" ? $name : h($TABLE)), $error);

$rights = array();
foreach ($fields as $key => $field) {
	$rights += $field["privileges"];
}
$adminer->selectLinks($table_status, (isset($rights["insert"]) || !support("table") ? "" : null));

$comment = $table_status["Comment"];
if ($comment != "") {
	echo "<p class='nowrap'>" . lang(49) . ": " . h($comment) . "\n";
}

if ($fields) {
	$adminer->tableStructurePrint($fields, $table_status);
}

if (support("indexes") && $driver->supportsIndex($table_status)) {
	echo "<h3 id='indexes'>" . lang(134) . "</h3>\n";
	$indexes = indexes($TABLE);
	if ($indexes) {
		$adminer->tableIndexesPrint($indexes);
	}
	echo '<p class="links"><a href="' . h(ME) . 'indexes=' . urlencode($TABLE) . '">' . lang(135) . "</a>\n";
}

if (!is_view($table_status)) {
	if (fk_support($table_status)) {
		echo "<h3 id='foreign-keys'>" . lang(101) . "</h3>\n";
		$foreign_keys = foreign_keys($TABLE);
		if ($foreign_keys) {
			echo "<table>\n";
			echo "<thead><tr><th>" . lang(136) . "<td>" . lang(137) . "<td>" . lang(104) . "<td>" . lang(103) . "<td></thead>\n";
			foreach ($foreign_keys as $name => $foreign_key) {
				echo "<tr title='" . h($name) . "'>";
				echo "<th><i>" . implode("</i>, <i>", array_map('Adminer\h', $foreign_key["source"])) . "</i>";
				$link = ($foreign_key["db"] != ""
					? preg_replace('~db=[^&]*~', "db=" . urlencode($foreign_key["db"]), ME)
					: ($foreign_key["ns"] != "" ? preg_replace('~ns=[^&]*~', "ns=" . urlencode($foreign_key["ns"]), ME) : ME)
				);
				echo "<td><a href='" . h($link . "table=" . urlencode($foreign_key["table"])) . "'>"
					. ($foreign_key["db"] != "" && $foreign_key["db"] != DB ? "<b>" . h($foreign_key["db"]) . "</b>." : "")
					. ($foreign_key["ns"] != "" && $foreign_key["ns"] != $_GET["ns"] ? "<b>" . h($foreign_key["ns"]) . "</b>." : "")
					. h($foreign_key["table"])
					. "</a>"
				;
				echo "(<i>" . implode("</i>, <i>", array_map('Adminer\h', $foreign_key["target"])) . "</i>)";
				echo "<td>" . h($foreign_key["on_delete"]);
				echo "<td>" . h($foreign_key["on_update"]);
				echo '<td><a href="' . h(ME . 'foreign=' . urlencode($TABLE) . '&name=' . urlencode($name)) . '">' . lang(138) . '</a>';
				echo "\n";
			}
			echo "</table>\n";
		}
		echo '<p class="links"><a href="' . h(ME) . 'foreign=' . urlencode($TABLE) . '">' . lang(139) . "</a>\n";
	}

	if (support("check")) {
		echo "<h3 id='checks'>" . lang(140) . "</h3>\n";
		$check_constraints = $driver->checkConstraints($TABLE);
		if ($check_constraints) {
			echo "<table>\n";
			foreach ($check_constraints as $key => $val) {
				echo "<tr title='" . h($key) . "'>";
				echo "<td><code class='jush-" . JUSH . "'>" . h($val);
				echo "<td><a href='" . h(ME . 'check=' . urlencode($TABLE) . '&name=' . urlencode($key)) . "'>" . lang(138) . "</a>";
				echo "\n";
			}
			echo "</table>\n";
		}
		echo '<p class="links"><a href="' . h(ME) . 'check=' . urlencode($TABLE) . '">' . lang(141) . "</a>\n";
	}
}

if (support(is_view($table_status) ? "view_trigger" : "trigger")) {
	echo "<h3 id='triggers'>" . lang(142) . "</h3>\n";
	$triggers = triggers($TABLE);
	if ($triggers) {
		echo "<table>\n";
		foreach ($triggers as $key => $val) {
			echo "<tr valign='top'><td>" . h($val[0]) . "<td>" . h($val[1]) . "<th>" . h($key) . "<td><a href='" . h(ME . 'trigger=' . urlencode($TABLE) . '&name=' . urlencode($key)) . "'>" . lang(138) . "</a>\n";
		}
		echo "</table>\n";
	}
	echo '<p class="links"><a href="' . h(ME) . 'trigger=' . urlencode($TABLE) . '">' . lang(143) . "</a>\n";
}

} elseif (isset($_GET["schema"])) {
	?>
<?php
page_header(lang(69), "", array(), h(DB . ($_GET["ns"] ? ".$_GET[ns]" : "")));

$table_pos = array();
$table_pos_js = array();
$SCHEMA = ($_GET["schema"] ?: $_COOKIE["adminer_schema-" . str_replace(".", "_", DB)]); // $_COOKIE["adminer_schema"] was used before 3.2.0 //! ':' in table name
preg_match_all('~([^:]+):([-0-9.]+)x([-0-9.]+)(_|$)~', $SCHEMA, $matches, PREG_SET_ORDER);
foreach ($matches as $i => $match) {
	$table_pos[$match[1]] = array($match[2], $match[3]);
	$table_pos_js[] = "\n\t'" . js_escape($match[1]) . "': [ $match[2], $match[3] ]";
}

$top = 0;
$base_left = -1;
$schema = array(); // table => array("fields" => array(name => field), "pos" => array(top, left), "references" => array(table => array(left => array(source, target))))
$referenced = array(); // target_table => array(table => array(left => target_column))
$lefts = array(); // float => bool
foreach (table_status('', true) as $table => $table_status) {
	if (is_view($table_status)) {
		continue;
	}
	$pos = 0;
	$schema[$table]["fields"] = array();
	foreach (fields($table) as $name => $field) {
		$pos += 1.25;
		$field["pos"] = $pos;
		$schema[$table]["fields"][$name] = $field;
	}
	$schema[$table]["pos"] = ($table_pos[$table] ?: array($top, 0));
	foreach ($adminer->foreignKeys($table) as $val) {
		if (!$val["db"]) {
			$left = $base_left;
			if ($table_pos[$table][1] || $table_pos[$val["table"]][1]) {
				$left = min(floatval($table_pos[$table][1]), floatval($table_pos[$val["table"]][1])) - 1;
			} else {
				$base_left -= .1;
			}
			while ($lefts[(string) $left]) {
				// find free $left
				$left -= .0001;
			}
			$schema[$table]["references"][$val["table"]][(string) $left] = array($val["source"], $val["target"]);
			$referenced[$val["table"]][$table][(string) $left] = $val["target"];
			$lefts[(string) $left] = true;
		}
	}
	$top = max($top, $schema[$table]["pos"][0] + 2.5 + $pos);
}

?>
<div id="schema" style="height: <?php echo $top; ?>em;">
<script<?php echo nonce(); ?>>
qs('#schema').onselectstart = () => false;
const tablePos = {<?php echo implode(",", $table_pos_js) . "\n"; ?>};
const em = qs('#schema').offsetHeight / <?php echo $top; ?>;
document.onmousemove = schemaMousemove;
document.onmouseup = partialArg(schemaMouseup, '<?php echo js_escape(DB); ?>');
</script>
<?php
foreach ($schema as $name => $table) {
	echo "<div class='table' style='top: " . $table["pos"][0] . "em; left: " . $table["pos"][1] . "em;'>";
	echo '<a href="' . h(ME) . 'table=' . urlencode($name) . '"><b>' . h($name) . "</b></a>";
	echo script("qsl('div').onmousedown = schemaMousedown;");

	foreach ($table["fields"] as $field) {
		$val = '<span' . type_class($field["type"]) . ' title="' . h($field["full_type"] . ($field["null"] ? " NULL" : '')) . '">' . h($field["field"]) . '</span>';
		echo "<br>" . ($field["primary"] ? "<i>$val</i>" : $val);
	}

	foreach ((array) $table["references"] as $target_name => $refs) {
		foreach ($refs as $left => $ref) {
			$left1 = $left - $table_pos[$name][1];
			$i = 0;
			foreach ($ref[0] as $source) {
				echo "\n<div class='references' title='" . h($target_name) . "' id='refs$left-" . ($i++) . "' style='left: $left1" . "em; top: " . $table["fields"][$source]["pos"] . "em; padding-top: .5em;'>"
					. "<div style='border-top: 1px solid gray; width: " . (-$left1) . "em;'></div></div>"
				;
			}
		}
	}

	foreach ((array) $referenced[$name] as $target_name => $refs) {
		foreach ($refs as $left => $columns) {
			$left1 = $left - $table_pos[$name][1];
			$i = 0;
			foreach ($columns as $target) {
				echo "\n<div class='references' title='" . h($target_name) . "' id='refd$left-" . ($i++) . "'"
					. " style='left: $left1" . "em; top: " . $table["fields"][$target]["pos"] . "em; height: 1.25em; background: url(" . h(preg_replace("~\\?.*~", "", ME) . "?file=arrow.gif) no-repeat right center;&version=5.1.0") . "'>"
					. "<div style='height: .5em; border-bottom: 1px solid gray; width: " . (-$left1) . "em;'></div>"
					. "</div>"
				;
			}
		}
	}

	echo "\n</div>\n";
}

foreach ($schema as $name => $table) {
	foreach ((array) $table["references"] as $target_name => $refs) {
		foreach ($refs as $left => $ref) {
			$min_pos = $top;
			$max_pos = -10;
			foreach ($ref[0] as $key => $source) {
				$pos1 = $table["pos"][0] + $table["fields"][$source]["pos"];
				$pos2 = $schema[$target_name]["pos"][0] + $schema[$target_name]["fields"][$ref[1][$key]]["pos"];
				$min_pos = min($min_pos, $pos1, $pos2);
				$max_pos = max($max_pos, $pos1, $pos2);
			}
			echo "<div class='references' id='refl$left' style='left: $left" . "em; top: $min_pos" . "em; padding: .5em 0;'><div style='border-right: 1px solid gray; margin-top: 1px; height: " . ($max_pos - $min_pos) . "em;'></div></div>\n";
		}
	}
}
?>
</div>
<p class="links"><a href="<?php echo h(ME . "schema=" . urlencode($SCHEMA)); ?>" id="schema-link"><?php echo lang(144); ?></a>
<?php
} elseif (isset($_GET["dump"])) {
	?>
<?php
$TABLE = $_GET["dump"];

if ($_POST && !$error) {
	save_settings(
		array_intersect_key($_POST, array_flip(array("output", "format", "db_style", "types", "routines", "events", "table_style", "auto_increment", "triggers", "data_style"))),
		"adminer_export"
	);
	$tables = array_flip((array) $_POST["tables"]) + array_flip((array) $_POST["data"]);
	$ext = dump_headers(
		(count($tables) == 1 ? key($tables) : DB),
		(DB == "" || count($tables) > 1)
	);
	$is_sql = preg_match('~sql~', $_POST["format"]);

	if ($is_sql) {
		echo "-- Adminer $VERSION " . $drivers[DRIVER] . " " . str_replace("\n", " ", $connection->server_info) . " dump\n\n";
		if (JUSH == "sql") {
			echo "SET NAMES utf8;
SET time_zone = '+00:00';
SET foreign_key_checks = 0;
" . ($_POST["data_style"] ? "SET sql_mode = 'NO_AUTO_VALUE_ON_ZERO';
" : "") . "
";
			$connection->query("SET time_zone = '+00:00'");
			$connection->query("SET sql_mode = ''");
		}
	}

	$style = $_POST["db_style"];
	$databases = array(DB);
	if (DB == "") {
		$databases = $_POST["databases"];
		if (is_string($databases)) {
			$databases = explode("\n", rtrim(str_replace("\r", "", $databases), "\n"));
		}
	}

	foreach ((array) $databases as $db) {
		$adminer->dumpDatabase($db);
		if ($connection->select_db($db)) {
			if ($is_sql && preg_match('~CREATE~', $style) && ($create = get_val("SHOW CREATE DATABASE " . idf_escape($db), 1))) {
				set_utf8mb4($create);
				if ($style == "DROP+CREATE") {
					echo "DROP DATABASE IF EXISTS " . idf_escape($db) . ";\n";
				}
				echo "$create;\n";
			}
			if ($is_sql) {
				if ($style) {
					echo use_sql($db) . ";\n\n";
				}
				$out = "";

				if ($_POST["types"]) {
					foreach (types() as $id => $type) {
						$enums = type_values($id);
						if ($enums) {
							$out .= ($style != 'DROP+CREATE' ? "DROP TYPE IF EXISTS " . idf_escape($type) . ";;\n" : "") . "CREATE TYPE " . idf_escape($type) . " AS ENUM ($enums);\n\n";
						} else {
							//! https://github.com/postgres/postgres/blob/REL_17_4/src/bin/pg_dump/pg_dump.c#L10846
							$out .= "-- Could not export type $type\n\n";
						}
					}
				}

				if ($_POST["routines"]) {
					foreach (routines() as $row) {
						$name = $row["ROUTINE_NAME"];
						$routine = $row["ROUTINE_TYPE"];
						$create = create_routine($routine, array("name" => $name) + routine($row["SPECIFIC_NAME"], $routine));
						set_utf8mb4($create);
						$out .= ($style != 'DROP+CREATE' ? "DROP $routine IF EXISTS " . idf_escape($name) . ";;\n" : "") . "$create;\n\n";
					}
				}

				if ($_POST["events"]) {
					foreach (get_rows("SHOW EVENTS", null, "-- ") as $row) {
						$create = remove_definer(get_val("SHOW CREATE EVENT " . idf_escape($row["Name"]), 3));
						set_utf8mb4($create);
						$out .= ($style != 'DROP+CREATE' ? "DROP EVENT IF EXISTS " . idf_escape($row["Name"]) . ";;\n" : "") . "$create;;\n\n";
					}
				}

				echo ($out && JUSH == 'sql' ? "DELIMITER ;;\n\n$out" . "DELIMITER ;\n\n" : $out);
			}

			if ($_POST["table_style"] || $_POST["data_style"]) {
				$views = array();
				foreach (table_status('', true) as $name => $table_status) {
					$table = (DB == "" || in_array($name, (array) $_POST["tables"]));
					$data = (DB == "" || in_array($name, (array) $_POST["data"]));
					if ($table || $data) {
						if ($ext == "tar") {
							$tmp_file = new TmpFile;
							ob_start(array($tmp_file, 'write'), 1e5);
						}

						$adminer->dumpTable($name, ($table ? $_POST["table_style"] : ""), (is_view($table_status) ? 2 : 0));
						if (is_view($table_status)) {
							$views[] = $name;
						} elseif ($data) {
							$fields = fields($name);
							$adminer->dumpData($name, $_POST["data_style"], "SELECT *" . convert_fields($fields, $fields) . " FROM " . table($name));
						}
						if ($is_sql && $_POST["triggers"] && $table && ($triggers = trigger_sql($name))) {
							echo "\nDELIMITER ;;\n$triggers\nDELIMITER ;\n";
						}

						if ($ext == "tar") {
							ob_end_flush();
							tar_file((DB != "" ? "" : "$db/") . "$name.csv", $tmp_file);
						} elseif ($is_sql) {
							echo "\n";
						}
					}
				}

				// add FKs after creating tables (except in MySQL which uses SET FOREIGN_KEY_CHECKS=0)
				if (function_exists('Adminer\foreign_keys_sql')) {
					foreach (table_status('', true) as $name => $table_status) {
						$table = (DB == "" || in_array($name, (array) $_POST["tables"]));
						if ($table && !is_view($table_status)) {
							echo foreign_keys_sql($name);
						}
					}
				}

				foreach ($views as $view) {
					$adminer->dumpTable($view, $_POST["table_style"], 1);
				}

				if ($ext == "tar") {
					echo pack("x512");
				}
			}
		}
	}

	$adminer->dumpFooter();
	exit;
}

page_header(lang(72), $error, ($_GET["export"] != "" ? array("table" => $_GET["export"]) : array()), h(DB));
?>

<form action="" method="post">
<table class="layout">
<?php
$db_style = array('', 'USE', 'DROP+CREATE', 'CREATE');
$table_style = array('', 'DROP+CREATE', 'CREATE');
$data_style = array('', 'TRUNCATE+INSERT', 'INSERT');
if (JUSH == "sql") { //! use insertUpdate() in all drivers
	$data_style[] = 'INSERT+UPDATE';
}
$row = get_settings("adminer_export");
if (!$row) {
	$row = array("output" => "text", "format" => "sql", "db_style" => (DB != "" ? "" : "CREATE"), "table_style" => "DROP+CREATE", "data_style" => "INSERT");
}
if (!isset($row["events"])) { // backwards compatibility
	$row["routines"] = $row["events"] = ($_GET["dump"] == "");
	$row["triggers"] = $row["table_style"];
}

echo "<tr><th>" . lang(145) . "<td>" . html_radios("output", $adminer->dumpOutput(), $row["output"]) . "\n";

echo "<tr><th>" . lang(146) . "<td>" . html_radios("format", $adminer->dumpFormat(), $row["format"]) . "\n";

echo (JUSH == "sqlite" ? "" : "<tr><th>" . lang(36) . "<td>" . html_select('db_style', $db_style, $row["db_style"])
	. (support("type") ? checkbox("types", 1, $row["types"], lang(31)) : "")
	. (support("routine") ? checkbox("routines", 1, $row["routines"], lang(147)) : "")
	. (support("event") ? checkbox("events", 1, $row["events"], lang(148)) : "")
);

echo "<tr><th>" . lang(125) . "<td>" . html_select('table_style', $table_style, $row["table_style"])
	. checkbox("auto_increment", 1, $row["auto_increment"], lang(50))
	. (support("trigger") ? checkbox("triggers", 1, $row["triggers"], lang(142)) : "")
;

echo "<tr><th>" . lang(149) . "<td>" . html_select('data_style', $data_style, $row["data_style"]);
?>
</table>
<p><input type="submit" value="<?php echo lang(72); ?>">
<?php echo input_token(); ?>

<table>
<?php
echo script("qsl('table').onclick = dumpClick;");
$prefixes = array();
if (DB != "") {
	$checked = ($TABLE != "" ? "" : " checked");
	echo "<thead><tr>";
	echo "<th style='text-align: left;'><label class='block'><input type='checkbox' id='check-tables'$checked>" . lang(125) . "</label>" . script("qs('#check-tables').onclick = partial(formCheck, /^tables\\[/);", "");
	echo "<th style='text-align: right;'><label class='block'>" . lang(149) . "<input type='checkbox' id='check-data'$checked></label>" . script("qs('#check-data').onclick = partial(formCheck, /^data\\[/);", "");
	echo "</thead>\n";

	$views = "";
	$tables_list = tables_list();
	foreach ($tables_list as $name => $type) {
		$prefix = preg_replace('~_.*~', '', $name);
		$checked = ($TABLE == "" || $TABLE == (substr($TABLE, -1) == "%" ? "$prefix%" : $name)); //! % may be part of table name
		$print = "<tr><td>" . checkbox("tables[]", $name, $checked, $name, "", "block");
		if ($type !== null && !preg_match('~table~i', $type)) {
			$views .= "$print\n";
		} else {
			echo "$print<td align='right'><label class='block'><span id='Rows-" . h($name) . "'></span>" . checkbox("data[]", $name, $checked) . "</label>\n";
		}
		$prefixes[$prefix]++;
	}
	echo $views;

	if ($tables_list) {
		echo script("ajaxSetHtml('" . js_escape(ME) . "script=db');");
	}

} else {
	echo "<thead><tr><th style='text-align: left;'>";
	echo "<label class='block'><input type='checkbox' id='check-databases'" . ($TABLE == "" ? " checked" : "") . ">" . lang(36) . "</label>";
	echo script("qs('#check-databases').onclick = partial(formCheck, /^databases\\[/);", "");
	echo "</thead>\n";
	$databases = $adminer->databases();
	if ($databases) {
		foreach ($databases as $db) {
			if (!information_schema($db)) {
				$prefix = preg_replace('~_.*~', '', $db);
				echo "<tr><td>" . checkbox("databases[]", $db, $TABLE == "" || $TABLE == "$prefix%", $db, "", "block") . "\n";
				$prefixes[$prefix]++;
			}
		}
	} else {
		echo "<tr><td><textarea name='databases' rows='10' cols='20'></textarea>";
	}
}
?>
</table>
</form>
<?php
$first = true;
foreach ($prefixes as $key => $val) {
	if ($key != "" && $val > 1) {
		echo ($first ? "<p>" : " ") . "<a href='" . h(ME) . "dump=" . urlencode("$key%") . "'>" . h($key) . "</a>";
		$first = false;
	}
}

} elseif (isset($_GET["privileges"])) {
	?>
<?php
page_header(lang(70));

echo '<p class="links"><a href="' . h(ME) . 'user=">' . lang(150) . "</a>";

$result = $connection->query("SELECT User, Host FROM mysql." . (DB == "" ? "user" : "db WHERE " . q(DB) . " LIKE Db") . " ORDER BY Host, User");
$grant = $result;
if (!$result) {
	// list logged user, information_schema.USER_PRIVILEGES lists just the current user too
	$result = $connection->query("SELECT SUBSTRING_INDEX(CURRENT_USER, '@', 1) AS User, SUBSTRING_INDEX(CURRENT_USER, '@', -1) AS Host");
}

echo "<form action=''><p>\n";
hidden_fields_get();
echo input_hidden("db", DB);
echo ($grant ? "" : input_hidden("grant"));
echo "<table class='odds'>\n";
echo "<thead><tr><th>" . lang(34) . "<th>" . lang(33) . "<th></thead>\n";

while ($row = $result->fetch_assoc()) {
	echo '<tr><td>' . h($row["User"]) . "<td>" . h($row["Host"]) . '<td><a href="' . h(ME . 'user=' . urlencode($row["User"]) . '&host=' . urlencode($row["Host"])) . '">' . lang(10) . "</a>\n";
}

if (!$grant || DB != "") {
	echo "<tr><td><input name='user' autocapitalize='off'><td><input name='host' value='localhost' autocapitalize='off'><td><input type='submit' value='" . lang(10) . "'>\n";
}

echo "</table>\n";
echo "</form>\n";

} elseif (isset($_GET["sql"])) {
	?>
<?php
if (!$error && $_POST["export"]) {
	save_settings(array("output" => $_POST["output"], "format" => $_POST["format"]), "adminer_import");
	dump_headers("sql");
	$adminer->dumpTable("", "");
	$adminer->dumpData("", "table", $_POST["query"]);
	$adminer->dumpFooter();
	exit;
}

restart_session();
$history_all = &get_session("queries");
$history = &$history_all[DB];
if (!$error && $_POST["clear"]) {
	$history = array();
	redirect(remove_from_uri("history"));
}

page_header((isset($_GET["import"]) ? lang(71) : lang(63)), $error);

if (!$error && $_POST) {
	$fp = false;
	if (!isset($_GET["import"])) {
		$query = $_POST["query"];
	} elseif ($_POST["webfile"]) {
		$sql_file_path = $adminer->importServerPath();
		$fp = @fopen((file_exists($sql_file_path)
			? $sql_file_path
			: "compress.zlib://$sql_file_path.gz"
		), "rb");
		$query = ($fp ? fread($fp, 1e6) : false);
	} else {
		$query = get_file("sql_file", true, ";");
	}

	if (is_string($query)) { // get_file() returns error as number, fread() as false
		if (function_exists('memory_get_usage') && ($memory_limit = ini_bytes("memory_limit")) != "-1") {
			@ini_set("memory_limit", max($memory_limit, 2 * strlen($query) + memory_get_usage() + 8e6)); // @ - may be disabled, 2 - substr and trim, 8e6 - other variables
		}

		if ($query != "" && strlen($query) < 1e6) { // don't add big queries
			$q = $query . (preg_match("~;[ \t\r\n]*\$~", $query) ? "" : ";"); //! doesn't work with DELIMITER |
			if (!$history || first(end($history)) != $q) { // no repeated queries
				restart_session();
				$history[] = array($q, time()); //! add elapsed time
				set_session("queries", $history_all); // required because reference is unlinked by stop_session()
				stop_session();
			}
		}

		$space = "(?:\\s|/\\*[\s\S]*?\\*/|(?:#|-- )[^\n]*\n?|--\r?\n)";
		$delimiter = ";";
		$offset = 0;
		$empty = true;
		$connection2 = connect($adminer->credentials()); // connection for exploring indexes and EXPLAIN (to not replace FOUND_ROWS()) //! PDO - silent error
		if (is_object($connection2) && DB != "") {
			$connection2->select_db(DB);
			if ($_GET["ns"] != "") {
				set_schema($_GET["ns"], $connection2);
			}
		}
		$commands = 0;
		$errors = array();
		$parse = '[\'"' . (JUSH == "sql" ? '`#' : (JUSH == "sqlite" ? '`[' : (JUSH == "mssql" ? '[' : ''))) . ']|/\*|-- |$' . (JUSH == "pgsql" ? '|\$[^$]*\$' : '');
		$total_start = microtime(true);
		$adminer_export = get_settings("adminer_import"); // this doesn't offer SQL export so we match the import/export style at select
		$dump_format = $adminer->dumpFormat();
		unset($dump_format["sql"]);

		while ($query != "") {
			if (!$offset && preg_match("~^$space*+DELIMITER\\s+(\\S+)~i", $query, $match)) {
				$delimiter = $match[1];
				$query = substr($query, strlen($match[0]));
			} else {
				preg_match('(' . preg_quote($delimiter) . "\\s*|$parse)", $query, $match, PREG_OFFSET_CAPTURE, $offset); // should always match
				list($found, $pos) = $match[0];
				if (!$found && $fp && !feof($fp)) {
					$query .= fread($fp, 1e5);
				} else {
					if (!$found && rtrim($query) == "") {
						break;
					}
					$offset = $pos + strlen($found);

					if ($found && rtrim($found) != $delimiter) { // find matching quote or comment end
						$c_style_escapes = $driver->hasCStyleEscapes() || (JUSH == "pgsql" && ($pos > 0 && strtolower($query[$pos - 1]) == "e"));

						$pattern = ($found == '/*' ? '\*/'
							: ($found == '[' ? ']'
							: (preg_match('~^-- |^#~', $found) ? "\n"
							: preg_quote($found) . ($c_style_escapes ? "|\\\\." : "")))
						);

						while (preg_match("($pattern|\$)s", $query, $match, PREG_OFFSET_CAPTURE, $offset)) {
							$s = $match[0][0];
							if (!$s && $fp && !feof($fp)) {
								$query .= fread($fp, 1e5);
							} else {
								$offset = $match[0][1] + strlen($s);
								if (!$s || $s[0] != "\\") {
									break;
								}
							}
						}

					} else { // end of a query
						$empty = false;
						$q = substr($query, 0, $pos);
						$commands++;
						$print = "<pre id='sql-$commands'><code class='jush-" . JUSH . "'>" . $adminer->sqlCommandQuery($q) . "</code></pre>\n";
						if (JUSH == "sqlite" && preg_match("~^$space*+ATTACH\\b~i", $q, $match)) {
							// PHP doesn't support setting SQLITE_LIMIT_ATTACHED
							echo $print;
							echo "<p class='error'>" . lang(151) . "\n";
							$errors[] = " <a href='#sql-$commands'>$commands</a>";
							if ($_POST["error_stops"]) {
								break;
							}
						} else {
							if (!$_POST["only_errors"]) {
								echo $print;
								ob_flush();
								flush(); // can take a long time - show the running query
							}
							$start = microtime(true);
							//! don't allow changing of character_set_results, convert encoding of displayed query
							if ($connection->multi_query($q) && is_object($connection2) && preg_match("~^$space*+USE\\b~i", $q)) {
								$connection2->query($q);
							}

							do {
								$result = $connection->store_result();

								if ($connection->error) {
									echo ($_POST["only_errors"] ? $print : "");
									echo "<p class='error'>" . lang(152) . ($connection->errno ? " ($connection->errno)" : "") . ": " . error() . "\n";
									$errors[] = " <a href='#sql-$commands'>$commands</a>";
									if ($_POST["error_stops"]) {
										break 2;
									}

								} else {
									$time = " <span class='time'>(" . format_time($start) . ")</span>"
										. (strlen($q) < 1000 ? " <a href='" . h(ME) . "sql=" . urlencode(trim($q)) . "'>" . lang(10) . "</a>" : "") // 1000 - maximum length of encoded URL in IE is 2083 characters
									;
									$affected = $connection->affected_rows; // getting warnings overwrites this
									$warnings = ($_POST["only_errors"] ? "" : $driver->warnings());
									$warnings_id = "warnings-$commands";
									if ($warnings) {
										$time .= ", <a href='#$warnings_id'>" . lang(45) . "</a>" . script("qsl('a').onclick = partial(toggle, '$warnings_id');", "");
									}
									$explain = null;
									$explain_id = "explain-$commands";
									if (is_object($result)) {
										$limit = $_POST["limit"];
										$orgtables = select($result, $connection2, array(), $limit);
										if (!$_POST["only_errors"]) {
											echo "<form action='' method='post'>\n";
											$num_rows = $result->num_rows;
											echo "<p class='sql-footer'>" . ($num_rows ? ($limit && $num_rows > $limit ? lang(153, $limit) : "") . lang(154, $num_rows) : "");
											echo $time;
											if ($connection2 && preg_match("~^($space|\\()*+SELECT\\b~i", $q) && ($explain = explain($connection2, $q))) {
												echo ", <a href='#$explain_id'>Explain</a>" . script("qsl('a').onclick = partial(toggle, '$explain_id');", "");
											}
											$id = "export-$commands";
											echo ", <a href='#$id'>" . lang(72) . "</a>" . script("qsl('a').onclick = partial(toggle, '$id');", "") . "<span id='$id' class='hidden'>: "
												. html_select("output", $adminer->dumpOutput(), $adminer_export["output"]) . " "
												. html_select("format", $dump_format, $adminer_export["format"])
												. input_hidden("query", $q)
												. "<input type='submit' name='export' value='" . lang(72) . "'>" . input_token() . "</span>\n"
												. "</form>\n"
											;
										}

									} else {
										if (preg_match("~^$space*+(CREATE|DROP|ALTER)$space++(DATABASE|SCHEMA)\\b~i", $q)) {
											restart_session();
											set_session("dbs", null); // clear cache
											stop_session();
										}
										if (!$_POST["only_errors"]) {
											echo "<p class='message' title='" . h(isset($connection->info) ? $connection->info : "") . "'>" . lang(155, $affected) . "$time\n";
										}
									}
									echo ($warnings ? "<div id='$warnings_id' class='hidden'>\n$warnings</div>\n" : "");
									if ($explain) {
										echo "<div id='$explain_id' class='hidden explain'>\n";
										select($explain, $connection2, $orgtables);
										echo "</div>\n";
									}
								}

								$start = microtime(true);
							} while ($connection->next_result());
						}

						$query = substr($query, $offset);
						$offset = 0;
					}

				}
			}
		}

		if ($empty) {
			echo "<p class='message'>" . lang(156) . "\n";
		} elseif ($_POST["only_errors"]) {
			echo "<p class='message'>" . lang(157, $commands - count($errors));
			echo " <span class='time'>(" . format_time($total_start) . ")</span>\n";
		} elseif ($errors && $commands > 1) {
			echo "<p class='error'>" . lang(152) . ": " . implode("", $errors) . "\n";
		}
		//! MS SQL - SET SHOWPLAN_ALL OFF

	} else {
		echo "<p class='error'>" . upload_error($query) . "\n";
	}
}
?>

<form action="" method="post" enctype="multipart/form-data" id="form">
<?php
$execute = "<input type='submit' value='" . lang(158) . "' title='Ctrl+Enter'>";
if (!isset($_GET["import"])) {
	$q = $_GET["sql"]; // overwrite $q from if ($_POST) to save memory
	if ($_POST) {
		$q = $_POST["query"];
	} elseif ($_GET["history"] == "all") {
		$q = $history;
	} elseif ($_GET["history"] != "") {
		$q = $history[$_GET["history"]][0];
	}
	echo "<p>";
	textarea("query", $q, 20);
	echo script(($_POST ? "" : "qs('textarea').focus();\n") . "qs('#form').onsubmit = partial(sqlSubmit, qs('#form'), '" . js_escape(remove_from_uri("sql|limit|error_stops|only_errors|history")) . "');");
	echo "<p>";
	$adminer->sqlPrintAfter();
	echo "$execute\n";
	echo lang(159) . ": <input type='number' name='limit' class='size' value='" . h($_POST ? $_POST["limit"] : $_GET["limit"]) . "'>\n";

} else {
	echo "<fieldset><legend>" . lang(160) . "</legend><div>";
	$gz = (extension_loaded("zlib") ? "[.gz]" : "");
	echo (ini_bool("file_uploads")
		? "SQL$gz (&lt; " . ini_get("upload_max_filesize") . "B): <input type='file' name='sql_file[]' multiple>\n$execute" // ignore post_max_size because it is for all form fields together and bytes computing would be necessary
		: lang(161)
	);
	echo "</div></fieldset>\n";
	$importServerPath = $adminer->importServerPath();
	if ($importServerPath) {
		echo "<fieldset><legend>" . lang(162) . "</legend><div>";
		echo lang(163, "<code>" . h($importServerPath) . "$gz</code>");
		echo ' <input type="submit" name="webfile" value="' . lang(164) . '">';
		echo "</div></fieldset>\n";
	}
	echo "<p>";
}

echo checkbox("error_stops", 1, ($_POST ? $_POST["error_stops"] : isset($_GET["import"]) || $_GET["error_stops"]), lang(165)) . "\n";
echo checkbox("only_errors", 1, ($_POST ? $_POST["only_errors"] : isset($_GET["import"]) || $_GET["only_errors"]), lang(166)) . "\n";
echo input_token();

if (!isset($_GET["import"]) && $history) {
	print_fieldset("history", lang(167), $_GET["history"] != "");
	for ($val = end($history); $val; $val = prev($history)) { // not array_reverse() to save memory
		$key = key($history);
		list($q, $time, $elapsed) = $val;
		echo '<a href="' . h(ME . "sql=&history=$key") . '">' . lang(10) . "</a>"
			. " <span class='time' title='" . @date('Y-m-d', $time) . "'>" . @date("H:i:s", $time) . "</span>" // @ - time zone may be not set
			. " <code class='jush-" . JUSH . "'>" . shorten_utf8(ltrim(str_replace("\n", " ", str_replace("\r", "", preg_replace('~^(#|-- ).*~m', '', $q)))), 80, "</code>")
			. ($elapsed ? " <span class='time'>($elapsed)</span>" : "")
			. "<br>\n"
		;
	}
	echo "<input type='submit' name='clear' value='" . lang(168) . "'>\n";
	echo "<a href='" . h(ME . "sql=&history=all") . "'>" . lang(169) . "</a>\n";
	echo "</div></fieldset>\n";
}
?>
</form>
<?php
} elseif (isset($_GET["edit"])) {
	?>
<?php
$TABLE = $_GET["edit"];
$fields = fields($TABLE);
$where = (isset($_GET["select"]) ? ($_POST["check"] && count($_POST["check"]) == 1 ? where_check($_POST["check"][0], $fields) : "") : where($_GET, $fields));
$update = (isset($_GET["select"]) ? $_POST["edit"] : $where);
foreach ($fields as $name => $field) {
	if (!isset($field["privileges"][$update ? "update" : "insert"]) || $adminer->fieldName($field) == "" || $field["generated"]) {
		unset($fields[$name]);
	}
}

if ($_POST && !$error && !isset($_GET["select"])) {
	$location = $_POST["referer"];
	if ($_POST["insert"]) { // continue edit or insert
		$location = ($update ? null : $_SERVER["REQUEST_URI"]);
	} elseif (!preg_match('~^.+&select=.+$~', $location)) {
		$location = ME . "select=" . urlencode($TABLE);
	}

	$indexes = indexes($TABLE);
	$unique_array = unique_array($_GET["where"], $indexes);
	$query_where = "\nWHERE $where";

	if (isset($_POST["delete"])) {
		queries_redirect(
			$location,
			lang(170),
			$driver->delete($TABLE, $query_where, !$unique_array)
		);

	} else {
		$set = array();
		foreach ($fields as $name => $field) {
			$val = process_input($field);
			if ($val !== false && $val !== null) {
				$set[idf_escape($name)] = $val;
			}
		}

		if ($update) {
			if (!$set) {
				redirect($location);
			}
			queries_redirect(
				$location,
				lang(171),
				$driver->update($TABLE, $set, $query_where, !$unique_array)
			);
			if (is_ajax()) {
				page_headers();
				page_messages($error);
				exit;
			}
		} else {
			$result = $driver->insert($TABLE, $set);
			$last_id = ($result ? last_id($result) : 0);
			queries_redirect($location, lang(172, ($last_id ? " $last_id" : "")), $result); //! link
		}
	}
}

$row = null;
if ($_POST["save"]) {
	$row = (array) $_POST["fields"];
} elseif ($where) {
	$select = array();
	foreach ($fields as $name => $field) {
		if (isset($field["privileges"]["select"])) {
			$as = ($_POST["clone"] && $field["auto_increment"] ? "''" : convert_field($field));
			$select[] = ($as ? "$as AS " : "") . idf_escape($name);
		}
	}
	$row = array();
	if (!support("table")) {
		$select = array("*");
	}
	if ($select) {
		$result = $driver->select($TABLE, $select, array($where), $select, array(), (isset($_GET["select"]) ? 2 : 1));
		if (!$result) {
			$error = error();
		} else {
			$row = $result->fetch_assoc();
			if (!$row) { // MySQLi returns null
				$row = false;
			}
		}
		if (isset($_GET["select"]) && (!$row || $result->fetch_assoc())) { // $result->num_rows != 1 isn't available in all drivers
			$row = null;
		}
	}
}

if (!support("table") && !$fields) {
	if (!$where) { // insert
		$result = $driver->select($TABLE, array("*"), $where, array("*"));
		$row = ($result ? $result->fetch_assoc() : false);
		if (!$row) {
			$row = array($driver->primary => "");
		}
	}
	if ($row) {
		foreach ($row as $key => $val) {
			if (!$where) {
				$row[$key] = null;
			}
			$fields[$key] = array("field" => $key, "null" => ($key != $driver->primary), "auto_increment" => ($key == $driver->primary));
		}
	}
}

edit_form($TABLE, $fields, $row, $update);

} elseif (isset($_GET["create"])) {
	?>
<?php
$TABLE = $_GET["create"];
$partition_by = array();
foreach (array('HASH', 'LINEAR HASH', 'KEY', 'LINEAR KEY', 'RANGE', 'LIST') as $key) {
	$partition_by[$key] = $key;
}

$referencable_primary = referencable_primary($TABLE);
$foreign_keys = array();
foreach ($referencable_primary as $table_name => $field) {
	$foreign_keys[str_replace("`", "``", $table_name) . "`" . str_replace("`", "``", $field["field"])] = $table_name; // not idf_escape() - used in JS
}

$orig_fields = array();
$table_status = array();
if ($TABLE != "") {
	$orig_fields = fields($TABLE);
	$table_status = table_status($TABLE);
	if (!$table_status) {
		$error = lang(9);
	}
}

$row = $_POST;
$row["fields"] = (array) $row["fields"];
if ($row["auto_increment_col"]) {
	$row["fields"][$row["auto_increment_col"]]["auto_increment"] = true;
}

if ($_POST) {
	save_settings(array("comments" => $_POST["comments"], "defaults" => $_POST["defaults"]));
}

if ($_POST && !process_fields($row["fields"]) && !$error) {
	if ($_POST["drop"]) {
		queries_redirect(substr(ME, 0, -1), lang(173), drop_tables(array($TABLE)));
	} else {
		$fields = array();
		$all_fields = array();
		$use_all_fields = false;
		$foreign = array();
		$orig_field = reset($orig_fields);
		$after = " FIRST";

		foreach ($row["fields"] as $key => $field) {
			$foreign_key = $foreign_keys[$field["type"]];
			$type_field = ($foreign_key !== null ? $referencable_primary[$foreign_key] : $field); //! can collide with user defined type
			if ($field["field"] != "") {
				if (!$field["generated"]) {
					$field["default"] = null;
				}
				$process_field = process_field($field, $type_field);
				$all_fields[] = array($field["orig"], $process_field, $after);
				if (!$orig_field || $process_field !== process_field($orig_field, $orig_field)) {
					$fields[] = array($field["orig"], $process_field, $after);
					if ($field["orig"] != "" || $after) {
						$use_all_fields = true;
					}
				}
				if ($foreign_key !== null) {
					$foreign[idf_escape($field["field"])] = ($TABLE != "" && JUSH != "sqlite" ? "ADD" : " ") . format_foreign_key(array(
						'table' => $foreign_keys[$field["type"]],
						'source' => array($field["field"]),
						'target' => array($type_field["field"]),
						'on_delete' => $field["on_delete"],
					));
				}
				$after = " AFTER " . idf_escape($field["field"]);
			} elseif ($field["orig"] != "") {
				$use_all_fields = true;
				$fields[] = array($field["orig"]);
			}
			if ($field["orig"] != "") {
				$orig_field = next($orig_fields);
				if (!$orig_field) {
					$after = "";
				}
			}
		}

		$partitioning = "";
		if (support("partitioning")) {
			if (isset($partition_by[$row["partition_by"]])) {
				$params = array();
				foreach ($row as $key => $val) {
					if (preg_match('~^partition~', $key)) {
						$params[$key] = $val;
					}
				}
				foreach ($params["partition_names"] as $key => $name) {
					if ($name == "") {
						unset($params["partition_names"][$key]);
						unset($params["partition_values"][$key]);
					}
				}
				if ($params != get_partitions_info($TABLE)) {
					$partitions = array();
					if ($params["partition_by"] == 'RANGE' || $params["partition_by"] == 'LIST') {
						foreach ($params["partition_names"] as $key => $name) {
							$value = $params["partition_values"][$key];
							$partitions[] = "\n  PARTITION " . idf_escape($name) . " VALUES " . ($params["partition_by"] == 'RANGE' ? "LESS THAN" : "IN") . ($value != "" ? " ($value)" : " MAXVALUE"); //! SQL injection
						}
					}
					// $params["partition"] can be expression, not only column
					$partitioning .= "\nPARTITION BY $params[partition_by]($params[partition])";
					if ($partitions) {
						$partitioning .= " (" . implode(",", $partitions) . "\n)";
					} elseif ($params["partitions"]) {
						$partitioning .= " PARTITIONS " . (+$params["partitions"]);
					}
				}
			} elseif (preg_match("~partitioned~", $table_status["Create_options"])) {
				$partitioning .= "\nREMOVE PARTITIONING";
			}
		}

		$message = lang(174);
		if ($TABLE == "") {
			cookie("adminer_engine", $row["Engine"]);
			$message = lang(175);
		}
		$name = trim($row["name"]);

		queries_redirect(ME . (support("table") ? "table=" : "select=") . urlencode($name), $message, alter_table(
			$TABLE,
			$name,
			(JUSH == "sqlite" && ($use_all_fields || $foreign) ? $all_fields : $fields),
			$foreign,
			($row["Comment"] != $table_status["Comment"] ? $row["Comment"] : null),
			($row["Engine"] && $row["Engine"] != $table_status["Engine"] ? $row["Engine"] : ""),
			($row["Collation"] && $row["Collation"] != $table_status["Collation"] ? $row["Collation"] : ""),
			($row["Auto_increment"] != "" ? number($row["Auto_increment"]) : ""),
			$partitioning
		));
	}
}

page_header(($TABLE != "" ? lang(43) : lang(73)), $error, array("table" => $TABLE), h($TABLE));

if (!$_POST) {
	$types = $driver->types();
	$row = array(
		"Engine" => $_COOKIE["adminer_engine"],
		"fields" => array(array("field" => "", "type" => (isset($types["int"]) ? "int" : (isset($types["integer"]) ? "integer" : "")), "on_update" => "")),
		"partition_names" => array(""),
	);

	if ($TABLE != "") {
		$row = $table_status;
		$row["name"] = $TABLE;
		$row["fields"] = array();
		if (!$_GET["auto_increment"]) { // don't prefill by original Auto_increment for the sake of performance and not reusing deleted ids
			$row["Auto_increment"] = "";
		}
		foreach ($orig_fields as $field) {
			$field["generated"] = $field["generated"] ?: (isset($field["default"]) ? "DEFAULT" : "");
			$row["fields"][] = $field;
		}

		if (support("partitioning")) {
			$row += get_partitions_info($TABLE);
			$row["partition_names"][] = "";
			$row["partition_values"][] = "";
		}
	}
}

$collations = collations();
$engines = $driver->engines();
// case of engine may differ
foreach ($engines as $engine) {
	if (!strcasecmp($engine, $row["Engine"])) {
		$row["Engine"] = $engine;
		break;
	}
}
?>

<form action="" method="post" id="form">
<p>
<?php
if (support("columns") || $TABLE == "") {
	echo lang(176) . ": <input name='name'" . ($TABLE == "" && !$_POST ? " autofocus" : "") . " data-maxlength='64' value='" . h($row["name"]) . "' autocapitalize='off'>\n";
	echo ($engines ? html_select("Engine", array("" => "(" . lang(177) . ")") + $engines, $row["Engine"]) . on_help("event.target.value", 1) . script("qsl('select').onchange = helpClose;") . "\n" : "");
	if ($collations) {
		echo "<datalist id='collations'>" . optionlist($collations) . "</datalist>";
		echo (preg_match("~sqlite|mssql~", JUSH) ? "" : "<input list='collations' name='Collation' value='" . h($row["Collation"]) . "' placeholder='(" . lang(102) . ")'>");
	}
	echo "<input type='submit' value='" . lang(14) . "'>\n";
}

if (support("columns")) {
	echo "<div class='scrollable'>\n";
	echo "<table id='edit-fields' class='nowrap'>\n";
	edit_fields($row["fields"], $collations, "TABLE", $foreign_keys);
	echo "</table>\n";
	echo script("editFields();");
	echo "</div>\n<p>\n";
	echo lang(50) . ": <input type='number' name='Auto_increment' class='size' value='" . h($row["Auto_increment"]) . "'>\n";
	echo checkbox("defaults", 1, ($_POST ? $_POST["defaults"] : get_setting("defaults")), lang(178), "columnShow(this.checked, 5)", "jsonly");
	$comments = ($_POST ? $_POST["comments"] : get_setting("comments"));
	echo (support("comment")
		? checkbox("comments", 1, $comments, lang(49), "editingCommentsClick(this, true);", "jsonly")
			. ' ' . (preg_match('~\n~', $row["Comment"])
				? "<textarea name='Comment' rows='2' cols='20'" . ($comments ? "" : " class='hidden'") . ">" . h($row["Comment"]) . "</textarea>"
				: '<input name="Comment" value="' . h($row["Comment"]) . '" data-maxlength="' . (min_version(5.5) ? 2048 : 60) . '"' . ($comments ? "" : " class='hidden'") . '>'
			)
		: '')
	;
	?>
<p>
<input type="submit" value="<?php echo lang(14); ?>">
<?php } ?>

<?php if ($TABLE != "") { ?>
<input type="submit" name="drop" value="<?php echo lang(129); ?>"><?php echo confirm(lang(179, $TABLE)); ?>
<?php } ?>
<?php
if (support("partitioning")) {
	$partition_table = preg_match('~RANGE|LIST~', $row["partition_by"]);
	print_fieldset("partition", lang(180), $row["partition_by"]);
	echo "<p>" . html_select("partition_by", array("" => "") + $partition_by, $row["partition_by"]) . on_help("event.target.value.replace(/./, 'PARTITION BY \$&')", 1) . script("qsl('select').onchange = partitionByChange;");
	echo "(<input name='partition' value='" . h($row["partition"]) . "'>)\n";
	echo lang(181) . ": <input type='number' name='partitions' class='size" . ($partition_table || !$row["partition_by"] ? " hidden" : "") . "' value='" . h($row["partitions"]) . "'>\n";
	echo "<table id='partition-table'" . ($partition_table ? "" : " class='hidden'") . ">\n";
	echo "<thead><tr><th>" . lang(182) . "<th>" . lang(183) . "</thead>\n";
	foreach ($row["partition_names"] as $key => $val) {
		echo '<tr>';
		echo '<td><input name="partition_names[]" value="' . h($val) . '" autocapitalize="off">';
		echo ($key == count($row["partition_names"]) - 1 ? script("qsl('input').oninput = partitionNameChange;") : '');
		echo '<td><input name="partition_values[]" value="' . h($row["partition_values"][$key]) . '">';
	}
	echo "</table>\n</div></fieldset>\n";
}
echo input_token();
?>
</form>
<?php
} elseif (isset($_GET["indexes"])) {
	?>
<?php
$TABLE = $_GET["indexes"];
$index_types = array("PRIMARY", "UNIQUE", "INDEX");
$table_status = table_status($TABLE, true);
if (preg_match('~MyISAM|M?aria' . (min_version(5.6, '10.0.5') ? '|InnoDB' : '') . '~i', $table_status["Engine"])) {
	$index_types[] = "FULLTEXT";
}
if (preg_match('~MyISAM|M?aria' . (min_version(5.7, '10.2.2') ? '|InnoDB' : '') . '~i', $table_status["Engine"])) {
	$index_types[] = "SPATIAL";
}
$indexes = indexes($TABLE);
$primary = array();
if (JUSH == "mongo") { // doesn't support primary key
	$primary = $indexes["_id_"];
	unset($index_types[0]);
	unset($indexes["_id_"]);
}
$row = $_POST;
if ($row) {
	save_settings(array("index_options" => $row["options"]));
}
if ($_POST && !$error && !$_POST["add"] && !$_POST["drop_col"]) {
	$alter = array();
	foreach ($row["indexes"] as $index) {
		$name = $index["name"];
		if (in_array($index["type"], $index_types)) {
			$columns = array();
			$lengths = array();
			$descs = array();
			$set = array();
			ksort($index["columns"]);
			foreach ($index["columns"] as $key => $column) {
				if ($column != "") {
					$length = $index["lengths"][$key];
					$desc = $index["descs"][$key];
					$set[] = idf_escape($column) . ($length ? "(" . (+$length) . ")" : "") . ($desc ? " DESC" : "");
					$columns[] = $column;
					$lengths[] = ($length ?: null);
					$descs[] = $desc;
				}
			}

			$existing = $indexes[$name];
			if ($existing) {
				ksort($existing["columns"]);
				ksort($existing["lengths"]);
				ksort($existing["descs"]);
				if (
					$index["type"] == $existing["type"]
					&& array_values($existing["columns"]) === $columns
					&& (!$existing["lengths"] || array_values($existing["lengths"]) === $lengths)
					&& array_values($existing["descs"]) === $descs
				) {
					// skip existing index
					unset($indexes[$name]);
					continue;
				}
			}
			if ($columns) {
				$alter[] = array($index["type"], $name, $set);
			}
		}
	}

	// drop removed indexes
	foreach ($indexes as $name => $existing) {
		$alter[] = array($existing["type"], $name, "DROP");
	}
	if (!$alter) {
		redirect(ME . "table=" . urlencode($TABLE));
	}
	queries_redirect(ME . "table=" . urlencode($TABLE), lang(184), alter_indexes($TABLE, $alter));
}

page_header(lang(134), $error, array("table" => $TABLE), h($TABLE));

$fields = array_keys(fields($TABLE));
if ($_POST["add"]) {
	foreach ($row["indexes"] as $key => $index) {
		if ($index["columns"][count($index["columns"])] != "") {
			$row["indexes"][$key]["columns"][] = "";
		}
	}
	$index = end($row["indexes"]);
	if ($index["type"] || array_filter($index["columns"], 'strlen')) {
		$row["indexes"][] = array("columns" => array(1 => ""));
	}
}
if (!$row) {
	foreach ($indexes as $key => $index) {
		$indexes[$key]["name"] = $key;
		$indexes[$key]["columns"][] = "";
	}
	$indexes[] = array("columns" => array(1 => ""));
	$row["indexes"] = $indexes;
}
$lengths = (JUSH == "sql" || JUSH == "mssql");
$show_options = ($_POST ? $_POST["options"] : get_setting("index_options"));
?>

<form action="" method="post">
<div class="scrollable">
<table class="nowrap">
<thead><tr>
<th id="label-type"><?php echo lang(185); ?>
<th><input type="submit" class="wayoff"><?php
echo lang(47) . ($lengths ? "<span class='idxopts" . ($show_options ? "" : " hidden") . "'> (" . lang(186) . ")</span>" : "");
if ($lengths || support("descidx")) {
	echo checkbox("options", 1, $show_options, lang(108), "indexOptionsShow(this.checked)", "jsonly") . "\n";
}
?>
<th id="label-name"><?php echo lang(187); ?>
<th><noscript><?php echo "<input type='image' class='icon' name='add[0]' src='" . h(preg_replace("~\\?.*~", "", ME) . "?file=plus.gif&version=5.1.0") . "' alt='+' title='" . lang(109) . "'>"; ?></noscript>
</thead>
<?php
if ($primary) {
	echo "<tr><td>PRIMARY<td>";
	foreach ($primary["columns"] as $key => $column) {
		echo select_input(" disabled", $fields, $column);
		echo "<label><input disabled type='checkbox'>" . lang(58) . "</label> ";
	}
	echo "<td><td>\n";
}
$j = 1;
foreach ($row["indexes"] as $index) {
	if (!$_POST["drop_col"] || $j != key($_POST["drop_col"])) {
		echo "<tr><td>" . html_select("indexes[$j][type]", array(-1 => "") + $index_types, $index["type"], ($j == count($row["indexes"]) ? "indexesAddRow.call(this);" : ""), "label-type");

		echo "<td>";
		ksort($index["columns"]);
		$i = 1;
		foreach ($index["columns"] as $key => $column) {
			echo "<span>" . select_input(
				" name='indexes[$j][columns][$i]' title='" . lang(47) . "'",
				($fields ? array_combine($fields, $fields) : $fields),
				$column,
				"partial(" . ($i == count($index["columns"]) ? "indexesAddColumn" : "indexesChangeColumn") . ", '" . js_escape(JUSH == "sql" ? "" : $_GET["indexes"] . "_") . "')"
			);
			echo "<span class='idxopts" . ($show_options ? "" : " hidden") . "'>";
			echo ($lengths ? "<input type='number' name='indexes[$j][lengths][$i]' class='size' value='" . h($index["lengths"][$key]) . "' title='" . lang(107) . "'>" : "");
			echo (support("descidx") ? checkbox("indexes[$j][descs][$i]", 1, $index["descs"][$key], lang(58)) : "");
			echo "</span> </span>";
			$i++;
		}

		echo "<td><input name='indexes[$j][name]' value='" . h($index["name"]) . "' autocapitalize='off' aria-labelledby='label-name'>\n";
		echo "<td><input type='image' class='icon' name='drop_col[$j]' src='" . h(preg_replace("~\\?.*~", "", ME) . "?file=cross.gif&version=5.1.0") . "' alt='x' title='" . lang(112) . "'>" . script("qsl('input').onclick = partial(editingRemoveRow, 'indexes\$1[type]');");
	}
	$j++;
}
?>
</table>
</div>
<p>
<input type="submit" value="<?php echo lang(14); ?>">
<?php echo input_token(); ?>
</form>
<?php
} elseif (isset($_GET["database"])) {
	?>
<?php
$row = $_POST;

if ($_POST && !$error && !isset($_POST["add_x"])) { // add is an image and PHP changes add.x to add_x
	$name = trim($row["name"]);
	if ($_POST["drop"]) {
		$_GET["db"] = ""; // to save in global history
		queries_redirect(remove_from_uri("db|database"), lang(188), drop_databases(array(DB)));
	} elseif (DB !== $name) {
		// create or rename database
		if (DB != "") {
			$_GET["db"] = $name;
			queries_redirect(preg_replace('~\bdb=[^&]*&~', '', ME) . "db=" . urlencode($name), lang(189), rename_database($name, $row["collation"]));
		} else {
			$databases = explode("\n", str_replace("\r", "", $name));
			$success = true;
			$last = "";
			foreach ($databases as $db) {
				if (count($databases) == 1 || $db != "") { // ignore empty lines but always try to create single database
					if (!create_database($db, $row["collation"])) {
						$success = false;
					}
					$last = $db;
				}
			}
			restart_session();
			set_session("dbs", null);
			queries_redirect(ME . "db=" . urlencode($last), lang(190), $success);
		}
	} else {
		// alter database
		if (!$row["collation"]) {
			redirect(substr(ME, 0, -1));
		}
		query_redirect("ALTER DATABASE " . idf_escape($name) . (preg_match('~^[a-z0-9_]+$~i', $row["collation"]) ? " COLLATE $row[collation]" : ""), substr(ME, 0, -1), lang(191));
	}
}

page_header(DB != "" ? lang(66) : lang(116), $error, array(), h(DB));

$collations = collations();
$name = DB;
if ($_POST) {
	$name = $row["name"];
} elseif (DB != "") {
	$row["collation"] = db_collation(DB, $collations);
} elseif (JUSH == "sql") {
	// propose database name with limited privileges
	foreach (get_vals("SHOW GRANTS") as $grant) {
		if (preg_match('~ ON (`(([^\\\\`]|``|\\\\.)*)%`\.\*)?~', $grant, $match) && $match[1]) {
			$name = stripcslashes(idf_unescape("`$match[2]`"));
			break;
		}
	}
}
?>

<form action="" method="post">
<p>
<?php
echo ($_POST["add_x"] || strpos($name, "\n")
	? '<textarea autofocus name="name" rows="10" cols="40">' . h($name) . '</textarea><br>'
	: '<input name="name" autofocus value="' . h($name) . '" data-maxlength="64" autocapitalize="off">'
) . "\n" . ($collations ? html_select("collation", array("" => "(" . lang(102) . ")") + $collations, $row["collation"]) . doc_link(array(
	'sql' => "charset-charsets.html",
	'mariadb' => "supported-character-sets-and-collations/",
	'mssql' => "relational-databases/system-functions/sys-fn-helpcollations-transact-sql",
)) : "");
?>
<input type="submit" value="<?php echo lang(14); ?>">
<?php
if (DB != "") {
	echo "<input type='submit' name='drop' value='" . lang(129) . "'>" . confirm(lang(179, DB)) . "\n";
} elseif (!$_POST["add_x"] && $_GET["db"] == "") {
	echo "<input type='image' class='icon' name='add' src='" . h(preg_replace("~\\?.*~", "", ME) . "?file=plus.gif&version=5.1.0") . "' alt='+' title='" . lang(109) . "'>\n";
}
echo input_token();
?>
</form>
<?php
} elseif (isset($_GET["scheme"])) {
	?>
<?php
$row = $_POST;

if ($_POST && !$error) {
	$link = preg_replace('~ns=[^&]*&~', '', ME) . "ns=";
	if ($_POST["drop"]) {
		query_redirect("DROP SCHEMA " . idf_escape($_GET["ns"]), $link, lang(192));
	} else {
		$name = trim($row["name"]);
		$link .= urlencode($name);
		if ($_GET["ns"] == "") {
			query_redirect("CREATE SCHEMA " . idf_escape($name), $link, lang(193));
		} elseif ($_GET["ns"] != $name) {
			query_redirect("ALTER SCHEMA " . idf_escape($_GET["ns"]) . " RENAME TO " . idf_escape($name), $link, lang(194)); //! sp_rename in MS SQL
		} else {
			redirect($link);
		}
	}
}

page_header($_GET["ns"] != "" ? lang(67) : lang(68), $error);

if (!$row) {
	$row["name"] = $_GET["ns"];
}
?>

<form action="" method="post">
<p><input name="name" autofocus value="<?php echo h($row["name"]); ?>" autocapitalize="off">
<input type="submit" value="<?php echo lang(14); ?>">
<?php
if ($_GET["ns"] != "") {
	echo "<input type='submit' name='drop' value='" . lang(129) . "'>" . confirm(lang(179, $_GET["ns"])) . "\n";
}
echo input_token();
?>
</form>
<?php
} elseif (isset($_GET["call"])) {
	?>
<?php
$PROCEDURE = ($_GET["name"] ?: $_GET["call"]);
page_header(lang(195) . ": " . h($PROCEDURE), $error);

$routine = routine($_GET["call"], (isset($_GET["callf"]) ? "FUNCTION" : "PROCEDURE"));
$in = array();
$out = array();
foreach ($routine["fields"] as $i => $field) {
	if (substr($field["inout"], -3) == "OUT") {
		$out[$i] = "@" . idf_escape($field["field"]) . " AS " . idf_escape($field["field"]);
	}
	if (!$field["inout"] || substr($field["inout"], 0, 2) == "IN") {
		$in[] = $i;
	}
}

if (!$error && $_POST) {
	$call = array();
	foreach ($routine["fields"] as $key => $field) {
		if (in_array($key, $in)) {
			$val = process_input($field);
			if ($val === false) {
				$val = "''";
			}
			if (isset($out[$key])) {
				$connection->query("SET @" . idf_escape($field["field"]) . " = $val");
			}
		}
		$call[] = (isset($out[$key]) ? "@" . idf_escape($field["field"]) : $val);
	}

	$query = (isset($_GET["callf"]) ? "SELECT" : "CALL") . " " . table($PROCEDURE) . "(" . implode(", ", $call) . ")";
	$start = microtime(true);
	$result = $connection->multi_query($query);
	$affected = $connection->affected_rows; // getting warnings overwrites this
	echo $adminer->selectQuery($query, $start, !$result);

	if (!$result) {
		echo "<p class='error'>" . error() . "\n";
	} else {
		$connection2 = connect($adminer->credentials());
		if (is_object($connection2)) {
			$connection2->select_db(DB);
		}

		do {
			$result = $connection->store_result();
			if (is_object($result)) {
				select($result, $connection2);
			} else {
				echo "<p class='message'>" . lang(196, $affected)
					. " <span class='time'>" . @date("H:i:s") . "</span>\n" // @ - time zone may be not set
				;
			}
		} while ($connection->next_result());

		if ($out) {
			select($connection->query("SELECT " . implode(", ", $out)));
		}
	}
}
?>

<form action="" method="post">
<?php
if ($in) {
	echo "<table class='layout'>\n";
	foreach ($in as $key) {
		$field = $routine["fields"][$key];
		$name = $field["field"];
		echo "<tr><th>" . $adminer->fieldName($field);
		$value = $_POST["fields"][$name];
		if ($value != "") {
			if ($field["type"] == "set") {
				$value = implode(",", $value);
			}
		}
		input($field, $value, (string) $_POST["function"][$name]); // param name can be empty
		echo "\n";
	}
	echo "</table>\n";
}
?>
<p>
<input type="submit" value="<?php echo lang(195); ?>">
<?php echo input_token(); ?>
</form>

<pre>
<?php
function pre_tr($s) {
	return preg_replace('~^~m', '<tr>', preg_replace('~\|~', '<td>', preg_replace('~\|$~m', "", rtrim($s))));
}
$table = '(\+--[-+]+\+\n)';
$row = '(\| .* \|\n)';
echo preg_replace_callback(
	"~^$table?$row$table?($row*)$table?~m",
	function ($match) {
		$first_row = pre_tr($match[2]);
		return "<table>\n" . ($match[1] ? "<thead>$first_row</thead>\n" : $first_row) . pre_tr($match[4]) . "\n</table>";
	},
	preg_replace(
		'~(\n(    -|mysql)&gt; )(.+)~',
		"\\1<code class='jush-sql'>\\3</code>",
		preg_replace('~(.+)\n---+\n~', "<b>\\1</b>\n", h($routine['comment']))
	)
);
?>
</pre>
<?php
} elseif (isset($_GET["foreign"])) {
	?>
<?php
$TABLE = $_GET["foreign"];
$name = $_GET["name"];
$row = $_POST;

if ($_POST && !$error && !$_POST["add"] && !$_POST["change"] && !$_POST["change-js"]) {
	if (!$_POST["drop"]) {
		$row["source"] = array_filter($row["source"], 'strlen');
		ksort($row["source"]); // enforce input order
		$target = array();
		foreach ($row["source"] as $key => $val) {
			$target[$key] = $row["target"][$key];
		}
		$row["target"] = $target;
	}

	if (JUSH == "sqlite") {
		$result = recreate_table($TABLE, $TABLE, array(), array(), array(" $name" => ($row["drop"] ? "" : " " . format_foreign_key($row))));
	} else {
		$alter = "ALTER TABLE " . table($TABLE);
		$result = ($name == "" || queries("$alter DROP " . (JUSH == "sql" ? "FOREIGN KEY " : "CONSTRAINT ") . idf_escape($name)));
		if (!$row["drop"]) {
			$result = queries("$alter ADD" . format_foreign_key($row));
		}
	}
	queries_redirect(
		ME . "table=" . urlencode($TABLE),
		($row["drop"] ? lang(197) : ($name != "" ? lang(198) : lang(199))),
		$result
	);
	if (!$row["drop"]) {
		$error = "$error<br>" . lang(200); //! no partitioning
	}
}

page_header(lang(201), $error, array("table" => $TABLE), h($TABLE));

if ($_POST) {
	ksort($row["source"]);
	if ($_POST["add"]) {
		$row["source"][] = "";
	} elseif ($_POST["change"] || $_POST["change-js"]) {
		$row["target"] = array();
	}
} elseif ($name != "") {
	$foreign_keys = foreign_keys($TABLE);
	$row = $foreign_keys[$name];
	$row["source"][] = "";
} else {
	$row["table"] = $TABLE;
	$row["source"] = array("");
}
?>

<form action="" method="post">
<?php
$source = array_keys(fields($TABLE)); //! no text and blob
if ($row["db"] != "") {
	$connection->select_db($row["db"]);
}
if ($row["ns"] != "") {
	$orig_schema = get_schema();
	set_schema($row["ns"]);
}
$referencable = array_keys(array_filter(table_status('', true), 'Adminer\fk_support'));
$target = array_keys(fields(in_array($row["table"], $referencable) ? $row["table"] : reset($referencable)));
$onchange = "this.form['change-js'].value = '1'; this.form.submit();";
echo "<p>" . lang(202) . ": " . html_select("table", $referencable, $row["table"], $onchange) . "\n";
if (support("scheme")) {
	$schemas = array_filter($adminer->schemas(), function ($schema) {
		return !preg_match('~^information_schema$~i', $schema);
	});
	echo lang(75) . ": " . html_select("ns", $schemas, $row["ns"] != "" ? $row["ns"] : $_GET["ns"], $onchange);
	if ($row["ns"] != "") {
		set_schema($orig_schema);
	}
} elseif (JUSH != "sqlite") {
	$dbs = array();
	foreach ($adminer->databases() as $db) {
		if (!information_schema($db)) {
			$dbs[] = $db;
		}
	}
	echo lang(74) . ": " . html_select("db", $dbs, $row["db"] != "" ? $row["db"] : $_GET["db"], $onchange);
}
echo input_hidden("change-js");
?>
<noscript><p><input type="submit" name="change" value="<?php echo lang(203); ?>"></noscript>
<table>
<thead><tr><th id="label-source"><?php echo lang(136); ?><th id="label-target"><?php echo lang(137); ?></thead>
<?php
$j = 0;
foreach ($row["source"] as $key => $val) {
	echo "<tr>";
	echo "<td>" . html_select("source[" . (+$key) . "]", array(-1 => "") + $source, $val, ($j == count($row["source"]) - 1 ? "foreignAddRow.call(this);" : ""), "label-source");
	echo "<td>" . html_select("target[" . (+$key) . "]", $target, $row["target"][$key], "", "label-target");
	$j++;
}
?>
</table>
<p>
<?php echo lang(104); ?>: <?php echo html_select("on_delete", array(-1 => "") + explode("|", $driver->onActions), $row["on_delete"]); ?>
 <?php echo lang(103); ?>: <?php echo html_select("on_update", array(-1 => "") + explode("|", $driver->onActions), $row["on_update"]); ?>
<?php echo doc_link(array(
	'sql' => "innodb-foreign-key-constraints.html",
	'mariadb' => "foreign-keys/",
	'pgsql' => "sql-createtable.html#SQL-CREATETABLE-REFERENCES",
	'mssql' => "t-sql/statements/create-table-transact-sql",
	'oracle' => "SQLRF01111",
)); ?>
<p>
<input type="submit" value="<?php echo lang(14); ?>">
<noscript><p><input type="submit" name="add" value="<?php echo lang(204); ?>"></noscript>
<?php if ($name != "") { ?>
<input type="submit" name="drop" value="<?php echo lang(129); ?>"><?php echo confirm(lang(179, $name)); ?>
<?php } ?>
<?php echo input_token(); ?>
</form>
<?php
} elseif (isset($_GET["view"])) {
	?>
<?php
$TABLE = $_GET["view"];
$row = $_POST;
$orig_type = "VIEW";
if (JUSH == "pgsql" && $TABLE != "") {
	$status = table_status($TABLE);
	$orig_type = strtoupper($status["Engine"]);
}

if ($_POST && !$error) {
	$name = trim($row["name"]);
	$as = " AS\n$row[select]";
	$location = ME . "table=" . urlencode($name);
	$message = lang(205);

	$type = ($_POST["materialized"] ? "MATERIALIZED VIEW" : "VIEW");

	if (!$_POST["drop"] && $TABLE == $name && JUSH != "sqlite" && $type == "VIEW" && $orig_type == "VIEW") {
		query_redirect((JUSH == "mssql" ? "ALTER" : "CREATE OR REPLACE") . " VIEW " . table($name) . $as, $location, $message);
	} else {
		$temp_name = $name . "_adminer_" . uniqid();
		drop_create(
			"DROP $orig_type " . table($TABLE),
			"CREATE $type " . table($name) . $as,
			"DROP $type " . table($name),
			"CREATE $type " . table($temp_name) . $as,
			"DROP $type " . table($temp_name),
			($_POST["drop"] ? substr(ME, 0, -1) : $location),
			lang(206),
			$message,
			lang(207),
			$TABLE,
			$name
		);
	}
}

if (!$_POST && $TABLE != "") {
	$row = view($TABLE);
	$row["name"] = $TABLE;
	$row["materialized"] = ($orig_type != "VIEW");
	if (!$error) {
		$error = error();
	}
}

page_header(($TABLE != "" ? lang(42) : lang(208)), $error, array("table" => $TABLE), h($TABLE));
?>

<form action="" method="post">
<p><?php echo lang(187); ?>: <input name="name" value="<?php echo h($row["name"]); ?>" data-maxlength="64" autocapitalize="off">
<?php echo (support("materializedview") ? " " . checkbox("materialized", 1, $row["materialized"], lang(131)) : ""); ?>
<p><?php textarea("select", $row["select"]); ?>
<p>
<input type="submit" value="<?php echo lang(14); ?>">
<?php if ($TABLE != "") { ?>
<input type="submit" name="drop" value="<?php echo lang(129); ?>"><?php echo confirm(lang(179, $TABLE)); ?>
<?php } ?>
<?php echo input_token(); ?>
</form>
<?php
} elseif (isset($_GET["event"])) {
	?>
<?php
$EVENT = $_GET["event"];
$intervals = array("YEAR", "QUARTER", "MONTH", "DAY", "HOUR", "MINUTE", "WEEK", "SECOND", "YEAR_MONTH", "DAY_HOUR", "DAY_MINUTE", "DAY_SECOND", "HOUR_MINUTE", "HOUR_SECOND", "MINUTE_SECOND");
$statuses = array("ENABLED" => "ENABLE", "DISABLED" => "DISABLE", "SLAVESIDE_DISABLED" => "DISABLE ON SLAVE");
$row = $_POST;

if ($_POST && !$error) {
	if ($_POST["drop"]) {
		query_redirect("DROP EVENT " . idf_escape($EVENT), substr(ME, 0, -1), lang(209));
	} elseif (in_array($row["INTERVAL_FIELD"], $intervals) && isset($statuses[$row["STATUS"]])) {
		$schedule = "\nON SCHEDULE " . ($row["INTERVAL_VALUE"]
			? "EVERY " . q($row["INTERVAL_VALUE"]) . " $row[INTERVAL_FIELD]"
			. ($row["STARTS"] ? " STARTS " . q($row["STARTS"]) : "")
			. ($row["ENDS"] ? " ENDS " . q($row["ENDS"]) : "") //! ALTER EVENT doesn't drop ENDS - MySQL bug #39173
			: "AT " . q($row["STARTS"])
			) . " ON COMPLETION" . ($row["ON_COMPLETION"] ? "" : " NOT") . " PRESERVE"
		;

		queries_redirect(
			substr(ME, 0, -1),
			($EVENT != "" ? lang(210) : lang(211)),
			queries(
				($EVENT != ""
				? "ALTER EVENT " . idf_escape($EVENT) . $schedule . ($EVENT != $row["EVENT_NAME"] ? "\nRENAME TO " . idf_escape($row["EVENT_NAME"]) : "")
				: "CREATE EVENT " . idf_escape($row["EVENT_NAME"]) . $schedule
				) . "\n" . $statuses[$row["STATUS"]] . " COMMENT " . q($row["EVENT_COMMENT"])
				. rtrim(" DO\n$row[EVENT_DEFINITION]", ";") . ";"
			)
		);
	}
}

page_header(($EVENT != "" ? lang(212) . ": " . h($EVENT) : lang(213)), $error);

if (!$row && $EVENT != "") {
	$rows = get_rows("SELECT * FROM information_schema.EVENTS WHERE EVENT_SCHEMA = " . q(DB) . " AND EVENT_NAME = " . q($EVENT));
	$row = reset($rows);
}
?>

<form action="" method="post">
<table class="layout">
<tr><th><?php echo lang(187); ?><td><input name="EVENT_NAME" value="<?php echo h($row["EVENT_NAME"]); ?>" data-maxlength="64" autocapitalize="off">
<tr><th title="datetime"><?php echo lang(214); ?><td><input name="STARTS" value="<?php echo h("$row[EXECUTE_AT]$row[STARTS]"); ?>">
<tr><th title="datetime"><?php echo lang(215); ?><td><input name="ENDS" value="<?php echo h($row["ENDS"]); ?>">
<tr><th><?php echo lang(216); ?><td><input type="number" name="INTERVAL_VALUE" value="<?php echo h($row["INTERVAL_VALUE"]); ?>" class="size"> <?php echo html_select("INTERVAL_FIELD", $intervals, $row["INTERVAL_FIELD"]); ?>
<tr><th><?php echo lang(119); ?><td><?php echo html_select("STATUS", $statuses, $row["STATUS"]); ?>
<tr><th><?php echo lang(49); ?><td><input name="EVENT_COMMENT" value="<?php echo h($row["EVENT_COMMENT"]); ?>" data-maxlength="64">
<tr><th><td><?php echo checkbox("ON_COMPLETION", "PRESERVE", $row["ON_COMPLETION"] == "PRESERVE", lang(217)); ?>
</table>
<p><?php textarea("EVENT_DEFINITION", $row["EVENT_DEFINITION"]); ?>
<p>
<input type="submit" value="<?php echo lang(14); ?>">
<?php if ($EVENT != "") { ?>
<input type="submit" name="drop" value="<?php echo lang(129); ?>"><?php echo confirm(lang(179, $EVENT)); ?>
<?php } ?>
<?php echo input_token(); ?>
</form>
<?php
} elseif (isset($_GET["procedure"])) {
	?>
<?php
$PROCEDURE = ($_GET["name"] ?: $_GET["procedure"]);
$routine = (isset($_GET["function"]) ? "FUNCTION" : "PROCEDURE");
$row = $_POST;
$row["fields"] = (array) $row["fields"];

if ($_POST && !process_fields($row["fields"]) && !$error) {
	$orig = routine($_GET["procedure"], $routine);
	$temp_name = "$row[name]_adminer_" . uniqid();
	foreach ($row["fields"] as $key => $field) {
		if ($field["field"] == "") {
			unset($row["fields"][$key]);
		}
	}
	drop_create(
		"DROP $routine " . routine_id($PROCEDURE, $orig),
		create_routine($routine, $row),
		"DROP $routine " . routine_id($row["name"], $row),
		create_routine($routine, array("name" => $temp_name) + $row),
		"DROP $routine " . routine_id($temp_name, $row),
		substr(ME, 0, -1),
		lang(218),
		lang(219),
		lang(220),
		$PROCEDURE,
		$row["name"]
	);
}

page_header(($PROCEDURE != "" ? (isset($_GET["function"]) ? lang(221) : lang(222)) . ": " . h($PROCEDURE) : (isset($_GET["function"]) ? lang(223) : lang(224))), $error);

if (!$_POST) {
	if ($PROCEDURE == "") {
		$row["language"] = "sql";
	} else {
		$row = routine($_GET["procedure"], $routine);
		$row["name"] = $PROCEDURE;
	}
}

$collations = get_vals("SHOW CHARACTER SET");
sort($collations);
$routine_languages = routine_languages();
echo ($collations ? "<datalist id='collations'>" . optionlist($collations) . "</datalist>" : "");
?>

<form action="" method="post" id="form">
<p><?php echo lang(187); ?>: <input name="name" value="<?php echo h($row["name"]); ?>" data-maxlength="64" autocapitalize="off">
<?php echo ($routine_languages ? lang(19) . ": " . html_select("language", $routine_languages, $row["language"]) . "\n" : ""); ?>
<input type="submit" value="<?php echo lang(14); ?>">
<div class="scrollable">
<table class="nowrap">
<?php
edit_fields($row["fields"], $collations, $routine);
if (isset($_GET["function"])) {
	echo "<tr><td>" . lang(225);
	edit_type("returns", $row["returns"], $collations, array(), (JUSH == "pgsql" ? array("void", "trigger") : array()));
}
?>
</table>
<?php echo script("editFields();"); ?>
</div>
<p><?php textarea("definition", $row["definition"]); ?>
<p>
<input type="submit" value="<?php echo lang(14); ?>">
<?php if ($PROCEDURE != "") { ?>
<input type="submit" name="drop" value="<?php echo lang(129); ?>"><?php echo confirm(lang(179, $PROCEDURE)); ?>
<?php } ?>
<?php echo input_token(); ?>
</form>
<?php
} elseif (isset($_GET["sequence"])) {
	?>
<?php
$SEQUENCE = $_GET["sequence"];
$row = $_POST;

if ($_POST && !$error) {
	$link = substr(ME, 0, -1);
	$name = trim($row["name"]);
	if ($_POST["drop"]) {
		query_redirect("DROP SEQUENCE " . idf_escape($SEQUENCE), $link, lang(226));
	} elseif ($SEQUENCE == "") {
		query_redirect("CREATE SEQUENCE " . idf_escape($name), $link, lang(227));
	} elseif ($SEQUENCE != $name) {
		query_redirect("ALTER SEQUENCE " . idf_escape($SEQUENCE) . " RENAME TO " . idf_escape($name), $link, lang(228));
	} else {
		redirect($link);
	}
}

page_header($SEQUENCE != "" ? lang(229) . ": " . h($SEQUENCE) : lang(230), $error);

if (!$row) {
	$row["name"] = $SEQUENCE;
}
?>

<form action="" method="post">
<p><input name="name" value="<?php echo h($row["name"]); ?>" autocapitalize="off">
<input type="submit" value="<?php echo lang(14); ?>">
<?php
if ($SEQUENCE != "") {
	echo "<input type='submit' name='drop' value='" . lang(129) . "'>" . confirm(lang(179, $SEQUENCE)) . "\n";
}
echo input_token();
?>
</form>
<?php
} elseif (isset($_GET["type"])) {
	?>
<?php
$TYPE = $_GET["type"];
$row = $_POST;

if ($_POST && !$error) {
	$link = substr(ME, 0, -1);
	if ($_POST["drop"]) {
		query_redirect("DROP TYPE " . idf_escape($TYPE), $link, lang(231));
	} else {
		query_redirect("CREATE TYPE " . idf_escape(trim($row["name"])) . " $row[as]", $link, lang(232));
	}
}

page_header($TYPE != "" ? lang(233) . ": " . h($TYPE) : lang(234), $error);

if (!$row) {
	$row["as"] = "AS ";
}
?>

<form action="" method="post">
<p>
<?php
if ($TYPE != "") {
	$types = $driver->types();
	$enums = type_values($types[$TYPE]);
	if ($enums) {
		echo "<code class='jush-" . JUSH . "'>ENUM (" . h($enums) . ")</code>\n<p>";
	}
	echo "<input type='submit' name='drop' value='" . lang(129) . "'>" . confirm(lang(179, $TYPE)) . "\n";
} else {
	echo lang(187) . ": <input name='name' value='" . h($row['name']) . "' autocapitalize='off'>\n";
	echo doc_link(array(
		'pgsql' => "datatype-enum.html",
	), "?");
	textarea("as", $row["as"]);
	echo "<p><input type='submit' value='" . lang(14) . "'>\n";
}
echo input_token();
?>
</form>
<?php
} elseif (isset($_GET["check"])) {
	?>
<?php
$TABLE = $_GET["check"];
$name = $_GET["name"];
$row = $_POST;

if ($row && !$error) {
	if (JUSH == "sqlite") {
		$result = recreate_table($TABLE, $TABLE, array(), array(), array(), 0, array(), $name, ($row["drop"] ? "" : $row["clause"]));
	} else {
		$result = ($name == "" || queries("ALTER TABLE " . table($TABLE) . " DROP CONSTRAINT " . idf_escape($name)));
		if (!$row["drop"]) {
			$result = queries("ALTER TABLE " . table($TABLE) . " ADD" . ($row["name"] != "" ? " CONSTRAINT " . idf_escape($row["name"]) : "") . " CHECK ($row[clause])"); //! SQL injection
		}
	}
	queries_redirect(
		ME . "table=" . urlencode($TABLE),
		($row["drop"] ? lang(235) : ($name != "" ? lang(236) : lang(237))),
		$result
	);
}

page_header(($name != "" ? lang(238) . ": " . h($name) : lang(141)), $error, array("table" => $TABLE));

if (!$row) {
	$checks = $driver->checkConstraints($TABLE);
	$row = array("name" => $name, "clause" => $checks[$name]);
}
?>

<form action="" method="post">
<p><?php
if (JUSH != "sqlite") {
	echo lang(187) . ': <input name="name" value="' . h($row["name"]) . '" data-maxlength="64" autocapitalize="off"> ';
}
echo doc_link(array(
	'sql' => "create-table-check-constraints.html",
	'mariadb' => "constraint/",
	'pgsql' => "ddl-constraints.html#DDL-CONSTRAINTS-CHECK-CONSTRAINTS",
	'mssql' => "relational-databases/tables/create-check-constraints",
	'sqlite' => "lang_createtable.html#check_constraints",
), "?");
?>
<p><?php textarea("clause", $row["clause"]); ?>
<p><input type="submit" value="<?php echo lang(14); ?>">
<?php if ($name != "") { ?>
<input type="submit" name="drop" value="<?php echo lang(129); ?>"><?php echo confirm(lang(179, $name)); ?>
<?php } ?>
<?php echo input_token(); ?>
</form>
<?php
} elseif (isset($_GET["trigger"])) {
	?>
<?php
$TABLE = $_GET["trigger"];
$name = $_GET["name"];
$trigger_options = trigger_options();
$row = (array) trigger($name, $TABLE) + array("Trigger" => $TABLE . "_bi");

if ($_POST) {
	if (!$error && in_array($_POST["Timing"], $trigger_options["Timing"]) && in_array($_POST["Event"], $trigger_options["Event"]) && in_array($_POST["Type"], $trigger_options["Type"])) {
		// don't use drop_create() because there may not be more triggers for the same action
		$on = " ON " . table($TABLE);
		$drop = "DROP TRIGGER " . idf_escape($name) . (JUSH == "pgsql" ? $on : "");
		$location = ME . "table=" . urlencode($TABLE);
		if ($_POST["drop"]) {
			query_redirect($drop, $location, lang(239));
		} else {
			if ($name != "") {
				queries($drop);
			}
			queries_redirect(
				$location,
				($name != "" ? lang(240) : lang(241)),
				queries(create_trigger($on, $_POST))
			);
			if ($name != "") {
				queries(create_trigger($on, $row + array("Type" => reset($trigger_options["Type"]))));
			}
		}
	}
	$row = $_POST;
}

page_header(($name != "" ? lang(242) . ": " . h($name) : lang(243)), $error, array("table" => $TABLE));
?>

<form action="" method="post" id="form">
<table class="layout">
<tr><th><?php echo lang(244); ?><td><?php echo html_select("Timing", $trigger_options["Timing"], $row["Timing"], "triggerChange(/^" . preg_quote($TABLE, "/") . "_[ba][iud]$/, '" . js_escape($TABLE) . "', this.form);"); ?>
<tr><th><?php echo lang(245); ?><td><?php echo html_select("Event", $trigger_options["Event"], $row["Event"], "this.form['Timing'].onchange();"); ?>
<?php echo (in_array("UPDATE OF", $trigger_options["Event"]) ? " <input name='Of' value='" . h($row["Of"]) . "' class='hidden'>": ""); ?>
<tr><th><?php echo lang(48); ?><td><?php echo html_select("Type", $trigger_options["Type"], $row["Type"]); ?>
</table>
<p><?php echo lang(187); ?>: <input name="Trigger" value="<?php echo h($row["Trigger"]); ?>" data-maxlength="64" autocapitalize="off">
<?php echo script("qs('#form')['Timing'].onchange();"); ?>
<p><?php textarea("Statement", $row["Statement"]); ?>
<p>
<input type="submit" value="<?php echo lang(14); ?>">
<?php if ($name != "") { ?>
<input type="submit" name="drop" value="<?php echo lang(129); ?>"><?php echo confirm(lang(179, $name)); ?>
<?php } ?>
<?php echo input_token(); ?>
</form>
<?php
} elseif (isset($_GET["user"])) {
	?>
<?php
$USER = $_GET["user"];
$privileges = array("" => array("All privileges" => ""));
foreach (get_rows("SHOW PRIVILEGES") as $row) {
	foreach (explode(",", ($row["Privilege"] == "Grant option" ? "" : $row["Context"])) as $context) {
		$privileges[$context][$row["Privilege"]] = $row["Comment"];
	}
}
$privileges["Server Admin"] += $privileges["File access on server"];
$privileges["Databases"]["Create routine"] = $privileges["Procedures"]["Create routine"]; // MySQL bug #30305
unset($privileges["Procedures"]["Create routine"]);
$privileges["Columns"] = array();
foreach (array("Select", "Insert", "Update", "References") as $val) {
	$privileges["Columns"][$val] = $privileges["Tables"][$val];
}
unset($privileges["Server Admin"]["Usage"]);
foreach ($privileges["Tables"] as $key => $val) {
	unset($privileges["Databases"][$key]);
}

$new_grants = array();
if ($_POST) {
	foreach ($_POST["objects"] as $key => $val) {
		$new_grants[$val] = (array) $new_grants[$val] + (array) $_POST["grants"][$key];
	}
}
$grants = array();
$old_pass = "";

if (isset($_GET["host"]) && ($result = $connection->query("SHOW GRANTS FOR " . q($USER) . "@" . q($_GET["host"])))) { //! use information_schema for MySQL 5 - column names in column privileges are not escaped
	while ($row = $result->fetch_row()) {
		if (preg_match('~GRANT (.*) ON (.*) TO ~', $row[0], $match) && preg_match_all('~ *([^(,]*[^ ,(])( *\([^)]+\))?~', $match[1], $matches, PREG_SET_ORDER)) { //! escape the part between ON and TO
			foreach ($matches as $val) {
				if ($val[1] != "USAGE") {
					$grants["$match[2]$val[2]"][$val[1]] = true;
				}
				if (preg_match('~ WITH GRANT OPTION~', $row[0])) { //! don't check inside strings and identifiers
					$grants["$match[2]$val[2]"]["GRANT OPTION"] = true;
				}
			}
		}
		if (preg_match("~ IDENTIFIED BY PASSWORD '([^']+)~", $row[0], $match)) {
			$old_pass = $match[1];
		}
	}
}

if ($_POST && !$error) {
	$old_user = (isset($_GET["host"]) ? q($USER) . "@" . q($_GET["host"]) : "''");
	if ($_POST["drop"]) {
		query_redirect("DROP USER $old_user", ME . "privileges=", lang(246));
	} else {
		$new_user = q($_POST["user"]) . "@" . q($_POST["host"]); // if $_GET["host"] is not set then $new_user is always different
		$pass = $_POST["pass"];
		if ($pass != '' && !$_POST["hashed"] && !min_version(8)) {
			// compute hash in a separate query so that plain text password is not saved to history
			$pass = get_val("SELECT PASSWORD(" . q($pass) . ")");
			$error = !$pass;
		}

		$created = false;
		if (!$error) {
			if ($old_user != $new_user) {
				$created = queries((min_version(5) ? "CREATE USER" : "GRANT USAGE ON *.* TO") . " $new_user IDENTIFIED BY " . (min_version(8) ? "" : "PASSWORD ") . q($pass));
				$error = !$created;
			} elseif ($pass != $old_pass) {
				queries("SET PASSWORD FOR $new_user = " . q($pass));
			}
		}

		if (!$error) {
			$revoke = array();
			foreach ($new_grants as $object => $grant) {
				if (isset($_GET["grant"])) {
					$grant = array_filter($grant);
				}
				$grant = array_keys($grant);
				if (isset($_GET["grant"])) {
					// no rights to mysql.user table
					$revoke = array_diff(array_keys(array_filter($new_grants[$object], 'strlen')), $grant);
				} elseif ($old_user == $new_user) {
					$old_grant = array_keys((array) $grants[$object]);
					$revoke = array_diff($old_grant, $grant);
					$grant = array_diff($grant, $old_grant);
					unset($grants[$object]);
				}
				if (
					preg_match('~^(.+)\s*(\(.*\))?$~U', $object, $match) && (
					!grant("REVOKE", $revoke, $match[2], " ON $match[1] FROM $new_user") //! SQL injection
					|| !grant("GRANT", $grant, $match[2], " ON $match[1] TO $new_user"))
				) {
					$error = true;
					break;
				}
			}
		}

		if (!$error && isset($_GET["host"])) {
			if ($old_user != $new_user) {
				queries("DROP USER $old_user");
			} elseif (!isset($_GET["grant"])) {
				foreach ($grants as $object => $revoke) {
					if (preg_match('~^(.+)(\(.*\))?$~U', $object, $match)) {
						grant("REVOKE", array_keys($revoke), $match[2], " ON $match[1] FROM $new_user");
					}
				}
			}
		}

		queries_redirect(ME . "privileges=", (isset($_GET["host"]) ? lang(247) : lang(248)), !$error);

		if ($created) {
			// delete new user in case of an error
			$connection->query("DROP USER $new_user");
		}
	}
}

page_header((isset($_GET["host"]) ? lang(34) . ": " . h("$USER@$_GET[host]") : lang(150)), $error, array("privileges" => array('', lang(70))));

$row = $_POST;
if ($row) {
	$grants = $new_grants;
} else {
	$row = $_GET + array("host" => get_val("SELECT SUBSTRING_INDEX(CURRENT_USER, '@', -1)")); // create user on the same domain by default
	$row["pass"] = $old_pass;
	if ($old_pass != "") {
		$row["hashed"] = true;
	}
	$grants[(DB == "" || $grants ? "" : idf_escape(addcslashes(DB, "%_\\"))) . ".*"] = array();
}

?>
<form action="" method="post">
<table class="layout">
<tr><th><?php echo lang(33); ?><td><input name="host" data-maxlength="60" value="<?php echo h($row["host"]); ?>" autocapitalize="off">
<tr><th><?php echo lang(34); ?><td><input name="user" data-maxlength="80" value="<?php echo h($row["user"]); ?>" autocapitalize="off">
<tr><th><?php echo lang(35); ?><td><input name="pass" id="pass" value="<?php echo h($row["pass"]); ?>" autocomplete="new-password">
<?php echo ($row["hashed"] ? "" : script("typePassword(qs('#pass'));")); ?>
<?php echo (min_version(8) ? "" : checkbox("hashed", 1, $row["hashed"], lang(249), "typePassword(this.form['pass'], this.checked);")); ?>
</table>

<?php
//! MAX_* limits, REQUIRE
echo "<table class='odds'>\n";
echo "<thead><tr><th colspan='2'>" . lang(70) . doc_link(array('sql' => "grant.html#priv_level"));
$i = 0;
foreach ($grants as $object => $grant) {
	echo '<th>' . ($object != "*.*"
		? "<input name='objects[$i]' value='" . h($object) . "' size='10' autocapitalize='off'>"
		: input_hidden("objects[$i]", "*.*") . "*.*"
	); //! separate db, table, columns, PROCEDURE|FUNCTION, routine
	$i++;
}
echo "</thead>\n";

foreach (
	array(
		"" => "",
		"Server Admin" => lang(33),
		"Databases" => lang(36),
		"Tables" => lang(133),
		"Columns" => lang(47),
		"Procedures" => lang(250),
	) as $context => $desc
) {
	foreach ((array) $privileges[$context] as $privilege => $comment) {
		echo "<tr><td" . ($desc ? ">$desc<td" : " colspan='2'") . ' lang="en" title="' . h($comment) . '">' . h($privilege);
		$i = 0;
		foreach ($grants as $object => $grant) {
			$name = "'grants[$i][" . h(strtoupper($privilege)) . "]'";
			$value = $grant[strtoupper($privilege)];
			if ($context == "Server Admin" && $object != (isset($grants["*.*"]) ? "*.*" : ".*")) {
				echo "<td>";
			} elseif (isset($_GET["grant"])) {
				echo "<td><select name=$name><option><option value='1'" . ($value ? " selected" : "") . ">" . lang(251) . "<option value='0'" . ($value == "0" ? " selected" : "") . ">" . lang(252) . "</select>";
			} else {
				echo "<td align='center'><label class='block'>";
				echo "<input type='checkbox' name=$name value='1'" . ($value ? " checked" : "") . ($privilege == "All privileges"
					? " id='grants-$i-all'>" //! uncheck all except grant if all is checked
					: ">" . ($privilege == "Grant option" ? "" : script("qsl('input').onclick = function () { if (this.checked) formUncheck('grants-$i-all'); };")));
				echo "</label>";
			}
			$i++;
		}
	}
}

echo "</table>\n";
?>
<p>
<input type="submit" value="<?php echo lang(14); ?>">
<?php if (isset($_GET["host"])) { ?>
<input type="submit" name="drop" value="<?php echo lang(129); ?>"><?php echo confirm(lang(179, "$USER@$_GET[host]")); ?>
<?php } ?>
<?php echo input_token(); ?>
</form>
<?php
} elseif (isset($_GET["processlist"])) {
	?>
<?php
if (support("kill")) {
	if ($_POST && !$error) {
		$killed = 0;
		foreach ((array) $_POST["kill"] as $val) {
			if (kill_process($val)) {
				$killed++;
			}
		}
		queries_redirect(ME . "processlist=", lang(253, $killed), $killed || !$_POST["kill"]);
	}
}

page_header(lang(117), $error);
?>

<form action="" method="post">
<div class="scrollable">
<table class="nowrap checkable odds">
<?php
echo script("mixin(qsl('table'), {onclick: tableClick, ondblclick: partialArg(tableClick, true)});");
// HTML valid because there is always at least one process
$i = -1;
foreach (process_list() as $i => $row) {
	if (!$i) {
		echo "<thead><tr lang='en'>" . (support("kill") ? "<th>" : "");
		foreach ($row as $key => $val) {
			echo "<th>$key" . doc_link(array(
				'sql' => "show-processlist.html#processlist_" . strtolower($key),
				'pgsql' => "monitoring-stats.html#PG-STAT-ACTIVITY-VIEW",
				'oracle' => "REFRN30223",
			));
		}
		echo "</thead>\n";
	}
	echo "<tr>" . (support("kill") ? "<td>" . checkbox("kill[]", $row[JUSH == "sql" ? "Id" : "pid"], 0) : "");
	foreach ($row as $key => $val) {
		echo "<td>" . (
			(JUSH == "sql" && $key == "Info" && preg_match("~Query|Killed~", $row["Command"]) && $val != "") ||
			(JUSH == "pgsql" && $key == "current_query" && $val != "<IDLE>") ||
			(JUSH == "oracle" && $key == "sql_text" && $val != "")
			? "<code class='jush-" . JUSH . "'>" . shorten_utf8($val, 100, "</code>") . ' <a href="' . h(ME . ($row["db"] != "" ? "db=" . urlencode($row["db"]) . "&" : "") . "sql=" . urlencode($val)) . '">' . lang(254) . '</a>'
			: h($val)
		);
	}
	echo "\n";
}
?>
</table>
</div>
<p>
<?php
if (support("kill")) {
	echo ($i + 1) . "/" . lang(255, max_connections());
	echo "<p><input type='submit' value='" . lang(256) . "'>\n";
}
echo input_token();
?>
</form>
<?php echo script("tableCheck();"); ?>
<?php
} elseif (isset($_GET["select"])) {
	?>
<?php
$TABLE = $_GET["select"];
$table_status = table_status1($TABLE);
$indexes = indexes($TABLE);
$fields = fields($TABLE);
$foreign_keys = column_foreign_keys($TABLE);
$oid = $table_status["Oid"];
$adminer_import = get_settings("adminer_import");

$rights = array(); // privilege => 0
$columns = array(); // selectable columns
$search_columns = array(); // searchable columns
$order_columns = array(); // searchable columns
$text_length = null;
foreach ($fields as $key => $field) {
	$name = $adminer->fieldName($field);
	$name_plain = html_entity_decode(strip_tags($name), ENT_QUOTES);
	if (isset($field["privileges"]["select"]) && $name != "") {
		$columns[$key] = $name_plain;
		if (is_shortable($field)) {
			$text_length = $adminer->selectLengthProcess();
		}
	}
	if (isset($field["privileges"]["where"]) && $name != "") {
		$search_columns[$key] = $name_plain;
	}
	if (isset($field["privileges"]["order"]) && $name != "") {
		$order_columns[$key] = $name_plain;
	}
	$rights += $field["privileges"];
}

list($select, $group) = $adminer->selectColumnsProcess($columns, $indexes);
$select = array_unique($select);
$group = array_unique($group);
$is_group = count($group) < count($select);
$where = $adminer->selectSearchProcess($fields, $indexes);
$order = $adminer->selectOrderProcess($fields, $indexes);
$limit = $adminer->selectLimitProcess();

if ($_GET["val"] && is_ajax()) {
	header("Content-Type: text/plain; charset=utf-8");
	foreach ($_GET["val"] as $unique_idf => $row) {
		$as = convert_field($fields[key($row)]);
		$select = array($as ?: idf_escape(key($row)));
		$where[] = where_check($unique_idf, $fields);
		$return = $driver->select($TABLE, $select, $where, $select);
		if ($return) {
			echo reset($return->fetch_row());
		}
	}
	exit;
}

$primary = $unselected = null;
foreach ($indexes as $index) {
	if ($index["type"] == "PRIMARY") {
		$primary = array_flip($index["columns"]);
		$unselected = ($select ? $primary : array());
		foreach ($unselected as $key => $val) {
			if (in_array(idf_escape($key), $select)) {
				unset($unselected[$key]);
			}
		}
		break;
	}
}
if ($oid && !$primary) {
	$primary = $unselected = array($oid => 0);
	$indexes[] = array("type" => "PRIMARY", "columns" => array($oid));
}

if ($_POST && !$error) {
	$where_check = $where;
	if (!$_POST["all"] && is_array($_POST["check"])) {
		$checks = array();
		foreach ($_POST["check"] as $check) {
			$checks[] = where_check($check, $fields);
		}
		$where_check[] = "((" . implode(") OR (", $checks) . "))";
	}
	$where_check = ($where_check ? "\nWHERE " . implode(" AND ", $where_check) : "");
	if ($_POST["export"]) {
		save_settings(array("output" => $_POST["output"], "format" => $_POST["format"]), "adminer_import");
		dump_headers($TABLE);
		$adminer->dumpTable($TABLE, "");
		$from = ($select ? implode(", ", $select) : "*")
			. convert_fields($columns, $fields, $select)
			. "\nFROM " . table($TABLE);
		$group_by = ($group && $is_group ? "\nGROUP BY " . implode(", ", $group) : "") . ($order ? "\nORDER BY " . implode(", ", $order) : "");
		$query = "SELECT $from$where_check$group_by";
		if (is_array($_POST["check"]) && !$primary) {
			$union = array();
			foreach ($_POST["check"] as $val) {
				// where is not unique so OR can't be used
				$union[] = "(SELECT" . limit($from, "\nWHERE " . ($where ? implode(" AND ", $where) . " AND " : "") . where_check($val, $fields) . $group_by, 1) . ")";
			}
			$query = implode(" UNION ALL ", $union);
		}
		$adminer->dumpData($TABLE, "table", $query);
		$adminer->dumpFooter();
		exit;
	}

	if (!$adminer->selectEmailProcess($where, $foreign_keys)) {
		if ($_POST["save"] || $_POST["delete"]) { // edit
			$result = true;
			$affected = 0;
			$set = array();
			if (!$_POST["delete"]) {
				foreach ($_POST["fields"] as $name => $val) {
					$val = process_input($fields[$name]);
					if ($val !== null && ($_POST["clone"] || $val !== false)) {
						$set[idf_escape($name)] = ($val !== false ? $val : idf_escape($name));
					}
				}
			}
			if ($_POST["delete"] || $set) {
				if ($_POST["clone"]) {
					$query = "INTO " . table($TABLE) . " (" . implode(", ", array_keys($set)) . ")\nSELECT " . implode(", ", $set) . "\nFROM " . table($TABLE);
				}
				if ($_POST["all"] || ($primary && is_array($_POST["check"])) || $is_group) {
					$result = ($_POST["delete"]
						? $driver->delete($TABLE, $where_check)
						: ($_POST["clone"]
							? queries("INSERT $query$where_check" . $driver->insertReturning($TABLE))
							: $driver->update($TABLE, $set, $where_check)
						)
					);
					$affected = $connection->affected_rows + (is_object($result) ? $result->num_rows : 0); // PostgreSQL with RETURNING fills num_rows
				} else {
					foreach ((array) $_POST["check"] as $val) {
						// where is not unique so OR can't be used
						$where2 = "\nWHERE " . ($where ? implode(" AND ", $where) . " AND " : "") . where_check($val, $fields);
						$result = ($_POST["delete"]
							? $driver->delete($TABLE, $where2, 1)
							: ($_POST["clone"]
								? queries("INSERT" . limit1($TABLE, $query, $where2))
								: $driver->update($TABLE, $set, $where2, 1)
							)
						);
						if (!$result) {
							break;
						}
						$affected += $connection->affected_rows;
					}
				}
			}
			$message = lang(257, $affected);
			if ($_POST["clone"] && $result && $affected == 1) {
				$last_id = last_id($result);
				if ($last_id) {
					$message = lang(172, " $last_id");
				}
			}
			queries_redirect(remove_from_uri($_POST["all"] && $_POST["delete"] ? "page" : ""), $message, $result);
			if (!$_POST["delete"]) {
				$post_fields = (array) $_POST["fields"];
				edit_form($TABLE, array_intersect_key($fields, $post_fields), $post_fields, !$_POST["clone"]);
				page_footer();
				exit;
			}

		} elseif (!$_POST["import"]) { // modify
			if (!$_POST["val"]) {
				$error = lang(258);
			} else {
				$result = true;
				$affected = 0;
				foreach ($_POST["val"] as $unique_idf => $row) {
					$set = array();
					foreach ($row as $key => $val) {
						$key = bracket_escape($key, 1); // 1 - back
						$set[idf_escape($key)] = (preg_match('~char|text~', $fields[$key]["type"]) || $val != "" ? $adminer->processInput($fields[$key], $val) : "NULL");
					}
					$result = $driver->update(
						$TABLE,
						$set,
						" WHERE " . ($where ? implode(" AND ", $where) . " AND " : "") . where_check($unique_idf, $fields),
						!$is_group && !$primary,
						" "
					);
					if (!$result) {
						break;
					}
					$affected += $connection->affected_rows;
				}
				queries_redirect(remove_from_uri(), lang(257, $affected), $result);
			}

		} elseif (!is_string($file = get_file("csv_file", true))) {
			$error = upload_error($file);
		} elseif (!preg_match('~~u', $file)) {
			$error = lang(259);
		} else {
			save_settings(array("output" => $adminer_import["output"], "format" => $_POST["separator"]), "adminer_import");
			$result = true;
			$cols = array_keys($fields);
			preg_match_all('~(?>"[^"]*"|[^"\r\n]+)+~', $file, $matches);
			$affected = count($matches[0]);
			$driver->begin();
			$separator = ($_POST["separator"] == "csv" ? "," : ($_POST["separator"] == "tsv" ? "\t" : ";"));
			$rows = array();
			foreach ($matches[0] as $key => $val) {
				preg_match_all("~((?>\"[^\"]*\")+|[^$separator]*)$separator~", $val . $separator, $matches2);
				if (!$key && !array_diff($matches2[1], $cols)) { //! doesn't work with column names containing ",\n
					// first row corresponds to column names - use it for table structure
					$cols = $matches2[1];
					$affected--;
				} else {
					$set = array();
					foreach ($matches2[1] as $i => $col) {
						$set[idf_escape($cols[$i])] = ($col == "" && $fields[$cols[$i]]["null"] ? "NULL" : q(preg_match('~^".*"$~s', $col) ? str_replace('""', '"', substr($col, 1, -1)) : $col));
					}
					$rows[] = $set;
				}
			}
			$result = (!$rows || $driver->insertUpdate($TABLE, $rows, $primary));
			if ($result) {
				$driver->commit();
			}
			queries_redirect(remove_from_uri("page"), lang(260, $affected), $result);
			$driver->rollback(); // after queries_redirect() to not overwrite error

		}
	}
}

$table_name = $adminer->tableName($table_status);
if (is_ajax()) {
	page_headers();
	ob_start();
} else {
	page_header(lang(52) . ": $table_name", $error);
}

$set = null;
if (isset($rights["insert"]) || !support("table")) {
	$params = array();
	foreach ((array) $_GET["where"] as $val) {
		if (
			isset($foreign_keys[$val["col"]]) && count($foreign_keys[$val["col"]]) == 1
			&& ($val["op"] == "=" || (!$val["op"] && (is_array($val["val"]) || !preg_match('~[_%]~', $val["val"])))) // LIKE in Editor
		) {
			$params["set" . "[" . bracket_escape($val["col"]) . "]"] = $val["val"];
		}
	}

	$set = $params ? "&" . http_build_query($params) : "";
}
$adminer->selectLinks($table_status, $set);

if (!$columns && support("table")) {
	echo "<p class='error'>" . lang(261) . ($fields ? "." : ": " . error()) . "\n";
} else {
	echo "<form action='' id='form'>\n";
	echo "<div style='display: none;'>";
	hidden_fields_get();
	echo (DB != "" ? input_hidden("db", DB) . (isset($_GET["ns"]) ? input_hidden("ns", $_GET["ns"]) : "") : ""); // not used in Editor
	echo input_hidden("select", $TABLE);
	echo "</div>\n";
	$adminer->selectColumnsPrint($select, $columns);
	$adminer->selectSearchPrint($where, $search_columns, $indexes);
	$adminer->selectOrderPrint($order, $order_columns, $indexes);
	$adminer->selectLimitPrint($limit);
	$adminer->selectLengthPrint($text_length);
	$adminer->selectActionPrint($indexes);
	echo "</form>\n";

	$page = $_GET["page"];
	if ($page == "last") {
		$found_rows = get_val(count_rows($TABLE, $where, $is_group, $group));
		$page = floor(max(0, $found_rows - 1) / $limit);
	}

	$select2 = $select;
	$group2 = $group;
	if (!$select2) {
		$select2[] = "*";
		$convert_fields = convert_fields($columns, $fields, $select);
		if ($convert_fields) {
			$select2[] = substr($convert_fields, 2);
		}
	}
	foreach ($select as $key => $val) {
		$field = $fields[idf_unescape($val)];
		if ($field && ($as = convert_field($field))) {
			$select2[$key] = "$as AS $val";
		}
	}
	if (!$is_group && $unselected) {
		foreach ($unselected as $key => $val) {
			$select2[] = idf_escape($key);
			if ($group2) {
				$group2[] = idf_escape($key);
			}
		}
	}
	$result = $driver->select($TABLE, $select2, $where, $group2, $order, $limit, $page, true);

	if (!$result) {
		echo "<p class='error'>" . error() . "\n";
	} else {
		if (JUSH == "mssql" && $page) {
			$result->seek($limit * $page);
		}
		$email_fields = array();
		echo "<form action='' method='post' enctype='multipart/form-data'>\n";
		$rows = array();
		while ($row = $result->fetch_assoc()) {
			if ($page && JUSH == "oracle") {
				unset($row["RNUM"]);
			}
			$rows[] = $row;
		}

		// use count($rows) without LIMIT, COUNT(*) without grouping, FOUND_ROWS otherwise (slowest)
		if ($_GET["page"] != "last" && $limit != "" && $group && $is_group && JUSH == "sql") {
			$found_rows = get_val(" SELECT FOUND_ROWS()"); // space to allow mysql.trace_mode
		}

		if (!$rows) {
			echo "<p class='message'>" . lang(12) . "\n";
		} else {
			$backward_keys = $adminer->backwardKeys($TABLE, $table_name);

			echo "<div class='scrollable'>";
			echo "<table id='table' class='nowrap checkable odds'>";
			echo script("mixin(qs('#table'), {onclick: tableClick, ondblclick: partialArg(tableClick, true), onkeydown: editingKeydown});");
			echo "<thead><tr>" . (!$group && $select
				? ""
				: "<td><input type='checkbox' id='all-page' class='jsonly'>" . script("qs('#all-page').onclick = partial(formCheck, /check/);", "")
					. " <a href='" . h($_GET["modify"] ? remove_from_uri("modify") : $_SERVER["REQUEST_URI"] . "&modify=1") . "'>" . lang(262) . "</a>");
			$names = array();
			$functions = array();
			reset($select);
			$rank = 1;
			foreach ($rows[0] as $key => $val) {
				if (!isset($unselected[$key])) {
					$val = $_GET["columns"][key($select)];
					$field = $fields[$select ? ($val ? $val["col"] : current($select)) : $key];
					$name = ($field ? $adminer->fieldName($field, $rank) : ($val["fun"] ? "*" : h($key)));
					if ($name != "") {
						$rank++;
						$names[$key] = $name;
						$column = idf_escape($key);
						$href = remove_from_uri('(order|desc)[^=]*|page') . '&order%5B0%5D=' . urlencode($key);
						$desc = "&desc%5B0%5D=1";
						$sortable = isset($field["privileges"]["order"]);
						echo "<th id='th[" . h(bracket_escape($key)) . "]'>" . script("mixin(qsl('th'), {onmouseover: partial(columnMouse), onmouseout: partial(columnMouse, ' hidden')});", "");
						$fun = apply_sql_function($val["fun"], $name); //! columns looking like functions
						echo ($sortable ? '<a href="' . h($href . ($order[0] == $column || $order[0] == $key || (!$order && $is_group && $group[0] == $column) ? $desc : '')) . '">' . "$fun</a>" : $fun); // $order[0] == $key - COUNT(*)
						echo "<span class='column hidden'>";
						if ($sortable) {
							echo "<a href='" . h($href . $desc) . "' title='" . lang(58) . "' class='text'> ↓</a>";
						}
						if (!$val["fun"] && isset($field["privileges"]["where"])) {
							echo '<a href="#fieldset-search" title="' . lang(55) . '" class="text jsonly"> =</a>';
							echo script("qsl('a').onclick = partial(selectSearch, '" . js_escape($key) . "');");
						}
						echo "</span>";
					}
					$functions[$key] = $val["fun"];
					next($select);
				}
			}

			$lengths = array();
			if ($_GET["modify"]) {
				foreach ($rows as $row) {
					foreach ($row as $key => $val) {
						$lengths[$key] = max($lengths[$key], min(40, strlen(utf8_decode($val))));
					}
				}
			}

			echo ($backward_keys ? "<th>" . lang(263) : "") . "</thead>\n";

			if (is_ajax()) {
				ob_end_clean();
			}

			foreach ($adminer->rowDescriptions($rows, $foreign_keys) as $n => $row) {
				$unique_array = unique_array($rows[$n], $indexes);
				if (!$unique_array) {
					$unique_array = array();
					foreach ($rows[$n] as $key => $val) {
						if (!preg_match('~^(COUNT\((\*|(DISTINCT )?`(?:[^`]|``)+`)\)|(AVG|GROUP_CONCAT|MAX|MIN|SUM)\(`(?:[^`]|``)+`\))$~', $key)) { //! columns looking like functions
							$unique_array[$key] = $val;
						}
					}
				}
				$unique_idf = "";
				foreach ($unique_array as $key => $val) {
					if ((JUSH == "sql" || JUSH == "pgsql") && preg_match('~char|text|enum|set~', $fields[$key]["type"]) && strlen($val) > 64) {
						$key = (strpos($key, '(') ? $key : idf_escape($key)); //! columns looking like functions
						$key = "MD5(" . (JUSH != 'sql' || preg_match("~^utf8~", $fields[$key]["collation"]) ? $key : "CONVERT($key USING " . charset($connection) . ")") . ")";
						$val = md5($val);
					}
					$unique_idf .= "&" . ($val !== null ? urlencode("where[" . bracket_escape($key) . "]") . "=" . urlencode($val === false ? "f" : $val) : "null%5B%5D=" . urlencode($key));
				}
				echo "<tr>" . (!$group && $select ? "" : "<td>"
					. checkbox("check[]", substr($unique_idf, 1), in_array(substr($unique_idf, 1), (array) $_POST["check"]))
					. ($is_group || information_schema(DB) ? "" : " <a href='" . h(ME . "edit=" . urlencode($TABLE) . $unique_idf) . "' class='edit'>" . lang(264) . "</a>")
				);

				foreach ($row as $key => $val) {
					if (isset($names[$key])) {
						$field = $fields[$key];
						$val = $driver->value($val, $field);
						if ($val != "" && (!isset($email_fields[$key]) || $email_fields[$key] != "")) {
							$email_fields[$key] = (is_mail($val) ? $names[$key] : ""); //! filled e-mails can be contained on other pages
						}

						$link = "";
						if (preg_match('~blob|bytea|raw|file~', $field["type"]) && $val != "") {
							$link = ME . 'download=' . urlencode($TABLE) . '&field=' . urlencode($key) . $unique_idf;
						}
						if (!$link && $val !== null) { // link related items
							foreach ((array) $foreign_keys[$key] as $foreign_key) {
								if (count($foreign_keys[$key]) == 1 || end($foreign_key["source"]) == $key) {
									$link = "";
									foreach ($foreign_key["source"] as $i => $source) {
										$link .= where_link($i, $foreign_key["target"][$i], $rows[$n][$source]);
									}
									$link = ($foreign_key["db"] != "" ? preg_replace('~([?&]db=)[^&]+~', '\1' . urlencode($foreign_key["db"]), ME) : ME) . 'select=' . urlencode($foreign_key["table"]) . $link; // InnoDB supports non-UNIQUE keys
									if ($foreign_key["ns"]) {
										$link = preg_replace('~([?&]ns=)[^&]+~', '\1' . urlencode($foreign_key["ns"]), $link);
									}
									if (count($foreign_key["source"]) == 1) {
										break;
									}
								}
							}
						}
						if ($key == "COUNT(*)") { //! columns looking like functions
							$link = ME . "select=" . urlencode($TABLE);
							$i = 0;
							foreach ((array) $_GET["where"] as $v) {
								if (!array_key_exists($v["col"], $unique_array)) {
									$link .= where_link($i++, $v["col"], $v["val"], $v["op"]);
								}
							}
							foreach ($unique_array as $k => $v) {
								$link .= where_link($i++, $k, $v);
							}
						}

						$val = select_value($val, $link, $field, $text_length);
						$id = h("val[$unique_idf][" . bracket_escape($key) . "]");
						$value = $_POST["val"][$unique_idf][bracket_escape($key)];
						$editable = !is_array($row[$key]) && is_utf8($val) && $rows[$n][$key] == $row[$key] && !$functions[$key] && !$field["generated"];
						$text = preg_match('~text|json|lob~', $field["type"]);
						echo "<td id='$id'" . (preg_match(number_type(), $field["type"]) && is_numeric(strip_tags($val)) ? " class='number'" : "");
						if (($_GET["modify"] && $editable) || $value !== null) {
							$h_value = h($value !== null ? $value : $row[$key]);
							echo ">" . ($text ? "<textarea name='$id' cols='30' rows='" . (substr_count($row[$key], "\n") + 1) . "'>$h_value</textarea>" : "<input name='$id' value='$h_value' size='$lengths[$key]'>");
						} else {
							$long = strpos($val, "<i>…</i>");
							echo " data-text='" . ($long ? 2 : ($text ? 1 : 0)) . "'"
								. ($editable ? "" : " data-warning='" . h(lang(265)) . "'")
								. ">$val"
							;
						}
					}
				}

				if ($backward_keys) {
					echo "<td>";
				}
				$adminer->backwardKeysPrint($backward_keys, $rows[$n]);
				echo "</tr>\n"; // close to allow white-space: pre
			}

			if (is_ajax()) {
				exit;
			}
			echo "</table>\n";
			echo "</div>\n";
		}

		if (!is_ajax()) {
			if ($rows || $page) {
				$exact_count = true;
				if ($_GET["page"] != "last") {
					if ($limit == "" || (count($rows) < $limit && ($rows || !$page))) {
						$found_rows = ($page ? $page * $limit : 0) + count($rows);
					} elseif (JUSH != "sql" || !$is_group) {
						$found_rows = ($is_group ? false : found_rows($table_status, $where));
						if ($found_rows < max(1e4, 2 * ($page + 1) * $limit)) {
							// slow with big tables
							$found_rows = first(slow_query(count_rows($TABLE, $where, $is_group, $group)));
						} else {
							$exact_count = false;
						}
					}
				}

				$pagination = ($limit != "" && ($found_rows === false || $found_rows > $limit || $page));
				if ($pagination) {
					echo (($found_rows === false ? count($rows) + 1 : $found_rows - $page * $limit) > $limit
						? '<p><a href="' . h(remove_from_uri("page") . "&page=" . ($page + 1)) . '" class="loadmore">' . lang(266) . '</a>'
							. script("qsl('a').onclick = partial(selectLoadMore, " . (+$limit) . ", '" . lang(267) . "…');", "")
						: ''
					);
					echo "\n";
				}
			}

			echo "<div class='footer'><div>\n";
			if ($rows || $page) {
				if ($pagination) {
					// display first, previous 4, next 4 and last page
					$max_page = ($found_rows === false
						? $page + (count($rows) >= $limit ? 2 : 1)
						: floor(($found_rows - 1) / $limit)
					);
					echo "<fieldset>";
					if (JUSH != "simpledb") {
						echo "<legend><a href='" . h(remove_from_uri("page")) . "'>" . lang(268) . "</a></legend>";
						echo script("qsl('a').onclick = function () { pageClick(this.href, +prompt('" . lang(268) . "', '" . ($page + 1) . "')); return false; };");
						echo pagination(0, $page) . ($page > 5 ? " …" : "");
						for ($i = max(1, $page - 4); $i < min($max_page, $page + 5); $i++) {
							echo pagination($i, $page);
						}
						if ($max_page > 0) {
							echo ($page + 5 < $max_page ? " …" : "");
							echo ($exact_count && $found_rows !== false
								? pagination($max_page, $page)
								: " <a href='" . h(remove_from_uri("page") . "&page=last") . "' title='~$max_page'>" . lang(269) . "</a>"
							);
						}
					} else {
						echo "<legend>" . lang(268) . "</legend>";
						echo pagination(0, $page) . ($page > 1 ? " …" : "");
						echo ($page ? pagination($page, $page) : "");
						echo ($max_page > $page ? pagination($page + 1, $page) . ($max_page > $page + 1 ? " …" : "") : "");
					}
					echo "</fieldset>\n";
				}

				echo "<fieldset>";
				echo "<legend>" . lang(270) . "</legend>";
				$display_rows = ($exact_count ? "" : "~ ") . $found_rows;
				$onclick = "const checked = formChecked(this, /check/); selectCount('selected', this.checked ? '$display_rows' : checked); selectCount('selected2', this.checked || !checked ? '$display_rows' : checked);";
				echo checkbox("all", 1, 0, ($found_rows !== false ? ($exact_count ? "" : "~ ") . lang(154, $found_rows) : ""), $onclick) . "\n";
				echo "</fieldset>\n";

				if ($adminer->selectCommandPrint()) {
					?>
<fieldset<?php echo ($_GET["modify"] ? '' : ' class="jsonly"'); ?>><legend><?php echo lang(262); ?></legend><div>
<input type="submit" value="<?php echo lang(14); ?>"<?php echo ($_GET["modify"] ? '' : ' title="' . lang(258) . '"'); ?>>
</div></fieldset>
<fieldset><legend><?php echo lang(128); ?> <span id="selected"></span></legend><div>
<input type="submit" name="edit" value="<?php echo lang(10); ?>">
<input type="submit" name="clone" value="<?php echo lang(254); ?>">
<input type="submit" name="delete" value="<?php echo lang(18); ?>"><?php echo confirm(); ?>
</div></fieldset>
<?php
				}

				$format = $adminer->dumpFormat();
				foreach ((array) $_GET["columns"] as $column) {
					if ($column["fun"]) {
						unset($format['sql']);
						break;
					}
				}
				if ($format) {
					print_fieldset("export", lang(72) . " <span id='selected2'></span>");
					$output = $adminer->dumpOutput();
					echo ($output ? html_select("output", $output, $adminer_import["output"]) . " " : "");
					echo html_select("format", $format, $adminer_import["format"]);
					echo " <input type='submit' name='export' value='" . lang(72) . "'>\n";
					echo "</div></fieldset>\n";
				}

				$adminer->selectEmailPrint(array_filter($email_fields, 'strlen'), $columns);
			}

			echo "</div></div>\n";

			if ($adminer->selectImportPrint()) {
				echo "<div>";
				echo "<a href='#import'>" . lang(71) . "</a>";
				echo script("qsl('a').onclick = partial(toggle, 'import');", "");
				echo "<span id='import'" . ($_POST["import"] ? "" : " class='hidden'") . ">: ";
				echo "<input type='file' name='csv_file'> ";
				echo html_select("separator", array("csv" => "CSV,", "csv;" => "CSV;", "tsv" => "TSV"), $adminer_import["format"]);
				echo " <input type='submit' name='import' value='" . lang(71) . "'>";
				echo "</span>";
				echo "</div>";
			}

			echo input_token();
			echo "</form>\n";
			echo (!$group && $select ? "" : script("tableCheck();"));
		}
	}
}

if (is_ajax()) {
	ob_end_clean();
	exit;
}

} elseif (isset($_GET["variables"])) {
	?>
<?php
$status = isset($_GET["status"]);
page_header($status ? lang(119) : lang(118));

$variables = ($status ? show_status() : show_variables());
if (!$variables) {
	echo "<p class='message'>" . lang(12) . "\n";
} else {
	echo "<table>\n";
	foreach ($variables as $row) {
		echo "<tr>";
		$key = array_shift($row);
		echo "<th><code class='jush-" . JUSH . ($status ? "status" : "set") . "'>" . h($key) . "</code>";
		foreach ($row as $val) {
			echo "<td>" . nl_br(h($val));
		}
	}
	echo "</table>\n";
}

} elseif (isset($_GET["script"])) {
	?>
<?php
header("Content-Type: text/javascript; charset=utf-8");

if ($_GET["script"] == "db") {
	$sums = array("Data_length" => 0, "Index_length" => 0, "Data_free" => 0);
	foreach (table_status() as $name => $table_status) {
		json_row("Comment-$name", h($table_status["Comment"]));
		if (!is_view($table_status)) {
			foreach (array("Engine", "Collation") as $key) {
				json_row("$key-$name", h($table_status[$key]));
			}
			foreach ($sums + array("Auto_increment" => 0, "Rows" => 0) as $key => $val) {
				if ($table_status[$key] != "") {
					$val = format_number($table_status[$key]);
					if ($val >= 0) {
						json_row("$key-$name", ($key == "Rows" && $val && $table_status["Engine"] == (JUSH == "pgsql" ? "table" : "InnoDB")
							? "~ $val"
							: $val
						));
					}
					if (isset($sums[$key])) {
						// ignore innodb_file_per_table because it is not active for tables created before it was enabled
						$sums[$key] += ($table_status["Engine"] != "InnoDB" || $key != "Data_free" ? $table_status[$key] : 0);
					}
				} elseif (array_key_exists($key, $table_status)) {
					json_row("$key-$name", "?");
				}
			}
		}
	}
	foreach ($sums as $key => $val) {
		json_row("sum-$key", format_number($val));
	}
	json_row("");

} elseif ($_GET["script"] == "kill") {
	$connection->query("KILL " . number($_POST["kill"]));

} else { // connect
	foreach (count_tables($adminer->databases()) as $db => $val) {
		json_row("tables-$db", $val);
		json_row("size-$db", db_size($db));
	}
	json_row("");
}

exit; // don't print footer

} else {
	?>
<?php
$tables_views = array_merge((array) $_POST["tables"], (array) $_POST["views"]);

if ($tables_views && !$error && !$_POST["search"]) {
	$result = true;
	$message = "";
	if (JUSH == "sql" && $_POST["tables"] && count($_POST["tables"]) > 1 && ($_POST["drop"] || $_POST["truncate"] || $_POST["copy"])) {
		queries("SET foreign_key_checks = 0"); // allows to truncate or drop several tables at once
	}

	if ($_POST["truncate"]) {
		if ($_POST["tables"]) {
			$result = truncate_tables($_POST["tables"]);
		}
		$message = lang(271);
	} elseif ($_POST["move"]) {
		$result = move_tables((array) $_POST["tables"], (array) $_POST["views"], $_POST["target"]);
		$message = lang(272);
	} elseif ($_POST["copy"]) {
		$result = copy_tables((array) $_POST["tables"], (array) $_POST["views"], $_POST["target"]);
		$message = lang(273);
	} elseif ($_POST["drop"]) {
		if ($_POST["views"]) {
			$result = drop_views($_POST["views"]);
		}
		if ($result && $_POST["tables"]) {
			$result = drop_tables($_POST["tables"]);
		}
		$message = lang(274);
	} elseif (JUSH == "sqlite" && $_POST["check"]) {
		foreach ((array) $_POST["tables"] as $table) {
			foreach (get_rows("PRAGMA integrity_check(" . q($table) . ")") as $row) {
				$message .= "<b>" . h($table) . "</b>: " . h($row["integrity_check"]) . "<br>";
			}
		}
	} elseif (JUSH != "sql") {
		$result = (JUSH == "sqlite"
			? queries("VACUUM")
			: apply_queries("VACUUM" . ($_POST["optimize"] ? "" : " ANALYZE"), $_POST["tables"])
		);
		$message = lang(275);
	} elseif (!$_POST["tables"]) {
		$message = lang(9);
	} elseif ($result = queries(($_POST["optimize"] ? "OPTIMIZE" : ($_POST["check"] ? "CHECK" : ($_POST["repair"] ? "REPAIR" : "ANALYZE"))) . " TABLE " . implode(", ", array_map('Adminer\idf_escape', $_POST["tables"])))) {
		while ($row = $result->fetch_assoc()) {
			$message .= "<b>" . h($row["Table"]) . "</b>: " . h($row["Msg_text"]) . "<br>";
		}
	}

	queries_redirect(substr(ME, 0, -1), $message, $result);
}

page_header(($_GET["ns"] == "" ? lang(36) . ": " . h(DB) : lang(75) . ": " . h($_GET["ns"])), $error, true);

if ($adminer->homepage()) {
	if ($_GET["ns"] !== "") {
		echo "<h3 id='tables-views'>" . lang(276) . "</h3>\n";
		$tables_list = tables_list();
		if (!$tables_list) {
			echo "<p class='message'>" . lang(9) . "\n";
		} else {
			echo "<form action='' method='post'>\n";
			if (support("table")) {
				echo "<fieldset><legend>" . lang(277) . " <span id='selected2'></span></legend><div>";
				echo "<input type='search' name='query' value='" . h($_POST["query"]) . "'>";
				echo script("qsl('input').onkeydown = partialArg(bodyKeydown, 'search');", "");
				echo " <input type='submit' name='search' value='" . lang(55) . "'>\n";
				echo "</div></fieldset>\n";
				if ($_POST["search"] && $_POST["query"] != "") {
					$_GET["where"][0]["op"] = $driver->convertOperator("LIKE %%");
					search_tables();
				}
			}
			echo "<div class='scrollable'>\n";
			echo "<table class='nowrap checkable odds'>\n";
			echo script("mixin(qsl('table'), {onclick: tableClick, ondblclick: partialArg(tableClick, true)});");
			echo '<thead><tr class="wrap">';
			echo '<td><input id="check-all" type="checkbox" class="jsonly">' . script("qs('#check-all').onclick = partial(formCheck, /^(tables|views)\[/);", "");
			echo '<th>' . lang(133);
			echo '<td>' . lang(278) . doc_link(array('sql' => 'storage-engines.html'));
			echo '<td>' . lang(124) . doc_link(array('sql' => 'charset-charsets.html', 'mariadb' => 'supported-character-sets-and-collations/'));
			echo '<td>' . lang(279) . doc_link(array('sql' => 'show-table-status.html', 'pgsql' => 'functions-admin.html#FUNCTIONS-ADMIN-DBOBJECT', 'oracle' => 'REFRN20286'));
			echo '<td>' . lang(280) . doc_link(array('sql' => 'show-table-status.html', 'pgsql' => 'functions-admin.html#FUNCTIONS-ADMIN-DBOBJECT'));
			echo '<td>' . lang(281) . doc_link(array('sql' => 'show-table-status.html'));
			echo '<td>' . lang(50) . doc_link(array('sql' => 'example-auto-increment.html', 'mariadb' => 'auto_increment/'));
			echo '<td>' . lang(282) . doc_link(array('sql' => 'show-table-status.html', 'pgsql' => 'catalog-pg-class.html#CATALOG-PG-CLASS', 'oracle' => 'REFRN20286'));
			echo (support("comment") ? '<td>' . lang(49) . doc_link(array('sql' => 'show-table-status.html', 'pgsql' => 'functions-info.html#FUNCTIONS-INFO-COMMENT-TABLE')) : '');
			echo "</thead>\n";

			$tables = 0;
			foreach ($tables_list as $name => $type) {
				$view = ($type !== null && !preg_match('~table|sequence~i', $type));
				$id = h("Table-" . $name);
				echo '<tr><td>' . checkbox(($view ? "views[]" : "tables[]"), $name, in_array($name, $tables_views, true), "", "", "", $id);
				echo '<th>' . (support("table") || support("indexes") ? "<a href='" . h(ME) . "table=" . urlencode($name) . "' title='" . lang(41) . "' id='$id'>" . h($name) . '</a>' : h($name));
				if ($view) {
					echo '<td colspan="6"><a href="' . h(ME) . "view=" . urlencode($name) . '" title="' . lang(42) . '">' . (preg_match('~materialized~i', $type) ? lang(131) : lang(132)) . '</a>';
					echo '<td align="right"><a href="' . h(ME) . "select=" . urlencode($name) . '" title="' . lang(40) . '">?</a>';
				} else {
					foreach (
						array(
							"Engine" => array(),
							"Collation" => array(),
							"Data_length" => array("create", lang(43)),
							"Index_length" => array("indexes", lang(135)),
							"Data_free" => array("edit", lang(44)),
							"Auto_increment" => array("auto_increment=1&create", lang(43)),
							"Rows" => array("select", lang(40)),
						) as $key => $link
					) {
						$id = " id='$key-" . h($name) . "'";
						echo ($link ? "<td align='right'>" . (support("table") || $key == "Rows" || (support("indexes") && $key != "Data_length")
							? "<a href='" . h(ME . "$link[0]=") . urlencode($name) . "'$id title='$link[1]'>?</a>"
							: "<span$id>?</span>"
						) : "<td id='$key-" . h($name) . "'>");
					}
					$tables++;
				}
				echo (support("comment") ? "<td id='Comment-" . h($name) . "'>" : "");
				echo "\n";
			}

			echo "<tr><td><th>" . lang(255, count($tables_list));
			echo "<td>" . h(JUSH == "sql" ? get_val("SELECT @@default_storage_engine") : "");
			echo "<td>" . h(db_collation(DB, collations()));
			foreach (array("Data_length", "Index_length", "Data_free") as $key) {
				echo "<td align='right' id='sum-$key'>";
			}
			echo "\n";

			echo "</table>\n";
			echo "</div>\n";
			if (!information_schema(DB)) {
				echo "<div class='footer'><div>\n";
				$vacuum = "<input type='submit' value='" . lang(283) . "'> " . on_help("'VACUUM'");
				$optimize = "<input type='submit' name='optimize' value='" . lang(284) . "'> " . on_help(JUSH == "sql" ? "'OPTIMIZE TABLE'" : "'VACUUM OPTIMIZE'");
				echo "<fieldset><legend>" . lang(128) . " <span id='selected'></span></legend><div>"
				. (JUSH == "sqlite" ? $vacuum . "<input type='submit' name='check' value='" . lang(285) . "'> " . on_help("'PRAGMA integrity_check'")
				: (JUSH == "pgsql" ? $vacuum . $optimize
				: (JUSH == "sql" ? "<input type='submit' value='" . lang(286) . "'> " . on_help("'ANALYZE TABLE'")
					. $optimize
					. "<input type='submit' name='check' value='" . lang(285) . "'> " . on_help("'CHECK TABLE'")
					. "<input type='submit' name='repair' value='" . lang(287) . "'> " . on_help("'REPAIR TABLE'")
				: "")))
				. "<input type='submit' name='truncate' value='" . lang(288) . "'> " . on_help(JUSH == "sqlite" ? "'DELETE'" : "'TRUNCATE" . (JUSH == "pgsql" ? "'" : " TABLE'")) . confirm()
				. "<input type='submit' name='drop' value='" . lang(129) . "'>" . on_help("'DROP TABLE'") . confirm() . "\n";
				$databases = (support("scheme") ? $adminer->schemas() : $adminer->databases());
				if (count($databases) != 1 && JUSH != "sqlite") {
					$db = (isset($_POST["target"]) ? $_POST["target"] : (support("scheme") ? $_GET["ns"] : DB));
					echo "<p>" . lang(289) . ": ";
					echo ($databases ? html_select("target", $databases, $db) : '<input name="target" value="' . h($db) . '" autocapitalize="off">');
					echo " <input type='submit' name='move' value='" . lang(290) . "'>";
					echo (support("copy") ? " <input type='submit' name='copy' value='" . lang(291) . "'> " . checkbox("overwrite", 1, $_POST["overwrite"], lang(292)) : "");
					echo "\n";
				}
				echo "<input type='hidden' name='all' value=''>"; // used by trCheck()
				echo script("qsl('input').onclick = function () { selectCount('selected', formChecked(this, /^(tables|views)\[/));" . (support("table") ? " selectCount('selected2', formChecked(this, /^tables\[/) || $tables);" : "") . " }");
				echo input_token();
				echo "</div></fieldset>\n";
				echo "</div></div>\n";
			}
			echo "</form>\n";
			echo script("tableCheck();");
		}

		echo "<p class='links'><a href='" . h(ME) . "create='>" . lang(73) . "</a>\n";
		echo (support("view") ? "<a href='" . h(ME) . "view='>" . lang(208) . "</a>\n" : "");

		if (support("routine")) {
			echo "<h3 id='routines'>" . lang(147) . "</h3>\n";
			$routines = routines();
			if ($routines) {
				echo "<table class='odds'>\n";
				echo '<thead><tr><th>' . lang(187) . '<td>' . lang(48) . '<td>' . lang(225) . "<td></thead>\n";
				foreach ($routines as $row) {
					$name = ($row["SPECIFIC_NAME"] == $row["ROUTINE_NAME"] ? "" : "&name=" . urlencode($row["ROUTINE_NAME"])); // not computed on the pages to be able to print the header first
					echo '<tr>';
					echo '<th><a href="' . h(ME . ($row["ROUTINE_TYPE"] != "PROCEDURE" ? 'callf=' : 'call=') . urlencode($row["SPECIFIC_NAME"]) . $name) . '">' . h($row["ROUTINE_NAME"]) . '</a>';
					echo '<td>' . h($row["ROUTINE_TYPE"]);
					echo '<td>' . h($row["DTD_IDENTIFIER"]);
					echo '<td><a href="' . h(ME . ($row["ROUTINE_TYPE"] != "PROCEDURE" ? 'function=' : 'procedure=') . urlencode($row["SPECIFIC_NAME"]) . $name) . '">' . lang(138) . "</a>";
				}
				echo "</table>\n";
			}
			echo '<p class="links">'
				. (support("procedure") ? '<a href="' . h(ME) . 'procedure=">' . lang(224) . '</a>' : '')
				. '<a href="' . h(ME) . 'function=">' . lang(223) . "</a>\n"
			;
		}

		if (support("sequence")) {
			echo "<h3 id='sequences'>" . lang(293) . "</h3>\n";
			$sequences = get_vals("SELECT sequence_name FROM information_schema.sequences WHERE sequence_schema = current_schema() ORDER BY sequence_name");
			if ($sequences) {
				echo "<table class='odds'>\n";
				echo "<thead><tr><th>" . lang(187) . "</thead>\n";
				foreach ($sequences as $val) {
					echo "<tr><th><a href='" . h(ME) . "sequence=" . urlencode($val) . "'>" . h($val) . "</a>\n";
				}
				echo "</table>\n";
			}
			echo "<p class='links'><a href='" . h(ME) . "sequence='>" . lang(230) . "</a>\n";
		}

		if (support("type")) {
			echo "<h3 id='user-types'>" . lang(31) . "</h3>\n";
			$user_types = types();
			if ($user_types) {
				echo "<table class='odds'>\n";
				echo "<thead><tr><th>" . lang(187) . "</thead>\n";
				foreach ($user_types as $val) {
					echo "<tr><th><a href='" . h(ME) . "type=" . urlencode($val) . "'>" . h($val) . "</a>\n";
				}
				echo "</table>\n";
			}
			echo "<p class='links'><a href='" . h(ME) . "type='>" . lang(234) . "</a>\n";
		}

		if (support("event")) {
			echo "<h3 id='events'>" . lang(148) . "</h3>\n";
			$rows = get_rows("SHOW EVENTS");
			if ($rows) {
				echo "<table>\n";
				echo "<thead><tr><th>" . lang(187) . "<td>" . lang(294) . "<td>" . lang(214) . "<td>" . lang(215) . "<td></thead>\n";
				foreach ($rows as $row) {
					echo "<tr>";
					echo "<th>" . h($row["Name"]);
					echo "<td>" . ($row["Execute at"] ? lang(295) . "<td>" . $row["Execute at"] : lang(216) . " " . $row["Interval value"] . " " . $row["Interval field"] . "<td>$row[Starts]");
					echo "<td>$row[Ends]";
					echo '<td><a href="' . h(ME) . 'event=' . urlencode($row["Name"]) . '">' . lang(138) . '</a>';
				}
				echo "</table>\n";
				$event_scheduler = get_val("SELECT @@event_scheduler");
				if ($event_scheduler && $event_scheduler != "ON") {
					echo "<p class='error'><code class='jush-sqlset'>event_scheduler</code>: " . h($event_scheduler) . "\n";
				}
			}
			echo '<p class="links"><a href="' . h(ME) . 'event=">' . lang(213) . "</a>\n";
		}

		if ($tables_list) {
			echo script("ajaxSetHtml('" . js_escape(ME) . "script=db');");
		}
	}
}

}

// each page calls its own page_header(), if the footer should not be called then the page exits
page_footer();
